/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <windows.h>
#pragma warning (push)
#pragma warning(disable : 4005)// for '_stprintf' : macro redefinition in tchar.h
#include <TCHAR.h>
#pragma warning (pop)
#include <stdio.h>
#include "resource.h"
#include "processesinfos.h"

#include "../Tools/Gui/ListView/Listview.h"
#include "../Tools/APIError/APIError.h"
#include "../Tools/Process/Memory/ProcessMemory.h"
#include "../Tools/Process/Dump/DumpToExe.h"
#include "../Tools/Process/Dump/Dump.h"
#include "../Tools/Gui/Dialog/DialogHelper.h"
#include "../Tools/Privilege/Privilege.h"

//////////////////////// functions //////////////////////// 
LRESULT CALLBACK DumperWndProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
void RefreshProcesses();
void RefreshModules(DWORD dwProcessNumber);
void Refresh();
void Dump();
void DumpToExe();
void Resize();
void Resize(RECT* pWinRect);
//////////////////////// globals //////////////////////// 
HINSTANCE mhInstance;
HWND mhWndDialog;
CProcessesInfos* mpCProcessesInfos;
CListview* mpCListviewProcesses;
CListview* mpCListviewModules;
HWND mhWndProcessListview=NULL;
HWND mhWndModulesListview=NULL;
int miOldSelectedProcessPosition=-1;
// listviews columns header
#define NB_PROCESSES_COLUMNS 4
COLUMN_INFO mpColumnInfoProcesses[NB_PROCESSES_COLUMNS]={{_T("Name"),140},{_T("ID"),80},{_T("Entry Point"),80},{_T("Size"),80}};
#define NB_MODULES_COLUMNS 4
COLUMN_INFO mpColumnInfoModules[NB_MODULES_COLUMNS]={{_T("Name"),300},{_T("Base Address"),80},{_T("Entry Point"),80},{_T("Size"),80}};



int WINAPI WinMain(HINSTANCE hInstance,
    HINSTANCE hPrevInstance,
    LPSTR lpCmdLine,
    int nCmdShow
)
{
    UNREFERENCED_PARAMETER(hPrevInstance);
    UNREFERENCED_PARAMETER(lpCmdLine);
    UNREFERENCED_PARAMETER(nCmdShow);

    HWND hwnd=NULL;
    mhInstance=hInstance;

    // to enable Xp style
    InitCommonControls();

    // try to set debug privilege to get more processes info
    CPrivilege* pCPrivilege=new CPrivilege(TRUE);
    pCPrivilege->SetPrivilege(SE_DEBUG_NAME,TRUE);

    mpCProcessesInfos=new CProcessesInfos(TRUE);
    
    DialogBox(hInstance, (LPCTSTR)IDD_DIALOG_DUMPER, hwnd, (DLGPROC)DumperWndProc);

    delete mpCProcessesInfos;
    delete pCPrivilege;
	return 0;
}


LRESULT CALLBACK DumperWndProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
            mhWndDialog=hDlg;
            // load icon
            CDialogHelper::SetIcon(hDlg,IDI_ICON_APP);

            // check first radio
            SendDlgItemMessage(mhWndDialog,IDC_RADIO_NONRAW,(UINT)BM_SETCHECK,BST_CHECKED,0);

            Resize();
           
            mhWndProcessListview=GetDlgItem(hDlg, IDC_LIST_PROCESSES);
            mpCListviewProcesses=new CListview(mhWndProcessListview);
            // full row select,gridline
            mpCListviewProcesses->SetStyle(TRUE,TRUE,FALSE);
            // load listview column names
            mpCListviewProcesses->InitListViewColumns(NB_PROCESSES_COLUMNS,mpColumnInfoProcesses);

            // full row select,gridline
            mhWndModulesListview=GetDlgItem(hDlg, IDC_LIST_MODULES);
            mpCListviewModules=new CListview(mhWndModulesListview);
            mpCListviewModules->SetStyle(TRUE,TRUE,FALSE);
            // load listview column names
            mpCListviewModules->InitListViewColumns(NB_MODULES_COLUMNS,mpColumnInfoModules);

            // load processes
            Refresh();

			return FALSE;
        case WM_SIZING:
            Resize((RECT*)lParam);
            break;
        case WM_SIZE:
            switch (wParam)
            {
            case SIZE_MAXIMIZED:
            case SIZE_RESTORED:
                Resize();
                break;
            }
            break;
		case WM_COMMAND:
            switch(LOWORD(wParam))
            {
            case IDC_BUTTON_EXIT:
            case IDCANCEL:
                delete mpCListviewProcesses;
                delete mpCListviewModules;
                EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
            case IDC_BUTTON_REFRESH:
                Refresh();
                return TRUE;
            case IDC_BUTTON_DUMP:
                Dump();
                return TRUE;
            case IDC_BUTTON_DUMP_TO_EXE:
                DumpToExe();
                return TRUE;
            }
            break;
        case WM_NOTIFY:
            LPNMHDR lpnm;
            lpnm=((LPNMHDR)lParam);
            if(!lpnm)
                return FALSE;
            // notify messages for processes list view
            if (lpnm->hwndFrom==mhWndProcessListview)
            {
                switch(lpnm->code)
                {
                case LVN_ITEMCHANGED:
                case NM_CLICK:
                    int ItemPosition=mpCListviewProcesses->GetSelectedIndex();
                    if (miOldSelectedProcessPosition!=ItemPosition)
                    {
                        miOldSelectedProcessPosition=ItemPosition;
                        RefreshModules(ItemPosition);
                    }
                    break;
                }
            }
            break;
	}
    return FALSE;

}

// refresh processes and modules list in listview
void Refresh()
{
    // retreive processes infos
    mpCProcessesInfos->RetrieveProcessesInfos();
    RefreshProcesses();
}

void RefreshProcesses()
{
    TCHAR** ppc;
    DWORD dwProcessIndex;
    // fill processes listview
    mpCListviewProcesses->Clear();

    ppc=new TCHAR*[4];
    for (dwProcessIndex=0;dwProcessIndex<4;dwProcessIndex++)
        ppc[dwProcessIndex]=new TCHAR[MAX_PATH];
    
    for (dwProcessIndex=0;dwProcessIndex<mpCProcessesInfos->dwNbProcessesSuccessfullyRetrieved;dwProcessIndex++)
    {
        _tcscpy(ppc[0],mpCProcessesInfos->sProcessesInfos[dwProcessIndex].szProcessName);
        _stprintf(ppc[1],_T("%u"),mpCProcessesInfos->sProcessesInfos[dwProcessIndex].dwProcessID);
        _stprintf(ppc[2],_T("0x%.8X"),(DWORD)mpCProcessesInfos->sProcessesInfos[dwProcessIndex].sModulesInfos[0].sModinfo.EntryPoint);
        _stprintf(ppc[3],_T("0x%.8X"),mpCProcessesInfos->sProcessesInfos[dwProcessIndex].sModulesInfos[0].sModinfo.SizeOfImage);
        mpCListviewProcesses->AddItemAndSubItems(4,ppc);
    }    
    for (dwProcessIndex=0;dwProcessIndex<4;dwProcessIndex++)
        delete ppc[dwProcessIndex];
    delete ppc;

    if (mpCProcessesInfos->dwNbProcessesSuccessfullyRetrieved>0)
        RefreshModules(0);
    else // clear module list only
    {
        mpCListviewModules->Clear();
    }
}
void RefreshModules(DWORD dwProcessNumber)
{
    TCHAR** ppc;
    DWORD dwModuleIndex;

    if ((dwProcessNumber>=mpCProcessesInfos->dwNbProcessesSuccessfullyRetrieved))
        return;
    
    // fill modules listview
    mpCListviewModules->Clear();

    ppc=new TCHAR*[4];
    for (dwModuleIndex=0;dwModuleIndex<4;dwModuleIndex++)
        ppc[dwModuleIndex]=new TCHAR[MAX_PATH];
    
    for (dwModuleIndex=0;dwModuleIndex<mpCProcessesInfos->sProcessesInfos[dwProcessNumber].dwNbModules;dwModuleIndex++)
    {
        _tcscpy(ppc[0],mpCProcessesInfos->sProcessesInfos[dwProcessNumber].sModulesInfos[dwModuleIndex].szModuleName);
        _stprintf(ppc[1],_T("0x%.8X"),(DWORD)mpCProcessesInfos->sProcessesInfos[dwProcessNumber].sModulesInfos[dwModuleIndex].sModinfo.lpBaseOfDll);
        _stprintf(ppc[2],_T("0x%.8X"),(DWORD)mpCProcessesInfos->sProcessesInfos[dwProcessNumber].sModulesInfos[dwModuleIndex].sModinfo.EntryPoint);
        _stprintf(ppc[3],_T("0x%.8X"),mpCProcessesInfos->sProcessesInfos[dwProcessNumber].sModulesInfos[dwModuleIndex].sModinfo.SizeOfImage);
        mpCListviewModules->AddItemAndSubItems(4,ppc);
    }    

    for (dwModuleIndex=0;dwModuleIndex<4;dwModuleIndex++)
        delete ppc[dwModuleIndex];
    delete ppc;
}


// Dump selected modules (or modules 0 if none is selected)
void Dump()
{
    int iProcessIndex=mpCListviewProcesses->GetSelectedIndex();
    int iModuleIndex=mpCListviewModules->GetSelectedIndex();

    if (iProcessIndex<0)
    {
        iProcessIndex=0;
        mpCListviewProcesses->SetSelectedIndex(0);
    }
    if (iModuleIndex<0)
    {
        iModuleIndex=0;
        mpCListviewModules->SetSelectedIndex(0);
    }
    CDump::Dump(mpCProcessesInfos->sProcessesInfos[iProcessIndex].dwProcessID,
        mpCProcessesInfos->sProcessesInfos[iProcessIndex].sModulesInfos[iModuleIndex].hModule);
}

void DumpToExe()
{
    if (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_NONRAW)==BST_CHECKED)
        CDumpToExe::RemoveUnusedMemory();
    else
        CDumpToExe::KeepAllAndModifyPe();
}

void Resize()
{
    // resize
    RECT Rect;
    GetWindowRect(mhWndDialog,&Rect);
    Resize(&Rect);
}

#define SPACE_BETWEEN_CONTROLS 5
#define RIGHT_PART_SIZE 165
void Resize(RECT* pWinRect)
{
    HWND hItem;
    RECT Rect;
    RECT RectListProcess;
    RECT RectWindow;
    RECT PreviousRect;

    int iListHeight;
    
    // check min widh and min height
    if ((pWinRect->right-pWinRect->left)<400)
        pWinRect->right=pWinRect->left+400;
    if ((pWinRect->bottom-pWinRect->top)<450)
        pWinRect->bottom=pWinRect->top+450;


    CDialogHelper::GetClientWindowRect(mhWndDialog,mhWndDialog,&RectWindow);

    // comput list height 
    // (WindowHeight-IDC_STATIC_PROCESSES.Height
    //              -IDC_STATIC_MODULES.Height
    //              -IDC_STATIC_PROCESSES.Top
    //              -SPACE_BETWEEN_CONTROLS
    iListHeight=(RectWindow.bottom-RectWindow.top)/2;
    // -IDC_STATIC_PROCESSES.Height-IDC_STATIC_PROCESSES.Top
    hItem=GetDlgItem(mhWndDialog,IDC_STATIC_PROCESSES);
    CDialogHelper::GetClientWindowRect(mhWndDialog,hItem,&Rect);
    iListHeight-=(Rect.bottom-Rect.top)+Rect.top;
    // -IDC_STATIC_MODULES.Height
    hItem=GetDlgItem(mhWndDialog,IDC_STATIC_MODULES);
    CDialogHelper::GetClientWindowRect(mhWndDialog,hItem,&Rect);
    iListHeight-=(Rect.bottom-Rect.top);
    // -SPACE_BETWEEN_CONTROLS
    iListHeight-=SPACE_BETWEEN_CONTROLS;


    // IDC_LIST_PROCESSES
    hItem=GetDlgItem(mhWndDialog,IDC_LIST_PROCESSES);
    CDialogHelper::GetClientWindowRect(mhWndDialog,hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,0,0,
        (RectWindow.right-RectWindow.left)-RIGHT_PART_SIZE-Rect.left*2,iListHeight,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOMOVE);
    CDialogHelper::GetClientWindowRect(mhWndDialog,hItem,&RectListProcess);
    
    // IDC_BUTTON_REFRESH
    hItem=GetDlgItem(mhWndDialog,IDC_BUTTON_REFRESH);
    CDialogHelper::GetClientWindowRect(mhWndDialog,hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,RectListProcess.right+SPACE_BETWEEN_CONTROLS,Rect.top,
        0,0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);
    CDialogHelper::GetClientWindowRect(mhWndDialog,hItem,&PreviousRect);

    // IDC_BUTTON_DUMP
    hItem=GetDlgItem(mhWndDialog,IDC_BUTTON_DUMP);
    CDialogHelper::GetClientWindowRect(mhWndDialog,hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,PreviousRect.left,Rect.top,
        0,0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);

    // IDC_BUTTON_EXIT
    hItem=GetDlgItem(mhWndDialog,IDC_BUTTON_EXIT);
    CDialogHelper::GetClientWindowRect(mhWndDialog,hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,PreviousRect.left,Rect.top,
        0,0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);

    // IDC_STATIC_DUMP_TO_EXE
    hItem=GetDlgItem(mhWndDialog,IDC_STATIC_DUMP_TO_EXE);
    CDialogHelper::GetClientWindowRect(mhWndDialog,hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,RectListProcess.right,Rect.top,
        0,0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);
    CDialogHelper::GetClientWindowRect(mhWndDialog,hItem,&PreviousRect);
    CDialogHelper::Redraw(hItem);

    // IDC_RADIO_NONRAW
    hItem=GetDlgItem(mhWndDialog,IDC_RADIO_NONRAW);
    CDialogHelper::GetClientWindowRect(mhWndDialog,hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,PreviousRect.left+1,Rect.top,
        0,0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);
    CDialogHelper::Redraw(hItem);

    // IDC_RADIO_MODIFY_PE
    hItem=GetDlgItem(mhWndDialog,IDC_RADIO_MODIFY_PE);
    CDialogHelper::GetClientWindowRect(mhWndDialog,hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,PreviousRect.left+1,Rect.top,
        0,0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);
    CDialogHelper::Redraw(hItem);

    // IDC_BUTTON_DUMP_TO_EXE
    hItem=GetDlgItem(mhWndDialog,IDC_BUTTON_DUMP_TO_EXE);
    CDialogHelper::GetClientWindowRect(mhWndDialog,hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,RectListProcess.right+RIGHT_PART_SIZE/2-(Rect.right-Rect.left)/2,Rect.top,
        0,0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);
    CDialogHelper::Redraw(hItem);

    // IDC_STATIC_MODULES
    hItem=GetDlgItem(mhWndDialog,IDC_STATIC_MODULES);
    CDialogHelper::GetClientWindowRect(mhWndDialog,hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,Rect.left,RectListProcess.bottom+SPACE_BETWEEN_CONTROLS,
        0,0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE); 
    CDialogHelper::GetClientWindowRect(mhWndDialog,hItem,&PreviousRect);

    // IDC_LIST_MODULES
    hItem=GetDlgItem(mhWndDialog,IDC_LIST_MODULES);
    CDialogHelper::GetClientWindowRect(mhWndDialog,hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,Rect.left,PreviousRect.bottom,
        (RectWindow.right-RectWindow.left)-Rect.left*2-SPACE_BETWEEN_CONTROLS,iListHeight,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER);
}