/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: class helper for dialog window
//-----------------------------------------------------------------------------

#include "DialogHelper.h"

//-----------------------------------------------------------------------------
// Name: SetIcon
// Object: set dialog big and small icon
// Parameters :
//     in  : HWND hWnd : window handle
//           int IdIconBig : id of big icon
//           int IdIconSmall : id of small icon
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDialogHelper::SetIcon(HWND hWnd,int IdIconBig,int IdIconSmall)
{
    SendMessage(hWnd, WM_SETICON, ICON_BIG,(LPARAM) 
        LoadIcon((HINSTANCE)(GetWindowLongPtr(hWnd, GWLP_HINSTANCE)), 
        MAKEINTRESOURCE(IdIconBig)));
    SendMessage(hWnd, WM_SETICON, ICON_SMALL, (LPARAM) 
        LoadIcon((HINSTANCE)(GetWindowLongPtr(hWnd, GWLP_HINSTANCE)), 
        MAKEINTRESOURCE(IdIconSmall)));
}
//-----------------------------------------------------------------------------
// Name: SetIcon
// Object: set dialog big and small icon
// Parameters :
//     in  : HWND hWnd : window handle
//           int IdIcon : id of big and small icon
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CDialogHelper::SetIcon(HWND hWnd,int IdIcon)
{
    CDialogHelper::SetIcon(hWnd,IdIcon,IdIcon);
}

//-----------------------------------------------------------------------------
// Name: GetClientWindowRect
// Object: translate lpRect from Screen coordinates to Client Coordinates
//          (used for resizing controls)
// Parameters :
//     in  : HWND hDlg : dialog handle
//           HWND hItem : item handle
//     out : LPRECT lpRect : Rect in dialog coordinates
//     return : 
//-----------------------------------------------------------------------------
void CDialogHelper::GetClientWindowRect(HWND hDlg,HWND hItem,LPRECT lpRect)
{
    GetWindowRect(hItem,lpRect);
    POINT p;
    p.x=lpRect->left;
    p.y=lpRect->top;
    ScreenToClient(hDlg,&p);
    lpRect->left=p.x;
    lpRect->top=p.y;
    p.x=lpRect->right;
    p.y=lpRect->bottom;
    ScreenToClient(hDlg,&p);
    lpRect->right=p.x;
    lpRect->bottom=p.y;
}


//-----------------------------------------------------------------------------
// Name: Redraw
// Object: redraw an item
// Parameters :
//     in  : HWND hItem : handle of item to be redraw
//     out : 
//     return : 
//-----------------------------------------------------------------------------
void CDialogHelper::Redraw(HWND hItem)
{
    RedrawWindow( hItem,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_FRAME | RDW_ALLCHILDREN);
}

//-----------------------------------------------------------------------------
// Name: SetTransparency
// Object: set dialog transparency
// Parameters :
//     in  : HWND hWnd : window handle
//           BYTE Opacity : 0 for transparent window, 100 for an opaque window
//     out : 
//     return : 
//-----------------------------------------------------------------------------
void CDialogHelper::SetTransparency(HWND hWnd,BYTE Opacity)
{
    LONG WindowLong;
    WindowLong=GetWindowLong(hWnd, GWL_EXSTYLE);
    if (Opacity<100)
    {
        if ((WindowLong&WS_EX_LAYERED)==0)
            // Set WS_EX_LAYERED on this window 
            SetWindowLong(hWnd,GWL_EXSTYLE, WindowLong | WS_EX_LAYERED);

        // Make this window Opacity% alpha
        SetLayeredWindowAttributes(hWnd, 0, (255 * Opacity) / 100, LWA_ALPHA);
        // Note that the third parameter of SetLayeredWindowAttributes
        // is a value that ranges from 0 to 255, with 0 making the window
        // completely transparent and 255 making it completely opaque.
        // This parameter mimics the more versatile BLENDFUNCTION of the AlphaBlend function.
        RedrawWindow( hWnd,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW );
    }
    else
    {
        // To make this window completely opaque again, 
        // remove the WS_EX_LAYERED bit by calling SetWindowLong and 
        // then ask the window to repaint. Removing the bit is desired 
        // to let the system know that it can free up some memory associated 
        // with layering and redirection. The code might look like this:
        // Remove WS_EX_LAYERED from this window styles
        if ((WindowLong&WS_EX_LAYERED)!=0)
            SetWindowLong(hWnd,GWL_EXSTYLE,WindowLong & ~WS_EX_LAYERED);
        CDialogHelper::Redraw(hWnd);
    }
}

//-----------------------------------------------------------------------------
// Name: Fading
// Object: Make a fading for specified window
// Parameters :
//     in  : HWND hWnd : window handle
//           BYTE BeginOpacity : begin opacity for fading (0 for transparent window, 100 for an opaque window)
//           BYTE EndOpacity : end opacity for fading (0 for transparent window, 100 for an opaque window)
//           BYTE Step : step size between 2 drawn transparancy
//                       (number of step will be abs(EndOpacity-EndOpacity)/Step)
//     out : 
//     return : 
//-----------------------------------------------------------------------------
void CDialogHelper::Fading(HWND hWnd,BYTE BeginOpacity,BYTE EndOpacity,BYTE Step)
{
    BYTE NbStep;
    BYTE CurrentOpacity;
    int StepDelta=Step;
    BeginOpacity=BeginOpacity<100?BeginOpacity:100;
    EndOpacity=EndOpacity<100?EndOpacity:100;
    if (BeginOpacity==EndOpacity)
    {
        CDialogHelper::SetTransparency(hWnd,BeginOpacity);
        return;
    }
    // comput requiered step
    if (BeginOpacity<EndOpacity)
    {
        NbStep=(BYTE)((EndOpacity-BeginOpacity)/StepDelta);
    }
    else
    {
        NbStep=(BYTE)((BeginOpacity-EndOpacity)/StepDelta);
        StepDelta=-StepDelta;
    }
    CurrentOpacity=BeginOpacity;
    for (BYTE cnt=0;cnt<NbStep;cnt++)
    {
        CDialogHelper::SetTransparency(hWnd,CurrentOpacity);
        CurrentOpacity=(BYTE)(CurrentOpacity+StepDelta);
    }
    CDialogHelper::SetTransparency(hWnd,EndOpacity);
}