/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: class helper for listview control
//-----------------------------------------------------------------------------

#include "listview.h"

#pragma message (__FILE__ " Information : Require the ComCtl32.dll version 6 and requires a manifest that specifies that version 6 of the dynamic-link library (DLL) must be used to make SetView method work")

#define CListview_REPLACED_CHAR_ARRAY_SIZE 3
#define CListview_REPLACED_CHAR_MAX_INCREASE 5
TCHAR* pppszReplacedChar[CListview_REPLACED_CHAR_ARRAY_SIZE][2]={{_T("&"),_T("&amp;")},{_T("<"),_T("&lt;")},{_T(">"),_T("&gt;")}};

//-----------------------------------------------------------------------------
// Name: CListview
// Object: Constructor. Assume that the common control DLL is loaded.
// Parameters :
//     in  : HWND hWndListView : handle to listview
//     out :
//     return : 
//-----------------------------------------------------------------------------
CListview::CListview(HWND hWndListView)
{
    this->hWndListView=hWndListView;

    // Ensure that the common control DLL is loaded.
    INITCOMMONCONTROLSEX icex;
    icex.dwSize = sizeof(INITCOMMONCONTROLSEX);
    icex.dwICC  = ICC_LISTVIEW_CLASSES;
    InitCommonControlsEx(&icex);

    hevtUnlocked=CreateEvent(NULL,FALSE,TRUE,NULL);
}
CListview::~CListview()
{
    CloseHandle(hevtUnlocked);
}

//-----------------------------------------------------------------------------
// Name: GetControlHandle
// Object: return the handle to the control
// Parameters :
//     in  : 
//     out :
//     return : HWND handle to the control
//-----------------------------------------------------------------------------
HWND CListview::GetControlHandle()
{
    return this->hWndListView;
}

//-----------------------------------------------------------------------------
// Name: Clear
// Object: Clear the listview.
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CListview::Clear()
{
    WaitForSingleObject(hevtUnlocked,LIST_VIEW_TIMEOUT);
    ListView_DeleteAllItems(this->hWndListView);
    SetEvent(hevtUnlocked);
}

//-----------------------------------------------------------------------------
// Name: InitListViewColumns
// Object: Sets columns headers and some styles.
// Parameters :
//     in  : 
//     out :
//     return : TRUE if successful,FALSE on error
//-----------------------------------------------------------------------------
BOOL CListview::InitListViewColumns(int NbColumns,PCOLUMN_INFO pColumnInfo)
{ 
    LVCOLUMN lvc; 
    int iCol; 

    // Initialize the LVCOLUMN structure.
    // The mask specifies that the format, width, text, and
    // subitem members of the structure are valid. 
    lvc.mask = LVCF_FMT | LVCF_WIDTH | LVCF_TEXT | LVCF_SUBITEM; 
	  
    // Add the columns. 
    for (iCol = 0; iCol < NbColumns; iCol++) 

	{ 
        lvc.iSubItem = iCol;
        lvc.pszText = pColumnInfo[iCol].pcName;
        lvc.cx = pColumnInfo[iCol].iWidth;     // width of column in pixels
        lvc.fmt =pColumnInfo[iCol].iTextAlign;

        if (ListView_InsertColumn(this->hWndListView, iCol,&lvc) == -1) 
            return FALSE; 
    } 
    return TRUE; 
}


//-----------------------------------------------------------------------------
// Name: SetView
// Object: Set listview view style. works only for XP and upper OS
//          require use of commctls 6 or upper
// Parameters :
//     in  : DWORD iView : new view to apply
//     out :
//     return : TRUE if successful,FALSE on error
//-----------------------------------------------------------------------------
DWORD CListview::SetView(DWORD iView)
{
    return ListView_SetView(this->hWndListView,iView);
}

//-----------------------------------------------------------------------------
// Name: AddItemAndSubItems
// Object: Add items and sub items contains in array of string ppcText at the 
//          end of listview
// Parameters :
//     in  : int NbItem : number of string in ppcText
//           TCHAR** ppcText : array of Item/SubItems string 
//     out :
//     return : TRUE if successful,FALSE on error
//-----------------------------------------------------------------------------
BOOL CListview::AddItemAndSubItems(int NbItem,TCHAR** ppcText)
{
    return this->AddItemAndSubItems(NbItem,ppcText,FALSE);
}

//-----------------------------------------------------------------------------
// Name: AddItemAndSubItems
// Object: Add items and sub items contains in array of string ppcText at the 
//          end of listview
// Parameters :
//     in  : int NbItem : number of string in ppcText
//           TCHAR** ppcText : array of Item/SubItems string 
//           BOOL ScrollToItem : TRUE to scroll to inserted item
//     out :
//     return : TRUE if successful,FALSE on error
//-----------------------------------------------------------------------------
BOOL CListview::AddItemAndSubItems(int NbItem,TCHAR** ppcText,BOOL ScrollToItem)
{
    return this->AddItemAndSubItems(NbItem,ppcText,this->GetItemCount(),ScrollToItem);
}

//-----------------------------------------------------------------------------
// Name: AddItemAndSubItems
// Object: Add items and sub items contains in array of string ppcText at  
//          position specified by ItemIndex
// Parameters :
//     in  : int NbItem : number of string in ppcText
//           TCHAR** ppcText : array of Item/SubItems string 
//           int ItemIndex : position in listview to and the new row
//           BOOL ScrollToItem : TRUE to scroll to inserted item
//     out :
//     return : TRUE if successful,FALSE on error
//-----------------------------------------------------------------------------
BOOL CListview::AddItemAndSubItems(int NbItem,TCHAR** ppcText,int ItemIndex,BOOL ScrollToItem)
{
    LVITEM lvI;
    int SubItemIndex;

    WaitForSingleObject(hevtUnlocked,LIST_VIEW_TIMEOUT);
    
    memset(&lvI,0,sizeof(LVITEM));
    lvI.mask = LVIF_TEXT; 
    lvI.iItem=ItemIndex;
	lvI.pszText =ppcText[0];						  
    ItemIndex=ListView_InsertItem(this->hWndListView, &lvI);
    if (ItemIndex<0)
        return FALSE;
    for (SubItemIndex=1;SubItemIndex<NbItem;SubItemIndex++)
    {
        ListView_SetItemText(
                            this->hWndListView,
                            ItemIndex,
                            SubItemIndex,
                            ppcText[SubItemIndex]
                            );
    }

    if (ScrollToItem)
        ListView_EnsureVisible(this->hWndListView,ItemIndex,FALSE);

    SetEvent(hevtUnlocked);
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: GetSelectedIndex
// Object: GetFirst selected item index  
// Parameters :
//     in  :
//     out :
//     return : first selected item index
//-----------------------------------------------------------------------------
int CListview::GetSelectedIndex()
{
    return ListView_GetSelectionMark(this->hWndListView);
}

//-----------------------------------------------------------------------------
// Name: SetSelectedIndex
// Object: Select item at the specified index  
// Parameters :
//     in  : int ItemIndex : index of item to select
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CListview::SetSelectedIndex(int ItemIndex)
{
    ListView_SetItemState (this->hWndListView,
                          ItemIndex,
                          LVIS_FOCUSED | LVIS_SELECTED, // item state
                          0x000F);                      // mask 
}

//-----------------------------------------------------------------------------
// Name: IsItemSelected
// Object: Check if item at the specified index is selected
// Parameters :
//     in  : int ItemIndex : index of item to check
//     out :
//     return : TRUE if item is selected
//-----------------------------------------------------------------------------
BOOL CListview::IsItemSelected(int ItemIndex)
{
    return (ListView_GetItemState(
        this->hWndListView,
        ItemIndex,
        LVIS_SELECTED)==LVIS_SELECTED);
}

//-----------------------------------------------------------------------------
// Name: GetItemText
// Object: get item text
// Parameters :
//     in  : int ItemIndex : index of item
//           int pcTextMaxSize : max buffer size for pcText
//     out : TCHAR* pcText : item text
//     return : 
//-----------------------------------------------------------------------------
void CListview::GetItemText(int ItemIndex,TCHAR* pcText,int pcTextMaxSize)
{
    this->GetItemText(
    ItemIndex,
    0,
    pcText,
    pcTextMaxSize
    );
}
//-----------------------------------------------------------------------------
// Name: GetItemText
// Object: get item text
// Parameters :
//     in  : int ItemIndex : item index
//           int SubItemIndex : sub item index
//           int pcTextMaxSize : max buffer size for pcText
//     out : TCHAR* pcText : subitem text
//     return : 
//-----------------------------------------------------------------------------
void CListview::GetItemText(int ItemIndex,int SubItemIndex,TCHAR* pcText,int pcTextMaxSize)
{
    *pcText=0;
    ListView_GetItemText(
    this->hWndListView,
    ItemIndex,
    SubItemIndex,
    pcText,
    pcTextMaxSize
    );
}

//-----------------------------------------------------------------------------
// Name: SetItemText
// Object: add item if requiered and set item text
// Parameters :
//     in  : int ItemIndex : item index
//           TCHAR* pcText : item text
//     out : 
//     return : 
//-----------------------------------------------------------------------------
void CListview::SetItemText(int ItemIndex,TCHAR* pcText)
{
    this->SetItemText(
    ItemIndex,
    0,
    pcText
    );
}
//-----------------------------------------------------------------------------
// Name: SetItemText
// Object: add item if requiered and set item text
// Parameters :
//     in  : int ItemIndex : item index
//           int SubItemIndex subitem index
//           TCHAR* pcText : subitem text
//     out : 
//     return : 
//-----------------------------------------------------------------------------
void CListview::SetItemText(int ItemIndex,int SubItemIndex,TCHAR* pcText)
{
    LVITEM lvI;
    memset(&lvI,0,sizeof(LVITEM));
    lvI.iItem=ItemIndex;

    // if item dont exist
    if (!ListView_GetItem(this->hWndListView,&lvI))
    {
        //add it
        lvI.mask = LVIF_TEXT; 
        lvI.iItem=ItemIndex;
        ItemIndex=ListView_InsertItem(this->hWndListView, &lvI);
    }

    ListView_SetItemText(
    this->hWndListView,
    ItemIndex,
    SubItemIndex,
    pcText
    );
}

//-----------------------------------------------------------------------------
// Name: GetItemCount
// Object: get number of items
// Parameters :
//     in  : 
//     out : 
//     return : number of items
//-----------------------------------------------------------------------------
int CListview::GetItemCount()
{
    return ListView_GetItemCount(this->hWndListView);
}
//-----------------------------------------------------------------------------
// Name: GetColumnCount
// Object: get number of columns
// Parameters :
//     in  : 
//     out : 
//     return : number of columns
//-----------------------------------------------------------------------------
int CListview::GetColumnCount()
{
    HWND hHeader = ListView_GetHeader(this->hWndListView);
	if (hHeader)
		return Header_GetItemCount(hHeader);
    return -1;
}

//-----------------------------------------------------------------------------
// Name: SetStyle
// Object: set style helper
// Parameters :
//     in  : BOOL bFullRowSelect : true for full row select
//           BOOL bGridLines : true for gridLines
//           BOOL bSendClickNotification : true for sending click notifications
//     out : 
//     return : 
//-----------------------------------------------------------------------------
void CListview::SetStyle(BOOL bFullRowSelect,BOOL bGridLines,BOOL bSendClickNotification)
{   
    DWORD dwStyles=ListView_GetExtendedListViewStyle(this->hWndListView);
    if (bFullRowSelect)
        dwStyles|=LVS_EX_FULLROWSELECT;
    else
        dwStyles&=~LVS_EX_FULLROWSELECT;
    if (bGridLines)
        dwStyles|=LVS_EX_GRIDLINES;
    else
        dwStyles&=~LVS_EX_GRIDLINES;
    if (bSendClickNotification)
    {
        dwStyles|=LVS_EX_ONECLICKACTIVATE;
        dwStyles|=LVS_EX_TWOCLICKACTIVATE;
    }
    else
    {
        dwStyles&=~LVS_EX_ONECLICKACTIVATE;
        dwStyles&=~LVS_EX_TWOCLICKACTIVATE;
    }
    ListView_SetExtendedListViewStyle(this->hWndListView,dwStyles);
}


//-----------------------------------------------------------------------------
// Name: Save
// Object: Save content of the listview. This function shows an interface to user
//         for choosing file
// Parameters :
//     in  : 
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CListview::Save()
{

    TCHAR pszFile[MAX_PATH]=_T("");

    // open dialog
    OPENFILENAME ofn;
    memset(&ofn,0,sizeof (OPENFILENAME));
    ofn.lStructSize=sizeof (OPENFILENAME);
    ofn.hwndOwner=NULL;
    ofn.hInstance=NULL;
    ofn.lpstrFilter=_T("xml\0*.xml\0txt\0*.txt\0");
    ofn.nFilterIndex = 1;
    ofn.Flags=OFN_EXPLORER|OFN_NOREADONLYRETURN|OFN_OVERWRITEPROMPT;
    ofn.lpstrDefExt=_T("xml");
    ofn.lpstrFile=pszFile;
    ofn.nMaxFile=MAX_PATH;

    if (!GetSaveFileName(&ofn))
        return TRUE;

    return this->Save(ofn.lpstrFile);

}

//-----------------------------------------------------------------------------
// Name: Save
// Object: Save content of the listview in the specified file.
//         Save depends of extension name. Save will be done in xml for .xml files
//         else it will be done in text
//         Warning Only saves done in xml can be reloaded
// Parameters :
//     in  : 
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CListview::Save(TCHAR* pszFileName)
{
    int iNbRow=this->GetItemCount();
    int iNbColumn=this->GetColumnCount();

    if (!pszFileName)
        return FALSE;

    if (!*pszFileName)
        return FALSE;

    if (iNbColumn<0)
        return FALSE;

    TCHAR psz[CListview_MAX_STRING_ITEM_SIZE];
    TCHAR pszOut[CListview_REPLACED_CHAR_MAX_INCREASE*CListview_MAX_STRING_ITEM_SIZE];
    BOOL bXML=FALSE;

    // check xml extension
    if (_tcslen(pszFileName)>4)
    {
        if (_tcsicmp(_T(".xml"),&pszFileName[_tcslen(pszFileName)-4])==0)
            bXML=TRUE;
    }

    FILE* hFile=_tfopen(pszFileName,_T("wb+"));
    if (!hFile)
    {
        _tcscpy(psz,_T("Can't create file "));
        _tcscat(psz,pszFileName);
        MessageBox(NULL,psz,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }

#if (defined(UNICODE)||defined(_UNICODE))
    // in unicode mode, write the unicode little endian header (FFFE)
    BYTE pbUnicodeHeader[2]={0xFF,0xFE};
    fwrite(pbUnicodeHeader,2,1,hFile);
#endif

    if (bXML)
    {
        fwrite(_T("<LISTVIEW>"),10*sizeof(TCHAR),1,hFile);
        // for each raw
        for (int cnt=0;cnt<iNbRow;cnt++)
        {
            fwrite(_T("<ROW>"),5*sizeof(TCHAR),1,hFile);
            // for each column
            for (int cnt2=0;cnt2<iNbColumn;cnt2++)
            {
                fwrite(_T("<COLUMN>"),8*sizeof(TCHAR),1,hFile);
                this->GetItemText(cnt,cnt2,psz,CListview_MAX_STRING_ITEM_SIZE);
                this->Convert(psz,pszOut,TRUE);
                fwrite(pszOut,_tcslen(pszOut)*sizeof(TCHAR),1,hFile);
                fwrite(_T("</COLUMN>"),9*sizeof(TCHAR),1,hFile);
            }
            fwrite(_T("</ROW>"),6*sizeof(TCHAR),1,hFile);
        }
        fwrite(_T("</LISTVIEW>"),11*sizeof(TCHAR),1,hFile);
    }
    else // save as text
    {
        // for each raw
        for (int cnt=0;cnt<iNbRow;cnt++)
        {
            // for each column
            for (int cnt2=0;cnt2<iNbColumn;cnt2++)
            {
                this->GetItemText(cnt,cnt2,psz,CListview_MAX_STRING_ITEM_SIZE);
                fwrite(psz,_tcslen(psz)*sizeof(TCHAR),1,hFile);
                // if not last column
                if (cnt2!=(iNbColumn-1))
                {
                    fwrite(_T("\t"),1*sizeof(TCHAR),1,hFile);
                }
            }
            fwrite(_T("\r\n"),2*sizeof(TCHAR),1,hFile);
        }
    }
    fclose(hFile);
    MessageBox(NULL,_T("Save Successfully Completed"),_T("Information"),MB_OK|MB_ICONINFORMATION|MB_TOPMOST);
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: Load
// Object: load content of the file and put it in listview. This function shows 
//         an interface to user for choosing file
// Parameters :
//     in  : 
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CListview::Load()
{
    TCHAR pszFile[MAX_PATH]=_T("");

    // open dialog
    OPENFILENAME ofn;
    memset(&ofn,0,sizeof (OPENFILENAME));
    ofn.lStructSize=sizeof (OPENFILENAME);
    ofn.hwndOwner=NULL;
    ofn.hInstance=NULL;
    ofn.lpstrFilter=_T("xml\0*.xml\0");
    ofn.nFilterIndex = 1;
    ofn.Flags=OFN_EXPLORER|OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST;
    ofn.lpstrDefExt=_T("xml");
    ofn.lpstrFile=pszFile;
    ofn.nMaxFile=MAX_PATH;
    
    if (!GetOpenFileName(&ofn))
        return TRUE;

    return this->Load(ofn.lpstrFile);
}

//-----------------------------------------------------------------------------
// Name: Load
// Object: load content of the file and put it in listview
// Parameters :
//     in  : 
//     out : 
//     return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CListview::Load(TCHAR* pszFileName)
{
    TCHAR pszMsg[MAX_PATH];
    TCHAR* psz=NULL;
    TCHAR* pszTagBegin;
    TCHAR* pszTagEnd;
    TCHAR* pszChildTagEnd;
    TCHAR* pszChildTagBegin;
    TCHAR* pszCurrentFilePos;
    TCHAR* pszOut;

    DWORD dwCurrentRow;
    DWORD dwCurrentColumn;
    DWORD dwpszSize=0;

    char* pcOriginalFile;
    TCHAR* pszFile;
    LONG FileSize;
    TCHAR* AllocatedHeap=NULL;
    BOOL bUnicodeFile=FALSE;

    if (!pszFileName)
        return FALSE;

    if (!*pszFileName)
        return FALSE;

    FILE* hFile=_tfopen(pszFileName,_T("rb"));
    if (!hFile)
    {
        _tcscpy(pszMsg,_T("Can't open file "));
        _tcscat(pszMsg,pszFileName);
        MessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }

    this->Clear();

    // get file size
    fseek(hFile,0,SEEK_END);
    FileSize=ftell(hFile);
    fseek(hFile,0,SEEK_SET);

    // check file size
    if (FileSize<=0)
    {
        fclose(hFile);

        _tcscpy(pszMsg,_T("File "));
        _tcscat(pszMsg,pszFileName);
        _tcscat(pszMsg,_T(" is empty"));
        MessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }

    // put file in memory
    pcOriginalFile=new char[FileSize];
    fread(pcOriginalFile,FileSize,1,hFile);

    // find the file type (ANSI or UNICODE)
    if (FileSize>=2)
    {
        // check FFFE
        if ((((BYTE)pcOriginalFile[0])==0xFF)
            &&(((BYTE)pcOriginalFile[1])==0xFE))
        {
            bUnicodeFile=TRUE;
        }
    }
#if (defined(UNICODE)||defined(_UNICODE))
    // if we work in unicode and the file is in ansi, convert it in unicode
    if (bUnicodeFile)
    {
        // remove signature
        FileSize-=2;
        // adjust file size : file size will now be in TCHAR count no more in bytes
        FileSize/=2;
        // just make a cast 
        pszFile=(TCHAR*)(pcOriginalFile+2);// point after Unicode header
    }
    else
    {
        // convert into unicode
        pszFile=(TCHAR*)HeapAlloc(GetProcessHeap(), 0,(FileSize+1)*2);
        AllocatedHeap=pszFile;
        if (!AllocatedHeap)
        {
            fclose(hFile);
            MessageBox(NULL,_T("Memory allocation error"),_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
            return FALSE;
        }
        if (!MultiByteToWideChar(CP_ACP, 0, (LPCSTR)pcOriginalFile, FileSize,pszFile,FileSize))
        {
            HeapFree(GetProcessHeap(), 0,AllocatedHeap);
            fclose(hFile);

            _sntprintf(pszMsg,2*MAX_PATH,_T("Error converting file %s in unicode"),pszFileName);
            MessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
            
            return FALSE;
        }
        pszFile[FileSize]=0;
    }
#else
    // if we work in ansi and the file is in unicode, convert it in ansi
    if (bUnicodeFile)
    {
        // convert into ansi
        pszFile=(TCHAR*)HeapAlloc(GetProcessHeap(), 0,(FileSize-2)/2+1);// -2 to remove signature
        AllocatedHeap=pszFile;
        if (!AllocatedHeap)
        {
            fclose(hFile);
            MessageBox(NULL,_T("Memory allocation error"),_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
            return FALSE;
        }
        if (!WideCharToMultiByte(CP_ACP, 0,(LPCWSTR)(&((PBYTE)pcOriginalFile)[2]), ((FileSize-2)/2), pszFile, (FileSize-2)/2, NULL, NULL))
        {
            HeapFree(GetProcessHeap(), 0,AllocatedHeap);
            fclose(hFile);

            _sntprintf(pszMsg,2*MAX_PATH,_T("Error converting file %s in ansi"),pszFileName);
            MessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
            
            return FALSE;
        }
        // adjust file size : file size will now be in TCHAR count no more in bytes
        FileSize=(FileSize-2)/2;
        pszFile[FileSize]=0;
    }
    else // file is already in ansi
    {
        pszFile=pcOriginalFile;
    }
#endif


    // search Listview tag
    pszCurrentFilePos=pszFile;
    pszTagBegin=_tcsstr(pszCurrentFilePos,_T("<LISTVIEW>"));
    if (!pszTagBegin)
    {
        fclose(hFile);
        delete pcOriginalFile;
        if (AllocatedHeap)
            HeapFree(GetProcessHeap(), 0,AllocatedHeap);

        _tcscpy(pszMsg,_T("Syntax error in file "));
        _tcscat(pszMsg,pszFileName);
        _tcscat(pszMsg,_T("\r\nNo LISTVIEW Tag found"));
        MessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }
    // go after Listview tag
    pszCurrentFilePos+=10;
    // search end of listview tag
    pszTagEnd=_tcsstr(pszCurrentFilePos,_T("</LISTVIEW>"));
    if (!pszTagEnd)
    {
        fclose(hFile);
        delete pcOriginalFile;
        if (AllocatedHeap)
            HeapFree(GetProcessHeap(), 0,AllocatedHeap);

        _tcscpy(pszMsg,_T("Syntax error in file "));
        _tcscat(pszMsg,pszFileName);
        _tcscat(pszMsg,_T("\r\nNo LISTVIEW end Tag found"));
        MessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }
    // ends string at begin of listviw tag end
    *pszTagEnd=0;

    // loop into row
    pszCurrentFilePos=_tcsstr(pszCurrentFilePos,_T("<ROW>"));
    dwCurrentRow=0;
    while (pszCurrentFilePos>0)
    {
        // go after Row tag
        pszCurrentFilePos+=5;
        // search end of row tag
        pszTagEnd=_tcsstr(pszCurrentFilePos,_T("</ROW>"));
        if (!pszTagEnd)
        {
            fclose(hFile);
            delete pcOriginalFile;
            if (psz)
                delete psz;
            if (AllocatedHeap)
                HeapFree(GetProcessHeap(), 0,AllocatedHeap);

            _tcscpy(pszMsg,_T("Syntax error in file "));
            _tcscat(pszMsg,pszFileName);
            _tcscat(pszMsg,_T("\r\nNo ROW end Tag found"));
            MessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
            return FALSE;
        }
        // search begin of column tag
        pszChildTagBegin=_tcsstr(pszCurrentFilePos,_T("<COLUMN>"));
        // loop into columns
        dwCurrentColumn=0;
        while ((pszTagEnd>pszChildTagBegin)&&(pszChildTagBegin))
        {
            // go after column tag
            pszCurrentFilePos+=8;
            // search end of column tag
            pszChildTagEnd=_tcsstr(pszCurrentFilePos,_T("</COLUMN>"));
            if(!pszChildTagEnd)
            {
                fclose(hFile);
                delete pcOriginalFile;
                if (psz)
                    delete psz;
                if (AllocatedHeap)
                    HeapFree(GetProcessHeap(), 0,AllocatedHeap);

                _tcscpy(pszMsg,_T("Syntax error in file "));
                _tcscat(pszMsg,pszFileName);
                _tcscat(pszMsg,_T("\r\nNo COLUMN end Tag found"));
                MessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                return FALSE;
            }
            if ((DWORD)(pszChildTagEnd-pszCurrentFilePos+1)>dwpszSize)
            {
                // if already allocated
                if (psz)
                    delete psz;

                // allocate new size
                dwpszSize=(DWORD)(pszChildTagEnd-pszCurrentFilePos+1);
                psz=new TCHAR[dwpszSize];
            }
            _tcsncpy(psz,pszCurrentFilePos,pszChildTagEnd-pszCurrentFilePos);
            psz[pszChildTagEnd-pszCurrentFilePos]=0;
            pszOut=new TCHAR[pszChildTagEnd-pszCurrentFilePos+1];
            // convert xml-like back to normal text
            this->Convert(psz,pszOut,FALSE);
            this->SetItemText(dwCurrentRow,dwCurrentColumn,pszOut);
            delete pszOut;

            // go after Column end tag 
            pszCurrentFilePos=pszChildTagEnd+9;
            // search begin of next column tag
            pszChildTagBegin=_tcsstr(pszCurrentFilePos,_T("<COLUMN>"));
            dwCurrentColumn++;
        } // end of column loop

        // go after row tag end
        pszCurrentFilePos=pszTagEnd+6;
        // search next row
        pszCurrentFilePos=_tcsstr(pszCurrentFilePos,_T("<ROW>"));
        dwCurrentRow++;
    } // end of row loop

    fclose(hFile);

    delete pcOriginalFile;
    if (psz)
        delete psz;
    if (AllocatedHeap)
        HeapFree(GetProcessHeap(), 0,AllocatedHeap);

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: Convert
// Object: convert string in xml like or xml like to string depending bToXML
//          Warning pszOutputString must be large enought no check is done
// Parameters :
//     in  : TCHAR* pszInputString : string to translate
//           BOOL bToXML : TRUE to convert to xml, FALSE to convert from XML
//     out : TCHAR* pszOutputString : result string
//     return : 
//-----------------------------------------------------------------------------
void CListview::Convert(TCHAR* pszInputString,TCHAR* pszOutputString,BOOL bToXML)
{
    if ((!pszInputString)||(!pszOutputString))
        return;

    int IndexOriginalString;
    int IndexReplaceString;

    if (bToXML)
    {
        IndexOriginalString=0;
        IndexReplaceString=1;
    }
    else
    {
        IndexOriginalString=1;
        IndexReplaceString=0;
    }
    // replace all strings defined in ppszReplacedChar
    for (int cnt=0;cnt<CListview_REPLACED_CHAR_ARRAY_SIZE;cnt++)
    {
        StrReplace(
            pszInputString,
            pszOutputString,
            pppszReplacedChar[cnt][IndexOriginalString],
            pppszReplacedChar[cnt][IndexReplaceString]
            );
        _tcscpy(pszInputString,pszOutputString);
    }
}

//-----------------------------------------------------------------------------
// Name: StrReplace
// Object: replace pszOldStr by pszNewStr in pszInputString and put the result in 
//         pszOutputString.
//         Warning pszOutputString must be large enought no check is done
// Parameters :
//     in  : TCHAR* pszInputString : string to translate
//           TCHAR* pszOldStr : string to replace
//           TCHAR* pszNewStr : replacing string
//     out : TCHAR* pszOutputString : result string
//     return : 
//-----------------------------------------------------------------------------
void CListview::StrReplace(TCHAR* pszInputString,TCHAR* pszOutputString,TCHAR* pszOldStr,TCHAR* pszNewStr)
{
    TCHAR* pszPos;
    TCHAR* pszOldPos;
    int SearchedItemSize;

    if ((!pszInputString)||(!pszOutputString)||(!pszOldStr)||(!pszNewStr))
        return;

    *pszOutputString=0;

    pszOldPos=pszInputString;
    // get searched item size
    SearchedItemSize=(int)_tcslen(pszOldStr);
    // look for next string to replace
    pszPos=_tcsstr(pszInputString,pszOldStr);
    while(pszPos)
    {
        // copy unchanged data
        _tcsncat(pszOutputString,pszOldPos,pszPos-pszOldPos);
        // copy replace string
        _tcscat(pszOutputString,pszNewStr);
        // update old position
        pszOldPos=pszPos+SearchedItemSize;
        // look for next string to replace
        pszPos=_tcsstr(pszOldPos,pszOldStr);
    }
    // copy remaining data
    _tcscat(pszOutputString,pszOldPos);
}