/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: class helper for doing api override or api monitoring
//         it manages the apioverride.dll
//         (It's the hart of project winapioverride)
//-----------------------------------------------------------------------------

#include "apioverride.h"

void CApiOverride::Initialize()
{
    TCHAR psz[MAX_PATH];
    TCHAR* pszLastSep;
    this->pCallBackUnexpectedUnloadFunc=NULL;
    this->pCallBackUnexpectedUnloadFuncUserParam=NULL;
    this->pCallBackLogFunc=NULL;
    this->pCallBackLogFuncUserParam=NULL;
    this->pListview=NULL;
    this->dwCurrentProcessId=0;
    this->bAPIOverrideDllLoaded=FALSE;
    this->bAPIOverrideDLLUnloadAsked=FALSE;

    this->hThreadWatchingEvents=NULL;
    this->hThreadLogging=NULL;

    this->pMailSlotServer=NULL;
    this->pMailSlotClient=NULL;

    this->hevtStartMonitoring=NULL;
    this->hevtStopMonitoring=NULL;
    this->hevtStartFaking=NULL;
    this->hevtStopFaking=NULL;
    this->hevtFreeProcess=NULL;

    this->hevtAPIOverrideDllProcessAttachCompleted=NULL;
    this->hevtAPIOverrideDllProcessDetachCompleted=NULL;
    this->hevtProcessFree=NULL;
    this->hevtMonitoringFileLoaded=NULL;
    this->hevtMonitoringFileUnloaded=NULL;
    this->hevtFakeAPIDLLLoaded=NULL;
    this->hevtFakeAPIDLLUnloaded=NULL;
    this->hevtError=NULL;

    this->ProcessInternalCallReply=NULL;
    this->hevtProcessInternalCallReply=CreateEvent(NULL,FALSE,FALSE,NULL);

    this->pFifo=new CFIFO(sizeof(tagLogEntry));
    this->hSemaphoreLog=CreateSemaphore(NULL,0,1024,NULL);
    this->bExternalCListView=FALSE;

    *this->ProcessName=0;

    // get current application directory
    GetModuleFileName(GetModuleHandle(NULL),pszAppPath,MAX_PATH);
    pszLastSep=_tcsrchr(pszAppPath,'\\');
    if (pszLastSep)
        *(pszLastSep+1)=0;// keep \

    // load injectlib dll
    _tcscpy(psz,this->pszAppPath);
    _tcscat(psz,INJECTLIB_DLL_NAME);
    this->hmodInjlib=LoadLibrary(psz);
    this->pInjectLib=(InjectLib)GetProcAddress(this->hmodInjlib,INJECTLIB_FUNC_NAME);
    this->pEjectLib=(EjectLib)GetProcAddress(this->hmodInjlib,EJECTLIB_FUNC_NAME);
}

//-----------------------------------------------------------------------------
// Name: CApiOverride
// Object: Constructor 
//         use SetMonitoringCallback or SetMonitoringListview next to monitor hooks
// Parameters :
//     in : 
// Return : 
//-----------------------------------------------------------------------------
CApiOverride::CApiOverride()
{
    this->Initialize();
}

//-----------------------------------------------------------------------------
// Name: CApiOverride
// Object: Constructor to manage yourself logging event
// Parameters :
//     in : tagCallBackLogFunc pCallBackLogFunc : monitoring callback 
//          warning we use mail slot so callback can be called few seconds after real function call
//          for real time function hooking just use a dll (see fake API dll sample)
// Return : 
//-----------------------------------------------------------------------------
CApiOverride::CApiOverride(tagCallBackLogFunc pCallBackLogFunc)
{
    this->Initialize();
    this->SetMonitoringCallback(pCallBackLogFunc,NULL);
}

//-----------------------------------------------------------------------------
// Name: CApiOverride
// Object: Constructor. Listview will be configured automatically, and it will be filled by monitoring events
// Parameters :
//     in : HWND hListView: Handle to a list view
// Return : 
//-----------------------------------------------------------------------------
CApiOverride::CApiOverride(HWND hListView)
{
    this->Initialize();
    this->SetMonitoringListview(hListView);
}


CApiOverride::~CApiOverride(void)
{
    this->Stop();

    if (this->pFifo)
    {
        delete this->pFifo;
        this->pFifo=NULL;
    }
    CleanCloseHandle(&this->hSemaphoreLog);

    if (this->pListview)
    {
        if (!this->bExternalCListView)
            delete this->pListview;
        this->pListview=NULL;
    }

    // unload injectlib dll
    this->pInjectLib=NULL;
    this->pEjectLib=NULL;
    if (this->hmodInjlib)
    {
        FreeLibrary(this->hmodInjlib);
    }

    CleanCloseHandle(&this->hevtProcessInternalCallReply);
    if (this->ProcessInternalCallReply)
        delete this->ProcessInternalCallReply;
}


//-----------------------------------------------------------------------------
// Name: ReportError
// Object: Show an error message merging current process number
// Parameters :
//      in : - TCHAR* pszErrorMessage : error message without process number
// Return : 
//-----------------------------------------------------------------------------
void CApiOverride::ReportError(TCHAR* pszErrorMessage)
{
    TCHAR pszMsg[2*MAX_PATH];
    if (this->dwCurrentProcessId!=0)
    {
        _sntprintf(pszMsg,2*MAX_PATH,_T("%s for process ID 0x%x"),pszErrorMessage,this->dwCurrentProcessId);
        if (*this->ProcessName)
        {
            _tcscat(pszMsg,_T(" ("));
            _tcscat(pszMsg,this->ProcessName);
            _tcscat(pszMsg,_T(")"));
        }
#if (!defined(TOOLS_NO_MESSAGEBOX))
        MessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
#endif
    }
    else
    {
#if (!defined(TOOLS_NO_MESSAGEBOX))
        MessageBox(NULL,pszErrorMessage,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
#endif
    }
}


//-----------------------------------------------------------------------------
// Name: GetProcessID
// Object: return the process ID with which CApioverride is working or has worked at last
// Parameters :
// Return : PID if CAPIOverride
//-----------------------------------------------------------------------------
DWORD CApiOverride::GetProcessID()
{
    return this->dwCurrentProcessId;
}

//-----------------------------------------------------------------------------
// Name: GetProcessName
// Object: return the process name with which CApioverride is working or has worked at last
// Parameters :
//      in : int ProcessNameMaxSize : max size of ProcessName in tchar
//      out: TCHAR* ProcessName : process name
// Return : TRUE on success
//-----------------------------------------------------------------------------
BOOL CApiOverride::GetProcessName(TCHAR* ProcessName,int ProcessNameMaxSize)
{
    if (IsBadWritePtr(ProcessName,ProcessNameMaxSize*sizeof(TCHAR)))
        return FALSE;

    _tcsncpy(ProcessName,this->ProcessName,ProcessNameMaxSize);

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: SetMonitoringCallback
// Object: Let you manage yourself logging event
// Parameters :
//    in : - tagCallBackLogFunc pCallBackLogFunc : monitoring callback 
//          warning we use mail slot so callback can be called few seconds after real function call
//          for real time function hooking just use a dll (see fake API dll sample)
//          if you want to stop callback call, just call SetMonitoringCallback with a NULL parameter
//         - LPVOID pUserParam : parameter for the callback
// Return : 
//-----------------------------------------------------------------------------
void CApiOverride::SetMonitoringCallback(tagCallBackLogFunc pCallBackLogFunc,LPVOID pUserParam)
{
    this->pCallBackLogFunc=pCallBackLogFunc;
    this->pCallBackLogFuncUserParam=pUserParam;
}

//-----------------------------------------------------------------------------
// Name: SetMonitoringListview
// Object: Listview will be configured automatically, and it will be field by monitoring events
//         you don't need to manage yourself logging events 
// Parameters :
//     in : HWND hListView: Handle to a list view
// Return : 
//-----------------------------------------------------------------------------
void CApiOverride::SetMonitoringListview(HWND hListView)
{
    if (this->pListview)
    {
        if (!this->bExternalCListView)
            delete this->pListview;
    }
    if (hListView==NULL)
    {
        this->pListview=NULL;
        return;
    }
    // init list view monitoring
    COLUMN_INFO mpColumnInfoProcesses[6]={  {_T("Dir"),30,LVCFMT_LEFT},
                                            {_T("Call"),360,LVCFMT_LEFT},
                                            {_T("Ret Value"),80,LVCFMT_CENTER},
                                            {_T("Caller Addr"),80,LVCFMT_CENTER},
                                            {_T("ProcessID"),80,LVCFMT_CENTER},
                                            {_T("Module Name"),100,LVCFMT_LEFT}
                                          };

    this->pListview=new CListview(hListView);
    this->pListview->SetStyle(TRUE,TRUE,FALSE);
    this->pListview->SetView(LV_VIEW_DETAILS);
    this->pListview->InitListViewColumns(6,mpColumnInfoProcesses);
}

//-----------------------------------------------------------------------------
// Name: SetMonitoringListview
// Object: Listview will be field by monitoring events
//         you don't need to manage yourself logging events
// Parameters :
//     in : CListview pListView: CListview object (warning it's not the MFC one)
// Return : 
//-----------------------------------------------------------------------------
void CApiOverride::SetMonitoringListview(CListview* pListView)
{
    this->bExternalCListView=TRUE;
    this->pListview=pListView;
}

//-----------------------------------------------------------------------------
// Name: LoadMonitoringFile
// Object: start monitoring API hooked by the file (multiple files can be hooked at the same time)
// Parameters :
//     in : TCHAR* pszFileName: api monitoring file
// Return : TRUE if file is partially loaded (even if some non fatal errors occurs)
//          and so needs a call to UnloadMonitoringFile to restore all hooked func,
//          FALSE if file not loaded at all
//-----------------------------------------------------------------------------
BOOL CApiOverride::LoadMonitoringFile(TCHAR* pszFileName)
{
    if (!this->bAPIOverrideDllLoaded)
    {
        this->ShowApiOverrideNotStartedMsg();
        return FALSE;
    }

    HANDLE pH[4]={this->hevtMonitoringFileLoaded,this->hevtError,this->hevtFreeProcess,this->hevtProcessFree};
    STRUCT_COMMAND Cmd;
    Cmd.dwCommand_ID=CMD_LOAD_MONITORING_FILE;
    _tcsncpy(Cmd.pszStringParam,pszFileName,MAX_PATH);
    // reset event in case of previous timeout
    ResetEvent(pH[0]);
    ResetEvent(pH[1]);
    if (!this->pMailSlotClient->Write(&Cmd,sizeof(STRUCT_COMMAND)))
    {
        ReportError(_T("Load Monitoring File error writing in MailSlot client"));
        return FALSE;
    }
    DWORD dwRet=WaitForMultipleObjects(4,pH,FALSE,TIME_REQUIERED_TO_LOAD);
    if (dwRet==WAIT_TIMEOUT)
    {
        ReportError(_T("Timeout Error for loading"));
        return FALSE;
    }

    if (dwRet!=WAIT_OBJECT_0)
        return FALSE;

    return TRUE;
}
//-----------------------------------------------------------------------------
// Name: UnloadMonitoringFile
// Object: stop monitoring API hooked by the file
// Parameters :
//     in : TCHAR* pszFileName: api monitoring file
// Return : FALSE on error, TRUE if success
//-----------------------------------------------------------------------------
BOOL CApiOverride::UnloadMonitoringFile(TCHAR* pszFileName)
{
    if (!this->bAPIOverrideDllLoaded)
    {
        this->ShowApiOverrideNotStartedMsg();
        return FALSE;
    }

    HANDLE pH[4]={this->hevtMonitoringFileUnloaded,this->hevtError,this->hevtFreeProcess,this->hevtProcessFree};
    STRUCT_COMMAND Cmd;
    Cmd.dwCommand_ID=CMD_UNLOAD_MONITORING_FILE;
    _tcsncpy(Cmd.pszStringParam,pszFileName,MAX_PATH);
    // reset event in case of previous timeout
    ResetEvent(pH[0]);
    ResetEvent(pH[1]);
    if (!this->pMailSlotClient->Write(&Cmd,sizeof(STRUCT_COMMAND)))
    {
        ReportError(_T("Unload Monitoring File Error writing in MailSlot client"));
        return FALSE;
    }
    DWORD dwRet=WaitForMultipleObjects(4,pH,FALSE,TIME_REQUIERED_TO_UNLOAD);
    if (dwRet==WAIT_TIMEOUT)
    {
        ReportError(_T("Timeout Error for unloading"));
        return FALSE;
    }

    if (dwRet!=WAIT_OBJECT_0)
        return FALSE;
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: LoadFakeAPI
// Object: load dll and start faking api/func specified in the specified in the dll
//         (see the Fake API sample for more infos on specifying API in dll)
//         multiple fake library can be hooked at the same time
// Parameters :
//     in : TCHAR* pszFileName: fake api dll name
// Return : TRUE if library is loaded (even if some non fatal errors occurs)
//          and so needs a call to UnloadFakeAPI to restore all hooked func,
//          FALSE if library not loaded 
//-----------------------------------------------------------------------------
BOOL CApiOverride::LoadFakeAPI(TCHAR* pszFileName)
{
    if (!this->bAPIOverrideDllLoaded)
    {
        this->ShowApiOverrideNotStartedMsg();
        return FALSE;
    }

    HANDLE pH[4]={this->hevtFakeAPIDLLLoaded,this->hevtError,this->hevtFreeProcess,this->hevtProcessFree};
    STRUCT_COMMAND Cmd;
    Cmd.dwCommand_ID=CMD_LOAD_FAKE_API_DLL;
    _tcsncpy(Cmd.pszStringParam,pszFileName,MAX_PATH);
    // reset event in case of previous timeout
    ResetEvent(pH[0]);
    ResetEvent(pH[1]);
    if (!this->pMailSlotClient->Write(&Cmd,sizeof(STRUCT_COMMAND)))
    {
        ReportError(_T("Load Fake API error writing in MailSlot client"));
        return FALSE;
    }
    DWORD dwRet=WaitForMultipleObjects(4,pH,FALSE,TIME_REQUIERED_TO_LOAD);
    if (dwRet==WAIT_TIMEOUT)
    {
        ReportError(_T("Timeout Error for loading"));
        return FALSE;
    }

    if (dwRet!=WAIT_OBJECT_0)
        return FALSE;
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: UnloadFakeAPI
// Object: stop faking api/func hooked by the dll before unloading this dll
// Parameters :
//     in : TCHAR* pszFileName: fake api dll name
// Return : FALSE on error, TRUE if success
//-----------------------------------------------------------------------------
BOOL CApiOverride::UnloadFakeAPI(TCHAR* pszFileName)
{
    if (!this->bAPIOverrideDllLoaded)
    {
        this->ShowApiOverrideNotStartedMsg();
        return FALSE;
    }

    HANDLE pH[4]={this->hevtFakeAPIDLLUnloaded,this->hevtError,this->hevtFreeProcess,this->hevtProcessFree};
    STRUCT_COMMAND Cmd;
    Cmd.dwCommand_ID=CMD_UNLOAD_FAKE_API_DLL;
    _tcsncpy(Cmd.pszStringParam,pszFileName,MAX_PATH);
    // reset event in case of previous timeout
    ResetEvent(pH[0]);
    ResetEvent(pH[1]);
    if (!this->pMailSlotClient->Write(&Cmd,sizeof(STRUCT_COMMAND)))
    {
        ReportError(_T("Unload Fake API error writing in MailSlot client"));
        return FALSE;
    }

    DWORD dwRet=WaitForMultipleObjects(4,pH,FALSE,TIME_REQUIERED_TO_UNLOAD);
    if (dwRet==WAIT_TIMEOUT)
    {
        ReportError(_T("Timeout Error for unloading"));
        return FALSE;
    }

    if (dwRet!=WAIT_OBJECT_0)
        return FALSE;
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: StartMonitoring
// Object: restore monitoring until the next StopMonitoring
//         API Override dll do monitoring by default (at start up)
//         So you only need to call this function after a StopMonitoring call
// Parameters :
//     in : 
// Return : TRUE on Success
//-----------------------------------------------------------------------------
BOOL CApiOverride::StartMonitoring()
{
    if (!this->bAPIOverrideDllLoaded)
    {
        this->ShowApiOverrideNotStartedMsg();
        return FALSE;
    }
    SetEvent(this->hevtStartMonitoring);
    return TRUE;
}
//-----------------------------------------------------------------------------
// Name: StopMonitoring
// Object: Temporary stop monitoring until the next StartMonitoring call
// Parameters :
//     in : 
// Return : TRUE on Success
//-----------------------------------------------------------------------------
BOOL CApiOverride::StopMonitoring()
{
    if (!this->bAPIOverrideDllLoaded)
    {
        this->ShowApiOverrideNotStartedMsg();
        return FALSE;
    }
    SetEvent(this->hevtStopMonitoring);
    return TRUE;
}
//-----------------------------------------------------------------------------
// Name: StartFaking
// Object: restore faking until the next StopFaking
//         API Override dll do faking by default (at start up)
//         So you only need to call this function after a StopFaking call
// Parameters :
//     in : 
// Return : TRUE on Success
//-----------------------------------------------------------------------------
BOOL CApiOverride::StartFaking()
{
    if (!this->bAPIOverrideDllLoaded)
    {
        this->ShowApiOverrideNotStartedMsg();
        return FALSE;
    }
    SetEvent(this->hevtStartFaking);
    return TRUE;
}
//-----------------------------------------------------------------------------
// Name: StopFaking
// Object: Temporary stop faking until the next StartFaking call
// Parameters :
//     in : 
// Return : TRUE on Success
//-----------------------------------------------------------------------------
BOOL CApiOverride::StopFaking()
{
    if (!this->bAPIOverrideDllLoaded)
    {
        this->ShowApiOverrideNotStartedMsg();
        return FALSE;
    }
    SetEvent(this->hevtStopFaking);
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: Stop
// Object: stop monitoring and faking and eject all dll of the current used process
// Parameters :
//     in : 
// Return : FALSE on error, TRUE if success
//-----------------------------------------------------------------------------
BOOL CApiOverride::Stop()
{
    TCHAR psz[2*MAX_PATH];
    int iMsgBoxRes=IDYES;
    DWORD dwRes;

    // put flag signaling we ask for unload
    this->bAPIOverrideDLLUnloadAsked=TRUE;

    // check if we are started (if not we try to remove all may allocated res in case of start failure)
    if (this->bAPIOverrideDllLoaded)
    {
        ResetEvent(this->hevtProcessFree);
        // ask the injected dll to stop its job
        SetEvent(this->hevtFreeProcess);

        // if api override is loaded wait for is read to be unloaded event
        if (this->bAPIOverrideDllLoaded)
        {
            while(iMsgBoxRes==IDYES)
            {
                
                dwRes=WaitForSingleObject(this->hevtProcessFree,TIME_REQUIERED_TO_UNLOAD);
                
                if (dwRes!=WAIT_OBJECT_0)
                {
                    // put the user aware and hope he will do actions that unlock fake api
                    // a good sample of blocking fake api are MessageBox
                    _stprintf(psz,_T("Warning %s seems to be still not unloaded from host process 0x%x"),
                                    API_OVERRIDE_DLL_NAME,this->dwCurrentProcessId);
                    if (this->ProcessName)
                    {
                        _tcscat(psz,_T(" ("));
                        _tcscat(psz,this->ProcessName);
                        _tcscat(psz,_T(")"));
                    }
                    _tcscat(psz,_T("\r\nAssume that Fake API are not in a blocking state.\r\nDo you want to wait more time ?"));
#if (!defined(TOOLS_NO_MESSAGEBOX))
                    iMsgBoxRes=MessageBox(NULL,psz,_T("Warning"),MB_YESNO|MB_ICONWARNING|MB_TOPMOST);
#else
                    iMsgBoxRes=IDNO;
#endif
                }
                else
                    break;
            }
        }
        // eject apioverride DLL
        if (this->pEjectLib)
        {
            _tcscpy(psz,this->pszAppPath);
            _tcscat(psz,API_OVERRIDE_DLL_NAME);
            this->pEjectLib(this->dwCurrentProcessId,psz);
        }
    }

    if (this->pMailSlotServer)
    {
        delete this->pMailSlotServer;
        this->pMailSlotServer=NULL;
    }
    if (this->pMailSlotClient)
    {
        delete this->pMailSlotClient;
        this->pMailSlotClient=NULL;
    }


    // hevtAPIOverrideDllProcessDetachCompleted SHOULD have been send,
    // but as we are waiting for INFINITE, and we want to be sure all thread are closed,
    // just send event again
    SetEvent(this->hevtAPIOverrideDllProcessDetachCompleted);
    // close handle associated with thread
    CleanCloseHandle(&this->hThreadWatchingEvents);
    CleanCloseHandle(&this->hThreadLogging);

    // close events handle
    CleanCloseHandle(&this->hevtStartMonitoring);
    CleanCloseHandle(&this->hevtStopMonitoring);
    CleanCloseHandle(&this->hevtStartFaking);
    CleanCloseHandle(&this->hevtStopFaking);
    CleanCloseHandle(&this->hevtFreeProcess);

    CleanCloseHandle(&this->hevtAPIOverrideDllProcessAttachCompleted);
    CleanCloseHandle(&this->hevtAPIOverrideDllProcessDetachCompleted);
    CleanCloseHandle(&this->hevtProcessFree);
    CleanCloseHandle(&this->hevtMonitoringFileLoaded);
    CleanCloseHandle(&this->hevtMonitoringFileUnloaded);
    CleanCloseHandle(&this->hevtFakeAPIDLLLoaded);
    CleanCloseHandle(&this->hevtFakeAPIDLLUnloaded);
    CleanCloseHandle(&this->hevtError);

    // set unloaded state
    this->bAPIOverrideDllLoaded=FALSE;

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: Start
// Object: inject API Override dll in selected process ID to allow monitoring and faking
// Parameters :
//     in : DWORD dwPID
// Return : FALSE on error, TRUE if success
//-----------------------------------------------------------------------------
BOOL CApiOverride::Start(DWORD dwPID)
{
    TCHAR psz[MAX_PATH];
    TCHAR pszMsg[MAX_PATH];
    TCHAR pszPID[9];
    HANDLE pH[4];

    // store pid
    this->dwCurrentProcessId=dwPID;

    *this->ProcessName=0;
    CProcessHelper::GetProcessName(dwPID,this->ProcessName,MAX_PATH);
    
    // check if we are started
    if (this->bAPIOverrideDllLoaded)
        this->Stop();

    // put flag signaling we don't have asked for unload
    this->bAPIOverrideDLLUnloadAsked=FALSE;

    // check if injlib dll is loaded
    if ((this->pInjectLib==NULL)||(this->pEjectLib==NULL))
    {
        _sntprintf(pszMsg,MAX_PATH,_T("Error loading %s. Make sure file exists and restart application"),INJECTLIB_DLL_NAME);
        ReportError(pszMsg);
        return FALSE;
    }

    // check if apioverride dll exists avoid lost of time
    _tcscpy(psz,this->pszAppPath);
    _tcscat(psz,API_OVERRIDE_DLL_NAME);

    HANDLE hFile = CreateFile(psz,             // open apioverride.dll 
                    GENERIC_READ,              // open for reading 
                    FILE_SHARE_READ,           // share for reading 
                    NULL,                      // no security 
                    OPEN_EXISTING,             // existing file only 
                    FILE_ATTRIBUTE_NORMAL,     // normal file 
                    NULL); 
    if (hFile==INVALID_HANDLE_VALUE)
    {
        _sntprintf(pszMsg,MAX_PATH,_T("Error %s not found. Make sure file exists and restart application"),API_OVERRIDE_DLL_NAME);
        ReportError(pszMsg);
        return FALSE;
    }
    CloseHandle(hFile);
   
    // pid -> string
    _stprintf(pszPID,_T("%x"),dwPID);

    // create interprocess communication events

    //HANDLE CreateEvent(
    //    LPSECURITY_ATTRIBUTES lpEventAttributes,
    //    BOOL bManualReset,
    //    BOOL bInitialState,
    //    LPCTSTR lpName
    //    );

    //(Injector -> APIOverride.dll)
    _tcscpy(psz,APIOVERRIDE_EVENT_START_MONITORING);
    _tcscat(psz,pszPID);
    this->hevtStartMonitoring=CreateEvent(NULL,FALSE,TRUE,psz);
    if (GetLastError()==ERROR_ALREADY_EXISTS)
    {
        CleanCloseHandle(&this->hevtStartMonitoring);
        _sntprintf(pszMsg,MAX_PATH,_T("Error another application seems to have inject API override"),pszPID);
        ReportError(pszMsg);
        return FALSE;
    }
    _tcscpy(psz,APIOVERRIDE_EVENT_STOP_MONITORING);
    _tcscat(psz,pszPID);
    this->hevtStopMonitoring=CreateEvent(NULL,FALSE,FALSE,psz);
    _tcscpy(psz,APIOVERRIDE_EVENT_START_FAKING);
    _tcscat(psz,pszPID);
    this->hevtStartFaking=CreateEvent(NULL,FALSE,TRUE,psz);
    _tcscpy(psz,APIOVERRIDE_EVENT_STOP_FAKING);
    _tcscat(psz,pszPID);
    this->hevtStopFaking=CreateEvent(NULL,FALSE,FALSE,psz);
    _tcscpy(psz,APIOVERRIDE_EVENT_FREE_PROCESS);
    _tcscat(psz,pszPID);
    this->hevtFreeProcess=CreateEvent(NULL,TRUE,FALSE,psz);
        
    // (APIOverride.dll -> Injector)
    _tcscpy(psz,APIOVERRIDE_EVENT_DLLPROCESS_ATTACH_COMPLETED);
    _tcscat(psz,pszPID);
    this->hevtAPIOverrideDllProcessAttachCompleted=CreateEvent(NULL,FALSE,FALSE,psz);
    _tcscpy(psz,APIOVERRIDE_EVENT_DLL_DETACHED_COMPLETED);
    _tcscat(psz,pszPID);
    this->hevtAPIOverrideDllProcessDetachCompleted=CreateEvent(NULL,FALSE,FALSE,psz);
    _tcscpy(psz,APIOVERRIDE_EVENT_PROCESS_FREE);
    _tcscat(psz,pszPID);
    this->hevtProcessFree=CreateEvent(NULL,TRUE,FALSE,psz);
    _tcscpy(psz,APIOVERRIDE_EVENT_MONITORING_FILE_LOADED);
    _tcscat(psz,pszPID);
    this->hevtMonitoringFileLoaded=CreateEvent(NULL,FALSE,FALSE,psz);
    _tcscpy(psz,APIOVERRIDE_EVENT_MONITORING_FILE_UNLOADED);
    _tcscat(psz,pszPID);
    this->hevtMonitoringFileUnloaded=CreateEvent(NULL,FALSE,FALSE,psz);
    _tcscpy(psz,APIOVERRIDE_EVENT_FAKE_API_DLL_LOADED);
    _tcscat(psz,pszPID);
    this->hevtFakeAPIDLLLoaded=CreateEvent(NULL,FALSE,FALSE,psz);
    _tcscpy(psz,APIOVERRIDE_EVENT_FAKE_API_DLL_UNLOADED);
    _tcscat(psz,pszPID);
    this->hevtFakeAPIDLLUnloaded=CreateEvent(NULL,FALSE,FALSE,psz);
    _tcscpy(psz,APIOVERRIDE_EVENT_ERROR);
    _tcscat(psz,pszPID);
    this->hevtError=CreateEvent(NULL,FALSE,FALSE,psz);


    if (!(this->hevtStartMonitoring&&this->hevtStopMonitoring&&this->hevtStartFaking&&this->hevtStopFaking&&this->hevtFreeProcess
            &&this->hevtAPIOverrideDllProcessAttachCompleted&&this->hevtAPIOverrideDllProcessDetachCompleted&&this->hevtProcessFree
            &&this->hevtMonitoringFileLoaded&&this->hevtMonitoringFileUnloaded
            &&this->hevtFakeAPIDLLLoaded&&this->hevtFakeAPIDLLUnloaded
            &&this->hevtError
            ))
    {
        ReportError(_T("Error creating named events"));
        return FALSE;
    }

    // start mailslot server for monitoring event logging (must be start before injecting API_OVERRIDE_DLL)
    _tcscpy(psz,APIOVERRIDE_MAILSLOT_TO_INJECTOR);
    _tcscat(psz,pszPID);
    pMailSlotServer=new CMailSlotServer(psz,StaticMailSlotServerCallback,this);
    if (!pMailSlotServer->Start())
    {
        _sntprintf(pszMsg,MAX_PATH,_T("Error starting mailslot server %s"),psz);
        ReportError(pszMsg);
        this->Stop();
        return FALSE;
    }

    // inject library
    _tcscpy(psz,this->pszAppPath);
    _tcscat(psz,API_OVERRIDE_DLL_NAME);

    // define events
    pH[0]=this->hevtAPIOverrideDllProcessAttachCompleted;
    pH[1]=this->hevtError;
    pH[2]=this->hevtFreeProcess;
    pH[3]=this->hevtProcessFree;

    // reset event in case of previous timeout
    ResetEvent(pH[0]);
    ResetEvent(pH[1]);

    // try to inject library
    if (!this->pInjectLib(this->dwCurrentProcessId,psz))
    {
        _sntprintf(pszMsg,MAX_PATH,_T("Error injecting library %s"),API_OVERRIDE_DLL_NAME);
        ReportError(pszMsg);
        this->Stop();
        return FALSE;
    }

    this->bAPIOverrideDllLoaded=TRUE;

    // wait for Ready to Work event
    DWORD dwRet=WaitForMultipleObjects(4,pH,FALSE,TIME_REQUIERED_TO_LOAD);
    switch(dwRet)
    {
        case WAIT_TIMEOUT:    // time out
            _sntprintf(pszMsg,MAX_PATH,_T("Error %s don't become ready"),API_OVERRIDE_DLL_NAME);
            ReportError(pszMsg);
            this->Stop();
            return FALSE;
        case WAIT_OBJECT_0:   // all is ok
            break;
        case WAIT_OBJECT_0+1: // error dll is no more loaded
            this->bAPIOverrideDllLoaded=FALSE;
            this->Stop();
            return FALSE;
        case WAIT_OBJECT_0+2: 
        case WAIT_OBJECT_0+3: 
            return FALSE;
        default:// wait failed
            this->Stop();
            return FALSE;
    }

    // start mailslot client for giving instructions (it can't be open before the Ready to Work event)
    _tcscpy(psz,APIOVERRIDE_MAILSLOT_FROM_INJECTOR);
    _tcscat(psz,pszPID);
    this->pMailSlotClient=new CMailSlotClient(psz);
    if (!pMailSlotClient->Open())
    {
        ReportError(_T("Can't open MailSlot"));
        this->Stop();
        return FALSE;
    }

    // start thread to watch hevtAPIOverrideDllProcessDetachCompleted
    this->hThreadWatchingEvents=CreateThread(NULL,0,CApiOverrideDllUnloadedThreadListener,this,0,NULL);

    // empty fifo
    this->pFifo->RemoveAllItems();
    // start thread for logging
    this->hThreadLogging=CreateThread(NULL,0,CApiOverrideLoggingThreadListener,this,0,NULL);

    // start monitoring
    this->StartMonitoring();
    // start faking
    this->StartFaking();

    return TRUE;
}
//-----------------------------------------------------------------------------
// Name: Start
// Object: start the software specified by pszFileName, inject API Override dll at start up
// Parameters :
//     in : TCHAR* pszFileName : path of software to start
// Return : FALSE on error, TRUE if success
//-----------------------------------------------------------------------------
BOOL CApiOverride::Start(TCHAR* pszFileName)
{
    return this->Start(pszFileName,NULL,NULL);
}

//-----------------------------------------------------------------------------
// Name: Start
// Object: start the software specified by pszFileName, inject API Override dll at start up,
//         call pCallBackFunc function  to allow to configure monitoring and faking
//         resume process when callback function returns
// Parameters :
//     in : TCHAR* pszFileName : path of software to start
//          FARPROC pCallBackFunc : instruction to do after pszFileName loading and before we resume the process
//                                          let us load monitoring file and fake api dll before software startup
//          LPVOID pUserParam : parameter for the callback
// Return : FALSE on error, TRUE if success
//-----------------------------------------------------------------------------
BOOL CApiOverride::Start(TCHAR* pszFileName,tagpCallBackBeforeAppResume pCallBackFunc,LPVOID pUserParam)
{
    return this->Start(pszFileName,_T(""),pCallBackFunc,pUserParam);
}

//-----------------------------------------------------------------------------
// Name: Start
// Object: start the software specified by pszFileName, inject API Override dll at start up,
//         call pCallBackFunc function  to allow to configure monitoring and faking
//         resume process when callback function returns
// Parameters :
//     in : TCHAR* pszFileName : path of software to start
//          TCHAR* pszCmdLine  : command line
//          FARPROC pCallBackFunc : instruction to do after pszFileName loading and before we resume the process
//                                          let us load monitoring file and fake api dll before software startup
//          LPVOID pUserParam : parameter for the callback
// Return : FALSE on error, TRUE if success
//-----------------------------------------------------------------------------
BOOL CApiOverride::Start(TCHAR* pszFileName,TCHAR* pszCmdLine,tagpCallBackBeforeAppResume pCallBackFunc,LPVOID pUserParam)
{
    return this->Start(pszFileName,pszCmdLine,pCallBackFunc,pUserParam,FALSE,0);
}


//-----------------------------------------------------------------------------
// Name: Start
// Object: start the software specified by pszFileName, inject API Override dll at start up,
//         call pCallBackFunc function  to allow to configure monitoring and faking
//         Process is resume at Startup during dwResumeTimeAtStartup ms
//         resume process when callback function returns
//         This function is used for some troubles on ResumeThread detected with some app (NotePad,Calc)
//         if someone knows the answer, just tell me :)
// Parameters :
//     in : TCHAR* pszFileName : path of software to start
//          TCHAR* pszCmdLine  : command line
//          FARPROC pCallBackFunc : instruction to do after pszFileName loading and before we resume the process
//                                          let us load monitoring file and fake api dll before software startup
//          LPVOID pUserParam : parameter for the callback
//          int iStartMethod : Suspended, AsSoonAsPossible,Sleep
//          DWORD dwResumeTimeAtStartup : Time in ms during which process will be resumed at startup
// Return : FALSE on error, TRUE if success
//-----------------------------------------------------------------------------
BOOL CApiOverride::Start(TCHAR* pszFileName,TCHAR* pszCmdLine,tagpCallBackBeforeAppResume pCallBackFunc,LPVOID pUserParam,int iStartMethod,DWORD dwResumeTimeAtStartup)
{
    STARTUPINFO StartupInfo;
    PROCESS_INFORMATION ProcessInformation;
    TCHAR* pszFileNameDirectory;
    TCHAR* pszLastSep=NULL;
    TCHAR* pszDir;
    BOOL bRet;
    TCHAR pszMsg[2*MAX_PATH];
    TCHAR ProcName[MAX_PATH];
    DWORD dwFirstTryTime;

    // check if file exists
    HANDLE hFile = CreateFile(pszFileName, GENERIC_READ, FILE_SHARE_READ, NULL,
        OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
    if (hFile == INVALID_HANDLE_VALUE)
    {
        _sntprintf(pszMsg,2*MAX_PATH,_T("File %s not found"),pszFileName);
        ReportError(pszMsg);
        return FALSE;
    }


    memset(&ProcessInformation,0,sizeof(PROCESS_INFORMATION));
    memset(&StartupInfo,0,sizeof(STARTUPINFO));
    StartupInfo.cb=sizeof(STARTUPINFO);

    // get software directory 
    pszDir=_tcsdup(pszFileName);
    pszLastSep=_tcsrchr(pszDir,'\\');
    if (pszLastSep)
    {
        *pszLastSep=0;
        pszFileNameDirectory=pszDir;
    }
    else // work in current dir
        pszFileNameDirectory=NULL;


    switch (iStartMethod)
    {
    case CApiOverride::StartWaySleep:
        // Load Process in a suspended mode
        bRet=CreateProcess( pszFileName,                //LPCTSTR lpApplicationName,
                            pszCmdLine,                 //LPTSTR lpCommandLine,
                            NULL,                       //LPSECURITY_ATTRIBUTES lpProcessAttributes,
                            NULL,                       //LPSECURITY_ATTRIBUTES lpThreadAttributes,
                            FALSE,                      //BOOL bInheritHandles,
                            0,                          //DWORD dwCreationFlags,
                            NULL,                       //LPVOID lpEnvironment,
                            pszFileNameDirectory,       //LPCTSTR lpCurrentDirectory,
                            &StartupInfo,               //LPSTARTUPINFO lpStartupInfo,
                            &ProcessInformation         //LPPROCESS_INFORMATION lpProcessInformation
                            );
        free(pszDir);
        if (!bRet)
        {
            // show last error
            CAPIError::ShowLastError();
            return FALSE;
        }
        // sleep if necessary
        if (dwResumeTimeAtStartup)
            Sleep(dwResumeTimeAtStartup);

        // Do all work
        if (!this->Start(ProcessInformation.dwProcessId))
        {
            TerminateProcess(ProcessInformation.hProcess,0xFFFFFFFF);
            CloseHandle(ProcessInformation.hThread);
            CloseHandle(ProcessInformation.hProcess);
            return FALSE;
        }

        // suspend Process main thread
        if (SuspendThread(ProcessInformation.hThread)==0xFFFFFFFF)
        {
            // show last error
            CAPIError::ShowLastError();
            TerminateProcess(ProcessInformation.hProcess,0xFFFFFFFF);
            CloseHandle(ProcessInformation.hThread);
            CloseHandle(ProcessInformation.hProcess);
            return FALSE;
        }
        break;
    case CApiOverride::StartWayAsSoonAsPossible:
        // Load Process in a suspended mode
        bRet=CreateProcess( pszFileName,                //LPCTSTR lpApplicationName,
                            pszCmdLine,                 //LPTSTR lpCommandLine,
                            NULL,                       //LPSECURITY_ATTRIBUTES lpProcessAttributes,
                            NULL,                       //LPSECURITY_ATTRIBUTES lpThreadAttributes,
                            FALSE,                      //BOOL bInheritHandles,
                            0,                          //DWORD dwCreationFlags,
                            NULL,                       //LPVOID lpEnvironment,
                            pszFileNameDirectory,       //LPCTSTR lpCurrentDirectory,
                            &StartupInfo,               //LPSTARTUPINFO lpStartupInfo,
                            &ProcessInformation         //LPPROCESS_INFORMATION lpProcessInformation
                            );
        free(pszDir);
        if (!bRet)
        {
            // show last error
            CAPIError::ShowLastError();
            return FALSE;
        }

        // until name retreival success or timeout
        for (int cnt=0;cnt<APIOVERRIDE_NB_SLEEP_FOR_GET_PROCESS_NAME_TIMEOUT;cnt++)
        {
            dwFirstTryTime=GetTickCount();
            while (((GetTickCount()-dwFirstTryTime)<(APIOVERRIDE_GET_PROCESS_NAME_TIMEOUT_IN_MS/APIOVERRIDE_NB_SLEEP_FOR_GET_PROCESS_NAME_TIMEOUT))
                    && (!CProcessHelper::GetProcessName(ProcessInformation.dwProcessId,ProcName,MAX_PATH)))
            {}
            // lets os do other job (like loading our target app)
            Sleep(10);
        }


        // Do all work
        if (!this->Start(ProcessInformation.dwProcessId))
        {
            TerminateProcess(ProcessInformation.hProcess,0xFFFFFFFF);
            CloseHandle(ProcessInformation.hThread);
            CloseHandle(ProcessInformation.hProcess);
            return FALSE;
        }

        // suspend Process main thread
        if (SuspendThread(ProcessInformation.hThread)==0xFFFFFFFF)
        {
            // show last error
            CAPIError::ShowLastError();
            TerminateProcess(ProcessInformation.hProcess,0xFFFFFFFF);
            CloseHandle(ProcessInformation.hThread);
            CloseHandle(ProcessInformation.hProcess);
            return FALSE;
        }
        break;
    case CApiOverride::StartWaySuspended:
        // Load Process in a suspended mode
        bRet=CreateProcess( pszFileName,                //LPCTSTR lpApplicationName,
                            pszCmdLine,                 //LPTSTR lpCommandLine,
                            NULL,                       //LPSECURITY_ATTRIBUTES lpProcessAttributes,
                            NULL,                       //LPSECURITY_ATTRIBUTES lpThreadAttributes,
                            FALSE,                      //BOOL bInheritHandles,
                            CREATE_SUSPENDED,           //DWORD dwCreationFlags,
                            NULL,                       //LPVOID lpEnvironment,
                            pszFileNameDirectory,       //LPCTSTR lpCurrentDirectory,
                            &StartupInfo,               //LPSTARTUPINFO lpStartupInfo,
                            &ProcessInformation         //LPPROCESS_INFORMATION lpProcessInformation
                            );
        free(pszDir);
        if (!bRet)
        {
            // show last error
            CAPIError::ShowLastError();
            return FALSE;
        }

        // Do all work
        if (!this->Start(ProcessInformation.dwProcessId))
        {
            TerminateProcess(ProcessInformation.hProcess,0xFFFFFFFF);
            CloseHandle(ProcessInformation.hThread);
            CloseHandle(ProcessInformation.hProcess);
            return FALSE;
        }
        break;
    default:
        return FALSE;
    }

    // call the call back if any (let us load monitoring file and fake api dll before software startup)
     if (!IsBadCodePtr((FARPROC)pCallBackFunc))
        pCallBackFunc(this->dwCurrentProcessId,pUserParam);


    // Resume Process main thread
    if (ResumeThread(ProcessInformation.hThread)==((DWORD)-1))
    {
        // show last error
        CAPIError::ShowLastError();
        this->Stop();
        TerminateProcess(ProcessInformation.hProcess,0xFFFFFFFF);
        CloseHandle(ProcessInformation.hThread);
        CloseHandle(ProcessInformation.hProcess);
        return FALSE;
    }

    CloseHandle(ProcessInformation.hThread);
    CloseHandle(ProcessInformation.hProcess);
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: StaticMailSlotServerCallback
// Object: callback for log events
// Parameters :
//     in : PVOID pData : pointer to a tagLogEntry struct
//          PVOID pUserData : pointer to CApiOverride object belonging the MailSlotServer which reach the event
// Return : 
//-----------------------------------------------------------------------------
void CApiOverride::StaticMailSlotServerCallback(PVOID pData,DWORD dwDataSize,PVOID pUserData)
{
    if (dwDataSize==0)
        return;
    // re enter object oriented progrmming
    ((CApiOverride*)pUserData)->MailSlotServerCallback(pData);
}

//-----------------------------------------------------------------------------
// Name: MailSlotServerCallback
// Object: internal callback for log events
// Parameters :
//     in : PVOID pData : pointer to a buffer send by injected dll
// Return : 
//-----------------------------------------------------------------------------
void CApiOverride::MailSlotServerCallback(PVOID pData)
{
    DWORD msgCmd;
    DWORD ReplySize;
    PBYTE pbData=(PBYTE)pData;

    memcpy(&msgCmd,pData,sizeof(DWORD));
    pbData+=sizeof(DWORD);
    switch (msgCmd)
    {
    case CMD_PROCESS_INTERNAL_CALL_REPLY:
        memcpy(&ReplySize,pbData,sizeof(DWORD));
        if (this->ProcessInternalCallReply)
            delete this->ProcessInternalCallReply;
        this->ProcessInternalCallReply=new BYTE[ReplySize];
        memcpy(this->ProcessInternalCallReply,pbData,ReplySize);
        SetEvent(this->hevtProcessInternalCallReply);
        return;
    case CMD_MONITORING_LOG:
        // As logging can block this callback (like listview SendMessage ;D)
        // and for better PROCESS_INTERNAL_CALL_REPLY speed
        // we just do log parsing/showing in another thread
        // so we just need to push data on fifo
        this->pFifo->Push(pbData);
        // signal new event
        ReleaseSemaphore(this->hSemaphoreLog,1,NULL);
        return;
    }
}

void CApiOverride::MonitoringCallback(tagLogEntry* ptagLogEntry)
{
    // if a callback has been specified
    if (!IsBadCodePtr((FARPROC)this->pCallBackLogFunc))
        this->pCallBackLogFunc(ptagLogEntry,this->pCallBackLogFuncUserParam);


    if (this->pListview)
    {
        TCHAR psz[MAX_PATH];
        TCHAR ppc[6][MAX_PATH+1];
        TCHAR* pp[6];

        // param direction
        switch (ptagLogEntry->bParamDirectionType)
        {
        case PARAM_DIR_TYPE_IN:
            _tcscpy(ppc[0],_T("In"));
            break;
        case PARAM_DIR_TYPE_OUT:
            _tcscpy(ppc[0],_T("Out"));
            break;
        case PARAM_DIR_TYPE_IN_NO_RETURN:
            _tcscpy(ppc[0],_T("InNoRet"));
            break;
        }

        // api name
        _tcsncpy(ppc[1],ptagLogEntry->szAPINameAndParams,MAX_PATH);
        ppc[1][MAX_PATH]=0;

        // ret value
        if (ptagLogEntry->bParamDirectionType==PARAM_DIR_TYPE_IN_NO_RETURN)
            *psz=0;
        else
            _sntprintf(psz,MAX_PATH,_T("0x%.8x"),ptagLogEntry->dwReturnValue);

        _tcscpy(ppc[2],psz);

        // origin address
        _sntprintf(psz,MAX_PATH,_T("0x%.8x"),(DWORD)ptagLogEntry->pvOriginAddress);
        _tcscpy(ppc[3],psz);

        // process ID
        _sntprintf(psz,MAX_PATH,_T("0x%.8x"),ptagLogEntry->dwProcessId);
        _tcscpy(ppc[4],psz);

        // module name
        _tcsncpy(ppc[5],ptagLogEntry->szModuleName,MAX_PATH);
        ppc[5][MAX_PATH]=0;

        // internal logging
        for (int cnt=0;cnt<6;cnt++)// convertion from TCHAR[][] to TCHAR**
            pp[cnt]=(TCHAR*)ppc+cnt*(MAX_PATH+1);
        this->pListview->AddItemAndSubItems(6,pp,TRUE);
    }
}
//-----------------------------------------------------------------------------
// Name: ProcessInternalCall
// Object: call specified function with parameters specified in pParams in the remote process
//          and store function return (eax) in pRet
// Parameters :
//      in: LPTSTR LibName : function address
//          LPTSTR FuncName
//          DWORD NbParams : nb params in pParams
//          PSTRUCT_FUNC_PARAM pParams : array of STRUCT_FUNC_PARAM. Can be null if no params
//      out : DWORD* ReturnValue : returned value
// Return : 
//-----------------------------------------------------------------------------
BOOL CApiOverride::ProcessInternalCall(LPTSTR LibName,LPTSTR FuncName,DWORD NbParams,PSTRUCT_FUNC_PARAM pParams,DWORD* ReturnValue)
{
    return this->ProcessInternalCall(LibName,FuncName,NbParams,pParams,ReturnValue,TIME_REQUIERED_TO_LOAD);
}
//-----------------------------------------------------------------------------
// Name: ProcessInternalCall
// Object: call specified function with parameters specified in pParams in the remote process
//          and store function return (eax) in pRet
// Parameters :
//      in: LPTSTR LibName : function address
//          LPTSTR FuncName
//          DWORD NbParams : nb params in pParams
//          PSTRUCT_FUNC_PARAM pParams : array of STRUCT_FUNC_PARAM. Can be null if no params
//          DWORD dwTimeOutMs : max time in ms to wait for function reply (0xFFFFFFFF for INFINITE)
//      out : DWORD* ReturnValue : returned value
// Return : 
//-----------------------------------------------------------------------------
BOOL CApiOverride::ProcessInternalCall(LPTSTR LibName,LPTSTR FuncName,DWORD NbParams,PSTRUCT_FUNC_PARAM pParams,DWORD* ReturnValue,DWORD dwTimeOutMs)
{
    TCHAR pszMsg[2*MAX_PATH];
    PBYTE pb;
    BOOL bRet;
    DWORD dwRet;
    DWORD dw;
    CApiOverrideFuncAndParams* pApiOverrideFuncAndParams;
    DWORD dwCnt;
    HANDLE pH[4];


    if (!this->bAPIOverrideDllLoaded)
    {
        this->ShowApiOverrideNotStartedMsg();
        return FALSE;
    }

    if (IsBadReadPtr(ReturnValue,sizeof(DWORD)))// don't check pParams because it can be NULL if no params
        return FALSE;

    // encode params and func name
    pApiOverrideFuncAndParams=new CApiOverrideFuncAndParams();
    if (!pApiOverrideFuncAndParams->Encode(LibName,FuncName,NbParams,pParams))
    {
        delete pApiOverrideFuncAndParams;
        return FALSE;
    }

    // send cmd buffer to injected lib
    pb=new BYTE[pApiOverrideFuncAndParams->EncodedBufferSize+sizeof(DWORD)];
    dw=CMD_PROCESS_INTERNAL_CALL_QUERY;
    memcpy(pb,&dw,sizeof(DWORD));
    memcpy(&pb[sizeof(DWORD)],pApiOverrideFuncAndParams->EncodedBuffer,pApiOverrideFuncAndParams->EncodedBufferSize);

    // define events
    pH[0]=this->hevtProcessInternalCallReply;
    pH[1]=this->hevtError;
    pH[2]=this->hevtFreeProcess;
    pH[3]=this->hevtProcessFree;

    // reset event in case of previous timeout before writing to mailslot (else event can occurs before we reset it)
    ResetEvent(pH[0]);
    ResetEvent(pH[1]);

    bRet=this->pMailSlotClient->Write(pb,pApiOverrideFuncAndParams->EncodedBufferSize+sizeof(DWORD));
    if (!bRet)
    {
        delete pb;
        delete pApiOverrideFuncAndParams;
        return FALSE;
    }

    // wait for injected lib reply
    dwRet=WaitForMultipleObjects(4,pH,FALSE,dwTimeOutMs);
    switch(dwRet)
    {
        case WAIT_TIMEOUT:
            _sntprintf(pszMsg,2*MAX_PATH,_T("Error no reply for %s:%s call in %d sec"),FuncName,LibName,dwTimeOutMs/1000);
            ReportError(pszMsg);
            return FALSE;
        case WAIT_OBJECT_0:
            break;
        case WAIT_OBJECT_0+2:
        case WAIT_OBJECT_0+3:
            return FALSE;
        case WAIT_OBJECT_0+1:
        default:
            _sntprintf(pszMsg,2*MAX_PATH,_T("Error during the call of %s:%s"),FuncName,LibName);
            ReportError(pszMsg);
            delete pb;
            delete pApiOverrideFuncAndParams;
            return FALSE;
    }


    // decode reply
    bRet=pApiOverrideFuncAndParams->Decode(this->ProcessInternalCallReply);
    if ((!bRet)||(NbParams!=pApiOverrideFuncAndParams->DecodedNbParams))
    {
        _sntprintf(pszMsg,2*MAX_PATH,_T("Error during the call of %s:%s"),FuncName,LibName);
        ReportError(pszMsg);
        delete pb;
        delete pApiOverrideFuncAndParams;
        return FALSE;
    }

    // copy data of received buffer
    *ReturnValue=pApiOverrideFuncAndParams->DecodedReturnedValue;

    // copy pParams
    for (dwCnt=0;dwCnt<pApiOverrideFuncAndParams->DecodedNbParams;dwCnt++)
    {
        // avoid buffer overflow here
        if (pApiOverrideFuncAndParams->DecodedParams[dwCnt].dwDataSize>pParams[dwCnt].dwDataSize)
            pApiOverrideFuncAndParams->DecodedParams[dwCnt].dwDataSize=pParams[dwCnt].dwDataSize;
        memcpy(pParams[dwCnt].pData,
            pApiOverrideFuncAndParams->DecodedParams[dwCnt].pData,
            pApiOverrideFuncAndParams->DecodedParams[dwCnt].dwDataSize=pParams[dwCnt].dwDataSize
            );
    }
    delete pb;
    delete pApiOverrideFuncAndParams;
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: ShowApiOverrideNotStartedMsg
// Object: show a standart error msg
// Parameters :
//     in : 
// Return : 
//-----------------------------------------------------------------------------
void CApiOverride::ShowApiOverrideNotStartedMsg()
{
    ReportError(_T("Error ApiOverride not Started"));
}

//-----------------------------------------------------------------------------
// Name: SetUnexpectedUnload
// Object: Set call back for unexpexted unload
//         This call back will be call if host process unload the dll without we ask it to do
//         It is call when host process close
// Parameters :
//     in : - FARPROC pCallBackFunc : callback function
//          - LPVOID pUserParam : parameter for the callback
// Return : 
//-----------------------------------------------------------------------------
void CApiOverride::SetUnexpectedUnload(tagCallBackUnexpectedUnload pCallBackFunc,LPVOID pUserParam)
{
    this->pCallBackUnexpectedUnloadFunc=pCallBackFunc;
    this->pCallBackUnexpectedUnloadFuncUserParam=pUserParam;
}
//-----------------------------------------------------------------------------
// Name: DllUnloadedCallBack
// Object: object callBack for dll unloading
// Parameters :
//     in : 
// Return : 
//-----------------------------------------------------------------------------
void CApiOverride::DllUnloadedCallBack()
{
    // check if internal unload asked (Stop methode as been called)
    if (this->bAPIOverrideDLLUnloadAsked)
        return;

    // signal that the dll is no more loaded (must be done before calling Stop)
    this->bAPIOverrideDllLoaded=FALSE;

    // Stop all threads and mailslots
    this->Stop();

    // else call process exit associated callback
    if (!IsBadCodePtr((FARPROC)this->pCallBackUnexpectedUnloadFunc))
        this->pCallBackUnexpectedUnloadFunc(this->dwCurrentProcessId,this->pCallBackUnexpectedUnloadFuncUserParam);
}
//-----------------------------------------------------------------------------
// Name: ShowApiOverrideNotStartedMsg
// Object: watch for dll unloading and call associated object methode
// Parameters :
//     in : LPVOID lpParam : CApiOverride*
// Return : 
//-----------------------------------------------------------------------------
DWORD WINAPI CApiOverrideDllUnloadedThreadListener(LPVOID lpParam)
{
    CApiOverride* pCApiOverride=(CApiOverride*)lpParam;
    // as Thread is started on each CApiOverride::Start event we just wait 1 event
    if (WaitForSingleObject(pCApiOverride->hevtAPIOverrideDllProcessDetachCompleted,INFINITE)!=WAIT_OBJECT_0)
    {
        CAPIError::ShowLastError();
        return 0xFFFFFFFF;
    }
    pCApiOverride->DllUnloadedCallBack();
    return 0;
}

//-----------------------------------------------------------------------------
// Name: CApiOverrideLoggingThreadListener
// Object: watch for monitoring log
// Parameters :
//     in : LPVOID lpParam : CApiOverride*
// Return : 
//-----------------------------------------------------------------------------
DWORD WINAPI CApiOverrideLoggingThreadListener(LPVOID lpParam)
{
    CApiOverride* pCApiOverride=(CApiOverride*)lpParam;
    HANDLE ph[3];
    ph[0]=pCApiOverride->hevtFreeProcess;
    ph[1]=pCApiOverride->hevtProcessFree;
    ph[2]=pCApiOverride->hSemaphoreLog;
    
    tagLogEntry Log;
    DWORD dwRes;
    for(;;)
    {
        dwRes=WaitForMultipleObjects(3,ph,FALSE,INFINITE);
        switch(dwRes)
        {
            case WAIT_OBJECT_0:// free process
            case WAIT_OBJECT_0+1:
                return 0;
            case WAIT_OBJECT_0+2:// evt log
                if (pCApiOverride->pFifo->Pop(&Log))
                    pCApiOverride->MonitoringCallback(&Log);
                break;
            default:// error
                CAPIError::ShowLastError();
                return 0xFFFFFFFF;
        }
    }
}