/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: class helper for doing api override or api monitoring
//         it manages the apioverride.dll
//         (It's the hart of project winapioverride)
//-----------------------------------------------------------------------------


#pragma once

#include <windows.h>
#include <stdio.h>
#pragma warning (push)
#pragma warning(disable : 4005)// for '_stprintf' : macro redefinition in tchar.h
#include <TCHAR.h>
#pragma warning (pop)
#include "interprocesscommunication.h"
#include "../InjLib/injlib.h"
#include "ApiOverrideFuncAndParams.h"
#include "../MailSlot/MailSlotClient.h"
#include "../MailSlot/MailSlotServer.h"
#include "../../GUI/ListView/ListView.h"
#include "../../FIFO/FIFO.h"
#include "../../APIError/ApiError.h"
#include "../../CleanCloseHandle/CleanCloseHandle.h"
#include "../ProcessHelper/ProcessHelper.h"

// dll  name
#define API_OVERRIDE_DLL_NAME _T("ApiOverride.dll")
#define INJECTLIB_DLL_NAME _T("InjLib.dll")

// timeouts in ms
#define TIME_REQUIERED_TO_LOAD 15000 // max time in sec requiered for dll/monitoring file to be loaded
#define TIME_REQUIERED_TO_UNLOAD 10000 // max time in sec requiered for dll/monitoring file to be unloaded

#define APIOVERRIDE_GET_PROCESS_NAME_TIMEOUT_IN_MS 4000 // max time in ms at application startup to wait for a GetProcessName to work (GetProcessName loop max time)
#define APIOVERRIDE_NB_SLEEP_FOR_GET_PROCESS_NAME_TIMEOUT 80 // max time the GetProcessName loop will sleep 10ms

typedef void (*tagCallBackLogFunc)(tagLogEntry* pLog,PVOID pUserParam);
typedef void (*tagCallBackUnexpectedUnload)(DWORD dwProcessID,PVOID pUserParam);
typedef void (*tagpCallBackBeforeAppResume)(DWORD dwProcessID,PVOID pUserParam);

class CApiOverride
{
    friend DWORD WINAPI CApiOverrideLoggingThreadListener(LPVOID lpParam);
    friend DWORD WINAPI CApiOverrideDllUnloadedThreadListener(LPVOID lpParam);
private:
    BOOL bAPIOverrideDLLUnloadAsked;
    BOOL bAPIOverrideDllLoaded;
    tagCallBackLogFunc pCallBackLogFunc;
    LPVOID pCallBackLogFuncUserParam;
    tagCallBackUnexpectedUnload pCallBackUnexpectedUnloadFunc;
    LPVOID pCallBackUnexpectedUnloadFuncUserParam;
    DWORD dwCurrentProcessId;
    CMailSlotServer* pMailSlotServer;
    CMailSlotClient* pMailSlotClient;
    HANDLE hSemaphoreLog;
    CFIFO* pFifo;
    CListview* pListview;
    BOOL bExternalCListView;
    InjectLib pInjectLib;
    EjectLib pEjectLib;
    HMODULE hmodInjlib;
    HANDLE hThreadWatchingEvents;
    HANDLE hThreadLogging;
    TCHAR pszAppPath[MAX_PATH];
    PBYTE ProcessInternalCallReply;
    HANDLE hevtProcessInternalCallReply;

    HANDLE hevtStartMonitoring;
    HANDLE hevtStopMonitoring;
    HANDLE hevtStartFaking;
    HANDLE hevtStopFaking;
    HANDLE hevtFreeProcess;
    
    HANDLE hevtAPIOverrideDllProcessAttachCompleted;
    HANDLE hevtAPIOverrideDllProcessDetachCompleted;
    HANDLE hevtProcessFree;
    HANDLE hevtMonitoringFileLoaded;
    HANDLE hevtMonitoringFileUnloaded;
    HANDLE hevtFakeAPIDLLLoaded;
    HANDLE hevtFakeAPIDLLUnloaded;
    HANDLE hevtError;

    void Initialize();
    void MailSlotServerCallback(PVOID pData);
    void MonitoringCallback(tagLogEntry* ptagLogEntry);
    void ShowApiOverrideNotStartedMsg();
    void DllUnloadedCallBack();
    void ReportError(TCHAR* pszErrorMessage);

    static void StaticMailSlotServerCallback(PVOID pData,DWORD dwDataSize,PVOID pUserData);
public:

    enum tagStartWays
    {
        StartWaySleep,
        StartWaySuspended,
        StartWayAsSoonAsPossible
    }StartWays;

    TCHAR ProcessName[MAX_PATH];

    CApiOverride();
    CApiOverride(tagCallBackLogFunc pCallBackLogFunc);
    CApiOverride(HWND hListView);
    ~CApiOverride(void);

    BOOL LoadMonitoringFile(TCHAR* pszFileName);
    BOOL UnloadMonitoringFile(TCHAR* pszFileName);
    BOOL LoadFakeAPI(TCHAR* pszFileName);
    BOOL UnloadFakeAPI(TCHAR* pszFileName);

    BOOL StartMonitoring();
    BOOL StopMonitoring();
    BOOL StartFaking();
    BOOL StopFaking();

    void SetUnexpectedUnload(tagCallBackUnexpectedUnload pCallBackFunc,LPVOID pUserParam);
    void SetMonitoringCallback(tagCallBackLogFunc pCallBackLogFunc,LPVOID pUserParam);
    void SetMonitoringListview(HWND hListView);
    void SetMonitoringListview(CListview* pListView);

    BOOL Stop();
    BOOL Start(DWORD dwPID);
    BOOL Start(TCHAR* pszFileName);
    BOOL Start(TCHAR* pszFileName,tagpCallBackBeforeAppResume pCallBackFunc,LPVOID pUserParam);
    BOOL Start(TCHAR* pszFileName,TCHAR* pszCmdLine,tagpCallBackBeforeAppResume pCallBackFunc,LPVOID pUserParam);
    BOOL Start(TCHAR* pszFileName,TCHAR* pszCmdLine,tagpCallBackBeforeAppResume pCallBackFunc,LPVOID pUserParam,int iStartMethod,DWORD dwResumeTimeAtStartup);

    BOOL ProcessInternalCall(LPTSTR LibName,LPTSTR FuncName,DWORD NbParams,PSTRUCT_FUNC_PARAM pParams,DWORD* ReturnValue);
    BOOL ProcessInternalCall(LPTSTR LibName,LPTSTR FuncName,DWORD NbParams,PSTRUCT_FUNC_PARAM pParams,DWORD* ReturnValue,DWORD dwTimeOutMs);

    DWORD GetProcessID();
    BOOL GetProcessName(TCHAR* ProcessName,int ProcessNameMaxSize);
};
