/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

// to avoid rebasing, use the linker/advanced/BaseAddress option 

#include "../../GenericFakeAPI.h"

// You just need to edit this file to add new fake api 
// WARNING YOUR FAKE API MUST HAVE THE SAME PARAMETERS AND CALLING CONVENTION AS THE REAL ONE,
//                  ELSE YOU WILL GET STACK ERRORS

///////////////////////////////////////////////////////////////////////////////
// fake API prototype MUST HAVE THE SAME PARAMETERS 
// for the same calling convention see MSDN : 
// "Using a Microsoft modifier such as __cdecl on a data declaration is an outdated practice"
///////////////////////////////////////////////////////////////////////////////

/*
BOOL WINAPI myWriteFile(
  HANDLE hFile,
  LPCVOID lpBuffer,
  DWORD nNumberOfBytesToWrite,
  LPDWORD lpNumberOfBytesWritten,
  LPOVERLAPPED lpOverlapped
);

BOOL WINAPI myReadFile(
  HANDLE hFile,
  LPVOID lpBuffer,
  DWORD nNumberOfBytesToRead,
  LPDWORD lpNumberOfBytesRead,
  LPOVERLAPPED lpOverlapped
);

HANDLE WINAPI myCreateFileA(
  LPCTSTR lpFileName,
  DWORD dwDesiredAccess,
  DWORD dwShareMode,
  LPSECURITY_ATTRIBUTES lpSecurityAttributes,
  DWORD dwCreationDisposition,
  DWORD dwFlagsAndAttributes,
  HANDLE hTemplateFile
);

HANDLE WINAPI myCreateFileW(
  LPCWSTR lpFileName,
  DWORD dwDesiredAccess,
  DWORD dwShareMode,
  LPSECURITY_ATTRIBUTES lpSecurityAttributes,
  DWORD dwCreationDisposition,
  DWORD dwFlagsAndAttributes,
  HANDLE hTemplateFile
);


int WINAPI  mysend(
  SOCKET s,
  const char* buf,
  int len,
  int flags
);

int WINAPI  myrecv(
  SOCKET s,
  char* buf,
  int len,
  int flags
);
*/

int  WINAPI mysendto(
  SOCKET s,
  const char* buf,
  int len,
  int flags,
  const struct sockaddr* to,
  int tolen
);

int WINAPI  myrecvfrom(
  SOCKET s,
  char* buf,
  int len,
  int flags,
  struct sockaddr* from,
  int* fromlen
);

  /*
int WINAPI myconnect(
  SOCKET s,
  const struct sockaddr* name,
  int namelen
);
*/

///////////////////////////////////////////////////////////////////////////////
// fake API array. Redirection are defined here
///////////////////////////////////////////////////////////////////////////////
STRUCT_FAKE_API pArrayFakeAPI[]=
{
    // library name ,function name, function handler, number of params (requiered to allocate enougth stack space)
 /*   {_T("Kernel32.dll"),_T("WriteFile"),(FARPROC)myWriteFile,5},
	{_T("Kernel32.dll"),_T("ReadFile"),(FARPROC)myReadFile,5},
	{_T("Kernel32.dll"),_T("CreateFileA"),(FARPROC)myCreateFileA,7},
	{_T("Kernel32.dll"),_T("CreateFileW"),(FARPROC)myCreateFileW,7},
	{_T("ws2_32.dll"),_T("connect"),(FARPROC)myconnect,3},
    {_T("ws2_32.dll"),_T("send"),(FARPROC)mysend,4},
	{_T("ws2_32.dll"),_T("recv"),(FARPROC)myrecv,4},*/
	{_T("ws2_32.dll"),_T("sendto"),(FARPROC)mysendto,6},
	{_T("ws2_32.dll"),_T("recvfrom"),(FARPROC)myrecvfrom,6},
    {_T(""),_T(""),NULL,0}// last element for ending loops
};

//////////////////////////////////////////////////////////////////////////////
// Only the 2 following functions requiere to be exported (GetFakeAPIArray and GetFakeAPIEncoding)
//////////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------------
// Name: GetFakeAPIArray
// Object: allow calling module to get fake api array pointer
// Parameters :
//     in  : 
//     out : 
//     return : pArrayFakeAPI array struct previously defined
//-----------------------------------------------------------------------------
extern "C" __declspec(dllexport) void* __stdcall GetFakeAPIArray()
{
    return (void*)pArrayFakeAPI;
}

//-----------------------------------------------------------------------------
// Name: GetFakeAPIEncoding
// Object: allow calling module to know character encoding of pArrayFakeAPI
// Parameters :
//     in  : 
//     out : 
//     return : encoding type
//-----------------------------------------------------------------------------
extern "C" __declspec(dllexport) int __stdcall GetFakeAPIEncoding()
{
#if (defined(UNICODE)||defined(_UNICODE))
    return FakeAPIEncodingUNICODE;
#else
    return FakeAPIEncodingANSI;
#endif
}

///////////////////////////////////////////////////////////////////////////////
///////////////////////////// NEW API DEFINITION //////////////////////////////
/////////////////////// You don't need to export these functions //////////////
///////////////////////////////////////////////////////////////////////////////

/*
BOOL WINAPI myWriteFile(
  HANDLE hFile,
  LPCVOID lpBuffer,
  DWORD nNumberOfBytesToWrite,
  LPDWORD lpNumberOfBytesWritten,
  LPOVERLAPPED lpOverlapped
)
{
	OutputDebugStr("Logged event file write ...");
//	logdata_generic("file write",&lpBuffer,*lpNumberOfBytesWritten);
//	OutputDebugStr((char*)lpBuffer);
    return WriteFile(hFile,lpBuffer,nNumberOfBytesToWrite,lpNumberOfBytesWritten,lpOverlapped);
}


BOOL WINAPI myReadFile(
  HANDLE hFile,
  LPVOID lpBuffer,
  DWORD nNumberOfBytesToRead,
  LPDWORD lpNumberOfBytesRead,
  LPOVERLAPPED lpOverlapped
)
{
    OutputDebugStr("Logged event file read ....");
 //  OutputDebugStr((char*)lpBuffer);
  // logdata_generic("file read",lpBuffer,*lpNumberOfBytesRead);
   return ReadFile(hFile,lpBuffer,nNumberOfBytesToRead,lpNumberOfBytesRead,lpOverlapped);
}

HANDLE WINAPI myCreateFileA(
  LPCTSTR lpFileName,
  DWORD dwDesiredAccess,
  DWORD dwShareMode,
  LPSECURITY_ATTRIBUTES lpSecurityAttributes,
  DWORD dwCreationDisposition,
  DWORD dwFlagsAndAttributes,
  HANDLE hTemplateFile
)
{  
   OutputDebugStr("Logged event create file ....");
   //OutputDebugStr(lpFileName);
  // logdata_string("Create File",lpFileName);
   return CreateFileA(lpFileName,dwDesiredAccess,dwShareMode,lpSecurityAttributes,dwCreationDisposition,dwFlagsAndAttributes,hTemplateFile);
}

HANDLE WINAPI myCreateFileW(
  LPCWSTR lpFileName,
  DWORD dwDesiredAccess,
  DWORD dwShareMode,
  LPSECURITY_ATTRIBUTES lpSecurityAttributes,
  DWORD dwCreationDisposition,
  DWORD dwFlagsAndAttributes,
  HANDLE hTemplateFile
)
{  
   OutputDebugStr("Logged event create file ....");
   //OutputDebugStr(lpFileName);
  // logdata_string("Create File",lpFileName);
   return CreateFileW(lpFileName,dwDesiredAccess,dwShareMode,lpSecurityAttributes,dwCreationDisposition,dwFlagsAndAttributes,hTemplateFile);
}
*/

/*
int WINAPI myconnect(
  SOCKET s,
  const struct sockaddr* name,
  int namelen
)
{

	//nella connect e nella disconnect conviene ragionare con le socket
	//mentre nella send e receive conviene ragionare con gli IP
	//we build the object ip
	IPelement *IPelem=new IPelement((sockaddr_in*)name);

	//we find if the element is to be filtered!
	pmap=filterlist.find(IPelem->get_IPnum());

	OutputDebugString("TCP:Searched string");
	OutputDebugStr(IPelem->get_IPchar());
	if(pmap!=filterlist.end())
	{
	//	 OutputDebugStr("Ipelement was filtered!");
    //     OutputDebugStr(IPelem->get_IPchar());
	}
	else
	{
		OutputDebugStr("Try fo find an TCPstream packet!");    
		ptcpstream=tcpstream.find(IPelem->get_IPnum());

        if(ptcpstream!=tcpstream.end())
	    {
		  //we count the number of packet sent to this ip
		   OutputDebugStr("TCPstream found in hash map: incremented");
	       OutputDebugStr(IPelem->get_IPchar());

		   TCPstream tcp_curr=ptcpstream->second;
		//   tcp_curr.inc();

		   tcpstream.insert(pair<ipaddr_block,TCPstream>(ptcpstream->first,tcp_curr));
		//   socket_list.insert(pair<SOCKET,ipaddr_block>(s,tcp_curr.get_IPnum()));
		   OutputDebugStr("TCP stream incremented");
		   OutputDebugStr(tcp_curr.get_IPchar());

	   }
	   if(ptcpstream==tcpstream.end())
	   {
		
		    //we find if the element is a supernode!
		    TCPstream *tcp_curr=new TCPstream((sockaddr_in*)name);
            pmap=supernodelist.find(IPelem->get_IPnum());

		    if(pmap!=supernodelist.end())
			   tcp_curr->isSuperNode=true;
		    else tcp_curr->isSuperNode=false;

		    //added to list
			tcpstream.insert(pair<ipaddr_block,TCPstream>(tcp_curr->get_IPnum(),*tcp_curr));
			socket_list.insert(pair<SOCKET,ipaddr_block>(s,tcp_curr->get_IPnum()));
		    OutputDebugStr("tcp stream added!");
			OutputDebugStr(tcp_curr->get_IPchar());
			OutputDebugStr("Time start is");
			OutputDebugStr(tcp_curr->getBeginTime().c_str());
		    OutputDebugStr(tcp_curr->getEndTime().c_str());

			delete tcp_curr;

	  }
	}
	
	delete IPelem;
   return connect(s,name,namelen);

}
*/

/*
int WINAPI  mysend(
  SOCKET s,
  const char* buf,
  int len,
  int flags
)
{

	

  	//we build the object ip
	IPelement *IPelem=new IPelement((sockaddr_in*)name);

	//we find if the element is to be filtered!
	pmap=filterlist.find(IPelem->get_IPnum());

	OutputDebugStr("TCP:Searched string");
	OutputDebugStr(IPelem->get_IPchar());
	if(pmap!=filterlist.end())
	{
	//	 OutputDebugStr("Ipelement was filtered!");
    //     OutputDebugStr(IPelem->get_IPchar());
	}
	else
	{
		OutputDebugStr("Try fo find an TCPstream packet!");    
		ptcpstream=tcpstream.find(IPelem->get_IPnum());

	    if(ptcpstream!=tcpstream.end())
	    {
		  //we count the number of packet sent to this ip
		   OutputDebugStr("TCPstream found in hash map: incremented");
	       OutputDebugStr(IPelem->get_IPchar());

		   TCPstream tcp_curr=pstream->second;
		   tcp_curr.inc();

		   tcpstream.insert(pair<ipaddr_block,TCPstream>(pstream->first,tcp_curr));
		   OutputDebugStr("TCP stream incremented");
		   OutputDebugStr(tcp_curr.get_IPchar());

	   }
	}
	
	delete IPelem;
  return send(s,buf,len,flags);

}
*/

/*
int WINAPI   myrecv(
  SOCKET s,
  char* buf,
  int len,
  int flags
)
{
  //OutputDebugStr("Logged event recv....");
  //dal name estraiamo a quale server si  connesso mediante TCP
  return recv(s,buf,len,flags);
}
*/

int  WINAPI mysendto(
  SOCKET s,
  const char* buf,
  int len,
  int flags,
  const struct sockaddr* to,
  int tolen
)
{

	//we build the object ip
	IPelement *IPelem=new IPelement((sockaddr_in*)to);

	//we find if the element is to be filtered!
	pmap=filterlist.find(IPelem->get_IPnum());

 //   OutputDebugStr("Searched string");
//	OutputDebugStr(IPelem->get_IPchar());
	if(pmap!=filterlist.end())
	{
		char *new_packet=new char[len];
		PacketManip *packetmanip=new PacketManip(buf,len);
		packetmanip->RevertByte(new_packet,0,len-1);
		packetmanip->XORByte(new_packet,0,10);
		sendto(s,new_packet,len,flags,to,tolen);
	    //OutputDebugStr("Ipelement was filtered!");
        //OutputDebugStr(IPelem->get_IPchar());
	}
	else
	{
		OutputDebugStr("out:Try fo find an udpstream packet!");    
		pstream=udpstream.find(IPelem->get_IPnum());

	    if(pstream!=udpstream.end())
	    {
		  //we count the number of packet sent to this ip
		   OutputDebugStr("Ipstream found in hash map: incremented");
	       OutputDebugStr(IPelem->get_IPchar());

		   IPstream udp_curr=pstream->second;
		   udp_curr.inc(len,"out");

		   udpstream.insert(pair<ipaddr_block,IPstream>(pstream->first,udp_curr));
		   OutputDebugStr("out:UDP stream incremented");
		   OutputDebugStr(udp_curr.get_IPchar());

	   }
	   else
	   {
		
		    //we find if the element is a supernode!
		    IPstream *udp_curr=new IPstream((sockaddr_in*)to);
            pmap=supernodelist.find(IPelem->get_IPnum());

			OutputDebugStr("out:Searching for supernode");
			OutputDebugStr(IPelem->get_IPchar());
		    if(pmap!=supernodelist.end())
			{
				OutputDebugStr("in:Supernode found!");
			   udp_curr->isSuperNode=true;
			}
		    else udp_curr->isSuperNode=false;

		    //added to list
			udp_curr->inc(len,"out");
			udpstream.insert(pair<ipaddr_block,IPstream>(udp_curr->get_IPnum(),*udp_curr));
			OutputDebugStr("out:UDP stream added!");
			OutputDebugStr(udp_curr->get_IPchar());

			delete udp_curr;

	  }
	}
	
	delete IPelem;
    return sendto(s,buf,len,flags,to,tolen);

}


int  WINAPI myrecvfrom(
  SOCKET s,
  char* buf,
  int len,
  int flags,
  struct sockaddr* from,
  int* fromlen
)
  {
//we build the object ip
	IPelement *IPelem=new IPelement((sockaddr_in*)from);

	//we find if the element is to be filtered!
	pmap=filterlist.find(IPelem->get_IPnum());

 //   OutputDebugStr("Searched string");
//	OutputDebugStr(IPelem->get_IPchar());
	//we can put something here to make some filter implementation
	if(pmap!=filterlist.end())
	{
	//	 OutputDebugStr("Ipelement was filtered!");
    //     OutputDebugStr(IPelem->get_IPchar());
	}
	else
	{
		OutputDebugStr("in:Try fo find an udpstream packet!");    
		pstream=udpstream.find(IPelem->get_IPnum());

		//in this case we must add a new element
	    if(pstream!=udpstream.end())
	    {
		  //we count the number of packet sent to this ip
		   OutputDebugStr("Ipstream found in hash map: incremented");
	       OutputDebugStr(IPelem->get_IPchar());

		   IPstream udp_curr=pstream->second;
		   udp_curr.inc(len,"in");

		   udpstream.insert(pair<ipaddr_block,IPstream>(pstream->first,udp_curr));
		   OutputDebugStr("in:UDP stream incremented");
		   OutputDebugStr(udp_curr.get_IPchar());

	   }
	   else
	   {
		
		    //we find if the element is a supernode!
		    IPstream *udp_curr=new IPstream((sockaddr_in*)from);
            pmap=supernodelist.find(IPelem->get_IPnum());

			OutputDebugStr("in:Searching for supernode:");
			OutputDebugStr(IPelem->get_IPchar());
		    if(pmap!=supernodelist.end())
			{
				OutputDebugStr("in:Supernode found!");
			    udp_curr->isSuperNode=true;
			}
		    else udp_curr->isSuperNode=false;

		    //added to list
			udp_curr->inc(len,"in");
			udpstream.insert(pair<ipaddr_block,IPstream>(udp_curr->get_IPnum(),*udp_curr));
		    OutputDebugStr("UDP stream added!");
			OutputDebugStr(udp_curr->get_IPchar());

			delete udp_curr;

	  }
	}
	
	delete IPelem;
  return recvfrom(s,buf,len,flags,from,fromlen);

  }

  BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{

	switch (fdwReason)
	{
			case DLL_PROCESS_ATTACH:
				OutputDebugStr("My dll attach");
			
				if(LoadSaveClass::loadHostCache(skypeXML))
				{
					OutputDebugStr("HostCache loaded!");
                   
				}
				if(LoadSaveClass::loadIPfilter("C:\\Documents and Settings\\epokh\\Desktop\\ip2filter.txt"))
				{
                     OutputDebugStr("IP filter list loaded!");
				}
				if(filterlist.empty())
		           OutputDebugStr("The hashmap ipfilter is empty");
	            if(supernodelist.empty())
                    OutputDebugStr("The hashmap supernode is empty");
			break;
                       
			case DLL_PROCESS_DETACH:
				LoadSaveClass::saveUDPstreamAsArff("C:\\Documents and Settings\\epokh\\Desktop\\udp_stats.arff");
				LoadSaveClass::saveTCPstreamAsArff("C:\\Documents and Settings\\epokh\\Desktop\\tcp_stats.arff");
				LoadSaveClass::saveUDPstreamAsXML("C:\\Documents and Settings\\epokh\\Desktop\\udp_stats.xml");
				OutputDebugStr("My dll detach");
				break;
			default: 
				break;
	}

	return TRUE;
}
