

#ifndef IPstream_H
#define IPstream_H
#include <time.h>

enum mode {in=1,out=0};
class infoPacket
{
private:
	int packet_len;
	mode packet_mode;
	int port;
	string num2str(long int n);
public:
	infoPacket(int len,string mode,int port)
	{
		packet_len=len;
		this->port=port;
		if(mode=="in")packet_mode=in;
		else packet_mode=out;

	}
	string get_mode();
	string get_port_str(){return num2str(port);}
	string get_len_str(){return num2str(packet_len);}


};

string infoPacket::get_mode()
{
   if(packet_mode==in) return "in";
   else return "out";
}

string infoPacket::num2str(long int n)
{
   char buffer[100];
   itoa(n,buffer,10);
   return string(buffer);

}


class IPstream:public IPelement
{
private:
	long int num_packets;
	long int num_packets_out;
	long int num_packets_in;
public:
	list<infoPacket> packet_infos;
public:
	IPstream(const struct sockaddr_in* to):IPelement(to){num_packets=0;num_packets_out=0;num_packets_in=0;}
	long int get_num_packets(){return num_packets;}
	void inc(int len,string mode);
	void dec(){num_packets--;}
	long int calc_total_packet(string mode);
	long int get_total_packet(string mode);

	string get_port_str(){return num2str(IPnum->port);}
	string get_num_packets_str(){return num2str(num_packets);}
	string get_total_packet_str(string mode);

};


void IPstream::inc(int len,string mode)
{
	
	num_packets++;
	if(mode=="in") num_packets_in++;
	else num_packets_out++;

	packet_infos.push_back(infoPacket(len,mode,IPnum->port));
    OutputDebugString("The infoPacket is added!");
}
long int IPstream::calc_total_packet(string mode)
{
	list<infoPacket>::iterator p;
	long int total=0;

	for(p=packet_infos.begin();p!=packet_infos.end();p++)
	{
      if(p->get_mode()==mode) total++;
	}

	return total;

}

long int IPstream::get_total_packet(string mode)
{
    if(mode=="in") return num_packets_in;
	else return num_packets_out;

}

string IPstream::get_total_packet_str(string mode)
{
    if(mode=="in") return num2str(num_packets_in);
	else return num2str(num_packets_out);

}

class TCPstream:public IPstream
{
private:
	struct tm * connBeginTime;
	struct tm * connEndTime;

public:
	TCPstream(const struct sockaddr_in* to);
	void setBeginTime();
	void setEndTime();
	string getBeginTime();
	string getEndTime();

};

TCPstream::TCPstream(const struct sockaddr_in* to):IPstream(to)
{

  time_t temptime;
  time ( &temptime );

  connBeginTime = localtime  ( &temptime );
  connEndTime = localtime  ( &temptime );

}

void TCPstream::setBeginTime()
{
  time_t temptime;
  time ( &temptime );

  connBeginTime = localtime  ( &temptime );

}

void TCPstream::setEndTime()
{
  time_t temptime;
  time ( &temptime );

  connEndTime = localtime  ( &temptime );


}

string TCPstream::getBeginTime()
{
	return string(asctime(connBeginTime));
}

string TCPstream::getEndTime()
{
   return string(asctime(connEndTime));
}


#endif