/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
originaly based from APISpy32 v2.1 from Yariv Kaplan @ WWW.INTERNALS.COM

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "APIMonitoringFileLoader.h"
extern CLinkList* pLinkListMonitoringFileID;
extern CLinkList* pLinkListAPIInfos;
// Known Parameter Table (used for parsing config file)
// see the ParseAPIParameters func to implement new formats
tagParamSpec ParamSpec[] = {
    //  ParamName,ParamType,ParamFormat,dwParamMask
    //  param name can be upper or lower case as an insensitive comparison is done
    {_T("SIZE_T"), PARAM_SIZE_T, FORMAT_UNSIGNED_LONG,  0xFFFFFFFF},
    {_T("PSIZE_T"),PARAM_PSIZE_T,FORMAT_HEXA, 0xFFFFFFFF},
    {_T("SIZE_T*"),PARAM_PSIZE_T,FORMAT_HEXA, 0xFFFFFFFF},
    
    {_T("PSTR"),   PARAM_PSTR,   FORMAT_HEXA, 0xFFFFFFFF},
    {_T("PCSTR"),  PARAM_PSTR,   FORMAT_HEXA, 0xFFFFFFFF},
    {_T("LPSTR"),  PARAM_PSTR,   FORMAT_HEXA, 0xFFFFFFFF},
    {_T("LPCSTR"), PARAM_PSTR,   FORMAT_HEXA, 0xFFFFFFFF},
    {_T("CHAR*"),  PARAM_PSTR,   FORMAT_HEXA, 0xFFFFFFFF},

    {_T("PWSTR"),  PARAM_PWSTR,  FORMAT_HEXA, 0xFFFFFFFF},
    {_T("PCWSTR"), PARAM_PWSTR,  FORMAT_HEXA, 0xFFFFFFFF},
    {_T("LPWSTR"), PARAM_PWSTR,  FORMAT_HEXA, 0xFFFFFFFF},
    {_T("LPCWSTR"),PARAM_PWSTR,  FORMAT_HEXA, 0xFFFFFFFF},
    {_T("wchar_t*"),PARAM_PWSTR, FORMAT_HEXA, 0xFFFFFFFF},

    {_T("PVOID"),  PARAM_PVOID,  FORMAT_HEXA, 0xFFFFFFFF},
    {_T("LPVOID"), PARAM_PVOID,  FORMAT_HEXA, 0xFFFFFFFF},
    {_T("LPCVOID"),PARAM_PVOID,  FORMAT_HEXA, 0xFFFFFFFF},

    {_T("CHAR"),   PARAM_CHAR,   FORMAT_UNSIGNED_LONG,  0x000000FF},
    {_T("UCHAR"),  PARAM_UCHAR,   FORMAT_UNSIGNED_LONG,  0x000000FF},
    {_T("BOOLEAN"),PARAM_BOOLEAN,FORMAT_UNSIGNED_LONG,  0x000000FF},
    {_T("BYTE"),   PARAM_BYTE,   FORMAT_UNSIGNED_LONG,  0x000000FF},
    {_T("BYTE*"),  PARAM_PBYTE,  FORMAT_HEXA, 0xFFFFFFFF},
    {_T("PBYTE"),  PARAM_PBYTE,  FORMAT_HEXA, 0xFFFFFFFF},
    {_T("BOOLEAN*"),PARAM_PBOOLEAN, FORMAT_HEXA, 0xFFFFFFFF},
    {_T("UCHAR*"),  PARAM_PUCHAR,  FORMAT_HEXA, 0xFFFFFFFF},

    {_T("BOOL"),   PARAM_BOOL,   FORMAT_UNSIGNED_LONG,  0xFFFFFFFF},
    {_T("BOOL*"),  PARAM_PBOOL,  FORMAT_HEXA, 0xFFFFFFFF},
    {_T("PBOOL"),  PARAM_PBOOL,  FORMAT_HEXA, 0xFFFFFFFF},
    {_T("LPBOOL"), PARAM_PBOOL,  FORMAT_HEXA, 0xFFFFFFFF},

    {_T("wchar_t"),PARAM_WCHAR,   FORMAT_UNSIGNED_LONG,  0x0000FFFF},
    {_T("WORD"),   PARAM_WORD,   FORMAT_UNSIGNED_LONG,  0x0000FFFF},
    {_T("short"),  PARAM_SHORT,   FORMAT_UNSIGNED_LONG,  0x0000FFFF},
    {_T("WPARAM"), PARAM_WPARAM,   FORMAT_UNSIGNED_LONG,  0x0000FFFF},
    {_T("WORD*"),  PARAM_PWORD,  FORMAT_HEXA, 0xFFFFFFFF},
    {_T("short*"), PARAM_PSHORT,  FORMAT_HEXA, 0xFFFFFFFF},
    {_T("PWORD"),  PARAM_PWORD,  FORMAT_HEXA, 0xFFFFFFFF},
    {_T("LPWORD"), PARAM_PWORD,  FORMAT_HEXA, 0xFFFFFFFF},

    {_T("INT"),    PARAM_INT,    FORMAT_LONG,  0xFFFFFFFF},
    {_T("INT*"),   PARAM_PINT,   FORMAT_HEXA, 0xFFFFFFFF},
    {_T("PINT"),   PARAM_PINT,   FORMAT_HEXA, 0xFFFFFFFF},
    {_T("LPINT"),  PARAM_PINT,   FORMAT_HEXA, 0xFFFFFFFF},

    {_T("UINT"),   PARAM_UINT,   FORMAT_UNSIGNED_LONG,  0xFFFFFFFF},
    {_T("UINT*"),  PARAM_PUINT,   FORMAT_HEXA, 0xFFFFFFFF},
    {_T("PUINT"),  PARAM_PUINT,   FORMAT_HEXA, 0xFFFFFFFF},
    {_T("LPUINT"), PARAM_PUINT,   FORMAT_HEXA, 0xFFFFFFFF},

    {_T("LONG"),   PARAM_LONG,   FORMAT_LONG, 0xFFFFFFFF},
    {_T("LONG_PTR"),PARAM_LONG_PTR,   FORMAT_LONG, 0xFFFFFFFF},
    {_T("LPARAM"), PARAM_LPARAM, FORMAT_LONG, 0xFFFFFFFF},
    {_T("LONG*"),  PARAM_PLONG,  FORMAT_HEXA, 0xFFFFFFFF},
    {_T("PLONG"),  PARAM_PLONG,  FORMAT_HEXA, 0xFFFFFFFF},
    {_T("LPLONG"), PARAM_PLONG,  FORMAT_HEXA, 0xFFFFFFFF},

    
    {_T("ULONG_PTR"),  PARAM_ULONG_PTR,  FORMAT_UNSIGNED_LONG,  0xFFFFFFFF},
    {_T("ULONG"),  PARAM_ULONG,  FORMAT_UNSIGNED_LONG,  0xFFFFFFFF},
    {_T("DWORD"),  PARAM_DWORD,  FORMAT_UNSIGNED_LONG,  0xFFFFFFFF},
    {_T("ULONG*"), PARAM_PULONG, FORMAT_HEXA, 0xFFFFFFFF},
    {_T("DWORD*"), PARAM_PDWORD, FORMAT_HEXA, 0xFFFFFFFF},
    {_T("PDWORD"), PARAM_PDWORD, FORMAT_HEXA, 0xFFFFFFFF},
    {_T("LPDWORD"),PARAM_PDWORD, FORMAT_HEXA, 0xFFFFFFFF},

    {_T("HANDLE"), PARAM_HANDLE, FORMAT_HEXA, 0xFFFFFFFF},
    {_T("HINSTANCE"), PARAM_HINSTANCE, FORMAT_HEXA, 0xFFFFFFFF},
    {_T("HWND"),   PARAM_HWND,   FORMAT_HEXA, 0xFFFFFFFF},
    {_T("HMODULE"),PARAM_HMODULE,FORMAT_HEXA, 0xFFFFFFFF},
    {_T("HMODULE*"),PARAM_PHMODULE,FORMAT_HEXA, 0xFFFFFFFF},
    {_T("PHMODULE"),PARAM_PHMODULE,FORMAT_HEXA, 0xFFFFFFFF},
    {_T("HDC"),    PARAM_HDC,    FORMAT_HEXA, 0xFFFFFFFF},
    {_T("WNDPROC"),PARAM_WNDPROC,FORMAT_HEXA, 0xFFFFFFFF},
    {_T("HMENU"),  PARAM_HMENU,  FORMAT_HEXA, 0xFFFFFFFF},
    {_T("DLGPROC"),PARAM_DLGPROC,FORMAT_HEXA, 0xFFFFFFFF},
    {_T("FARPROC"),PARAM_FARPROC,FORMAT_HEXA, 0xFFFFFFFF},

    {_T("UNKNOWN"),PARAM_UNKNOWN,FORMAT_HEXA, 0xFFFFFFFF}// must be let as last param for loop seeking
};

//-----------------------------------------------------------------------------
// Name: LoadMonitoringFile
// Object: Parse config file and attach hooks.
//
// lines are like
// KERNEL32.DLL|GetProcAddress(HANDLE, PSTR)                       for classical definition
// KERNEL32.DLL|GetProcAddress(HANDLE, PSTR)|InOut                 for definition with parameter spy direction
// EXE_INTERNAL@0x12345|SupposedFuncName(HANDLE, PSTR)|Out         for direct adressing (allow to spy exe internal func), adress is the virtual one not the raw one
//
// for default direction see DEFAULT_PARAMETER_DIRECTION_SPYING in APIMonitoringFileLoader.h
//
// Works only with lines ending with \r\n
//
// lines begining with ; are considered as comments
//
// Parameters :
//     in  : TCHAR* pszFileName : config file name
// Return : TRUE if file is partially loaded (even if some non fatal errors occurs),
//          FALSE if file not loaded at all
//-----------------------------------------------------------------------------
BOOL LoadMonitoringFile(TCHAR* pszFileName)
{
    HANDLE hFile;
    HANDLE hMap;
    DWORD dwIndex;
    TCHAR* pszParameterList;
    TCHAR* pszParameter;
    TCHAR* pszModuleName;
    TCHAR* pszParameterDirection;
    TCHAR* pszAPIName;
    TCHAR* pcFile;
    TCHAR* pcFileOrig;
    DWORD dwFileSize;
    DWORD dwCurrentLine;
    BYTE cParamIndex;
    tagAPIInfo *pAPIInfo;
    CLinkListItem *pItemAPIInfo;
    CLinkListItem *pItemFileInfo;
    CLinkListItem* pListItemMonitoringFile;
    tagFileID* pMonitoringFileID;
    tagFileID* pFileID;
    TCHAR szAPIDefinition[MAX_CONFIG_FILE_LINE_SIZE];
    TCHAR* pszAPIDefinition;
    TCHAR* pc;
    PBYTE pbAPI=0;
    BOOL bAlreadyHooked=FALSE;
    ULONG_PTR FileID;
    TCHAR pszMsg[2*MAX_PATH];
    BOOL bUnicodeFile=FALSE;
    TCHAR* AllocatedHeap=NULL;

    if (*pszFileName==0)
    {
        DynamicMessageBox(NULL,_T("Empty file name"),_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }

    // check if file is already loaded
    pListItemMonitoringFile = pLinkListMonitoringFileID->Head;
    // look throw pItemMonitoringFileInfo
    while (pListItemMonitoringFile)
    {
        pMonitoringFileID=(tagFileID*)pListItemMonitoringFile->ItemData;
        if (_tcsicmp(pszFileName,pMonitoringFileID->szFileName)==0)
        {
            _sntprintf(pszMsg,2*MAX_PATH,_T("File %s already loaded"),pszFileName);
            DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
            return FALSE;
        }
        pListItemMonitoringFile = pListItemMonitoringFile->NextItem;
    }

    // open config file
    hFile = CreateFile(pszFileName, GENERIC_READ, FILE_SHARE_READ, NULL,
        OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
    if (hFile == INVALID_HANDLE_VALUE)
    {
        _sntprintf(pszMsg,2*MAX_PATH,_T("File %s not found"),pszFileName);
        DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }
    dwFileSize = GetFileSize(hFile, NULL);
    if (dwFileSize == 0xFFFFFFFF)
    {
        CloseHandle(hFile);
        _sntprintf(pszMsg,2*MAX_PATH,_T("Can't get file size for %s"),pszFileName);
        DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }
    hMap = CreateFileMapping(hFile, NULL, PAGE_READONLY, 0, 0, NULL);
    if (hMap == NULL) 
    {
        CloseHandle(hFile);
        _sntprintf(pszMsg,2*MAX_PATH,_T("Can't create file mapping for %s"),pszFileName);
        DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }
    pcFileOrig = (TCHAR*)MapViewOfFile(hMap, FILE_MAP_READ, 0, 0, 0);
    pcFile=pcFileOrig;
    if (pcFile == NULL) 
    {
        CloseHandle(hMap);
        CloseHandle(hFile);
        _sntprintf(pszMsg,2*MAX_PATH,_T("Can't map view of file %s"),pszFileName);
        DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }


    // check if file is unicode or ansi file
    // unicode files begin with FFFE in little endian
    if (dwFileSize>=2)
    {
        // check FFFE (we have to cast pcFileOrig in PBYTE as we don't know if it's 8 or 16 byte pointer)
        if ((((PBYTE)pcFileOrig)[0]==0xFF)
            &&(((PBYTE)pcFileOrig)[1]==0xFE))
        {
            bUnicodeFile=TRUE;
            // point after Unicode header
            pcFile=(TCHAR*)(&((PBYTE)pcFileOrig)[2]);
        }
    }
#if (defined(UNICODE)||defined(_UNICODE))
    // if we work in unicode and the file is in ansi, convert it in unicode
    if (bUnicodeFile)
    {
        // remove signature
        dwFileSize-=2;
        // adjust file size : file size will now be in TCHAR count no more in bytes
        dwFileSize/=2;
    }
    else
    {
        // convert into unicode
        pcFile=(TCHAR*)HeapAlloc(GetProcessHeap(), 0,(dwFileSize+1)*2);
        AllocatedHeap=pcFile;
        if (!AllocatedHeap)
        {
            CloseHandle(hMap);
            CloseHandle(hFile);
            DynamicMessageBox(NULL,_T("Memory allocation error"),_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
            return FALSE;
        }
        if (!MultiByteToWideChar(CP_ACP, 0, (LPCSTR)pcFileOrig, (int)dwFileSize,pcFile,(int)dwFileSize))
        {
            CloseHandle(hMap);
            CloseHandle(hFile);
            HeapFree(GetProcessHeap(), 0,AllocatedHeap);

            _sntprintf(pszMsg,2*MAX_PATH,_T("Error converting file %s in unicode"),pszFileName);
            DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
            
            return FALSE;
        }
        pcFile[dwFileSize]=0;
    }
#else
    // if we work in ansi and the file is in unicode, convert it in ansi
    if (bUnicodeFile)
    {
        // convert into ansi
        pcFile=(TCHAR*)HeapAlloc(GetProcessHeap(), 0,(dwFileSize-2)/2+1);// -2 to remove signature
        AllocatedHeap=pcFile;
        if (!AllocatedHeap)
        {
            CloseHandle(hMap);
            CloseHandle(hFile);

            DynamicMessageBox(NULL,_T("Memory allocation error"),_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
            return FALSE;
        }
        if (!WideCharToMultiByte(CP_ACP, 0,(LPCWSTR)(&((PBYTE)pcFileOrig)[2]), (int)(dwFileSize-2)/2, pcFile, (int)(dwFileSize-2)/2, NULL, NULL))
        {
            CloseHandle(hMap);
            CloseHandle(hFile);
            HeapFree(GetProcessHeap(), 0,AllocatedHeap);

            _sntprintf(pszMsg,2*MAX_PATH,_T("Error converting file %s in ansi"),pszFileName);
            DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
            return FALSE;
        }
        // adjust file size : file size will now be in TCHAR count no more in bytes
        dwFileSize=(dwFileSize-2)/2;
        pcFile[dwFileSize]=0;
    }
#endif

    // store info for file unload
    FileID=GetTickCount();
    pListItemMonitoringFile=pLinkListMonitoringFileID->AddItem();
    if (pListItemMonitoringFile)
    {
        pMonitoringFileID=(tagFileID*)pListItemMonitoringFile->ItemData;
        _tcsncpy(pMonitoringFileID->szFileName,pszFileName,MAX_PATH);
        pMonitoringFileID->Id=FileID;
    }

    // parse config file
    dwCurrentLine=1;
    while (dwFileSize)
    {
        // read char until end of line or end char (= end of file)
        for (dwIndex = 0; *pcFile != 13 && dwFileSize; dwIndex++)
        {
            szAPIDefinition[dwIndex] = *pcFile;
            pcFile++;
            dwFileSize--;
        }
        // ends string buffer
        szAPIDefinition[dwIndex] = 0;
        pszAPIDefinition=TrimString(szAPIDefinition);

        // if not empty or commented line
        if ((*pszAPIDefinition!=0)&&(*pszAPIDefinition!=';'))
        {
            // pszAPIDefinition is like KERNEL32.DLL|GetProcAddress(HANDLE, PSTR)
            //                       or KERNEL32.DLL|GetProcAddress(HANDLE, PSTR)|In
            //                       or EXE_INTERNAL@0x411B70|Add(int x,int y)
            //                       or EXE_INTERNAL@0x411B70|Add(int x,int y)|In
            pszModuleName = pszAPIDefinition;
            pszAPIName = _tcschr(pszModuleName, FIELDS_SEPARATOR);
                
            if (!pszAPIName)
            {
                // parsing error ask if we should continue
                _sntprintf(pszMsg,2*MAX_PATH,_T("Parsing Error in %s at line: %d\r\n%s\r\nContinue parsing ?"),pszFileName,dwCurrentLine,pszAPIDefinition);
                if (DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_YESNO|MB_ICONERROR|MB_TOPMOST)!=IDYES)
                {
                    // stop parsing
                    break;
                }
            }
            else
            {
                // ends pszModuleName string by replacing | by \0
                *pszAPIName = 0;
                // get char after | for pszAPIName
                pszAPIName++;

                // look for parameter direction
                pszParameterDirection=_tcsrchr(pszAPIName,FIELDS_SEPARATOR);
                // if direction is specified
                if (pszParameterDirection)
                    // ends pszAPIName
                    *pszParameterDirection=0;

                pszParameterList = _tcsrchr(pszAPIName, '(');
                if (pszParameterList)
                {
                    // ends pszAPIName string by replacing ( by \0
                    *pszParameterList=0;
                    // pszParameterList begins after (
                    pszParameterList++;

                    // trim pszAPIName
                    pszAPIName=TrimString(pszAPIName);

                    // remove return type in pszAPIName if any
                    // search last ' ' in pszAPIName
                    pc=_tcschr(pszAPIName,' ');
                    // keep only data after last space
                    if (pc)
                    {
                        pc++;// point to char after space
                        pszAPIName=pc;
                    }

                    //////////////////////////////////////
                    // Get function address
                    //////////////////////////////////////

                    // check if address is specified
                    if (_tcsnicmp(pszModuleName,EXE_INTERNAL_PREFIX,EXE_INTERNAL_PREFIX_SIZE-1)==0)// -1 because count \0
                        // remove prefixe and get value
                        _stscanf(&pszModuleName[EXE_INTERNAL_PREFIX_SIZE-1-2],_T("%x"),&pbAPI);// -2 to keep 0x
                    else
                        // get address with loadlibrary getprocaddress
                        pbAPI=GetFuncAddr(pszModuleName,pszAPIName);

                    if (IsBadCodePtr((FARPROC)pbAPI))
                    {
                        _sntprintf(pszMsg,2*MAX_PATH,_T("Bad code pointer for function %s in dll\r\n%s\r\nFunction not hooked"),pszAPIName,pszModuleName);
                        DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                    }
                    else
                    {
                        pItemAPIInfo=GetAssociatedItemAPIInfo(pbAPI,&bAlreadyHooked);
                        if (pItemAPIInfo)
                        {
                            pAPIInfo=(tagAPIInfo*)pItemAPIInfo->ItemData;
                            if (pAPIInfo)
                            {
                                // monitoring is already hooked by another file
                                if (pAPIInfo->MonitoringFileID!=0)
                                {
                                    _sntprintf(pszMsg,2*MAX_PATH,_T("A monitoring file is was already defined for\r\nmodule %s\r\nand API name %s\r\nby "),pszModuleName,pszAPIName);

                                    // get already hooking file name
                                    pItemFileInfo = pLinkListMonitoringFileID->Head;
                                    // look throw pItemFileInfo
                                    while (pItemFileInfo)
                                    {
                                        pFileID=(tagFileID*)pItemFileInfo->ItemData;
                                        if (pAPIInfo->MonitoringFileID==pFileID->Id)
                                        {
                                            // filename found add it to szMsg
                                            _tcscat(pszMsg,pFileID->szFileName);
                                            break;// go out of while
                                        }
                                        pItemFileInfo = pItemFileInfo->NextItem;
                                    }

                                    _tcscat(pszMsg,_T("\r\nDo you want this new monitoring file take the ownership of the hook ?"));
                                    if(DynamicMessageBox(NULL,pszMsg,_T("API Override Warning"),MB_ICONWARNING|MB_YESNO|MB_TOPMOST)==IDYES)
                                    {
                                        // if user want to update, update monitoring file ID
                                        pAPIInfo->MonitoringFileID=FileID;
                                    }
                                }
                                else// monitoring is not already hooked by another file
                                {
                                    //////////////////////////////////////
                                    // copy module name
                                    //////////////////////////////////////
                                    pAPIInfo->szModuleName = (TCHAR*)HeapAlloc(GetProcessHeap(), 0, (_tcslen(pszModuleName) + 1)*sizeof(TCHAR));
                                    if (pAPIInfo->szModuleName == NULL)
                                    {
                                        if (!bAlreadyHooked)// if no fake hook is installed
                                            pLinkListAPIInfos->RemoveItem(pItemAPIInfo); // hook is not installed don't remove it
                                        UnmapViewOfFile(pcFileOrig);
                                        CloseHandle(hMap);
                                        CloseHandle(hFile);

                                        DynamicMessageBox(NULL,_T("Memory allocation error"),_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);

                                        if (AllocatedHeap)
                                            HeapFree(GetProcessHeap(), 0,AllocatedHeap);

                                        return TRUE;// file has begin to be loaded --> must be unloaded
                                    }
                                    _tcscpy(pAPIInfo->szModuleName, pszModuleName);

                                    //////////////////////////////////////
                                    // copy API name
                                    //////////////////////////////////////
                                    pAPIInfo->szAPIName = (TCHAR*)HeapAlloc(GetProcessHeap(), 0, (_tcslen(pszAPIName) + 1)*sizeof(TCHAR));
                                    if (pAPIInfo->szAPIName == NULL)
                                    {
                                        HeapFree(GetProcessHeap(), 0, pAPIInfo->szModuleName);
                                        if (!bAlreadyHooked)// if no fake hook is installed
                                            pLinkListAPIInfos->RemoveItem(pItemAPIInfo); // hook is not installed don't remove it
                                        UnmapViewOfFile(pcFileOrig);
                                        CloseHandle(hMap);
                                        CloseHandle(hFile);
                                        DynamicMessageBox(NULL,_T("Memory allocation error"),_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);

                                        if (AllocatedHeap)
                                            HeapFree(GetProcessHeap(), 0,AllocatedHeap);

                                        return TRUE;// file has begin to be loaded --> must be unloaded
                                    }
                                    _tcscpy(pAPIInfo->szAPIName, pszAPIName);


                                    //////////////////////////////////////
                                    // parse all func parameters
                                    //////////////////////////////////////
                                    cParamIndex = 0;
                                    pszParameter = _tcstok(pszParameterList, _T(",;)"));

                                    // by default put all params to unknown in case of bad param number
                                    memset(pAPIInfo->ParamList,PARAM_UNKNOWN,MAX_PARAM);

                                    // fill given params type
                                    while (pszParameter)
                                    {
                                        pszParameter = TrimString(pszParameter);

                                        // remove usless keywords ( const, struct, IN, OUT, INOUT, FAR)
                                        // must be done before checking for " *" (like for "char FAR *")

                                        // convert param in upper case to make insensitive search
                                        _tcsupr(pszParameter);

                                        // remove CONST keyword
                                        pc=_tcsstr(pszParameter,_T("CONST "));// lets space to assume its not the begin of type
                                        if (pc)
                                        {
                                            // pc must be point first TCHAR or previous TCHAR must be space
                                            if (FirstCharOrPreviousCharSpace(pc,pszParameter))
                                                _tcscpy(&pszParameter[pc-pszParameter],&pszParameter[pc-pszParameter+6]);// len of "CONST " in TCHAR
                                        }

                                        // remove STRUCT keyword
                                        pc=_tcsstr(pszParameter,_T("STRUCT "));// lets space to assume its not the begin of type
                                        if (pc)
                                        {
                                            // pc must be point first TCHAR or previous TCHAR must be space
                                            if (FirstCharOrPreviousCharSpace(pc,pszParameter))
                                                _tcscpy(&pszParameter[pc-pszParameter],&pszParameter[pc-pszParameter+7]);// len of "STRUCT " in TCHAR
                                        }

                                        // remove IN keyword
                                        pc=_tcsstr(pszParameter,_T("IN "));// lets space to assume its not the begin of type
                                        if (pc)
                                        {
                                            // pc must be point first TCHAR or previous TCHAR must be space
                                            if (FirstCharOrPreviousCharSpace(pc,pszParameter))
                                                _tcscpy(&pszParameter[pc-pszParameter],&pszParameter[pc-pszParameter+3]);// len of "IN " in TCHAR
                                        }

                                        // remove OUT keyword
                                        pc=_tcsstr(pszParameter,_T("OUT "));// lets space to assume its not the begin of type
                                        if (pc)
                                        {
                                            // pc must be point first TCHAR or previous TCHAR must be space
                                            if (FirstCharOrPreviousCharSpace(pc,pszParameter))
                                                _tcscpy(&pszParameter[pc-pszParameter],&pszParameter[pc-pszParameter+4]);// len of OUT in TCHAR
                                        }

                                        // remove INOUT keyword
                                        pc=_tcsstr(pszParameter,_T("INOUT "));// lets space to assume its not the begin of type
                                        if (pc)
                                        {
                                            // pc must be point first TCHAR or previous TCHAR must be space
                                            if (FirstCharOrPreviousCharSpace(pc,pszParameter))
                                                _tcscpy(&pszParameter[pc-pszParameter],&pszParameter[pc-pszParameter+6]);// len of "INOUT " in TCHAR
                                        }

                                        // remove FAR keyword
                                        pc=_tcsstr(pszParameter,_T("FAR "));// lets space to assume its not the begin of type
                                        if (pc)
                                        {
                                            // pc must be point first TCHAR or previous TCHAR must be space
                                            if (FirstCharOrPreviousCharSpace(pc,pszParameter))
                                                _tcscpy(&pszParameter[pc-pszParameter],&pszParameter[pc-pszParameter+4]);// len of "FAR " in TCHAR
                                        }


                                        // looking for " *" for definitions like "char *psz" to keep "char*"
                                        pc=_tcsstr(pszParameter,_T(" *"));
                                        if (pc)
                                        {
                                            // replace space by *
                                            *pc='*';
                                            pc++;
                                            // check for over * like char **
                                            while (*(pc+1)=='*')
                                            {
                                                *pc='*';
                                                pc++;
                                            }
                                            // ends pszParameter string
                                            *pc=0;
                                        }

                                        // check if there's still param name
                                        pc=_tcsrchr(pszParameter,' ');
                                        if (pc)// if it is the case
                                            *pc=0;// remove param name
                                        pAPIInfo->ParamList[cParamIndex] = GetParamType(pszParameter);
                                        cParamIndex++;

                                        if (cParamIndex == MAX_PARAM)
                                            break;

                                        pszParameter = _tcstok(NULL, _T(",;)"));
                                    }
                                    // too much param --> dont hook func (MAX_PARAM can be changed in APIOverrideKernel.h)
                                    if (cParamIndex == MAX_PARAM)
                                    {
                                        HeapFree(GetProcessHeap(), 0, pAPIInfo->szAPIName);
                                        HeapFree(GetProcessHeap(), 0, pAPIInfo->szModuleName);
                                        if (!bAlreadyHooked)// if no fake hook is installed
                                            pLinkListAPIInfos->RemoveItem(pItemAPIInfo);
                                    }
                                    else
                                        pAPIInfo->ParamCount = cParamIndex;


                                    if (cParamIndex < MAX_PARAM)
                                    {
                                        //////////////////////////////////////
                                        // specify parameter direction spying
                                        //////////////////////////////////////

                                        // if direction is specified
                                        if (pszParameterDirection)
                                        {
                                            // pszParameterDirection points to char after |
                                            pszParameterDirection++;
                                            // trim
                                            pszParameterDirection=TrimString(pszParameterDirection);
                                            // find type
                                            if (_tcsicmp(pszParameterDirection,PARAM_DIR_IN_NAME)==0)
                                                pAPIInfo->ParamDirectionType=PARAM_DIR_IN;
                                            else if (_tcsicmp(pszParameterDirection,PARAM_DIR_OUT_NAME)==0)
                                                pAPIInfo->ParamDirectionType=PARAM_DIR_OUT;
                                            else if (_tcsicmp(pszParameterDirection,PARAM_DIR_INOUT_NAME)==0)
                                                pAPIInfo->ParamDirectionType=PARAM_DIR_INOUT;
                                            else if (_tcsicmp(pszParameterDirection,PARAM_DIR_NONE_NAME)==0)
                                                pAPIInfo->ParamDirectionType=PARAM_DIR_NONE;
                                            else if (_tcsicmp(pszParameterDirection,PARAM_DIR_INNORET_NAME)==0)
                                                pAPIInfo->ParamDirectionType=PARAM_DIR_IN_NO_RETURN;
                                            else
                                                pAPIInfo->ParamDirectionType=DEFAULT_PARAMETER_DIRECTION_SPYING;
                                        }
                                        else
                                            pAPIInfo->ParamDirectionType=DEFAULT_PARAMETER_DIRECTION_SPYING;

                                        // store reference to file
                                        pAPIInfo->MonitoringFileID=FileID;


                                        if (!bAlreadyHooked)
                                        {
                                            //////////////////////////////////////
                                            // save API address
                                            //////////////////////////////////////
                                            pAPIInfo->APIAddress = (FARPROC)pbAPI;

                                            //////////////////////////////////////
                                            // create a critical section to avoid simultaneous func hooks
                                            //////////////////////////////////////
                                            InitializeCriticalSection(&pAPIInfo->CriticalSection);

                                            //////////////////////////////////////
                                            // install hook now (only now when pAPIInfo is fully completed else you'll get unexpected results :D)
                                            //////////////////////////////////////
                                            if (!HookAPIFunction(pAPIInfo))
                                            {
                                                DeleteCriticalSection(&pAPIInfo->CriticalSection);
                                                HeapFree(GetProcessHeap(), 0, pAPIInfo->szAPIName);
                                                HeapFree(GetProcessHeap(), 0, pAPIInfo->szModuleName);
                                                pLinkListAPIInfos->RemoveItem(pItemAPIInfo); // hook is not installed don't remove it
                                            }
                                        }// end if (!bAlreadyHooked)
                                    }// end if cParamIndex < MAX_PARAM
                                }// end if (pAPIInfo->MonitoringFileID==0)
                            } // end if (pAPIInfo)
                        }// end if (pItemAPIInfo)
                    }// end if (pbAPI)
                }// end if (pszParameterList)
            }// end if (pszAPIName)
        }// end if (*pszAPIDefinition)

        // checks EOF
        if (dwFileSize == 0) 
            break;

        // end of line as we are in windows OS -->\r\n --> go 2 chars away and reduce remaining file size
        pcFile += 2;// work as pcFile is in TCHAR
        dwFileSize -= 2;// work as dwFileSize is in TCHAR count
        dwCurrentLine++;

    }// end while(dwFileSize)

    UnmapViewOfFile(pcFileOrig);
    CloseHandle(hMap);
    CloseHandle(hFile);

    if (AllocatedHeap)
        HeapFree(GetProcessHeap(), 0,AllocatedHeap);

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: UnloadMonitoringFile
// Object: try to unload all associated hook for this monitoring file
// Parameters :
//     in  : TCHAR* pszFileName : monitoring file name (full path)
// Return : TRUE if file is not hooked (even if error), FALSE if it is still loaded
//-----------------------------------------------------------------------------
BOOL UnloadMonitoringFile(TCHAR* pszFileName)
{
    ULONG_PTR FileID=0;
    tagFileID* pMonitoringFileID;
    CLinkListItem* pItemMonitoringFileInfo;
    tagAPIInfo *pAPIInfo;
    CLinkListItem* pItemAPIInfo;
    CLinkListItem* pNextItemAPIInfo;
    BOOL bRet;
    TCHAR pszMsg[MAX_PATH];

    if (*pszFileName==0)
    {
        DynamicMessageBox(NULL,_T("Empty file name"),_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }

    // get file ID
    pItemMonitoringFileInfo = pLinkListMonitoringFileID->Head;
    // look throw pItemMonitoringFileInfo
    while (pItemMonitoringFileInfo)
    {
        pMonitoringFileID=(tagFileID*)pItemMonitoringFileInfo->ItemData;
        if (_tcsicmp(pszFileName,pMonitoringFileID->szFileName)==0)
        {
            FileID=pMonitoringFileID->Id;
            break;
        }
        pItemMonitoringFileInfo = pItemMonitoringFileInfo->NextItem;
    }
    if (FileID==0)
    {
        _sntprintf(pszMsg,MAX_PATH,_T("File %s was not loaded"),pszFileName);
        DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return TRUE;
    }

    // look in pLinkListAPIInfos for hooks with file ID == dwFileID
    pItemAPIInfo = pLinkListAPIInfos->Head;
    bRet=TRUE;
    while (pItemAPIInfo)
    {
        pAPIInfo=(tagAPIInfo*)pItemAPIInfo->ItemData;
        pNextItemAPIInfo=pItemAPIInfo->NextItem;//UnhookAPIFunction release pItemAPIInfo data
        if (pAPIInfo->MonitoringFileID == FileID)
        {
            pAPIInfo->MonitoringFileID=0;
            // if there's no Fake API address configured, there was only a monitoring hook
            if (pAPIInfo->FakeAPIAddress==NULL)
            {
                // unhook function
                bRet=UnhookAPIFunction(pItemAPIInfo)&&bRet;
            }
        }
        pItemAPIInfo = pNextItemAPIInfo;
    }
    if (bRet)
        // remove pItemMonitoringFileInfo
        pLinkListMonitoringFileID->RemoveItem(pItemMonitoringFileInfo);
    return bRet;
}


//-----------------------------------------------------------------------------
// Name: TrimString
// Object: remove begin and ending space of a string
// Parameters :
//     inout  : TCHAR* pszStr
//     return : TCHAR* pszStr
//-----------------------------------------------------------------------------
TCHAR* TrimString(TCHAR* pszStr)
{
    TCHAR* pcBlankChar;

    if (!*pszStr)
        return pszStr;

    pcBlankChar = pszStr + _tcslen(pszStr) - 1;

    // remove last spaces
    while ((*pcBlankChar == ' ') && (pszStr<=pcBlankChar))
    {
        *pcBlankChar = '\0';
        pcBlankChar--;
    }

    // remove first spaces
    while (*pszStr == ' ') pszStr++;

    return pszStr;
}

//-----------------------------------------------------------------------------
// Name: GetParamType
// Object: get param type from tagParamSpec array (ParamSpec) for specified param name
// Parameters :
//     in : TCHAR* pszParameter : parameter name
//     out : 
//     return : param type (param UNKNOWN type if not found)
//-----------------------------------------------------------------------------
DWORD GetParamType(TCHAR* pszParameter)
{
    BYTE cIndex;

    for (cIndex = 0; ParamSpec[cIndex].ParamType != PARAM_UNKNOWN; cIndex++)
        if (_tcsicmp(pszParameter, ParamSpec[cIndex].ParamName) == 0)
            return ParamSpec[cIndex].ParamType;

    // if not found return UNKNOWN info
    return ParamSpec[cIndex].ParamType;
}

//-----------------------------------------------------------------------------
// Name: GetParamFormat
// Object: get param format from tagParamSpec array (ParamSpec) for specified param type
// Parameters :
//     in : DWORD ParamType : parameter type
//     out : 
//     return : param format (param UNKNOWN format if not found)
//-----------------------------------------------------------------------------
BYTE GetParamFormat(DWORD ParamType)
{
    BYTE cIndex;

    for (cIndex = 0; ParamSpec[cIndex].ParamType != PARAM_UNKNOWN; cIndex++)
        if (ParamSpec[cIndex].ParamType == ParamType)
            return ParamSpec[cIndex].ParamFormat;

    // if not found return UNKNOWN info
    return ParamSpec[cIndex].ParamFormat;
}


//-----------------------------------------------------------------------------
// Name: GetParamName
// Object: get param name from tagParamSpec array (ParamSpec) for specified param type
// Parameters :
//     in : DWORD ParamType : parameter type
//     out : 
//     return : param name (param UNKNOWN name if not found)
//-----------------------------------------------------------------------------
TCHAR* GetParamName(DWORD ParamType)
{
    BYTE cIndex;

    for (cIndex = 0; ParamSpec[cIndex].ParamType != PARAM_UNKNOWN; cIndex++)
        if (ParamSpec[cIndex].ParamType == ParamType)
            return ParamSpec[cIndex].ParamName;

    // if not found return UNKNOWN info
    return ParamSpec[cIndex].ParamName;
}

//-----------------------------------------------------------------------------
// Name: GetParamMask
// Object: get param mask from tagParamSpec array (ParamSpec) for specified param type
// Parameters :
//     in : DWORD ParamType : parameter type
//     out : 
//     return : param mask (param UNKNOWN mask if not found)
//-----------------------------------------------------------------------------
DWORD GetParamMask(DWORD ParamType)
{
    BYTE cIndex;

    for (cIndex = 0; ParamSpec[cIndex].ParamType != PARAM_UNKNOWN; cIndex++)
        if (ParamSpec[cIndex].ParamType == ParamType)
            return ParamSpec[cIndex].dwParamMask;

    // if not found return UNKNOWN info
    return ParamSpec[cIndex].dwParamMask;
}


//-----------------------------------------------------------------------------
// Name: ParseAPIParameters
// Object: parse parameters and put them to pszLogString
// Parameters :
//      in: tagAPIInfo *pAPIInfo : API hook info
//          DWORD pszLogStringSize : max size of pszLogString
//      out : TCHAR* pszLogString : pretty string containing func name and params values
// Return : func pointer on success, NULL on error
//-----------------------------------------------------------------------------
void ParseAPIParameters(tagAPIInfo *pAPIInfo, TCHAR* pszLogString,DWORD pszLogStringSize, ...)
{
    va_list Marker;
    DWORD dwParamValue;
    size_t UsedSize;
    TCHAR szParamStr[MAX_PATH];
    TCHAR szUnicodeStr[MAX_PATH];
    BYTE cIndex;
    BOOL bNotEnoughtSpace=FALSE;
    int iStringSize;
    
    // argument retreival
    va_start(Marker, pszLogStringSize);

    _tcsncpy(pszLogString, pAPIInfo->szAPIName,pszLogStringSize-2);
    pszLogString[pszLogStringSize-2]=0;
    _tcscat(pszLogString, _T("("));
    UsedSize=_tcslen(pszLogString);

    // loop throught parameters
    for (cIndex = 0; cIndex < pAPIInfo->ParamCount; cIndex++)
    {
        dwParamValue = va_arg(Marker, DWORD);

        dwParamValue &= GetParamMask(pAPIInfo->ParamList[cIndex]);

        // if buffer is not big enougth
        if (pszLogStringSize<(UsedSize+MAX_PARAM_NAME_STRING_SIZE+1+MAX_PARAM_FORMAT_STRING_SIZE+1))
        {
            bNotEnoughtSpace=TRUE;
            break;
        }
        // else

        // add param name
        _tcscat(pszLogString, GetParamName(pAPIInfo->ParamList[cIndex]));
        _tcscat(pszLogString, _T(":"));

        // add param value
        // wsprintf(szParamStr, GetParamFormat(pAPIInfo->ParamList[cIndex]), dwParamValue);
        switch(GetParamFormat(pAPIInfo->ParamList[cIndex]))
        {
        case FORMAT_LONG:
            _ltot(dwParamValue,szParamStr,10);
            break;
        case FORMAT_UNSIGNED_LONG:
            _ultot(dwParamValue,szParamStr,10);
            break;
        case FORMAT_HEXA:
            _tcscpy(szParamStr,_T("0x"));
            _ultot(dwParamValue,&szParamStr[2],16);
            break;
        }
        _tcscat(pszLogString, szParamStr);

        // check if param is a pointer to add pointed data value
        _tcscat(pszLogString, _T(":"));
        UsedSize=_tcslen(pszLogString);
        switch (pAPIInfo->ParamList[cIndex])
        {
        case PARAM_PSTR:
            if (IsBadReadPtr((PVOID)dwParamValue, sizeof(char)) == 0)
            {
#if (defined(UNICODE)||defined(_UNICODE))
                iStringSize=SecureStrlen((LPSTR)dwParamValue);
                if (iStringSize>=0)
                {
                    // convert ansi to unicode
                    iStringSize=MultiByteToWideChar(CP_ACP, 0, (LPCSTR)dwParamValue, (int)iStringSize,szUnicodeStr, MAX_PATH);
                    if (!iStringSize)
                        *szUnicodeStr=0;
                    else
                    {
                        if (iStringSize>=MAX_PATH)
                            iStringSize=MAX_PATH-1;
                        szUnicodeStr[iStringSize] = 0;
                    }
                    // format string
                    FormatString(szUnicodeStr, szParamStr, MAX_PATH);
                }
                else
                    _tcscpy(szParamStr, _T("?"));
#else
                FormatString((TCHAR*)dwParamValue, szParamStr, MAX_PATH);
#endif
            }
            else
            {
                _tcscpy(szParamStr, _T("?"));
            }
            break;

        case PARAM_PINT:
        case PARAM_PLONG:
            if (IsBadReadPtr((PVOID)dwParamValue, sizeof(LONG)) == 0)
            {
                // wsprintf(szParamStr, _T("%d"), *(int *)dwParamValue);
                _itot(*(int *)dwParamValue,szParamStr,10);
            }
            else
                _tcscpy(szParamStr, _T("?"));
            break;

        case PARAM_PDWORD:
        case PARAM_PSIZE_T:
        case PARAM_PHMODULE:
            if (IsBadReadPtr((PVOID)dwParamValue, sizeof(DWORD)) == 0)
            {
                // wsprintf(szParamStr, _T("%u"), *(DWORD *)dwParamValue);
                _ultot(*(DWORD *)dwParamValue,szParamStr,10);
            }
            else
                _tcscpy(szParamStr, _T("?"));
            break;

        case PARAM_PWORD:
            if (IsBadReadPtr((PVOID)dwParamValue, sizeof(WORD)) == 0)
            {
                // wsprintf(szParamStr, _T("%u"), LOWORD(*(WORD *)dwParamValue));
                _ultot(LOWORD(*(WORD *)dwParamValue),szParamStr,10);
            }
            else
               _tcscpy(szParamStr, _T("?"));
            break;

        case PARAM_PBYTE:
            if (IsBadReadPtr((PVOID)dwParamValue, sizeof(BYTE)) == 0)
            {
                // wsprintf(szParamStr, _T("%u"), LOBYTE(*(BYTE *)dwParamValue));
                _ultot(LOWORD(LOBYTE(*(BYTE *)dwParamValue)),szParamStr,10);
            }
            else
                _tcscpy(szParamStr, _T("?"));
            break;

        case PARAM_PWSTR:
            if (IsBadReadPtr((PVOID)dwParamValue, sizeof(wchar_t)) == 0)
            {
#if (defined(UNICODE)||defined(_UNICODE))
                // format string
                FormatString((TCHAR*)dwParamValue, szParamStr, MAX_PATH);
#else
                iStringSize=SecureWstrlen((LPWSTR)dwParamValue);
                if (iStringSize>=0)
                {
                    // convert into ansi
                    iStringSize=WideCharToMultiByte(CP_ACP, 0, (LPCWSTR)dwParamValue, iStringSize, szUnicodeStr, MAX_PATH, NULL, NULL);
                    if (!iStringSize)
                        *szUnicodeStr=0;
                    else
                    {
                        if (iStringSize>=MAX_PATH)
                            iStringSize=MAX_PATH-1;
                        szUnicodeStr[iStringSize] = 0;
                    }

                    // format string
                    FormatString(szUnicodeStr, szParamStr, MAX_PATH);
                }
                else
                    _tcscpy(szParamStr, _T("?"));
#endif
            }
            else
                _tcscpy(szParamStr, _T("?"));
            break;
        default:
                // remove ':'
                UsedSize--;
                pszLogString[UsedSize]=0;
                // no param
                *szParamStr=0;
            break;
        }
        
        
        if (pszLogStringSize<(UsedSize+_tcslen(szParamStr)))
        {
            bNotEnoughtSpace=TRUE;
            // put the most info we can do
            _tcsncat(pszLogString, szParamStr,pszLogStringSize-UsedSize-4);// 3 for adding ... at end of string
            // exit parameter parsing
            break;
        }
        // else copy param infos
        _tcscat(pszLogString, szParamStr);
        _tcscat(pszLogString, _T(", "));

        // get new string size
        UsedSize=_tcslen(pszLogString);
    }
    
    // if buffer is not big enougth
    if (bNotEnoughtSpace)
    {
        // Add "..." if we can
        if (pszLogStringSize>UsedSize+3)
            _tcscat(pszLogString,_T("..."));
    }
    else
    // add )
    {
        if (pAPIInfo->ParamCount > 0)
            pszLogString[_tcslen(pszLogString) - 2] = ')';// -2 for removing last ", "
        else
            _tcscat(pszLogString, _T(")"));
    }
    va_end(Marker);
}
//-----------------------------------------------------------------------------
// Name: FormatString
// Object: format string parameter by replacing \r \n and \t and assume string size
//         can be send throught mailslots
// Parameters :
//      in: TCHAR* pszSrcStr : src string
//          DWORD dwMaxSize : pszDstStr max size
//      out : TCHAR* pszDstStr : dest string
// Return : func pointer on success, NULL on error
//-----------------------------------------------------------------------------
void FormatString(TCHAR* pszSrcStr, TCHAR* pszDstStr, DWORD dwMaxSize)
{
    DWORD dwDstSize = 0;

    // check src string content
    if (IsBadStringPtr(pszSrcStr, dwMaxSize))
    {
        *pszDstStr++ = '?';
        *pszDstStr = '\0';
        return;
    }

    // add "
    *pszDstStr++ = '"';
    dwDstSize++;

    // for each char
    while (*pszSrcStr != '\0')
    {
        // replace '\n' by "\n"
        if (*pszSrcStr == '\n')
        {
            *pszDstStr++ = '\\';
            dwDstSize++;
            *pszDstStr = 'n';
        }
        // replace '\t' by "\t"
        else if (*pszSrcStr == '\t')
        {
            *pszDstStr++ = '\\';
            dwDstSize++;
            *pszDstStr = 't';
        }
        // replace '\r' by "\r"
        else if (*pszSrcStr == '\r')
        {
            *pszDstStr++ = '\\';
            dwDstSize++;
            *pszDstStr = 'r';
        }
        else// just copy char
            *pszDstStr = *pszSrcStr;

        // increase counter
        dwDstSize++;
        pszSrcStr++;
        pszDstStr++;

        // check string size and if not enought size ends string with ...
        if (dwDstSize >= dwMaxSize-5-1)// -5 for following 5 chars completion, -1 for a possible \r\n\t next char
        {
            *pszDstStr++ = '.';
            *pszDstStr++ = '.';
            *pszDstStr++ = '.';

            break;
        }
    }

    // add "
    *pszDstStr++ = '"';
    // ends string
    *pszDstStr = '\0';
}

//-----------------------------------------------------------------------------
// Name: FirstCharOrPreviousCharSpace
// Object: check if pc points to first char of Origin, or if char before pc in Origin is a space
// Parameters :
//      in: TCHAR* pc : pointer string to check
//          TCHAR* Origin : reference string
//      out : 
// Return : TRUE if pc points to first char of Origin or if char before pc 
//          in Origin is a space
//-----------------------------------------------------------------------------
BOOL FirstCharOrPreviousCharSpace(TCHAR* pc,TCHAR* Origin)
{
    BOOL bFound=FALSE;
    if (pc==Origin)
        bFound=TRUE;
    else
    {
        pc--;
        if (*pc==' ')
            bFound=TRUE;
        pc++;
    }
    return bFound;
}