/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
originaly based from APISpy32 v2.1 from Yariv Kaplan @ WWW.INTERNALS.COM

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include "APIOverrideKernel.h"

extern tagParamSpec ParamSpec[];// table of params types. Defined in APIMonitoringFileLoader

extern BOOL bMonitoring;
extern BOOL bFaking;
extern CLinkList* pLinkListAPIInfos;
extern CLinkList* pLinkListMonitoringFileID;
extern CLinkList* pLinkListFakeAPIFileID;
extern DWORD dwSystemPageSize;

#pragma check_stack(off)  // don't check stack
#pragma runtime_checks( "", off ) // don't check stack

//////////////////////////////////////////////////////////////////////
// APIHandler :
// function call at API start
//////////////////////////////////////////////////////////////////////
void APIHandler()
{

    tagAPIInfo *pAPIInfo;
    tagFileID* pFileID;
    CLinkListItem* pItemFileInfo;
    PBYTE pbAfterCall;
    PDWORD pdwParam;
    DWORD dwParamSize;
    DWORD dwESPReservedSize;
    PDWORD pdwESP;
    PDWORD pdwESPAfterFuncCall;
    PDWORD pdwESPAfterFuncCallCalleeCleans;
    BOOL bStackCleanedByCallee;
    void *pvReturnAddr;
    DWORD dwReturnValue;
    TCHAR szLogStringIn[MAX_PATH];
    TCHAR szLogStringOut[MAX_PATH];
    TCHAR szMsg[3*MAX_PATH];
    BOOL bLogInputParameters;
    BOOL bLogOuputParameters;
    BOOL bLogInputParametersWithoutReturn;
    BOOL bUnhookDoneDuringCalling;
    
    ULONG_PTR FakeAPIFileID;
    ULONG_PTR MonitoringFileID;

    //////////////////////////////////////////////////////////////////////
    // get informations from the stack
    //////////////////////////////////////////////////////////////////////
    //Each function call creates a new stack frame with the following layout, note that high memory is at the top of the list: 
    // Function parameters 
    // Function return address 
 
    // call order was
    //      1) push API params
    //      2) call API --> push return address of hooked func
    //      3) code modification of first bytes of API
    //          a) push pAPIInfo
    //          b) call APIHandler --> push return address of our asm call (HookedFuncAddr+OPCODE_REPLACEMENT_SIZE)
    _asm
    {
        Mov EAX, [EBP + 4]// <-- return address of our asm call (==HookedFuncAddr+OPCODE_REPLACEMENT_SIZE)
        Mov [pbAfterCall], EAX

        Mov EAX, [EBP + 8]
        Mov [pAPIInfo], EAX

        Mov EAX, [EBP + 12] // <-- return address of hooked func
        Mov [pvReturnAddr], EAX

        Lea EAX, [EBP + 16]
        Mov [pdwParam], EAX
    }
    *szLogStringIn=0;
    *szLogStringOut=0;

    // if unhook has been called between hook func entry and now (may will never appear)
    // WARNING unhook must have restore original opcode else you go in infinite loop with a push inside (bad for our stack isn't it ? :D)
    bUnhookDoneDuringCalling=IsBadReadPtr(pAPIInfo,sizeof(tagAPIInfo));
    if (bUnhookDoneDuringCalling==FALSE)
        bUnhookDoneDuringCalling=IsBadReadPtr(pAPIInfo->szAPIName,1)||IsBadReadPtr(pAPIInfo->szModuleName,4);

    if (bUnhookDoneDuringCalling==FALSE)
    {
        // we can access our critical section part
        EnterCriticalSection(&pAPIInfo->CriticalSection);
        // check unhooking again as EnterCriticalSection is blocking
        bUnhookDoneDuringCalling=IsBadReadPtr(pAPIInfo,sizeof(tagAPIInfo));
        if (bUnhookDoneDuringCalling==FALSE)
            bUnhookDoneDuringCalling=IsBadReadPtr(pAPIInfo->szAPIName,1)||IsBadReadPtr(pAPIInfo->szModuleName,4);
    }
    if (bUnhookDoneDuringCalling)
    {
        // make the original call only with a supposed enought stack with
        dwParamSize=MAX_PARAM*4;// take max param size
        dwESPReservedSize=dwParamSize+ESP_SECURITY_SIZE* 4;
        // duplicate all parameters in the stack for our func
        _asm
        {
            Sub ESP, [dwESPReservedSize] // get enougth space in stack frame (remember push=esp--)
            Mov [pdwESP], ESP            // get ESP pointer
        }
        // memcpy(pdwESP, pdwParam, dwParamSize);// copy params from there original position
        memcpy(pdwESP, pdwParam, dwESPReservedSize); // copy even more param in case of bad config file
        // call original func (as func is unhooked original opcodes are already restored)
        _asm
        {
            Mov     EAX , [pbAfterCall]
            Sub     EAX , OPCODE_REPLACEMENT_SIZE
            Call    EAX
        }
    }
    else
    {
        
        // copy original stored instructions to execute original API func after the hook
        memcpy(pAPIInfo->APIAddress, pAPIInfo->Opcodes, OPCODE_REPLACEMENT_SIZE);

        //////////////////////////////////////////////////////////////////////
        // copy original information onto the stack
        //////////////////////////////////////////////////////////////////////
        dwParamSize = pAPIInfo->ParamCount * 4;
        // pAPIInfo->ParamCount gives a supposed size (size given by the config file)
        // in case of error in your config file you can crash your esp --> adding security size
        dwESPReservedSize=dwParamSize+ESP_SECURITY_SIZE* 4;

        // duplicate all parameters in the stack for our func
        _asm
        {
            Sub ESP, [dwESPReservedSize] // get enougth space in stack frame (remember push=esp-4)
            Mov [pdwESP], ESP            // get ESP pointer
        }
        // memcpy(pdwESP, pdwParam, dwParamSize);// copy params from there original position
        memcpy(pdwESP, pdwParam, dwESPReservedSize); // copy even more param in case of bad config file

        //////////////////////////////////////////////////////////////////////
        // log input parameters if requiered
        //////////////////////////////////////////////////////////////////////
        bLogInputParametersWithoutReturn=(pAPIInfo->ParamDirectionType==PARAM_DIR_IN_NO_RETURN);
        bLogInputParameters=((pAPIInfo->ParamDirectionType==PARAM_DIR_IN)
                              ||(pAPIInfo->ParamDirectionType==PARAM_DIR_INOUT)
                             );
        // if we log parameter in
        if ((bLogInputParameters||bLogInputParametersWithoutReturn)
            && bMonitoring && (pAPIInfo->MonitoringFileID!=0))
        {
            ParseAPIParameters(pAPIInfo, szLogStringIn,MAX_PATH);// ParseAPIParameters: parse parameters and put them in szLogString
            // calling this way, params are used without consuming them

            if (bLogInputParametersWithoutReturn)
                AddLogEntry(GetCurrentProcessId(),pAPIInfo->szModuleName, szLogStringIn, 0, (PBYTE)pvReturnAddr - ASM_CALL_INSRUCTION_SIZE,PARAM_DIR_TYPE_IN_NO_RETURN);
        }

        // save some infos to check if unhook has been done during call
        FakeAPIFileID=pAPIInfo->FakeAPIFileID;
        MonitoringFileID=pAPIInfo->MonitoringFileID;

        //////////////////////////////////////////////////////////////////////
        // call API or api override
        //////////////////////////////////////////////////////////////////////

        // if an api replacement has been configured execute it instead of real api
        if (pAPIInfo->FakeAPIAddress && bFaking)
            pAPIInfo->FakeAPIAddress();
        else // execute original api
            pAPIInfo->APIAddress();
    }
    //////////////////////////////////////////////////////////////////////
    // save return
    //////////////////////////////////////////////////////////////////////
    _asm
    {
        Mov [dwReturnValue], EAX // store func ret in dwReturnValue
    }

    //////////////////////////////////////////////////////////////////////
    // Check stack to know calling convention
    //////////////////////////////////////////////////////////////////////
    _asm
    {
        // checking stack
        Mov ECX,ESP
        // get ESP pointer after func call to known calling convention (stdcall or cdecl) and so if you have to clean the stack or not
        Mov [pdwESPAfterFuncCall], ECX

        // comput esp in case of callee cleaning to check the config file number of args
        Sub ECX,[dwParamSize]
        Mov [pdwESPAfterFuncCallCalleeCleans], ECX

        // comput real parameter size and free local stack

        // real parameter size=pdwESPAfterFuncCall-pdwESP
        Mov ECX, [pdwESPAfterFuncCall]
        Sub ECX, [pdwESP]
        Mov [dwParamSize],ECX // store real param size
        // dwESPSizeToFree=dwESPReservedSize-real param size
        Mov ECX,[dwESPReservedSize]
        Sub ECX,[dwParamSize]
       
        //Mov [dwESPSizeToFree],ECX
        //Add ESP, [dwESPSizeToFree]
        Add ESP,ECX
    }



    ////////////////////////////////
    // get Calling convention
    ////////////////////////////////
    if (pdwESPAfterFuncCall==pdwESP)//if stack cleaned by caller or no params
        bStackCleanedByCallee=FALSE;
    else
    {
        bStackCleanedByCallee=TRUE;
        // check given param size
        if (pdwESPAfterFuncCallCalleeCleans!=pdwESP)
        {
            // bogus config file
            // comput real parameter size to clean stack at hook return
            pAPIInfo->ParamCount=(BYTE)dwParamSize/4;
            // as we change parameter size in pAPIInfo struct, only 1 warning will be done

            // signal it
            _tcscpy(szMsg,_T("Error in config file : "));
            _itot((int)pAPIInfo->ParamCount,&szMsg[_tcslen(szMsg)],10);
            _tcscat(szMsg,_T(" parameters are requiered."));
            
            if (pAPIInfo->MonitoringFileID!=0)
            {
                _tcscat(szMsg,_T("\r\nConfig File: "));
                // get file name
                pItemFileInfo = pLinkListMonitoringFileID->Head;
                // look throw pItemFileInfo
                while (pItemFileInfo)
                {
                    pFileID=(tagFileID*)pItemFileInfo->ItemData;
                    if (pAPIInfo->MonitoringFileID==pFileID->Id)
                    {
                        // filename found add it to szMsg
                        _tcscat(szMsg,pFileID->szFileName);
                        break;// go out of while
                    }
                    pItemFileInfo = pItemFileInfo->NextItem;
                }
            }
            else // fake API
            {
                _tcscat(szMsg,_T("\r\nFake API Dll: "));
                // get file name
                pItemFileInfo = pLinkListFakeAPIFileID->Head;
                // look throw pItemFileInfo
                while (pItemFileInfo)
                {
                    pFileID=(tagFileID*)pItemFileInfo->ItemData;
                    if (pAPIInfo->FakeAPIFileID==pFileID->Id)
                    {
                        // filename found add it to szMsg
                        _tcscat(szMsg,pFileID->szFileName);
                        break;// go out of while
                    }
                    pItemFileInfo = pItemFileInfo->NextItem;
                }
            }
            _tcscat(szMsg,_T("\r\nModule: "));
            _tcscat(szMsg,pAPIInfo->szModuleName);
            _tcscat(szMsg,_T("\r\nApi: "));
            _tcscat(szMsg,pAPIInfo->szAPIName);            
            DynamicMessageBox(NULL,szMsg,_T("API Override Warning"),MB_ICONWARNING|MB_OK|MB_TOPMOST);
        }
    }
   

    //////////////////////////////////////////////////////////////////////
    // log output parameters if requiered (do the same as input parameters)
    //////////////////////////////////////////////////////////////////////
    bLogOuputParameters=(pAPIInfo->ParamDirectionType==PARAM_DIR_OUT)||(pAPIInfo->ParamDirectionType==PARAM_DIR_INOUT);
    if (bLogOuputParameters && bMonitoring && (pAPIInfo->MonitoringFileID!=0))
    {
        // we get original parameters and parse them again because only pointer content as been changed,
        // not the pointer itself. Pointer content parsing is done by ParseAPIParameters, so let it do the job

        // duplicate all parameters in the stack for our func
        _asm
        {
            Sub ESP, [dwParamSize] // get enougth space in stack frame (remember push=esp--)
            Mov [pdwESP], ESP      // get ESP pointer
        }
        memcpy(pdwESP, pdwParam, dwParamSize);// copy params from there original position
        ParseAPIParameters(pAPIInfo, szLogStringOut,MAX_PATH);// parse parameters and put them in szLogString
        // calling this way, params are used without being consumed

        // as we don't call any func, free stack
        _asm
        {
            Add ESP, [dwParamSize]
        }
    }

    //////////////////////////////////////////////////////////////////////
    // send logging information to monitoring application
    //////////////////////////////////////////////////////////////////////
    if (bLogInputParameters && bMonitoring && (pAPIInfo->MonitoringFileID!=0))
        AddLogEntry(GetCurrentProcessId(),pAPIInfo->szModuleName, szLogStringIn, dwReturnValue, (PBYTE)pvReturnAddr - ASM_CALL_INSRUCTION_SIZE,PARAM_DIR_TYPE_IN);
    if (bLogOuputParameters && bMonitoring && (pAPIInfo->MonitoringFileID!=0))
        AddLogEntry(GetCurrentProcessId(),pAPIInfo->szModuleName, szLogStringOut, dwReturnValue, (PBYTE)pvReturnAddr - ASM_CALL_INSRUCTION_SIZE,PARAM_DIR_TYPE_OUT);

    //////////////////////////////////////////////////////////////////////
    // Restore Hook
    //////////////////////////////////////////////////////////////////////

    // restore hook for the next API call
    memcpy(pAPIInfo->APIAddress,pAPIInfo->pbHookCodes,OPCODE_REPLACEMENT_SIZE);
    // exit exclusion
    LeaveCriticalSection(&pAPIInfo->CriticalSection);


    //////////////////////////////////////////////////////////////////////
    // Restore Stack (make the same job real Api do without hooking)
    //////////////////////////////////////////////////////////////////////
    if (bStackCleanedByCallee)
    {
        _asm
        {
            Mov EAX, [dwReturnValue]// restore API return value
            Mov ECX, [dwParamSize] 
            Mov EDX, [pvReturnAddr]

            Pop EDI
            Pop ESI
            Pop EBX

            Mov ESP, EBP
            Pop EBP      // from now you can't access your function local var
            Add ESP, 8   // remove frame pointer and return addr
            Add ESP, 4   // remove our pAPIInfo param from stack
            // now esp points to second ret address --> ret will use return address of hooked func

            Add ESP, ECX // remove params from stack

            Push EDX     // push return addr

            Ret
        }
    }
    else
    {
        _asm
        {
            Mov EAX, [dwReturnValue]// restore API return value
            Mov EDX, [pvReturnAddr]

            Pop EDI
            Pop ESI
            Pop EBX

            Mov ESP, EBP
            Pop EBP      // from now you can't access your function local var
            Add ESP, 8   // remove frame pointer and return addr
            Add ESP, 4   // remove our pAPIInfo param from stack
            // now esp points to second ret address --> ret will use return address of hooked func

            Push EDX     // push return addr

            Ret
        }
    }

}
#pragma runtime_checks( "", restore )


//-----------------------------------------------------------------------------
// Name: RemoveProtection
// Object: Remove protection of pAPIInfo->APIAddress, and store old Protection flags in
//               pAPIInfo->dwOldProtectionFlags
// Parameters :
//     inout  : tagAPIInfo *pAPIInfo
// Return : TRUE
//-----------------------------------------------------------------------------
BOOL RemoveProtection(tagAPIInfo *pAPIInfo)
{
    MEMORY_BASIC_INFORMATION mbi;
    DWORD dwProtectionFlags;
    DWORD dwScratch;

    // Get page protection of API
    VirtualQuery(pAPIInfo->APIAddress, &mbi, sizeof(mbi));
    dwProtectionFlags = mbi.Protect;
    pAPIInfo->dwOldProtectionFlags = dwProtectionFlags;

    // Remove page protection from API
    dwProtectionFlags &= ~PAGE_READONLY;
    dwProtectionFlags &= ~PAGE_EXECUTE_READ;
    dwProtectionFlags |= PAGE_READWRITE;

    VirtualProtect(pAPIInfo->APIAddress, dwSystemPageSize, dwProtectionFlags, &dwScratch);

    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: GetAssociatedItemAPIInfo
// Object: Get List Item containing APIInfo Struct Associated to given API addresse
// Parameters :
//     in  : PBYTE pbAPI virtual address of func to hook 
//     out : BOOL* pbAlreadyHooked : true if func is already hooked
// Return : NULL on error
//          on success 
//               - if Hook was not existing Allocate and return a new List Item containing an empty APIInfo struct
//               - else return the old List Item containing APIInfo struct to allow update of some fields
//-----------------------------------------------------------------------------
CLinkListItem *GetAssociatedItemAPIInfo(PBYTE pbAPI,BOOL* pbAlreadyHooked)
{
    tagAPIInfo *pAPIInfo;
    CLinkListItem* pItemAPIInfo;

    if ((pbAPI==NULL)||(pbAlreadyHooked==NULL))
        return NULL;
    *pbAlreadyHooked=FALSE;
    // Is it already hooked ?
    pItemAPIInfo = pLinkListAPIInfos->Head;
    while (pItemAPIInfo)
    {
        pAPIInfo=(tagAPIInfo*)pItemAPIInfo->ItemData;
        if (pAPIInfo->APIAddress == (FARPROC)pbAPI)
        {
            // return old hook to allow update some tagAPIInfo fields
            *pbAlreadyHooked=TRUE;
            return pItemAPIInfo;
        }
        pItemAPIInfo = pItemAPIInfo->NextItem;
    }

    // No, so add a new item
    pItemAPIInfo =pLinkListAPIInfos->AddItem();

    return pItemAPIInfo;
}

//-----------------------------------------------------------------------------
// Name: HookAPIFunction
// Object: Patch Opcode located at pAPIInfo->APIAddress to make a call to APIHandler
//         Save original bytes in pAPIInfo->Opcodes
//         Save modified bytes in pAPIInfo->pbHookCodes to avoid recomputing on each hook restoration
// Parameters :
//     inout  : tagAPIInfo *pAPIInfo : pAPIInfo->APIAddress contains virtual address of API to patch
// Return : FALSE on error, TRUE if success
//-----------------------------------------------------------------------------
BOOL HookAPIFunction(tagAPIInfo *pAPIInfo)
{
    PDWORD pdwAPI;
    BOOL Result;

    if (pAPIInfo == NULL)
        return FALSE;

    // allow write access to memory
    Result = RemoveProtection(pAPIInfo);
    if (Result == FALSE)
        return FALSE;

    // Save first OPCODE_REPLACEMENT_SIZE bytes of API
    memcpy(pAPIInfo->Opcodes, pAPIInfo->APIAddress, OPCODE_REPLACEMENT_SIZE);

    // Write a call to the hook function
    // push pAPIInfo
    pAPIInfo->pbHookCodes[0] = 0x68; // PUSH asm instruction
    pdwAPI = (DWORD *)&pAPIInfo->pbHookCodes[1];
    pdwAPI[0]=(DWORD)pAPIInfo;

    pAPIInfo->pbHookCodes[5] = 0xE8;// CALL asm instruction
    // cast from our byte pointer to dword pointer for storing call address
    pdwAPI = (DWORD *)&pAPIInfo->pbHookCodes[6];
    // comput relative address from API address to our API hooking function (APIHandler)
    pdwAPI[0] = (DWORD)APIHandler - (DWORD)pAPIInfo->APIAddress - OPCODE_REPLACEMENT_SIZE;
    
    // change code to CALL APIHandler func
    memcpy(pAPIInfo->APIAddress,pAPIInfo->pbHookCodes,OPCODE_REPLACEMENT_SIZE);

    return TRUE;
}


//-----------------------------------------------------------------------------
// Name: UnhookAPIFunction
// Object: try to remove hook (dll in use are not free to avoid crash)
// Parameters :
//     inout  : CLinkListItem *pItemAPIInfo : free on success
// Return : FALSE on error or func is not unhook now, TRUE if success
//-----------------------------------------------------------------------------
BOOL UnhookAPIFunction(CLinkListItem *pItemAPIInfo)
{
    tagAPIInfo *pAPIInfo;
    BOOL Result;
    DWORD dwScratch;
    int iMsgBoxRes;
    int cnt;
    TCHAR pszMsg[2*MAX_PATH];
    HANDLE hBoostedThread=NULL;
    BOOL bInsideCriticalSection=FALSE;

    if (pItemAPIInfo==NULL)
        return FALSE;
    pAPIInfo=(tagAPIInfo*)pItemAPIInfo->ItemData;
    if (pAPIInfo==NULL)
        return FALSE;

    // change pbHookCodes to Opcodes to avoid hook to be restore at the end of hooking
    memcpy(pAPIInfo->pbHookCodes, pAPIInfo->Opcodes, OPCODE_REPLACEMENT_SIZE);

    // restore original opcode 
    Result = RemoveProtection(pAPIInfo);
    if (Result == TRUE)
        memcpy(pAPIInfo->APIAddress, pAPIInfo->Opcodes, OPCODE_REPLACEMENT_SIZE);

    Result=TRUE;
    // Make sur we are not in hooking func
    iMsgBoxRes=IDYES;

    // wait end of hook
    // monitoring could be unhooked without waiting, but this can cause trouble because we can't know
    // if we can unload this dll or not (if we unload it and a monitoring hook is in use, the return
    // address will be in our unloaded dll --> beautiful crash)

    Result=FALSE;
    while(iMsgBoxRes==IDYES)
    {
        for (cnt=0;cnt*UNHOOK_POOLING_WAIT<UNHOOK_MAX_WAIT_TIME;cnt++)
        {
            if (TryEnterCriticalSection(&pAPIInfo->CriticalSection))
            {
                bInsideCriticalSection=TRUE;
                Result=TRUE;
                break;
            }
            else
            {
                if (hBoostedThread==NULL)
                {
                    hBoostedThread=pAPIInfo->CriticalSection.OwningThread;
                    // boost critical section owner thread
                    SetThreadPriorityBoost(hBoostedThread, FALSE);
                }

                Sleep(UNHOOK_POOLING_WAIT);
            }
        }
        if (Result)
            break;
        else
        {
            // put the user aware and hope he will do actions that unlock fake api
            // a good sample of blocking call are DynamicMessageBox
            _sntprintf(pszMsg,2*MAX_PATH,_T("Warning %s in module %s\r\n is in use.\r\nDo you want to wait more time ?"),pAPIInfo->szAPIName,pAPIInfo->szModuleName);
            iMsgBoxRes=DynamicMessageBox(NULL,pszMsg,_T("Warning"),MB_YESNO|MB_ICONWARNING|MB_TOPMOST);
        }
    }
    if (hBoostedThread)
        // restore thread priority if changed
        SetThreadPriorityBoost(hBoostedThread,TRUE);

    if (Result)
    {
        // free allocated memory for string
        HeapFree(GetProcessHeap(), 0, pAPIInfo->szAPIName);
        HeapFree(GetProcessHeap(), 0, pAPIInfo->szModuleName);

        LeaveCriticalSection(&pAPIInfo->CriticalSection);

        // restore original protection
        VirtualProtect(pAPIInfo->APIAddress, dwSystemPageSize, pAPIInfo->dwOldProtectionFlags, &dwScratch);
        
        DeleteCriticalSection(&pAPIInfo->CriticalSection);

        // remove item from link list
        pLinkListAPIInfos->RemoveItem(pItemAPIInfo);
    }
    else
    {
        if (bInsideCriticalSection)
            LeaveCriticalSection(&pAPIInfo->CriticalSection);
    }
    return Result;
}

//-----------------------------------------------------------------------------
// Name: UnhookAllAPIFunctions
// Object: try to remove all hooks (dll in use are not free to avoid crash)
// Parameters :
// Return : 
//-----------------------------------------------------------------------------
void UnhookAllAPIFunctions()
{
    CLinkListItem *pItemAPIInfo;
    CLinkListItem *pTempItemAPIInfo;
    tagFileID* pFileID;

    // remove all remaining hooks 
    if (pLinkListAPIInfos==NULL)
        return;
    
    pItemAPIInfo =pLinkListAPIInfos->Head;
    while (pItemAPIInfo)
    {
        pTempItemAPIInfo = pItemAPIInfo->NextItem;// because pItemAPIInfo can be free by UnhookAPIFunction
        UnhookAPIFunction(pItemAPIInfo);
        pItemAPIInfo = pTempItemAPIInfo;
    }
    
    if (pLinkListMonitoringFileID)
    {
        // free pLinkListMonitoringFileID
        pItemAPIInfo =pLinkListMonitoringFileID->Head;
        while (pItemAPIInfo)
        {
            pFileID=(tagFileID*)pItemAPIInfo->ItemData;
            pTempItemAPIInfo = pItemAPIInfo->NextItem; // because pItemAPIInfo can be free by UnloadMonitoringFile
            UnloadMonitoringFile(pFileID->szFileName);
            pItemAPIInfo = pTempItemAPIInfo;
        }
    }

    if (pLinkListFakeAPIFileID)
    {
        // free pLinkListFakeAPIFileID
        pItemAPIInfo =pLinkListFakeAPIFileID->Head;
        while (pItemAPIInfo)
        {
            pFileID=(tagFileID*)pItemAPIInfo->ItemData;
            pTempItemAPIInfo = pItemAPIInfo->NextItem;// because pItemAPIInfo can be free by UnloadFakeApiDll
            UnloadFakeApiDll(pFileID->szFileName);
            pItemAPIInfo = pTempItemAPIInfo;
        }
    }
}



//-----------------------------------------------------------------------------
// Name: GetFuncAddr
// Object: Load Library specified by pszModuleName only if not already loaded and
//          get function address of specified func name pszAPIName
// Parameters :
//      in: TCHAR* pszModuleName : dll name
//          TCHAR* pszAPIName : function name
//      out : HMODULE* phModule module handle 
// Return : func pointer on success, NULL on error
//-----------------------------------------------------------------------------
PBYTE GetFuncAddr(TCHAR* pszModuleName,TCHAR* pszAPIName,HMODULE* phModule)
{
    PBYTE pbAPI;
#if (defined(UNICODE)||defined(_UNICODE))
    CHAR pcFuncName[MAX_PATH];
    int iStringSize;
#endif
    BOOL bLibraryLoaded=FALSE;
    if (phModule == NULL)
        return NULL;

    // get module handle
    *phModule = GetModuleHandle(pszModuleName);
    if (*phModule == NULL)
    {
        // if module handle not found load library
        *phModule=LoadLibrary(pszModuleName);
        if (*phModule == NULL)
            return NULL;
        bLibraryLoaded=TRUE;
    }
#if (defined(UNICODE)||defined(_UNICODE))
    // convert into ansi until GetProcAddress don't support unicode
    iStringSize=WideCharToMultiByte(CP_ACP, 0, (LPCWSTR)pszAPIName, (int)wcslen(pszAPIName)+1, pcFuncName, MAX_PATH, NULL, NULL);
    if (!iStringSize)
        *pcFuncName=0;
    else
    {
        if (iStringSize>=MAX_PATH)
            iStringSize=MAX_PATH-1;
        pcFuncName[iStringSize] = 0;
    }
    pbAPI = (PBYTE)GetProcAddress(*phModule, pcFuncName);
#else
    pbAPI = (PBYTE)GetProcAddress(*phModule, pszAPIName);
#endif
    if (pbAPI == NULL)
    {
        if (bLibraryLoaded)
            FreeLibrary(*phModule);
        return NULL;
    }
    return pbAPI;
}

//-----------------------------------------------------------------------------
// Name: GetFuncAddr
// Object: Load Library specified by pszModuleName only if not already loaded and
//          get function address of specified func name pszAPIName
// Parameters :
//      in: TCHAR* pszModuleName : dll name
//          TCHAR* pszAPIName : function name
// Return : func pointer on success, NULL on error
//-----------------------------------------------------------------------------
PBYTE GetFuncAddr(TCHAR* pszModuleName,TCHAR* pszAPIName)
{
    HMODULE hModule;
    return GetFuncAddr(pszModuleName,pszAPIName,&hModule);
}


//-----------------------------------------------------------------------------
// Name: SecureStrlen
// Object: assume to not throw error as strlen can do for any string pointer (valid or not)
// Parameters :
//      in: char* pc : src string
//      out : 
// Return : -1 on error (bad string pointer) else size of string pointed by pc
//-----------------------------------------------------------------------------
int SecureStrlen(char* pc)
{
    char* p=pc;
    for(;;)
    {
        // check pointer validity
        if (IsBadReadPtr(p,sizeof(char)))
            return -1;
        // check for end char
        if (*p==0)
            // end char found, return sizeof string
            return (int)(p-pc);
        p++;
    }
}
//-----------------------------------------------------------------------------
// Name: SecureWstrlen
// Object: assume to not throw error as strlen can do for any string pointer (valid or not)
// Parameters :
//      in: wchar_t* pc : src string
//      out : 
// Return : -1 on error (bad string pointer) else size of string pointed by pc
//-----------------------------------------------------------------------------
int SecureWstrlen(wchar_t* pc)
{
    wchar_t* p=pc;
    for(;;)
    {
        // check pointer validity
        if (IsBadReadPtr(p,sizeof(wchar_t)))
            return -1;
        // check for end char
        if (*p==0)
            // end char found, return sizeof string
            return (int)(p-pc);
        p++;
    }
}