/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
originaly based from APISpy32 v2.1 from Yariv Kaplan @ WWW.INTERNALS.COM

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#pragma once

#ifndef _WIN32_WINNT
    #define _WIN32_WINNT 0x0400 // for TryEnterCriticalSection
#endif
#include <windows.h>
#include "APIMonitoringFileLoader.h"
#include "FakeAPILoader.h"
#include "LogAPI.h"
#include "../../../LinkList/LinkList.h"

#include "DynamicMessageBox.h"


#define ESP_SECURITY_SIZE 10 // this number is the number of args that can be unfortunately forget in config file
// --> esp will be changed of ESP_SECURITY_SIZE*4 bits on 32bits platforms
#define MAX_PARAM    20 // max Api params

// tag to specify internal address or software instead of a libname/funcname
#define EXE_INTERNAL_PREFIX _T("EXE_INTERNAL@0x")
#define EXE_INTERNAL_PREFIX_SIZE 16 // size in TCHAR including \0

#define UNHOOK_MAX_WAIT_TIME 5000 // max time for executing hook, used as max wait time for unhooking
#define UNHOOK_POOLING_WAIT  500

#define OPCODE_REPLACEMENT_SIZE 10
#define ASM_CALL_INSRUCTION_SIZE 5 // quite wrong with CALL dword ptr FF 15 --> size = 6

// monitoring parameter direction def
enum tagParamDirection
{
    PARAM_DIR_IN=0,
    PARAM_DIR_OUT,
    PARAM_DIR_INOUT,
    PARAM_DIR_IN_NO_RETURN,
    PARAM_DIR_NONE // for only modify an api without logging it
};
#define DEFAULT_PARAMETER_DIRECTION_SPYING PARAM_DIR_IN

// monitoring available param type
enum tagParamType
{
    PARAM_BYTE,
    PARAM_UCHAR,
    PARAM_PBYTE,
    PARAM_WORD,
    PARAM_PWORD,
    PARAM_SHORT,
    PARAM_PSHORT,
    PARAM_WPARAM,
    PARAM_ULONG,
    PARAM_PULONG,
    PARAM_ULONG_PTR,
    PARAM_INT,
    PARAM_PINT,
    PARAM_UINT,
    PARAM_PUINT,
    PARAM_LONG,
    PARAM_PLONG,
    PARAM_LONG_PTR,
    PARAM_LPARAM,
    PARAM_DWORD,
    PARAM_PDWORD,
    PARAM_SIZE_T,
    PARAM_PSIZE_T,
    PARAM_PSTR,
    PARAM_PWSTR,
    PARAM_PVOID,
    PARAM_HANDLE,
    PARAM_BOOL,
    PARAM_PBOOL,
    PARAM_BOOLEAN,
    PARAM_PBOOLEAN,
    PARAM_CHAR,
    PARAM_PUCHAR,
    PARAM_WCHAR,
    PARAM_HINSTANCE,
    PARAM_HWND,
    PARAM_HMODULE,
    PARAM_PHMODULE,
    PARAM_HDC,
    PARAM_HMENU,
    PARAM_WNDPROC,
    PARAM_DLGPROC,
    PARAM_FARPROC,
    PARAM_UNKNOWN
};
enum tagParamFormat
{
    FORMAT_LONG,
    FORMAT_UNSIGNED_LONG,
    FORMAT_HEXA
};

struct tagParamSpec
{
    TCHAR *ParamName;
    DWORD ParamType;
    BYTE  ParamFormat;
    DWORD dwParamMask;
};

struct tagFileID
{
    TCHAR szFileName[MAX_PATH];
    ULONG_PTR Id;
};

struct tagAPIInfo
{
    BYTE pbHookCodes[OPCODE_REPLACEMENT_SIZE]; // used to install hook
    BYTE Opcodes[OPCODE_REPLACEMENT_SIZE];     // used to uninstall hook
    FARPROC APIAddress;
    FARPROC FakeAPIAddress;

    DWORD dwOldProtectionFlags;
    CRITICAL_SECTION CriticalSection;

    TCHAR* szModuleName;
    TCHAR* szAPIName;

    DWORD ParamList[MAX_PARAM];
    BYTE ParamCount;
    BYTE ParamDirectionType;

    ULONG_PTR MonitoringFileID;
    ULONG_PTR FakeAPIFileID;
};



///////////////////////////////////////////////////////
//////////////          functions        //////////////
///////////////////////////////////////////////////////
PBYTE GetFuncAddr(TCHAR* pszModuleName,TCHAR* pszAPIName);
PBYTE GetFuncAddr(TCHAR* pszModuleName,TCHAR* pszAPIName,HMODULE* phModule);
CLinkListItem *GetAssociatedItemAPIInfo(PBYTE pbAPI,BOOL* pbAlreadyHooked);
BOOL HookAPIFunction(tagAPIInfo *pAPIInfo);
BOOL UnhookAPIFunction(CLinkListItem *pItemAPIInfo);
void UnhookAllAPIFunctions();
int SecureStrlen(char* pc);
int SecureWstrlen(wchar_t* pc);