/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "fakeapiloader.h"

extern CLinkList* pLinkListAPIInfos;
extern CLinkList* pLinkListFakeAPIFileID;


//-----------------------------------------------------------------------------
// Name: LoadFakeApiDll
// Object: Load a fake API dll and attach hooks. See Fake API dll example
//         Standart exported func of dll gives a array of address hooking
//         array of type STRUCT_FAKE_API
// Parameters :
//     in  : TCHAR* szFakeAPIDLLFullPath : dll file name (full path)
// Return : TRUE if library is loaded (even if not all functions are loaded),
//          FALSE if library is not loaded at all
//-----------------------------------------------------------------------------
BOOL LoadFakeApiDll(TCHAR* szFakeAPIDLLFullPath)
{
    tagGetFakeAPIArrayFuncPointer pGetFakeAPIArrayFuncPointer;
    tagGetFakeAPIEncoding pGetFakeAPIEncoding;
    int iFakeAPIEncoding=FakeAPIEncodingANSI;
    int iStringSize;
    PSTRUCT_FAKE_API_UNICODE FakeAPIArrayUnicode=NULL;
    PSTRUCT_FAKE_API_ANSI FakeAPIArrayAnsi=NULL;
    int Index=0;
    PBYTE pbAPI=0;
    tagAPIInfo *pAPIInfo;
    CLinkListItem *pItemAPIInfo;
    CLinkListItem *pItemFileInfo;
    CLinkListItem *pItemFakeApiFileInfo;
    tagFileID* pFileID;
    tagFileID* pOldFileID;
    BOOL bAlreadyHooked=FALSE;
    HMODULE hModule;
    TCHAR pszMsg[3*MAX_PATH];
    BOOL bNoMoreItems=TRUE;
    TCHAR pszModuleName[MAX_PATH];
    TCHAR pszAPIName[MAX_PATH];
    FARPROC FakeAPI;
    BYTE ParamCount;

    // check dll name
    if (*szFakeAPIDLLFullPath==0)
    {
        DynamicMessageBox(NULL,_T("Empty file name"),_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }

    // check if lib is already loaded
    if (GetModuleHandle(szFakeAPIDLLFullPath))
    {
        _sntprintf(pszMsg,3*MAX_PATH,_T("File %s already loaded"),szFakeAPIDLLFullPath);
        DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }

    // check if dll exists
    HANDLE hFile = CreateFile(szFakeAPIDLLFullPath, GENERIC_READ, FILE_SHARE_READ, NULL,
        OPEN_EXISTING, FILE_FLAG_SEQUENTIAL_SCAN, NULL);
    if (hFile == INVALID_HANDLE_VALUE)
    {
        _sntprintf(pszMsg,3*MAX_PATH,_T("File %s not found"),szFakeAPIDLLFullPath);
        DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }

    // load library and get proc address to retreive PSTRUCT_FAKE_API array
    pGetFakeAPIArrayFuncPointer=(tagGetFakeAPIArrayFuncPointer)GetFuncAddr(szFakeAPIDLLFullPath,GET_FAKE_API_ARRAY_FUNCTION_NAME,&hModule);
    // get proc address of to retreive dll char encoding
    pGetFakeAPIEncoding=(tagGetFakeAPIEncoding)GetFuncAddr(szFakeAPIDLLFullPath,GET_FAKE_API_ENCODING_FUNCTION_NAME,&hModule);

    // check library loading status
    if (hModule==NULL)
    {
        _sntprintf(pszMsg,3*MAX_PATH,_T("Load Library failed for dll %s"),szFakeAPIDLLFullPath);
        DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }

    // if we can get encoding params (else iFakeAPIEncoding is defaulted to ANSI for backward compatibility)
    if (pGetFakeAPIEncoding)
        iFakeAPIEncoding=pGetFakeAPIEncoding();

    // func GET_FAKE_API_ARRAY_FUNCTION_NAME not exported
    if (pGetFakeAPIArrayFuncPointer==NULL)
    {
        _sntprintf(pszMsg,3*MAX_PATH,_T("Exported function %s not found in dll\r\n %s"),GET_FAKE_API_ARRAY_FUNCTION_NAME,szFakeAPIDLLFullPath);
        DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        FreeLibrary(hModule);
        return FALSE;
    }

    bNoMoreItems=TRUE;
    // get a pointer to faking array
    if (iFakeAPIEncoding==FakeAPIEncodingANSI)
    {
        FakeAPIArrayAnsi=(PSTRUCT_FAKE_API_ANSI)pGetFakeAPIArrayFuncPointer();
        if (FakeAPIArrayAnsi)
            bNoMoreItems=FALSE;
    }
    else
    {
        FakeAPIArrayUnicode=(PSTRUCT_FAKE_API_UNICODE)pGetFakeAPIArrayFuncPointer();
        if (FakeAPIArrayUnicode)
            bNoMoreItems=FALSE;
    }

    // check status of pGetFakeAPIArrayFuncPointer() call
    if (bNoMoreItems)
    {
        _sntprintf(pszMsg,3*MAX_PATH,_T("Exported function %s in dll\r\n %s\r\nreturn an empty pointer"),GET_FAKE_API_ARRAY_FUNCTION_NAME,szFakeAPIDLLFullPath);
        DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        FreeLibrary(hModule);
        return FALSE;
    }

    // add an item to the link list pLinkListFakeAPIFileID
    pItemFakeApiFileInfo=pLinkListFakeAPIFileID->AddItem();
    if (pItemFakeApiFileInfo==NULL)
    {
        DynamicMessageBox(NULL,_T("Not enought memory"),_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        FreeLibrary(hModule);
        return FALSE;
    }
    // fill item info
    pFileID=(tagFileID*)pItemFakeApiFileInfo->ItemData;
    pFileID->Id=(ULONG_PTR)hModule;
    _tcsncpy(pFileID->szFileName,szFakeAPIDLLFullPath,MAX_PATH);
    pFileID->szFileName[MAX_PATH-1]=0;

    Index=-1;
    bNoMoreItems=FALSE;
    // parse all array: remember last  element of array as it FakeAPI field put to null
    while(!bNoMoreItems)
    {
        Index++;

        ////////////////////////////////////////////////////
        // compare caracter encoding of fake dll to caracter encoding of this module
        // and do convertion if necessary
        ////////////////////////////////////////////////////
        if (iFakeAPIEncoding==FakeAPIEncodingANSI)
        {
            FakeAPI=FakeAPIArrayAnsi[Index].FakeAPI;
            // check for the end of the array before try to convert data
            bNoMoreItems=(FakeAPI==NULL);
            if (bNoMoreItems)
                break;
            ParamCount=FakeAPIArrayAnsi[Index].ParamCount;
#if (defined(UNICODE)||defined(_UNICODE))
            // we have to translate pszModuleName and pszAPIName from ansi to unicode

            // convert pszModulzName
            iStringSize=SecureStrlen(FakeAPIArrayAnsi[Index].pszModuleName);
            if (iStringSize<0)
            {
                _sntprintf(pszMsg,3*MAX_PATH,_T("Can't get module name of item %d of fake api dll array"),Index+1);
                DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                continue;
            }
            iStringSize=MultiByteToWideChar(CP_ACP, 0, 
                FakeAPIArrayAnsi[Index].pszModuleName, 
                iStringSize,
                pszModuleName,
                MAX_PATH);
            if (!iStringSize)
            {
                _sntprintf(pszMsg,3*MAX_PATH,_T("Error converting module name of item %d of fake api dll array to unicode"),Index+1);
                DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                continue;
            }
            if (iStringSize>=MAX_PATH)
                iStringSize=MAX_PATH-1;
            pszModuleName[iStringSize]=0;

            // convert pszAPIName
            iStringSize=SecureStrlen(FakeAPIArrayAnsi[Index].pszAPIName);
            if (iStringSize<0)
            {
                _sntprintf(pszMsg,3*MAX_PATH,_T("Can't get API name of item %d of fake api dll array"),Index+1);
                DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                continue;
            }
            iStringSize=MultiByteToWideChar(CP_ACP, 0, 
                FakeAPIArrayAnsi[Index].pszAPIName, 
                iStringSize,
                pszAPIName,
                MAX_PATH);
            if (!iStringSize)
            {
                _sntprintf(pszMsg,3*MAX_PATH,_T("Error converting API name of item %d of fake api dll array to unicode"),Index+1);
                DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                continue;
            }
            if (iStringSize>=MAX_PATH)
                iStringSize=MAX_PATH-1;
            pszAPIName[iStringSize]=0;

#else
            // if we are in ansi mode, just copy data
            _tcsncpy(pszModuleName,FakeAPIArrayAnsi[Index].pszModuleName,MAX_PATH);
            _tcsncpy(pszAPIName,FakeAPIArrayAnsi[Index].pszAPIName,MAX_PATH);
#endif
        }
        else// unicode encoding
        {
            FakeAPI=FakeAPIArrayUnicode[Index].FakeAPI;
            // check for the end of the array before try to convert data
            bNoMoreItems=(FakeAPI==NULL);
            if (bNoMoreItems)
                break;
            ParamCount=FakeAPIArrayUnicode[Index].ParamCount;
#if (defined(UNICODE)||defined(_UNICODE))
            // if we are in unicode mode just copy data
            _tcsncpy(pszModuleName,FakeAPIArrayUnicode[Index].pszModuleName,MAX_PATH);
            _tcsncpy(pszAPIName,FakeAPIArrayUnicode[Index].pszAPIName,MAX_PATH);
#else
            // translate from unicode to ansi pszModuleName and pszAPIName

            // translate pszModuleName
            iStringSize=SecureWstrlen(FakeAPIArrayUnicode[Index].pszModuleName);
            if (iStringSize<0)
            {
                _sntprintf(pszMsg,3*MAX_PATH,_T("Can't get module name of item %d of fake api dll array"),Index+1);
                DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                continue;
            }
            iStringSize=WideCharToMultiByte(CP_ACP, 0, 
                FakeAPIArrayUnicode[Index].pszModuleName,
                iStringSize, 
                pszModuleName, 
                MAX_PATH,
                NULL,
                NULL);
            if (!iStringSize)
            {
                _sntprintf(pszMsg,3*MAX_PATH,_T("Error converting module name of item %d of fake api dll array to ansi"),Index+1);
                DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                continue;
            }
            else
            {
                if (iStringSize>=MAX_PATH)
                    iStringSize=MAX_PATH-1;
                pszModuleName[iStringSize] = 0;
            }

            // translate pszAPIName
            iStringSize=SecureWstrlen(FakeAPIArrayUnicode[Index].pszAPIName);
            if (iStringSize<0)
            {
                _sntprintf(pszMsg,3*MAX_PATH,_T("Can't get API name of item %d of fake api dll array"),Index+1);
                DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                continue;
            }
            iStringSize=WideCharToMultiByte(CP_ACP, 0, 
                FakeAPIArrayUnicode[Index].pszAPIName,
                iStringSize, 
                pszAPIName, 
                MAX_PATH,
                NULL,
                NULL);
            if (!iStringSize)
            {
                _sntprintf(pszMsg,3*MAX_PATH,_T("Error converting API name of item %d of fake api dll array to ansi"),Index+1);
                DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                continue;
            }
            else
            {
                if (iStringSize>=MAX_PATH)
                    iStringSize=MAX_PATH-1;
                pszAPIName[iStringSize] = 0;
            }
#endif
        }
        ////////////////////////////////////////////////////
        // END OF 
        // compare caracter encoding of fake dll to caracter encoding of this module
        // and do convertion if necessary
        ////////////////////////////////////////////////////


        // get hook address
        if (_tcsnicmp(pszModuleName,EXE_INTERNAL_PREFIX,EXE_INTERNAL_PREFIX_SIZE-1)==0)// -1 because count \0
            // remove prefixe and get value
            _stscanf(&pszModuleName[EXE_INTERNAL_PREFIX_SIZE-1-2],_T("%x"),&pbAPI);// -2 to keep 0x
        else
            // get address with loadlibrary getprocaddress
            pbAPI=GetFuncAddr(pszModuleName,pszAPIName);

        if (IsBadCodePtr((FARPROC)pbAPI))
        {
            _sntprintf(pszMsg,3*MAX_PATH,_T("Bad code pointer for function %s in dll\r\n %s\r\nFunction not hooked"),pszAPIName,pszModuleName);
            DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
            continue;
        }

        // get Item in the pLinkListAPIInfos list
        pItemAPIInfo=GetAssociatedItemAPIInfo(pbAPI,&bAlreadyHooked);
        if (!pItemAPIInfo)
            continue;

        pAPIInfo=(tagAPIInfo*)pItemAPIInfo->ItemData;
        if (!pAPIInfo)
            continue;

        // if api was already hooked
        if (bAlreadyHooked)
        {
            // if a fake api is already installed on this func
            // if no fake api was installed
            if (pAPIInfo->FakeAPIAddress==NULL)
            {
                // set fake api address
                pAPIInfo->FakeAPIAddress=FakeAPI;
                pAPIInfo->FakeAPIFileID=(ULONG_PTR)hModule;
                continue;
            }
            // else

            // a fake api is already installed for this func

            // ask for update fake api
            _sntprintf(pszMsg,3*MAX_PATH,_T("A Fake API was already defined for\r\nmodule %s\r\nand API name %s\r\nby "),pszModuleName,pszAPIName);

            // get file name
            pItemFileInfo = pLinkListFakeAPIFileID->Head;
            // look throw pItemFileInfo
            while (pItemFileInfo)
            {
                pOldFileID=(tagFileID*)pItemFileInfo->ItemData;
                if (pAPIInfo->FakeAPIFileID==pOldFileID->Id)
                {
                    // filename found add it to szMsg
                    _tcscat(pszMsg,pOldFileID->szFileName);
                    break;// go out of while
                }
                pItemFileInfo = pItemFileInfo->NextItem;
            }

            _tcscat(pszMsg,_T("\r\nDo you want to change it with this new dll ?"));
            if(DynamicMessageBox(NULL,pszMsg,_T("API Override Warning"),MB_ICONWARNING|MB_YESNO|MB_TOPMOST)==IDYES)
            {
                // update fake api address and hmodule
                pAPIInfo->FakeAPIAddress=FakeAPI;
                pAPIInfo->FakeAPIFileID=(ULONG_PTR)hModule;
            }
        }
        else // api was not hooked
        {
            pAPIInfo->FakeAPIAddress=FakeAPI;
            pAPIInfo->FakeAPIFileID=(ULONG_PTR)hModule;

            //////////////////////////////////////
            // save API address
            //////////////////////////////////////
            pAPIInfo->APIAddress = (FARPROC)pbAPI;

            //////////////////////////////////////
            // copy module name
            //////////////////////////////////////
            pAPIInfo->szModuleName = (TCHAR*)HeapAlloc(GetProcessHeap(), 0, (_tcslen(pszModuleName) + 1)*sizeof(TCHAR));
            if (pAPIInfo->szModuleName == NULL)
            {
                pLinkListAPIInfos->RemoveItem(pItemAPIInfo); // hook is not installed don't remove it

                DynamicMessageBox(NULL,_T("Memory allocation error"),_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                return TRUE;// dll has begin to be loaded --> must be unloaded
            }
            _tcscpy(pAPIInfo->szModuleName, pszModuleName);

            //////////////////////////////////////
            // copy API name
            //////////////////////////////////////
            pAPIInfo->szAPIName = (TCHAR*)HeapAlloc(GetProcessHeap(), 0, (_tcslen(pszAPIName) + 1)*sizeof(TCHAR));
            if (pAPIInfo->szAPIName == NULL)
            {
                HeapFree(GetProcessHeap(), 0, pAPIInfo->szModuleName);
                pLinkListAPIInfos->RemoveItem(pItemAPIInfo); // hook is not installed don't remove it

                DynamicMessageBox(NULL,_T("Memory allocation error"),_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                return TRUE;// dll has begin to be loaded --> must be unloaded
            }
            _tcscpy(pAPIInfo->szAPIName, pszAPIName);

            //////////////////////////////////////
            // create a critical section to avoid simultaneous func hooks
            //////////////////////////////////////
            InitializeCriticalSection(&pAPIInfo->CriticalSection);

            //////////////////////////////////////
            // specify parameter number for allocating enought stack size
            //////////////////////////////////////
            pAPIInfo->ParamCount = ParamCount;

            //////////////////////////////////////
            // specify no parameter direction spying
            //////////////////////////////////////
            pAPIInfo->ParamDirectionType=PARAM_DIR_NONE;

            //////////////////////////////////////
            // install hook now (only now when pAPIInfo is fully completed else you'll get unexpected results :D)
            //////////////////////////////////////
            if (!HookAPIFunction(pAPIInfo))//==FALSE)
            {
                DeleteCriticalSection(&pAPIInfo->CriticalSection);
                HeapFree(GetProcessHeap(), 0, pAPIInfo->szAPIName);
                HeapFree(GetProcessHeap(), 0, pAPIInfo->szModuleName);
                pLinkListAPIInfos->RemoveItem(pItemAPIInfo); // hook is not installed don't remove it
            }
        }// end if(!bAlreadyHooked)
    }// end while
    return TRUE;
}


//-----------------------------------------------------------------------------
// Name: UnloadFakeApiDll
// Object: try to unload all associated hook for this dll
// Parameters :
//     in  : PSTR pszFileName : dll file name (full path)
// Return : TRUE if library is free (even if error), FALSE if dll is still loaded
//-----------------------------------------------------------------------------
BOOL UnloadFakeApiDll(TCHAR* szFakeAPIDLLFullPath)
{
    ULONG_PTR FileID=0;
    tagFileID* pFileID;
    CLinkListItem* pItemFakeApiFileInfo;
    tagAPIInfo *pAPIInfo;
    CLinkListItem* pItemAPIInfo;
    CLinkListItem* pNextItemAPIInfo;
    TCHAR pszMsg[2*MAX_PATH];
    BOOL bUnHookSuccessFull=TRUE;
    BOOL bRet=FALSE;
    HMODULE hModule;

    if (*szFakeAPIDLLFullPath==0)
    {
        DynamicMessageBox(NULL,_T("Empty file name"),_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        return FALSE;
    }

    // get file ID
    pItemFakeApiFileInfo = pLinkListFakeAPIFileID->Head;
    // look throw pLinkListFakeAPIFileID
    while (pItemFakeApiFileInfo)
    {
        pFileID=(tagFileID*)pItemFakeApiFileInfo->ItemData;
        if (_tcsicmp(szFakeAPIDLLFullPath,pFileID->szFileName)==0)
        {
            FileID=pFileID->Id;
            break;
        }
        pItemFakeApiFileInfo = pItemFakeApiFileInfo->NextItem;
    }
    if (FileID==0)// not use for hooking
    {
        // check if lib is already loaded
        hModule=GetModuleHandle(szFakeAPIDLLFullPath);
        if (hModule)// should never appear
        {
            FreeLibrary(hModule);
        }
        else
        {
            _sntprintf(pszMsg,2*MAX_PATH,_T("File %s was not loaded"),szFakeAPIDLLFullPath);
            DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        }
        return TRUE;
    }
    // look in pLinkListAPIInfos for hooks with file ID == dwFileID
    pItemAPIInfo = pLinkListAPIInfos->Head;
    while (pItemAPIInfo)
    {
        pAPIInfo=(tagAPIInfo*)pItemAPIInfo->ItemData;
        pNextItemAPIInfo=pItemAPIInfo->NextItem;//UnhookAPIFunction release pItemAPIInfo data
        if (pAPIInfo->FakeAPIFileID == FileID)
        {
            // Fake address won't be call anymore 
            pAPIInfo->FakeAPIAddress=NULL;

            // if no monitoring hook on the same func
            if (pAPIInfo->MonitoringFileID==0)
            {
                // unhook function
                bRet=UnhookAPIFunction(pItemAPIInfo);
                bUnHookSuccessFull=bUnHookSuccessFull && bRet;
            }
        }
        pItemAPIInfo = pNextItemAPIInfo;
    }

    // Free library
    if (bUnHookSuccessFull) // else we are sure to crash host app as 
    {                       // at end of blocking func you are returning in our dll memory... which is Unloaded
        // remove pItemFakeApiFileInfo
        pLinkListFakeAPIFileID->RemoveItem(pItemFakeApiFileInfo);
        FreeLibrary((HMODULE)FileID);
        return TRUE;
    }
    return FALSE;
}