/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "ProcessInternalCallRequest.h"

//-----------------------------------------------------------------------------
// Name: ProcessInternalCallRequest
// Object: call function at address pFunc with parameters specified in pParams
//          and store function return (eax) in pRet
// Parameters :
//      in: PBYTE pBuffer
//          CMailSlotClient* pMailSlotClient
// Return : TRUE on success, FALSE on error
//-----------------------------------------------------------------------------
BOOL ProcessInternalCallRequest(PBYTE pBuffer,CMailSlotClient* pMailSlotClient)
{
    FARPROC pFunc;
    PBYTE Data;
    DWORD dw;
    BOOL bRes=FALSE;
    CApiOverrideFuncAndParams* pApiOverrideFuncAndParams=new CApiOverrideFuncAndParams();

    // decode specified buffer
    pApiOverrideFuncAndParams->Decode(pBuffer);

    //////////////////////////
    // get hook address
    //////////////////////////

    // if EXE_INTERNAL
    if (_tcsnicmp(pApiOverrideFuncAndParams->DecodedLibName,EXE_INTERNAL_PREFIX,EXE_INTERNAL_PREFIX_SIZE-1)==0)// -1 because count of \0
        // remove prefixe and get value
        _stscanf(&pApiOverrideFuncAndParams->DecodedLibName[EXE_INTERNAL_PREFIX_SIZE-1-2],_T("%x"),&pFunc);// -2 to keep 0x
    else // if dll func
        pFunc=(FARPROC)GetFuncAddr(pApiOverrideFuncAndParams->DecodedLibName,
                                    pApiOverrideFuncAndParams->DecodedFuncName
                                    );
    // check func pointer
    if (IsBadCodePtr(pFunc))
        return FALSE;

    // make call
    ProcessInternalCallRequest(
            (FARPROC)pFunc,
            pApiOverrideFuncAndParams->DecodedNbParams,
            pApiOverrideFuncAndParams->DecodedParams,
            &pApiOverrideFuncAndParams->DecodedReturnedValue
            );
    
    // encode result
    if (pApiOverrideFuncAndParams->Encode(
        pApiOverrideFuncAndParams->DecodedLibName,
        pApiOverrideFuncAndParams->DecodedFuncName,
        pApiOverrideFuncAndParams->DecodedNbParams,
        pApiOverrideFuncAndParams->DecodedParams,
        pApiOverrideFuncAndParams->DecodedReturnedValue
        ))
    {
        // send results throw mailslot
        Data=new BYTE[pApiOverrideFuncAndParams->EncodedBufferSize+sizeof(DWORD)];
        dw=CMD_PROCESS_INTERNAL_CALL_REPLY;
        memcpy(Data,&dw,sizeof(DWORD));
        memcpy(&Data[sizeof(DWORD)],pApiOverrideFuncAndParams->EncodedBuffer,pApiOverrideFuncAndParams->EncodedBufferSize);
        bRes=pMailSlotClient->Write(Data,pApiOverrideFuncAndParams->EncodedBufferSize+sizeof(DWORD));
        delete Data;
    }
    delete pApiOverrideFuncAndParams;

    return bRes;
}

//-----------------------------------------------------------------------------
// Name: ProcessInternalCallRequest
// Object: call function at address pFunc with parameters specified in pParams
//          and store function return (eax) in pRet
// Parameters :
//      in: FARPROC pFunc : function address
//          int NbParams : nb params in pParams
//          PSTRUCT_FUNC_PARAM pParams : array of STRUCT_FUNC_PARAM. Can be null if no params
//      out : DWORD* pRet : returned value
// Return : 
//-----------------------------------------------------------------------------
void ProcessInternalCallRequest(FARPROC pFunc,int NbParams,PSTRUCT_FUNC_PARAM pParams,DWORD* pRet)
{
    int cnt;
    int cnt2;
    DWORD dw;
    BYTE b;
    USHORT us;
    PSTRUCT_FUNC_PARAM pCurrentParam;
    DWORD dwOriginalESP;

    _asm
    {
        // store esp to restore it without carring about calling convention
        mov [dwOriginalESP],ESP
    }
    // for each param
    for (cnt=NbParams-1;cnt>=0;cnt--)
    {
        pCurrentParam=&pParams[cnt];

        // if params should be passed as ref
        if (pCurrentParam->bPassAsRef)
        {
            // push param address
            dw=(DWORD)pCurrentParam->pData;
            _asm
            {
                mov eax,dw
                push eax
            }
        }
        else // we have to push param value
        {
            // byte
            if (pCurrentParam->dwDataSize==1)
            {
                b=pCurrentParam->pData[0];
                _asm
                {
                    mov al,b
                    push eax
                }
            }
            // short
            else if (pCurrentParam->dwDataSize==2)
            {
                memcpy(&us,pCurrentParam->pData,2);
                _asm
                {
                    mov ax,us
                    push eax
                }
            }
            // dword
            else if (pCurrentParam->dwDataSize==4)
            {
                memcpy(&dw,pCurrentParam->pData,4);
                _asm
                {
                    mov eax,dw
                    push eax
                }
            }
            // more than dword
            else
            {
                for (cnt2=pCurrentParam->dwDataSize-4;cnt2>=0;cnt2-=4)
                {
                    memcpy(&dw,&pCurrentParam->pData[cnt2],4);
                    _asm
                    {
                        mov eax,dw
                        push eax
                    }
                }
            }
        }
    }
    // now all params are pushed in stack --> just make call
    _asm
    {
        // call func
        call pFunc
        // put pointer to return address in ecx
        mov ecx,[pRet]
        // put return value in the adress pointed by ecx --> *pRet=eax
        mov [ecx],eax
        // restore esp (works for both calling convention)
        mov ESP,[dwOriginalESP]
    }
}