/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


// to avoid rebasing, use the linker/advanced/BaseAddress option 

#include <windows.h>
#include "APIMonitoringFileLoader.h"
#include "APIOverrideKernel.h"
#include "FakeApiLoader.h"
#include "ProcessInternalCallRequest.h"
#include "../../MailSlot/MailSlotClient.h"
#include "../InterProcessCommunication.h"
#include "../../../CleanCloseHandle/CleanCloseHandle.h"
#include "../../../APIError/APIError.h"
#include "../../singleinstance/csingleinstance.h"

// we don't want to link to the user32.dll, so don't use msg box inside MailSlotServer
// assume CMAILSLOTSERVER_NO_MESSAGEBOX is defined in project options/c++/preprocessor
#include "../../MailSlot/MailSlotServer.h"

#include "DynamicMessageBox.h"

DWORD dwSystemPageSize=4096;
CLinkList* pLinkListAPIInfos=NULL;
CLinkList* pLinkListMonitoringFileID=NULL;
CLinkList* pLinkListFakeAPIFileID=NULL;
CMailSlotClient* pMailSlotClt=NULL;
CMailSlotServer* pMailSlotSrv=NULL;
CSingleInstance* pSingleInstance=NULL;

HANDLE hThreadWatchingEvents=NULL;
BOOL bMonitoring=TRUE;
BOOL bFaking=TRUE;

BOOL bProcessFreeDone=FALSE;

//(Injector -> APIOverride.dll)
HANDLE hevtStartMonitoring=NULL;
HANDLE hevtStopMonitoring=NULL;
HANDLE hevtStartFaking=NULL;
HANDLE hevtStopFaking=NULL;
HANDLE hevtFreeProcess=NULL;
    
// (APIOverride.dll -> Injector)
HANDLE hevtAPIOverrideDllProcessAttachCompleted=NULL;
HANDLE hevtAPIOverrideDllProcessDetachCompleted=NULL;
HANDLE hevtProcessFree=NULL;
HANDLE hevtMonitoringFileLoaded=NULL;
HANDLE hevtMonitoringFileUnloaded=NULL;
HANDLE hevtFakeAPIDLLLoaded=NULL;
HANDLE hevtFakeAPIDLLUnloaded=NULL;
HANDLE hevtError=NULL;
HANDLE hevtUnload=NULL;


void FreeAll();
void MailSlotSrvCallback(PVOID pData,DWORD dwDataSize,PVOID pUserData);
DWORD WINAPI ThreadWatchingEventsProc(LPVOID lpParameter);

BOOL WINAPI DllMain(HINSTANCE hInstDLL, DWORD dwReason, PVOID pvReserved)
{
    UNREFERENCED_PARAMETER(pvReserved);

    TCHAR psz[MAX_PATH+9];
    TCHAR pszMsg[2*MAX_PATH];
    TCHAR pszPID[9];
    SYSTEM_INFO siSysInfo;

    switch (dwReason)
    {
        case DLL_PROCESS_ATTACH:
            // avoid DLL_THREAD_ATTACH and DLL_THREAD_DETACH notifications
            DisableThreadLibraryCalls(hInstDLL);

            GetSystemInfo(&siSysInfo); 
            dwSystemPageSize=siSysInfo.dwPageSize;

            // pid -> string
            _itot(GetCurrentProcessId(),pszPID,16);

            _tcscpy(psz,APIOVERRIDE_EVENT_ERROR);
            _tcscat(psz,pszPID);
            hevtError=OpenEvent(EVENT_ALL_ACCESS,TRUE,psz);

            _tcscpy(psz,APIOVERRIDE_MUTEX);
            _tcscat(psz,pszPID);
            pSingleInstance=new CSingleInstance(psz);
            if (pSingleInstance->IsAnotherInstanceRunning())
            {
                if (hevtError)
                    SetEvent(hevtError);
                _sntprintf(pszMsg,2*MAX_PATH,_T("Error API override already loaded for Process Id %s"),pszPID);
                DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                delete pSingleInstance;
                return FALSE;
            }
            //////////////////////////
            // try to open events
            //////////////////////////

            //(Injector -> APIOverride.dll)
            _tcscpy(psz,APIOVERRIDE_EVENT_START_MONITORING);
            _tcscat(psz,pszPID);
            hevtStartMonitoring=OpenEvent(EVENT_ALL_ACCESS,FALSE,psz);
            _tcscpy(psz,APIOVERRIDE_EVENT_STOP_MONITORING);
            _tcscat(psz,pszPID);
            hevtStopMonitoring=OpenEvent(EVENT_ALL_ACCESS,FALSE,psz);
            _tcscpy(psz,APIOVERRIDE_EVENT_START_FAKING);
            _tcscat(psz,pszPID);
            hevtStartFaking=OpenEvent(EVENT_ALL_ACCESS,FALSE,psz);
            _tcscpy(psz,APIOVERRIDE_EVENT_STOP_FAKING);
            _tcscat(psz,pszPID);
            hevtStopFaking=OpenEvent(EVENT_ALL_ACCESS,FALSE,psz);
            _tcscpy(psz,APIOVERRIDE_EVENT_FREE_PROCESS);
            _tcscat(psz,pszPID);
            hevtFreeProcess=OpenEvent(EVENT_ALL_ACCESS,FALSE,psz);

            // (APIOverride.dll -> Injector)
            _tcscpy(psz,APIOVERRIDE_EVENT_DLLPROCESS_ATTACH_COMPLETED);
            _tcscat(psz,pszPID);
            hevtAPIOverrideDllProcessAttachCompleted=OpenEvent(EVENT_ALL_ACCESS,TRUE,psz);
            _tcscpy(psz,APIOVERRIDE_EVENT_DLL_DETACHED_COMPLETED);
            _tcscat(psz,pszPID);
            hevtAPIOverrideDllProcessDetachCompleted=OpenEvent(EVENT_ALL_ACCESS,TRUE,psz);
            _tcscpy(psz,APIOVERRIDE_EVENT_PROCESS_FREE);
            _tcscat(psz,pszPID);
            hevtProcessFree=OpenEvent(EVENT_ALL_ACCESS,TRUE,psz);
            _tcscpy(psz,APIOVERRIDE_EVENT_MONITORING_FILE_LOADED);
            _tcscat(psz,pszPID);
            hevtMonitoringFileLoaded=OpenEvent(EVENT_ALL_ACCESS,TRUE,psz);
            _tcscpy(psz,APIOVERRIDE_EVENT_MONITORING_FILE_UNLOADED);
            _tcscat(psz,pszPID);
            hevtMonitoringFileUnloaded=OpenEvent(EVENT_ALL_ACCESS,TRUE,psz);
            _tcscpy(psz,APIOVERRIDE_EVENT_FAKE_API_DLL_LOADED);
            _tcscat(psz,pszPID);
            hevtFakeAPIDLLLoaded=OpenEvent(EVENT_ALL_ACCESS,TRUE,psz);
            _tcscpy(psz,APIOVERRIDE_EVENT_FAKE_API_DLL_UNLOADED);
            _tcscat(psz,pszPID);
            hevtFakeAPIDLLUnloaded=OpenEvent(EVENT_ALL_ACCESS,TRUE,psz);

            if (!(hevtStartMonitoring&&hevtStopMonitoring&&hevtStartFaking&&hevtStopFaking&&hevtFreeProcess
                &&hevtAPIOverrideDllProcessAttachCompleted&&hevtAPIOverrideDllProcessDetachCompleted&&hevtProcessFree
                &&hevtMonitoringFileLoaded&&hevtMonitoringFileUnloaded
                &&hevtFakeAPIDLLLoaded&&hevtFakeAPIDLLUnloaded
                &&hevtError))
            {
                if (hevtError)
                    SetEvent(hevtError);

                _sntprintf(pszMsg,2*MAX_PATH,_T("Error opening named events for Process Id %s"),pszPID);
                DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                FreeAll();
                CleanCloseHandle(&hevtAPIOverrideDllProcessDetachCompleted);
                return FALSE;
            }

            //////////////////////////
            // create mailslots 
            //////////////////////////

            // client for sending logs to spying process
            _tcscpy(psz,APIOVERRIDE_MAILSLOT_TO_INJECTOR);
            _tcscat(psz,pszPID);

            pMailSlotClt=new CMailSlotClient(psz);
            if (!pMailSlotClt->Open())
            {
                SetEvent(hevtError);
                _sntprintf(pszMsg,2*MAX_PATH,_T("Can't open MailSlot %s"),psz);
                DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                
                FreeAll();
                CleanCloseHandle(&hevtAPIOverrideDllProcessDetachCompleted);
                return FALSE;
            }

            // server to get slow commands from spying process
            _tcscpy(psz,APIOVERRIDE_MAILSLOT_FROM_INJECTOR);
            _tcscat(psz,pszPID);
            pMailSlotSrv=new CMailSlotServer(psz,MailSlotSrvCallback,NULL);
            if (!pMailSlotSrv->Start())
            {
                SetEvent(hevtError);

                _sntprintf(pszMsg,2*MAX_PATH,_T("Error starting MailSlot server %s"),psz);
                DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                
                FreeAll();
                CleanCloseHandle(&hevtAPIOverrideDllProcessDetachCompleted);
                return FALSE;
            }

            //////////////////////////
            // create link lists 
            //////////////////////////

            // create a link list for API informations
            pLinkListAPIInfos=new CLinkList(sizeof(tagAPIInfo));

            // create a link list for api monitoring file ID
            pLinkListMonitoringFileID=new CLinkList(sizeof(tagFileID));

            // create a link list for fake api dll file ID
            pLinkListFakeAPIFileID=new CLinkList(sizeof(tagFileID));
            


            // start Thread for looking Injector -> APIOverride.dll events and local unload event
            hThreadWatchingEvents=CreateThread(
                                    NULL,//LPSECURITY_ATTRIBUTES lpThreadAttributes,
                                    0,// SIZE_T dwStackSize,
                                    ThreadWatchingEventsProc,//LPTHREAD_START_ROUTINE lpStartAddress,
                                    NULL,//LPVOID lpParameter,
                                    0,//DWORD dwCreationFlags,
                                    NULL//LPDWORD lpThreadId
                                    );

            if (!hThreadWatchingEvents)
            {
                SetEvent(hevtError);

                _sntprintf(pszMsg,2*MAX_PATH,_T("Error creating watching thread for Process Id %s"),pszPID);
                DynamicMessageBox(NULL,pszMsg,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);

                
                FreeAll();
                CleanCloseHandle(&hevtAPIOverrideDllProcessDetachCompleted);
                return FALSE;
            }

            hevtUnload=CreateEvent(NULL,TRUE,FALSE,NULL);

            SetEvent(hevtAPIOverrideDllProcessAttachCompleted);
            break;

        case DLL_PROCESS_DETACH: // not called if DLL_PROCESS_ATTACH return FALSE --> free memory in DLL_PROCESS_ATTACHbefore returning FALSE
            if (hevtUnload)
                SetEvent(hevtUnload);
            FreeAll();

            // set hevtAPIOverrideDllProcessDetachCompleted event before closing it
            SetEvent(hevtAPIOverrideDllProcessDetachCompleted);
            CleanCloseHandle(&hevtAPIOverrideDllProcessDetachCompleted);
            CleanCloseHandle(&hThreadWatchingEvents);
            break;
    }

    return TRUE;
}
//-----------------------------------------------------------------------------
// Name: MailSlotSrvCallback
// Object: callback for commands with data
// Parameters :
//      in: PVOID pData : PSTRUCT_COMMAND
//          DWORD dwDataSize : pData size
//          PVOID pUserData : not used
// Return : 
//-----------------------------------------------------------------------------
void MailSlotSrvCallback(PVOID pData,DWORD dwDataSize,PVOID pUserData)
{
    UNREFERENCED_PARAMETER(pUserData);
    if (dwDataSize<sizeof(PSTRUCT_COMMAND))
        return;
    PSTRUCT_COMMAND pCmd=(PSTRUCT_COMMAND)pData;

    if ((dwDataSize<sizeof(STRUCT_COMMAND))&&(pCmd->dwCommand_ID!=CMD_PROCESS_INTERNAL_CALL_QUERY))
        return;
    
    switch(pCmd->dwCommand_ID)
    {
        case CMD_LOAD_MONITORING_FILE:
            if (LoadMonitoringFile(pCmd->pszStringParam))
                // signal file is loaded (no information returned about success or not)
                SetEvent(hevtMonitoringFileLoaded);
            else
                SetEvent(hevtError);
            break;
        case CMD_UNLOAD_MONITORING_FILE:
            if (UnloadMonitoringFile(pCmd->pszStringParam))
                // signal file is unloaded (no information returned about success or not)
                SetEvent(hevtMonitoringFileUnloaded);
            else
                SetEvent(hevtError);
            break;
        case CMD_LOAD_FAKE_API_DLL:
            if (LoadFakeApiDll(pCmd->pszStringParam))
                // signal dll is loaded (no information returned about success or not)
                SetEvent(hevtFakeAPIDLLLoaded);
            else
                SetEvent(hevtError);
            break;
        case CMD_UNLOAD_FAKE_API_DLL:
            if (UnloadFakeApiDll(pCmd->pszStringParam))
                // signal dll is unloaded (no information returned about success or not)
                SetEvent(hevtFakeAPIDLLUnloaded);
            else
                SetEvent(hevtError);
            break;
        case CMD_PROCESS_INTERNAL_CALL_QUERY:// pData is not a PSTRUCT_COMMAND
            if (!ProcessInternalCallRequest(&((PBYTE)pData)[sizeof(DWORD)],pMailSlotClt))
                SetEvent(hevtError);
            break;
    }
}

//-----------------------------------------------------------------------------
// Name: ThreadWatchingEventsProc
// Object: thread for watching events (commands with no data)
// Parameters :
//      in: LPVOID lpParameter : not used
// Return : 0 on success -1 on error
//-----------------------------------------------------------------------------
DWORD WINAPI ThreadWatchingEventsProc(LPVOID lpParameter)
{
    UNREFERENCED_PARAMETER(lpParameter);

    DWORD dwRes;
    DWORD dwLastError;
    TCHAR pcError[MAX_PATH];

    HANDLE pHandles[6]={
                        hevtFreeProcess,
                        hevtStartMonitoring,
                        hevtStopMonitoring,
                        hevtStartFaking,
                        hevtStopFaking,
                        hevtUnload
                       };

    for(;;)
    {
        dwRes=WaitForMultipleObjects(6,pHandles,FALSE,INFINITE);
        switch(dwRes)
        {
        case WAIT_OBJECT_0://hevtFreeProcess
            FreeAll();
            return 0;
        case WAIT_OBJECT_0+1://hevtStartMonitoring
            bMonitoring=TRUE;
            break;
        case WAIT_OBJECT_0+2://hevtStopMonitoring
            bMonitoring=FALSE;
            break;
        case WAIT_OBJECT_0+3://hevtStartFaking
            bFaking=TRUE;
            break;
        case WAIT_OBJECT_0+4://hevtStopFaking
            bFaking=FALSE;
            break;
        case WAIT_OBJECT_0+5://hevtUnload
            return 0xFFFFFFFF;
        case WAIT_FAILED:
            dwLastError=GetLastError();
            dwRes=FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
                                    NULL,
                                    dwLastError,
                                    GetUserDefaultLangID(),//GetSystemDefaultLangID()
                                    pcError,
                                    MAX_PATH-1,
                                    NULL);
    
            //If the function succeeds, the return value is the number of TCHARs stored in the output buffer,
            //  excluding the terminating null character.
            //If the function fails, the return value is zero
            if(dwRes!=0)
                DynamicMessageBox(NULL,pcError,_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);

            return WAIT_FAILED;
        }
    }
}

//-----------------------------------------------------------------------------
// Name: FreeAll
// Object: free all allocated resources
// Parameters :
//      in: 
// Return : 
//-----------------------------------------------------------------------------
void FreeAll()
{
    if (bProcessFreeDone)
        return;
    bProcessFreeDone=TRUE;
    bMonitoring=FALSE;
    bFaking=FALSE;

    //////////////////////////
    // unhook all
    //////////////////////////
    UnhookAllAPIFunctions();

    //////////////////////////
    // clear link lists must be after UnhookAllAPIFunctions
    //////////////////////////
    if (pLinkListAPIInfos)
        delete pLinkListAPIInfos;

    if (pLinkListMonitoringFileID)
        delete pLinkListMonitoringFileID;

    if (pLinkListFakeAPIFileID)
        delete pLinkListFakeAPIFileID;

    //////////////////////////
    // Free MailSlots
    //////////////////////////
    if (pMailSlotClt)
        delete pMailSlotClt;            

    if (pMailSlotSrv)
    {
        pMailSlotSrv->Stop();
        delete pMailSlotSrv;
    }

    //////////////////////////
    // close events handles
    //////////////////////////

    //(Injector -> APIOverride.dll)
    CleanCloseHandle(&hevtStartMonitoring);
    CleanCloseHandle(&hevtStopMonitoring);
    CleanCloseHandle(&hevtStartFaking);
    CleanCloseHandle(&hevtStopFaking);
    CleanCloseHandle(&hevtFreeProcess);
        
    // (APIOverride.dll -> Injector)
    CleanCloseHandle(&hevtAPIOverrideDllProcessAttachCompleted);
    
    CleanCloseHandle(&hevtMonitoringFileLoaded);
    CleanCloseHandle(&hevtMonitoringFileUnloaded);
    CleanCloseHandle(&hevtFakeAPIDLLLoaded);
    CleanCloseHandle(&hevtFakeAPIDLLUnloaded);
    CleanCloseHandle(&hevtError);

    CleanCloseHandle(&hevtUnload);
    SetEvent(hevtProcessFree);
    CleanCloseHandle(&hevtProcessFree);

    if (pSingleInstance)
        delete pSingleInstance;    
}