/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.



This Class has been written thanks to the very good
WinSpy Project Copyright (c) by 2003 Robert Kuster
*/

#include "codeinject.h"

#ifdef _DEBUG
#pragma message (__FILE__ " ")
#pragma message (__FILE__ " ************************************************************")
#pragma message (__FILE__ "    WARNING CODE INJECTION WILL ONLY WORK IN RELEASE MODE")
#pragma message (__FILE__ " ************************************************************")
#pragma message (__FILE__ " ")
#endif

//-----------------------------------------------------------------------------
// Name: RemoteGetModuleHandleThreadProc
// Object: Code injected in remote process to get a module handle
// Parameters :
//     in/out  : CCodeInject::PGETMODULEHANDLE_PARAM lpParameter
//     return : 
//-----------------------------------------------------------------------------
static DWORD WINAPI RemoteGetModuleHandleThreadProc (CCodeInject::PGETMODULEHANDLE_PARAM lpParameter)
{
    lpParameter->bFreeLibraryShoudBeCall=FALSE;
    // try get module handle
    lpParameter->RemotehModule=lpParameter->pGetModuleHandleA(lpParameter->pszDllName);
    if (lpParameter->RemotehModule)
        return 0;
    // if library not loaded, just load it
    lpParameter->bFreeLibraryShoudBeCall=TRUE;
    lpParameter->RemotehModule=lpParameter->pLoadLibraryA(lpParameter->pszDllName);
 	return 0;
}
static DWORD WINAPI RemoteGetModuleHandleAfterThreadProc (void) {return 0;}

//-----------------------------------------------------------------------------
// Name: RemoteFreeLibraryThreadProc
// Object: Code injected in remote process free a library
// Parameters :
//     in     : CCodeInject::PFREELIBRARY_PARAM lpParameter
//     return : 
//-----------------------------------------------------------------------------
static DWORD WINAPI RemoteFreeLibraryThreadProc (CCodeInject::PFREELIBRARY_PARAM lpParameter)
{
    return lpParameter->pFreeLibrary(lpParameter->hModule);
}
static DWORD WINAPI RemoteFreeLibraryAfterThreadProc (void) {return 0;}

//-----------------------------------------------------------------------------
// Name: RemoteGetProcAddressThreadProc
// Object: Code injected in remote process to get a proc address
// Parameters :
//     in/out  : CCodeInject::PGETPROCADDRESS_PARAM lpParameter
//     return : 
//-----------------------------------------------------------------------------
static DWORD WINAPI RemoteGetProcAddressThreadProc (CCodeInject::PGETPROCADDRESS_PARAM lpParameter)
{
    lpParameter->RemotepFunction=lpParameter->pGetProcAddressA(lpParameter->hModule,lpParameter->pszProcName);
    return 0;
}
static DWORD WINAPI RemoteGetProcAddressAfterThreadProc (void) {return 0;}


//-----------------------------------------------------------------------------
// Name: CCodeInject
// Object: constructor
// Parameters :
//     in  : HANDLE hProcess : Handle to a process open with PROCESS_CREATE_THREAD |PROCESS_QUERY_INFORMATION|PROCESS_VM_OPERATION|PROCESS_VM_WRITE flags
//                                you can add the PROCESS_VM_READ to retrive data information
//     out : 
//     return : 
//-----------------------------------------------------------------------------
CCodeInject::CCodeInject(HANDLE hProcess)
{
    SYSTEM_INFO siSysInfo;
    GetSystemInfo(&siSysInfo); 
    this->dwPageSize=siSysInfo.dwPageSize;

    this->hProcess=hProcess;
    this->pCodeRemote=NULL;
    this->pDataRemote=NULL;
    this->pDataRemoteBackup=NULL;
    this->pCodeRemoteBackup=NULL;

    // only because kernell32 is mapped at the same address for all the process
    HMODULE hkernel32=GetModuleHandle("kernel32.dll");
    if (!hkernel32)
    {
        hkernel32=LoadLibrary("kernel32.dll");
	    return;
    }
    // load libraries
    this->pLoadLibraryA=(pfLoadLibraryA)GetProcAddress(hkernel32, "LoadLibraryA");
    this->pGetModuleHandleA=(pfGetModuleHandleA)GetProcAddress(hkernel32, "GetModuleHandleA");
    this->pGetProcAddressA=(pfGetProcAddressA)GetProcAddress(hkernel32, "GetProcAddress");
    this->pFreeLibrary=(pfFreeLibrary)GetProcAddress(hkernel32, "FreeLibrary");
}

CCodeInject::~CCodeInject(void)
{
    this->FreeRemoteAllocatedMemory();
}

//-----------------------------------------------------------------------------
// Name: FreeRemoteAllocatedMemory
// Object: Free remote process allocated memory of members pDataRemote and pCodeRemote
// Parameters :
//     in  : 
//     out : 
//     return : 
//-----------------------------------------------------------------------------
void CCodeInject::FreeRemoteAllocatedMemory()
{
    if (this->pDataRemote)
        VirtualFreeEx( this->hProcess, this->pDataRemote, 0, MEM_RELEASE );

    if (this->pCodeRemote)
        VirtualFreeEx( this->hProcess, this->pCodeRemote, 0, MEM_RELEASE );
}

//-----------------------------------------------------------------------------
// Name: SetParameter
// Object: Inject parameters data into remote process
// Parameters :
//     in  : PVOID Param : param to the ThreadProc
//           SIZE_T ParamSize : param size
//     out : 
//     return : FALSE on error, TRUE on success
//-----------------------------------------------------------------------------
BOOL CCodeInject::SetParameter(PVOID Param,SIZE_T ParamSize)
{
    DWORD dwNumBytesXferred = 0; // number of bytes written/read to/from the remote process;

    if (IsBadReadPtr(Param,ParamSize))
        return FALSE;

    if (this->pDataRemote)
    {
        VirtualFreeEx( this->hProcess, this->pDataRemote, 0, MEM_RELEASE );
        this->pDataRemote=NULL;
    }

    // 1. Allocate memory in the remote process for Param
    // 2. Write a copy of DataLocal to the allocated memory
    this->pDataRemote = VirtualAllocEx( this->hProcess, 0, ParamSize, MEM_COMMIT, PAGE_READWRITE );
    if (this->pDataRemote==NULL)
        return FALSE;

    if (!WriteProcessMemory( this->hProcess, this->pDataRemote, Param, ParamSize, &dwNumBytesXferred ))
    {
        VirtualFreeEx( this->hProcess, this->pDataRemote, 0, MEM_RELEASE );
        this->pDataRemote=NULL;
        return FALSE;
    }
    return TRUE;
}


//-----------------------------------------------------------------------------
// Name: GetParameter
// Object: read parameters data into remote process
//         Use it to retreive data after a Remote function call
// Parameters :
//     in  : SIZE_T ParamSize : param size
//     out : PVOID Param : param transmitted to the ThreadProc
//     return : FALSE on error, TRUE on success
//-----------------------------------------------------------------------------
BOOL CCodeInject::GetParameter(PVOID Param,SIZE_T ParamSize)
{
    SIZE_T nbBytesReads=0;
    if (IsBadWritePtr(Param,ParamSize))
        return FALSE;
    return ReadProcessMemory(this->hProcess,this->pDataRemote,Param,ParamSize,&nbBytesReads);
}

//-----------------------------------------------------------------------------
// Name: SetCode
// Object: Inject code into remote process
//         You don't need to use it if you want to call an api or an already loaded code, just 
//         use execute(apiAddress,lpExitCode) instead
// Parameters :
//     in  : FARPROC ThreadProcStartAdress : ThreadProc
//           DWORD dwThreadProcCodeSize = ((LPBYTE) AfterThreadProc - (LPBYTE) ThreadProc);
//
//                                      where ThreadProc and AfterThreadProc are like the following
//                        typedef LRESULT     (WINAPI *pfSendMessage)(HWND,UINT,WPARAM,LPARAM);
//                        typedef struct
//                        {
//                            // params we want to transmit
//                            HWND    hwnd;
//
//                            // func pointer
//                            FARPROC	pfunc;
//                        }INJDATA,*PINJDATA;
//                        //static DWORD WINAPI ShowPasswordContentThreadProc (LPVOID lpParameter)
//                        static DWORD WINAPI ShowPasswordContentThreadProc (PINJDATA lpParameter)
//                        {
//                            ((pfSendMessage)(lpParameter->pfunc))(lpParameter->hwnd,(UINT) EM_SETPASSWORDCHAR,'x',0);
// 	                          return 0;
//                        }
//                        // This function marks the memory address after ThreadProc.
//                        static DWORD WINAPI  ShowPasswordContentAfterThreadProc (void) {return 0;}
//
//     out : 
//     return : FALSE on error, TRUE on success
//-----------------------------------------------------------------------------
BOOL CCodeInject::SetCode(FARPROC ThreadProcStartAdress,DWORD dwThreadProcCodeSize)
{
    // check dwThreadProcCodeSize (with some compiler options, memory flags don't
    // work anymore so check it)
    if (dwThreadProcCodeSize>this->dwPageSize)
        // the following is not disturbing as at least a memory page is allocated by a VirtualAllocEx call
        // (of course only if code to inject is less than a memory page)
        dwThreadProcCodeSize=this->dwPageSize;

    DWORD dwNumBytesXferred = 0; // number of bytes written/read to/from the remote process;

    if (IsBadReadPtr(ThreadProcStartAdress,dwThreadProcCodeSize))
        return FALSE;

    if (this->pCodeRemote)
    {
        VirtualFreeEx( this->hProcess, this->pCodeRemote, 0, MEM_RELEASE );
        this->pCodeRemote=NULL;
    }

    // 1. Allocate memory in the remote process for the injected ThreadProc
    // 2. Write a copy of ThreadProc to the allocated memory
    this->pCodeRemote = (FARPROC) VirtualAllocEx( this->hProcess, 0, dwThreadProcCodeSize, MEM_COMMIT, PAGE_EXECUTE_READWRITE );        
    if (this->pCodeRemote==NULL)
        return FALSE;

    if (!WriteProcessMemory( this->hProcess, this->pCodeRemote, ThreadProcStartAdress, dwThreadProcCodeSize, &dwNumBytesXferred ))
    {
        VirtualFreeEx( this->hProcess, this->pCodeRemote, 0, MEM_RELEASE );
        this->pCodeRemote=NULL;
        return FALSE;
    }
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: Execute
// Object: execute TreadProc in remote process (blocking call : function will not return until thread as finished)
// Parameters :
//     in  : 
//     out : 
//     return : FALSE on error
//-----------------------------------------------------------------------------
BOOL CCodeInject::Execute()
{
    DWORD dwExitCode=0;
    return this->Execute(&dwExitCode);
}
//-----------------------------------------------------------------------------
// Name: Execute
// Object: execute thread proc in remote process (blocking call : function will not return until thread as finished)
// Parameters :
//     in  : 
//     out : LPDWORD lpExitCode : exit code of TreadProc
//     return : FALSE on error
//-----------------------------------------------------------------------------
BOOL CCodeInject::Execute(LPDWORD lpExitCode)
{
    // check if InjectCode was successfully called before
    if (!this->pCodeRemote)
        return FALSE;
    return this->Execute(this->pCodeRemote,lpExitCode);
}
//-----------------------------------------------------------------------------
// Name: Execute
// Object: execute RemoteProcFuncAddress proc in remote process (blocking call : function will not return until thread as finished)
//         Use this function if you don't have injected code (func already exist in remote process)
//         WARNING A DIRECT CALL WILL WORK ONLY FOR FUNC HAVING 1 PARAMETER AND IF THIS PARAMETER IS A POINTER
//         else you have to do code injection like done in RemoteGetProcAddress, RemoteGetModuleHandleThreadProc
//         RemoteFreeLibrary
// Parameters :
//     in  : FARPROC RemoteProcFunc : remote address of the proc func to execute
//     out : LPDWORD lpExitCode : exit code of TreadProc
//     return : FALSE on error
//-----------------------------------------------------------------------------
BOOL CCodeInject::Execute(FARPROC RemoteProcFuncAddress,LPDWORD lpExitCode)
{
    HANDLE       hThread = NULL; // the handle to the thread executing the remote copy of ThreadProc;
    DWORD        dwThreadId = 0;

    if (IsBadWritePtr(lpExitCode,sizeof(DWORD)))
        return FALSE;

    *lpExitCode=0;

    // Start execution of remote ThreadProc
    hThread = CreateRemoteThread(this->hProcess,
                                NULL,
                                0,
                                (LPTHREAD_START_ROUTINE) RemoteProcFuncAddress,
                                this->pDataRemote,
                                0,
                                &dwThreadId
                                );
    if (!hThread)
        return FALSE;

    // wait for end of thread
    if (WaitForSingleObject(hThread, INFINITE)!=WAIT_OBJECT_0)
    {
        CloseHandle(hThread);
        return FALSE;
    }

    GetExitCodeThread(hThread, lpExitCode);
    CloseHandle(hThread);
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: SaveRemotePointers
// Object: used for internal function for allocating and freeing temporary 
//              pDataRemote and pCodeRemote
// Parameters :
//     in  : 
//     out : 
//     return : 
//-----------------------------------------------------------------------------
void CCodeInject::SaveRemotePointers()
{
    // save local data
    this->pDataRemoteBackup=this->pDataRemote;
    this->pCodeRemoteBackup=this->pCodeRemote;
    // avoid memory to be free as the test condition is this->pDataRemote
    // if we don't put this after restoration our pointer will point on freed memory
    this->pDataRemote=NULL;
    this->pCodeRemote=NULL;
}

//-----------------------------------------------------------------------------
// Name: FreeRemoteAllocatedMemory
// Object: used for internal function for restoring pDataRemote and pCodeRemote
// Parameters :
//     in  : 
//     out : 
//     return : 
//-----------------------------------------------------------------------------
void CCodeInject::RestoreRemotePointers()
{
    this->pDataRemote=this->pDataRemoteBackup;
    this->pCodeRemote=this->pCodeRemoteBackup;
}

//-----------------------------------------------------------------------------
// Name: RemoteGetModuleHandle
// Object: return the HMODULE of the specified library in the remote process
//          if module as been retreive by LoadLibrary, bFreeLibraryShoudBeCall is put to TRUE
//          if it has been retreive by GetModuleHandle, bFreeLibraryShoudBeCall is put to FALSE
//         Use this function if your injected code requiere a non kernell32|user32 library call
// Parameters :
//     in  : 
//     out : HMODULE* hModule : module handle in the remote process
//           BOOL* bFreeLibraryShoudBeCall : TRUE if LoadLibrary as been called (and so FreeLibrary should be called)
//                                           FALE if HMODULE as been retreive by GetModuleHandle
//     return : FALSE on error
//-----------------------------------------------------------------------------
BOOL CCodeInject::RemoteGetModuleHandle(char* pszDllName,HMODULE* phModule,BOOL* pbFreeLibraryShoudBeCall)
{
    GETMODULEHANDLE_PARAM data;
    DWORD dwThreadProcCodeSize;
    BOOL bRet=TRUE;

    if (IsBadReadPtr(pszDllName,1))
        return FALSE;
    if (IsBadReadPtr(phModule,sizeof(HMODULE)))
        return FALSE;
    if (IsBadReadPtr(pbFreeLibraryShoudBeCall,sizeof(BOOL)))
        return FALSE;

    // backup remote pointer pDataRemote and pCodeRemote
    this->SaveRemotePointers();

    // comput code size
    dwThreadProcCodeSize = ((LPBYTE) RemoteGetModuleHandleAfterThreadProc - (LPBYTE) RemoteGetModuleHandleThreadProc);
    // fill data struct infos
    data.pLoadLibraryA=this->pLoadLibraryA;
    data.pGetModuleHandleA=this->pGetModuleHandleA;
    strncpy(data.pszDllName,pszDllName,MAX_PATH-1);
    data.pszDllName[MAX_PATH-1]=0;
    
    if (!this->SetParameter(&data,sizeof(data)))
    {
        this->FreeRemoteAllocatedMemory();
        this->RestoreRemotePointers();
        return FALSE;
    }
    if (!this->SetCode((FARPROC)RemoteGetModuleHandleThreadProc,dwThreadProcCodeSize))
    {
        this->FreeRemoteAllocatedMemory();
        this->RestoreRemotePointers();
        return FALSE;
    }

    if (this->Execute())
    {
        if (this->GetParameter(&data,sizeof(data)))
        {
            *phModule=data.RemotehModule;
            *pbFreeLibraryShoudBeCall=data.bFreeLibraryShoudBeCall;
        }
        else
            bRet=FALSE;
    }
    else
        bRet=FALSE;
   
    // free memory allocated by SetParameter and SetCode
    this->FreeRemoteAllocatedMemory();

    // restore remote pointer pDataRemote and pCodeRemote
    this->RestoreRemotePointers();

    return bRet;
}

//-----------------------------------------------------------------------------
// Name: RemoteFreeLibrary
// Object: Free library specified by hModule in remote process
//         Use this function if your injected code requiere a non kernell32|user32 library call
// Parameters :
//     in  : HMODULE hModule
//     out : 
//     return : FALSE on error
//-----------------------------------------------------------------------------
BOOL CCodeInject::RemoteFreeLibrary(HMODULE hModule)
{
    FREELIBRARY_PARAM data;
    DWORD dwThreadProcCodeSize;
    BOOL bRet=TRUE;

    // backup remote pointer pDataRemote and pCodeRemote
    this->SaveRemotePointers();

    // comput code size
    dwThreadProcCodeSize = ((LPBYTE) RemoteFreeLibraryAfterThreadProc - (LPBYTE) RemoteFreeLibraryThreadProc);
    // fill data struct infos
    data.hModule=hModule;
    data.pFreeLibrary=this->pFreeLibrary;
    
    if (!this->SetParameter(&data,sizeof(data)))
    {
        this->FreeRemoteAllocatedMemory();
        this->RestoreRemotePointers();
        return FALSE;
    }
    if (!this->SetCode((FARPROC)RemoteFreeLibraryThreadProc,dwThreadProcCodeSize))
    {
        this->FreeRemoteAllocatedMemory();
        this->RestoreRemotePointers();
        return FALSE;
    }

    if (!this->Execute())
        bRet=FALSE;
   
    // free memory allocated by SetParameter and SetCode
    this->FreeRemoteAllocatedMemory();

    // restore remote pointer pDataRemote and pCodeRemote
    this->RestoreRemotePointers();

    return bRet;
}

//-----------------------------------------------------------------------------
// Name: RemoteGetProcAddress
// Object: find proc address in remote process
//         Use this function if your injected code requiere a non kernell32|user32 library call
// Parameters :
//     in  : HMODULE hModule
//           char* pszProcName
//     out : FARPROC* ppFunc : proc address in remote process
//     return : FALSE on error
//-----------------------------------------------------------------------------
BOOL CCodeInject::RemoteGetProcAddress(HMODULE hModule,char* pszProcName,FARPROC* ppFunc)
{
    GETPROCADDRESS_PARAM data;
    DWORD dwThreadProcCodeSize;
    BOOL bRet=TRUE;

    if (IsBadReadPtr(pszProcName,1))
        return FALSE;
    if (IsBadWritePtr(ppFunc,sizeof(FARPROC)))
        return FALSE;

    // backup remote pointer pDataRemote and pCodeRemote
    this->SaveRemotePointers();

    // comput code size
    dwThreadProcCodeSize = ((LPBYTE) RemoteGetProcAddressAfterThreadProc - (LPBYTE) RemoteGetProcAddressThreadProc);
    // fill data struct infos
    data.hModule=hModule;
    data.pGetProcAddressA=this->pGetProcAddressA;
    strncpy(data.pszProcName,pszProcName,MAX_PATH-1);
    data.pszProcName[MAX_PATH-1]=0;
    
    if (!this->SetParameter(&data,sizeof(data)))
    {
        this->FreeRemoteAllocatedMemory();
        this->RestoreRemotePointers();
        return FALSE;
    }
    if (!this->SetCode((FARPROC)RemoteGetProcAddressThreadProc,dwThreadProcCodeSize))
    {
        this->FreeRemoteAllocatedMemory();
        this->RestoreRemotePointers();
        return FALSE;
    }

    if (this->Execute())
    {
        if (this->GetParameter(&data,sizeof(data)))
        {
            *ppFunc=data.RemotepFunction;
        }
        else
            bRet=FALSE;
    }
    else
        bRet=FALSE;
   
    // free memory allocated by SetParameter and SetCode
    this->FreeRemoteAllocatedMemory();

    // restore remote pointer pDataRemote and pCodeRemote
    this->RestoreRemotePointers();

    return bRet;
}