/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: class helper for dumping process
//-----------------------------------------------------------------------------

#include "Dump.h"

// process must have PROCESS_QUERY_INFORMATION|PROCESS_VM_READ rights
BOOL CDump::Dump(HANDLE hProcess,HMODULE hModule)
{
    MODULEINFO Modinfo;
    PBYTE pbuffer;
    DWORD dwReallyRead=0;
    TCHAR szFile[MAX_PATH]; 
    OPENFILENAME ofn;
    TCHAR* pcExt;

    // get process filename
    if (GetModuleBaseName(hProcess,hModule,szFile,MAX_PATH))
    {
        // change its extension to .dmp
        pcExt=_tcsrchr(szFile,'.');
        if (pcExt)
        {
            *pcExt=0;
            _tcscat(szFile,_T(".dmp"));
        }
    }
    else
        _tcscpy(szFile,_T(""));

    // get module info
    if (!GetModuleInformation(hProcess,hModule,&Modinfo,sizeof(MODULEINFO)))
    {
        CAPIError::ShowLastError();
        return FALSE;
    }

    // open for read only
    
    pbuffer=new BYTE[Modinfo.SizeOfImage];
    dwReallyRead=0;
    if (!ReadProcessMemory(hProcess,
            Modinfo.lpBaseOfDll,
            (LPVOID)pbuffer,
            Modinfo.SizeOfImage,//SIZE_T nSize,
            &dwReallyRead
        ))
    {
        CAPIError::ShowLastError();
        delete pbuffer;
        return FALSE;
    }

    // save file dialog
    memset(&ofn,0,sizeof (OPENFILENAME));
    ofn.lStructSize=sizeof (OPENFILENAME);
    ofn.hwndOwner=NULL;
    ofn.hInstance=NULL;
    ofn.lpstrFilter=_T("dmp\0*.dmp\0All\0*.*\0");
    ofn.nFilterIndex = 1;
    ofn.Flags=OFN_EXPLORER|OFN_NOREADONLYRETURN|OFN_OVERWRITEPROMPT;
    ofn.lpstrDefExt=_T("dmp");
    ofn.lpstrFile=szFile;
    ofn.nMaxFile=MAX_PATH;
    
    if (!GetSaveFileName(&ofn))
    {
        delete pbuffer;
        return TRUE;
    }

    // write dump
    HANDLE hFile = CreateFile(
        ofn.lpstrFile,
        GENERIC_READ|GENERIC_WRITE,
        FILE_SHARE_READ|FILE_SHARE_WRITE, 
        NULL,
        CREATE_ALWAYS,
        FILE_ATTRIBUTE_NORMAL,
        NULL);
    if (hFile==INVALID_HANDLE_VALUE)
    {
        CAPIError::ShowLastError();
        delete pbuffer;
        return FALSE;
    }

    if (!WriteFile(hFile,
                    pbuffer,
                    dwReallyRead,
                    &dwReallyRead,
                    NULL
                    ))
    {
        CAPIError::ShowLastError();
        CloseHandle(hFile);
        delete pbuffer;
        return FALSE;
    }

    CloseHandle(hFile);
    delete pbuffer;
    MessageBox(NULL,_T("Dump successfully completed"),_T("Information"),MB_OK|MB_ICONINFORMATION|MB_TOPMOST);
    return TRUE;
}

BOOL CDump::Dump(DWORD dwProcessID)
{
    HANDLE hProcess;
    DWORD dwNeededBytes;
    HMODULE hModule;


    BOOL bRet;
    // get process handle
    hProcess = OpenProcess(PROCESS_QUERY_INFORMATION |PROCESS_VM_READ,
                                        FALSE,dwProcessID);

    // get module handle
    if (!EnumProcessModules(hProcess, &hModule,sizeof(HMODULE),&dwNeededBytes))
    {
        CAPIError::ShowLastError();
        CloseHandle(hProcess);
        return FALSE;
    }

    bRet=CDump::Dump(hProcess,hModule);

    // close hande to process
    CloseHandle(hProcess);

    return bRet;
}

BOOL CDump::Dump(DWORD dwProcessID,HMODULE hModule)
{
    HANDLE hProcess;

    BOOL bRet;
    // get process handle
    hProcess = OpenProcess(PROCESS_QUERY_INFORMATION |PROCESS_VM_READ,
                                        FALSE,dwProcessID);

    bRet=CDump::Dump(hProcess,hModule);

    // close hande to process
    CloseHandle(hProcess);

    return bRet;
}