#include "ProcessAndThreadID.h"

CProcessAndThreadID::CProcessAndThreadID()
{
    this->NtDllLibLoaded=FALSE;
    this->Kernel32LibLoaded=FALSE;
    this->pNtQueryInformationThread=NULL;
    this->pNtQueryInformationProcess=NULL;
    this->pRtlNtStatusToDosError=NULL;
    this->pGetProcessIdOfThread =NULL;
    this->pGetThreadId=NULL;
    this->pGetProcessId=NULL;

    this->hModuleNtDll=GetModuleHandle("Ntdll.dll");
    if (!this->hModuleNtDll)
    {
        this->NtDllLibLoaded=TRUE;
        this->hModuleNtDll=LoadLibrary("Ntdll.dll");
    }
    this->hModuleKernel32=GetModuleHandle("Kernel32.dll");
    if (!this->hModuleKernel32)
    {
        this->Kernel32LibLoaded=TRUE;
        this->hModuleKernel32=LoadLibrary("Kernel32.dll");
    }
    if (this->hModuleNtDll)
    {
        this->pNtQueryInformationThread=(ptrNtQueryInformationThread)GetProcAddress(this->hModuleNtDll,"NtQueryInformationThread");
        this->pNtQueryInformationProcess=(ptrNtQueryInformationProcess)GetProcAddress(this->hModuleNtDll,"NtQueryInformationProcess");
        this->pRtlNtStatusToDosError=(ptrRtlNtStatusToDosError)GetProcAddress(this->hModuleNtDll,"RtlNtStatusToDosError");
    }
    if (this->hModuleKernel32)
    {
        this->pGetProcessIdOfThread =(ptrGetProcessIdOfThread)GetProcAddress(this->hModuleKernel32,"GetProcessIdOfThread");
        this->pGetThreadId=(ptrGetThreadId)GetProcAddress(this->hModuleKernel32,"GetThreadId");
        this->pGetProcessId=(ptrGetProcessId)GetProcAddress(this->hModuleKernel32,"GetProcessId");
    }
}
CProcessAndThreadID::~CProcessAndThreadID()
{
    if (this->NtDllLibLoaded)
    {
        FreeLibrary(this->hModuleNtDll);
        this->hModuleNtDll=NULL;
    }
}

//-----------------------------------------------------------------------------
// Name: GetProcessIdOfThreadOldOS
// Object: GetProcessIdOfThread for xp or older OS for new os just use GetProcessIdOfThread
// Parameters :
//     in  : HANDLE hThread : thread handle
//     out :
//     return : ProcessId or 0 if Failed
//-----------------------------------------------------------------------------
DWORD CProcessAndThreadID::GetProcessIdOfThreadOldOS(HANDLE hThread)
{
    DWORD dwProcessId=0;
    HANDLE        hThreadSnap = NULL;
    THREADENTRY32 te32        = {0}; 
    DWORD dwThreadId;

    dwThreadId=this->GetThreadIdOldOS(hThread);
    if (dwThreadId==0)
        return 0;

    // Take a snapshot of all threads currently in the system. 
    hThreadSnap = CreateToolhelp32Snapshot(TH32CS_SNAPTHREAD, 0); 
    if (hThreadSnap == INVALID_HANDLE_VALUE) 
        return dwProcessId; 
 
    // Fill in the size of the structure before using it. 
    te32.dwSize = sizeof(THREADENTRY32); 
 
    // Walk the thread snapshot to find all threads of the process. 
    // If the thread belongs to the process, add its information 
    // to the display list.
    if (Thread32First(hThreadSnap, &te32)) 
    { 
        do 
        { 
            if (dwThreadId == te32.th32ThreadID) 
            { 
                dwProcessId=te32.th32OwnerProcessID;
                break;
            } 
        } 
        while (Thread32Next(hThreadSnap, &te32)); 
    } 
    else 
    {
        CAPIError::ShowLastError();
    }

    // Do not forget to clean up the snapshot object. 
    CloseHandle (hThreadSnap); 

    return dwProcessId;
}
DWORD CProcessAndThreadID::GetThreadIdOldOS(HANDLE hThread)
{
    NTSTATUS                 Status;
    THREAD_BASIC_INFORMATION tbi;
    HANDLE                   hDupHandle;
    HANDLE                   hCurrentProcess;

    if ((this->pNtQueryInformationThread==NULL)||(this->pRtlNtStatusToDosError==NULL))
    {
        MessageBox(NULL,"Error loading NtQueryInformationThread or RtlNtStatusToDosError in Ntdll.dll",
            "Error",MB_OK|MB_ICONERROR|MB_TOPMOST);
        return 0;
    }

    hCurrentProcess = GetCurrentProcess();

    // Use DuplicateHandle() to get THREAD_QUERY_INFORMATION access right
    if (!DuplicateHandle(hCurrentProcess, 
                         hThread, 
                         hCurrentProcess, 
                         &hDupHandle, 
                         THREAD_QUERY_INFORMATION, 
                         FALSE, 
                         0))
    {
        SetLastError(ERROR_ACCESS_DENIED);
        CAPIError::ShowLastError();
        return 0;
    }

    /*
    NTSTATUS
    NtQueryInformationThread (
        IN HANDLE ThreadHandle,
        IN THREADINFOCLASS ThreadInformationClass,
        OUT PVOID ThreadInformation,
        IN ULONG ThreadInformationLength,
        OUT PULONG ReturnLength OPTIONAL
        );
    */

    // get thread id
    Status = this->pNtQueryInformationThread(hDupHandle,
                                      (THREADINFOCLASS)0,//ThreadBasicInformation
                                      &tbi,
                                      sizeof(tbi),
                                      NULL
                                      );
    CloseHandle(hDupHandle);
    if (!NT_SUCCESS(Status))
    {
        SetLastError(this->pRtlNtStatusToDosError(Status));
        CAPIError::ShowLastError();
        return 0;
    }
    return (DWORD)tbi.ClientId.UniqueThread;
}

DWORD CProcessAndThreadID::GetProcessIdOldOS(HANDLE hProcess)
{
    NTSTATUS                  Status;
    PROCESS_BASIC_INFORMATION pbi;
    HANDLE                    hDupHandle;
    HANDLE                    hCurrentProcess;

    if ((this->pNtQueryInformationThread==NULL)||(this->pRtlNtStatusToDosError==NULL))
    {
        MessageBox(NULL,"Error loading NtQueryInformationThread or RtlNtStatusToDosError in Ntdll.dll",
            "Error",MB_OK|MB_ICONERROR|MB_TOPMOST);
        return 0;
    }

    hCurrentProcess = GetCurrentProcess();

    // Use DuplicateHandle() to get PROCESS_QUERY_INFORMATION access right
    if (!DuplicateHandle(hCurrentProcess, 
                         hProcess, 
                         hCurrentProcess, 
                         &hDupHandle, 
                         PROCESS_QUERY_INFORMATION, 
                         FALSE, 
                         0))
    {
        SetLastError(ERROR_ACCESS_DENIED);
        CAPIError::ShowLastError();
        return 0;
    }

    /*
    NTSTATUS
    NtQueryInformationProcess (
        IN HANDLE ProcessHandle,
        IN PROCESSINFOCLASS ProcessInformationClass,
        OUT PVOID ProcessInformation,
        IN ULONG ProcessInformationLength,
        OUT PULONG ReturnLength OPTIONAL
        );
    */
    Status = this->pNtQueryInformationProcess(hDupHandle, 
                                       ProcessBasicInformation, 
                                       &pbi,
                                       sizeof(pbi), 
                                       NULL);

    CloseHandle(hDupHandle);

    if (!NT_SUCCESS(Status))
    {        
        SetLastError(this->pRtlNtStatusToDosError(Status));
        return 0;
    }
    // Return PID
    return (DWORD)pbi.UniqueProcessId;
}

DWORD CProcessAndThreadID::GetProcessIdOfThread(HANDLE hThread)
{
    if(this->pGetProcessIdOfThread)
        return this->pGetProcessIdOfThread(hThread);
    else
        return this->GetProcessIdOfThreadOldOS(hThread);
}
DWORD CProcessAndThreadID::GetThreadId(HANDLE hThread)
{
    if(this->pGetThreadId)
        return this->pGetThreadId(hThread);
    return this->GetThreadIdOldOS(hThread);
}

DWORD CProcessAndThreadID::GetProcessId(HANDLE hProcess)
{
    if(this->pGetProcessId)
        return this->pGetProcessId(hProcess);
    return this->GetProcessIdOldOS(hProcess);
}