/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: Process helper class. It allows easy operation on processes
//         like retreiving name or checking if a process is still running
//-----------------------------------------------------------------------------

#include "ProcessHelper.h"

//-----------------------------------------------------------------------------
// Name: IsAlive
// Object: check if a process is still running 
// Parameters :
//     in : DWORD ProcessID : ID of process we want to check
// Return : TRUE if process is running, else FALSE 
//          FALSE on EnumProcesses error 
//-----------------------------------------------------------------------------
BOOL CProcessHelper::IsAlive(DWORD ProcessID)
{
    HANDLE hProcess = OpenProcess( PROCESS_QUERY_INFORMATION,FALSE, ProcessID );
    if (hProcess)
    {
        CloseHandle(hProcess);
        return TRUE;
    }
    return FALSE;
}

//-----------------------------------------------------------------------------
// Name: GetProcessName
// Object: get the name of the given process ID
// Parameters :
//     in : DWORD ProcessID : ID of process we require name
//          DWORD ProcessNameMaxSize :size of ProcessName count in TCHAR
//     out : TCHAR* ProcessName : name of the process
// Return : TRUE on success, FALSE on error
//-----------------------------------------------------------------------------
BOOL CProcessHelper::GetProcessName(DWORD ProcessID,TCHAR* ProcessName,DWORD ProcessNameMaxSize)
{
    HMODULE hMod;
    DWORD cbNeeded;
    HANDLE hProcess;

    // check output pointer
    if (IsBadWritePtr(ProcessName,ProcessNameMaxSize*sizeof(TCHAR)))
    {
        if (!IsBadWritePtr(ProcessName,sizeof(TCHAR)))
            *ProcessName=0;
        return FALSE;
    }

    // Get a handle to the process.
    hProcess = OpenProcess( PROCESS_QUERY_INFORMATION |
                                PROCESS_VM_READ,
                                FALSE, ProcessID );
    // check success status
    if (!hProcess)
    {
        *ProcessName=0;
        return FALSE;
    }

    // Get the process name.

    // we just need first module handle (the application one)
    // so we don't send an array of HMODULE, but only address of hMod
    if (!EnumProcessModules( hProcess, &hMod, sizeof(hMod),&cbNeeded))
    {
        // close process handle
        CloseHandle( hProcess );
        *ProcessName=0;
        return FALSE;
    }

    // retreive module base name
    if (!GetModuleBaseName(hProcess,hMod,ProcessName,ProcessNameMaxSize))
    {
        // close process handle
        CloseHandle( hProcess );
        *ProcessName=0;
        return FALSE;
    }


    // close process handle
    CloseHandle( hProcess );
    return TRUE;

}