/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: class helper to convert ansi to unicode, and unicode to ansi
//-----------------------------------------------------------------------------

#include "ansiunicodeconvert.h"

//-----------------------------------------------------------------------------
// Name: AnsiToUnicode
// Object: AnsiToUnicode converts the ANSI string pszA to a Unicode string
// and returns the Unicode string through ppszW. Space for the
// converted string is allocated by AnsiToUnicode. Must be free using free(*ppszW)
// Parameters :
//     in  : LPCSTR pszA : string to convert
//     out : LPWSTR* ppszW : converted string
//     return : win32 error code
//-----------------------------------------------------------------------------
DWORD CAnsiUnicodeConvert::AnsiToUnicode(LPCSTR pszA, LPWSTR* ppszW)
{

    ULONG cCharacters;
    DWORD dwError;

    // If input is null then just return the same.
    if (NULL == pszA)
    {
        *ppszW = NULL;
        return NOERROR;
    }

    // Determine number of wide characters to be allocated for the
    // Unicode string.
    cCharacters =  (ULONG)strlen(pszA)+1;

    *ppszW = (LPWSTR) malloc(cCharacters*2);
    if (NULL == *ppszW)
        return ERROR_NOT_ENOUGH_MEMORY;

    // Convert to Unicode.
    if (0 == MultiByteToWideChar(CP_ACP, 0, pszA, cCharacters,
                  *ppszW, cCharacters))
    {
        dwError = GetLastError();
        free(*ppszW);
        *ppszW = NULL;
        return dwError;
    }

    return ERROR_SUCCESS;
}

//-----------------------------------------------------------------------------
// Name: UnicodeToAnsi
// Object: UnicodeToAnsi converts the Unicode string pszW to an ANSI string
// and returns the ANSI string through ppszA. Space for the
// converted string is allocated by UnicodeToAnsi. Must be free using free(*ppszA)
// Parameters :
//     in  : LPCWSTR* ppszW : string to convert
//     out : LPSTR pszA : converted string
//     return : win32 error code
//-----------------------------------------------------------------------------
DWORD CAnsiUnicodeConvert::UnicodeToAnsi(LPCWSTR pszW, LPSTR* ppszA)
{

    ULONG cbAnsi, cCharacters;
    DWORD dwError;

    // If input is null then just return the same.
    if (pszW == NULL)
    {
        *ppszA = NULL;
        return NOERROR;
    }

    cCharacters = (ULONG)wcslen(pszW)+1;
    // Determine number of bytes to be allocated for ANSI string. An
    // ANSI string can have at most 2 bytes per character (for Double
    // Byte Character Strings.)
    cbAnsi = cCharacters*2;

    *ppszA = (LPSTR) malloc(cbAnsi);
    if (NULL == *ppszA)
        return ERROR_NOT_ENOUGH_MEMORY;

    // Convert to ANSI.
    if (0 == WideCharToMultiByte(CP_ACP, 0, pszW, cCharacters, *ppszA,
                  cbAnsi, NULL, NULL))
    {
        dwError = GetLastError();
        free(*ppszA);
        *ppszA = NULL;
        return dwError;
    }
    return ERROR_SUCCESS;
} 
