/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: manages the main dialog resize operations
//-----------------------------------------------------------------------------

#include "ResizeMainDialog.h"

extern HWND mhWndDialog;


// translate lpRect from Screen coordinates to Client Coordinates
// (used for resizing controls)
void GetClientWindowRect(HWND hItem,LPRECT lpRect)
{
    GetWindowRect(hItem,lpRect);
    POINT p;
    p.x=lpRect->left;
    p.y=lpRect->top;
    ScreenToClient(mhWndDialog,&p);
    lpRect->left=p.x;
    lpRect->top=p.y;
    p.x=lpRect->right;
    p.y=lpRect->bottom;
    ScreenToClient(mhWndDialog,&p);
    lpRect->right=p.x;
    lpRect->bottom=p.y;
}

void Resize()
{
    // resize
    RECT Rect;
    GetWindowRect(mhWndDialog,&Rect);
    Resize(&Rect);
}


#define SPACE_BETWEEN_CONTROLS 5

void Resize(RECT* pWinRect)
{
    HWND hItem;
    RECT Rect;
    RECT RectStaticStart;

    RECT RectStaticMonitoring;
    RECT RectStaticBrowseMonitoring;
    RECT RectStaticEditMonitoring;
    RECT RectStaticLoadMonitoring;

    RECT RectStaticFaking;
    RECT RectStaticEditFaking;
    RECT RectStaticLoadFaking;

    RECT PreviousRect;

    RECT RectWindow;

    // check min widh and min height
    if ((pWinRect->right-pWinRect->left)<670)
        pWinRect->right=pWinRect->left+670;
    if ((pWinRect->bottom-pWinRect->top)<380)
        pWinRect->bottom=pWinRect->top+380;


    GetClientWindowRect(mhWndDialog,&RectWindow);

    // IDC_STATIC_MONITORING
    hItem=GetDlgItem(mhWndDialog,IDC_STATIC_MONITORING);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,0,0,
        (RectWindow.right-RectWindow.left)/2-Rect.left-SPACE_BETWEEN_CONTROLS,Rect.bottom-Rect.top,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOMOVE);
    GetClientWindowRect(hItem,&RectStaticMonitoring);

    // IDC_BUTTON_BROWSE_MONITORING_FILE
    hItem=GetDlgItem(mhWndDialog,IDC_BUTTON_BROWSE_MONITORING_FILE);
    GetClientWindowRect(hItem,&Rect);
    // use IDC_STATIC_MONITORING rect
    SetWindowPos(hItem,HWND_NOTOPMOST,
        RectStaticMonitoring.right-SPACE_BETWEEN_CONTROLS-(Rect.right-Rect.left),Rect.top,
        0,0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);
    GetClientWindowRect(hItem,&RectStaticBrowseMonitoring);

    // IDC_EDIT_MONITORING_FILE
    hItem=GetDlgItem(mhWndDialog,IDC_EDIT_MONITORING_FILE);
    GetClientWindowRect(hItem,&Rect);
    // use IDC_STATIC_MONITORING && IDC_BUTTON_BROWSE_MONITORING_FILE rect
    SetWindowPos(hItem,HWND_NOTOPMOST,
        RectStaticMonitoring.left+SPACE_BETWEEN_CONTROLS,Rect.top,
        RectStaticMonitoring.right-RectStaticMonitoring.left-3*SPACE_BETWEEN_CONTROLS-(RectStaticBrowseMonitoring.right-RectStaticBrowseMonitoring.left),Rect.bottom-Rect.top,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER);
    GetClientWindowRect(hItem,&RectStaticEditMonitoring);

    // IDC_BUTTON_LOAD_MONITORING_FILE
    hItem=GetDlgItem(mhWndDialog,IDC_BUTTON_LOAD_MONITORING_FILE);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,
        RectStaticMonitoring.right-SPACE_BETWEEN_CONTROLS-(Rect.right-Rect.left),
        RectStaticBrowseMonitoring.bottom+SPACE_BETWEEN_CONTROLS,
        0,0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);
    GetClientWindowRect(hItem,&RectStaticLoadMonitoring);

    // IDC_BUTTON_UNLOAD_MONITORING_FILE
    hItem=GetDlgItem(mhWndDialog,IDC_BUTTON_UNLOAD_MONITORING_FILE);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,
        RectStaticLoadMonitoring.left,RectStaticLoadMonitoring.bottom+SPACE_BETWEEN_CONTROLS,
        0,0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);

    // IDC_LIST_MONITORING_FILES
    hItem=GetDlgItem(mhWndDialog,IDC_LIST_MONITORING_FILES);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,
        RectStaticMonitoring.left+SPACE_BETWEEN_CONTROLS,
        RectStaticBrowseMonitoring.bottom+SPACE_BETWEEN_CONTROLS,
        RectStaticMonitoring.right-RectStaticMonitoring.left-3*SPACE_BETWEEN_CONTROLS-(RectStaticLoadMonitoring.right-RectStaticLoadMonitoring.left),
        Rect.bottom-Rect.top,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER);

    // IDC_STATIC_FAKING
    hItem=GetDlgItem(mhWndDialog,IDC_STATIC_FAKING);
    GetClientWindowRect(hItem,&Rect);
    // use IDC_STATIC_MONITORING rect
    SetWindowPos(hItem,HWND_NOTOPMOST,RectStaticMonitoring.right+SPACE_BETWEEN_CONTROLS,RectStaticMonitoring.top,
        RectStaticMonitoring.right-RectStaticMonitoring.left,RectStaticMonitoring.bottom-RectStaticMonitoring.top,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER);
    GetClientWindowRect(hItem,&RectStaticFaking);

    // IDC_EDIT_FAKING_FILE
    hItem=GetDlgItem(mhWndDialog,IDC_EDIT_FAKING_FILE);
    // use IDC_STATIC_FAKING rect
    SetWindowPos(hItem,HWND_NOTOPMOST,RectStaticFaking.left+SPACE_BETWEEN_CONTROLS,RectStaticEditMonitoring.top,
        RectStaticEditMonitoring.right-RectStaticEditMonitoring.left,RectStaticEditMonitoring.bottom-RectStaticEditMonitoring.top,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER);
    GetClientWindowRect(hItem,&RectStaticEditFaking);

    // IDC_BUTTON_BROWSE_FAKING_FILE
    hItem=GetDlgItem(mhWndDialog,IDC_BUTTON_BROWSE_FAKING_FILE);
    // use IDC_EDIT_FAKING_FILE rect
    SetWindowPos(hItem,HWND_NOTOPMOST,
        RectStaticEditFaking.right+SPACE_BETWEEN_CONTROLS,RectStaticEditFaking.top,
        0,0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);


    // IDC_BUTTON_LOAD_FAKING_FILE
    hItem=GetDlgItem(mhWndDialog,IDC_BUTTON_LOAD_FAKING_FILE);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,
        RectStaticFaking.right-SPACE_BETWEEN_CONTROLS-(Rect.right-Rect.left),
        RectStaticBrowseMonitoring.bottom+SPACE_BETWEEN_CONTROLS,
        0,0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);
    GetClientWindowRect(hItem,&RectStaticLoadFaking);

    // IDC_BUTTON_UNLOAD_FAKING_FILE
    hItem=GetDlgItem(mhWndDialog,IDC_BUTTON_UNLOAD_FAKING_FILE);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,
        RectStaticLoadFaking.left,
        RectStaticLoadFaking.bottom+SPACE_BETWEEN_CONTROLS,
        0,0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);

    // IDC_LIST_FAKING_FILES
    hItem=GetDlgItem(mhWndDialog,IDC_LIST_FAKING_FILES);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,
        RectStaticFaking.left+SPACE_BETWEEN_CONTROLS,
        RectStaticBrowseMonitoring.bottom+SPACE_BETWEEN_CONTROLS,
        RectStaticFaking.right-RectStaticFaking.left-3*SPACE_BETWEEN_CONTROLS-(RectStaticLoadFaking.right-RectStaticLoadFaking.left),
        Rect.bottom-Rect.top,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER);



    // IDC_LISTLOGMONITORING
    hItem=GetDlgItem(mhWndDialog,IDC_LISTLOGMONITORING);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,0,0,
        RectWindow.right-RectWindow.left-Rect.left-3*SPACE_BETWEEN_CONTROLS,RectWindow.bottom-Rect.top-2*SPACE_BETWEEN_CONTROLS,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOMOVE);

    // IDC_BUTTON_START_STOP_HOOKS
    hItem=GetDlgItem(mhWndDialog,IDC_BUTTON_START_STOP_HOOKS);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,
        RectWindow.right-(Rect.right-Rect.left)-2*SPACE_BETWEEN_CONTROLS,Rect.top,
        0,0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);
    GetClientWindowRect(hItem,&RectStaticStart);

    // IDC_BUTTON_ABOUT
    hItem=GetDlgItem(mhWndDialog,IDC_BUTTON_ABOUT);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,
        RectStaticStart.left,Rect.top,
        0,0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);

    // IDC_BUTTON_EXIT
    hItem=GetDlgItem(mhWndDialog,IDC_BUTTON_EXIT);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,
        RectStaticStart.left,Rect.top,
        0,0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);

    // IDC_STATICPIDSELECT
    hItem=GetDlgItem(mhWndDialog,IDC_STATICPIDSELECT);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,
        0,0,
        RectStaticStart.left-SPACE_BETWEEN_CONTROLS-Rect.left,
        Rect.bottom-Rect.top,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOMOVE);
    GetClientWindowRect(hItem,&PreviousRect);

    // IDC_EDIT_CMD_LINE
    hItem=GetDlgItem(mhWndDialog,IDC_EDIT_CMD_LINE);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,
        PreviousRect.right-SPACE_BETWEEN_CONTROLS-(PreviousRect.right-PreviousRect.left)/4,Rect.top,
        (PreviousRect.right-PreviousRect.left)/4,
        Rect.bottom-Rect.top,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER);
    GetClientWindowRect(hItem,&PreviousRect);

    // IDC_STATIC_COMMAND_LINE
    hItem=GetDlgItem(mhWndDialog,IDC_STATIC_COMMAND_LINE);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,
        PreviousRect.left-SPACE_BETWEEN_CONTROLS-(Rect.right-Rect.left),Rect.top,
        0,
        0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);
    GetClientWindowRect(hItem,&PreviousRect);

    // IDC_BUTTON_BROWSE_APP_PATH
    hItem=GetDlgItem(mhWndDialog,IDC_BUTTON_BROWSE_APP_PATH);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,
        PreviousRect.left-SPACE_BETWEEN_CONTROLS-(Rect.right-Rect.left),Rect.top,
        0,
        0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);
    GetClientWindowRect(hItem,&PreviousRect);

    // IDC_EDIT_APP_PATH
    hItem=GetDlgItem(mhWndDialog,IDC_EDIT_APP_PATH);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,0,0,
        PreviousRect.left-SPACE_BETWEEN_CONTROLS-Rect.left,Rect.bottom-Rect.top,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOMOVE);

    // IDC_BUTTON_CLEAR_MONITORING_LISTVIEW
    hItem=GetDlgItem(mhWndDialog,IDC_BUTTON_CLEAR_MONITORING_LISTVIEW);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,
        RectStaticFaking.left+SPACE_BETWEEN_CONTROLS,
        Rect.top,
        0,
        0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);
    GetClientWindowRect(hItem,&PreviousRect);
    RedrawWindow( hItem,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE);

    // IDC_BUTTON_SAVE_LOGS
    hItem=GetDlgItem(mhWndDialog,IDC_BUTTON_SAVE_LOGS);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,
        PreviousRect.right+SPACE_BETWEEN_CONTROLS,
        Rect.top,
        0,
        0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);
    GetClientWindowRect(hItem,&PreviousRect);
    RedrawWindow( hItem,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE);

    // IDC_BUTTON_LOAD_LOGS
    hItem=GetDlgItem(mhWndDialog,IDC_BUTTON_LOAD_LOGS);
    GetClientWindowRect(hItem,&Rect);
    SetWindowPos(hItem,HWND_NOTOPMOST,
        PreviousRect.right+SPACE_BETWEEN_CONTROLS,
        Rect.top,
        0,
        0,
        SWP_ASYNCWINDOWPOS|SWP_NOACTIVATE|SWP_NOZORDER|SWP_NOSIZE);
    RedrawWindow( hItem,NULL,NULL,RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE);
}