/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

//-----------------------------------------------------------------------------
// Object: manages the filters dialog
//-----------------------------------------------------------------------------

#include "filters.h"

CFilters* pFiltersWndProc=NULL;

CFilters::CFilters()
{
    this->hWndFiltersDialog=NULL;

    this->pdwFiltersCurrentPorcessID=NULL;
    this->pdwFiltersParentPorcessID=NULL;
    this->pszFiltersInclusion=NULL;
    this->pszFiltersExclusion=NULL;
    this->pdwFiltersNewPorcessIDToWatch=NULL;
    this->pdwFiltersPorcessIDToRelease=NULL;

    this->pdwFiltersCurrentPorcessIDSize=0;
    this->pdwFiltersParentPorcessIDSize=0;
    this->pszFiltersInclusionSize=0;
    this->pszFiltersExclusionSize=0;
    this->pdwFiltersNewPorcessIDToWatchSize=0;
    this->pdwFiltersPorcessIDToReleaseSize=0;

    *this->AppNameIncludeFilters=0;
    *this->AppNameExcludeFilters=0;
    *this->AppParentIDFilters=0;
}
CFilters::~CFilters()
{
    this->FreeMemory();
}

//-----------------------------------------------------------------------------
// Name: FreeMemory
// Object: Free memory allocated by the class
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CFilters::FreeMemory()
{
    // free pdwFiltersCurrentPorcessID
    if (this->pdwFiltersCurrentPorcessID)
    {
        delete[] this->pdwFiltersCurrentPorcessID;
        this->pdwFiltersCurrentPorcessID=NULL;
    }

    // free pdwFiltersParentPorcessID
    if (this->pdwFiltersParentPorcessID)
    {
        delete[] this->pdwFiltersParentPorcessID;
        this->pdwFiltersParentPorcessID=NULL;
    }

    // free pdwFiltersNewPorcessIDToWatch
    if (this->pdwFiltersNewPorcessIDToWatch)
    {
        delete[] this->pdwFiltersNewPorcessIDToWatch;
        this->pdwFiltersNewPorcessIDToWatch=NULL;
    }

    // free pdwFiltersPorcessIDToRelease
    if (this->pdwFiltersPorcessIDToRelease)
    {
        delete[] this->pdwFiltersPorcessIDToRelease;
        this->pdwFiltersPorcessIDToRelease=NULL;
    }

    // free pszFiltersInclusion and its content
    if (this->pszFiltersInclusion)
    {
        CMultipleElementsParsing::ParseStringArrayFree(this->pszFiltersInclusion,pszFiltersInclusionSize);
        this->pszFiltersInclusion=NULL;
    }

    // free pszFiltersExclusion and its content
    if (this->pszFiltersExclusion)
    {
        CMultipleElementsParsing::ParseStringArrayFree(this->pszFiltersExclusion,pszFiltersExclusionSize);
        this->pszFiltersExclusion=NULL;
    }

    this->pdwFiltersCurrentPorcessIDSize=0;
    this->pdwFiltersParentPorcessIDSize=0;
    this->pszFiltersInclusionSize=0;
    this->pszFiltersExclusionSize=0;
}

//-----------------------------------------------------------------------------
// Name: ReloadOptions
// Object: reload previously saved options
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CFilters::ReloadOptions()
{
    // fill inclusion filter field
    SetDlgItemText(this->hWndFiltersDialog,IDC_EDIT_INCLUSION_FILTERS,this->AppNameIncludeFilters);

    // fill exclusion filter field
    SetDlgItemText(this->hWndFiltersDialog,IDC_EDIT_EXCLUSION_FILTERS,this->AppNameExcludeFilters);

    // fill parent filter field
    SetDlgItemText(this->hWndFiltersDialog,IDC_EDIT_PARENT_PID_FILTERS,this->AppParentIDFilters);

    // select previous defined process
    this->ReSelectProcess();
}


//-----------------------------------------------------------------------------
// Name: ReSelectProcess
// Object: Reselect process number saved in options in listbox.
//         Only process still running are reselected
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CFilters::ReSelectProcess()
{
    if (this->pdwFiltersCurrentPorcessIDSize==0)
        return;
    DWORD dwNbItems;
    DWORD dwCnt;
    DWORD dwCnt2;
    DWORD dw;
    TCHAR ItemText[MAX_PATH];
    TCHAR ProcName[MAX_PATH];
    int iSscanfNbRes;


    // remove no more alive processes from this->pdwFiltersCurrentPorcessID
    for (dwCnt=0;dwCnt<this->pdwFiltersCurrentPorcessIDSize;dwCnt++)
    {
        // if process is no more alive
        if (!CProcessHelper::IsAlive(this->pdwFiltersCurrentPorcessID[dwCnt]))
        {
            // if last elment
            if (dwCnt==(this->pdwFiltersCurrentPorcessIDSize-1))
            {
                // just decrease array size and break
                this->pdwFiltersCurrentPorcessIDSize--;
                break;
            }
            // else
            // switch with last element
            this->pdwFiltersCurrentPorcessID[dwCnt]=this->pdwFiltersCurrentPorcessID[this->pdwFiltersCurrentPorcessIDSize-1];
            // decrease number of elements
            this->pdwFiltersCurrentPorcessIDSize--;
            // decrease counter to check the new element at pos dwCnt
            dwCnt--;
        }
    }


    // get number of items in listbox
    dwNbItems=(DWORD)SendMessage(this->hwndFilterDialogListBox,LB_GETCOUNT,0,0);
    if ((dwNbItems!=LB_ERR)&&(dwNbItems>0))
    {
        // for each item in listbox
        for (dwCnt=0;dwCnt<dwNbItems;dwCnt++)
        {
            // retreive text (we don't send the LB_GETTEXTLEN message as we have filled listbox with
            // MAX_PATH length buffer
            if (SendMessage(this->hwndFilterDialogListBox,LB_GETTEXT,dwCnt,(LPARAM)ItemText)!=LB_ERR)
            {
                // text is like "ProcName (0xPid)"
                // retreive process ID
                dw=0;
                iSscanfNbRes=_stscanf(ItemText,_T("%s (0x%x)"),ProcName,&dw);
                if ((iSscanfNbRes!=2)||(dw==0))
                    continue;

                // search Process ID in current process array
                for(dwCnt2=0;dwCnt2<this->pdwFiltersCurrentPorcessIDSize;dwCnt2++)
                {
                    // if we found it
                    if (dw==this->pdwFiltersCurrentPorcessID[dwCnt2])
                        // select item in listbox
                        SendMessage(this->hwndFilterDialogListBox,(UINT) LB_SETSEL,TRUE,dwCnt);
                }
            }
        }
    }
}

//-----------------------------------------------------------------------------
// Name: StoreOptions
// Object: Store options for a next filter dialog display.
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CFilters::StoreOptions()
{
    // save inclusion filter field data
    GetDlgItemText(this->hWndFiltersDialog,IDC_EDIT_INCLUSION_FILTERS,this->AppNameIncludeFilters,MAX_PATH);

    // save exclusion filter field data
    GetDlgItemText(this->hWndFiltersDialog,IDC_EDIT_EXCLUSION_FILTERS,this->AppNameExcludeFilters,MAX_PATH);

    // save parent ID filter field data
    GetDlgItemText(this->hWndFiltersDialog,IDC_EDIT_PARENT_PID_FILTERS,this->AppParentIDFilters,MAX_PATH);
}

//-----------------------------------------------------------------------------
// Name: ShowFilterDialog
// Object: show the filter dialog box
// Parameters :
//     in  : HINSTANCE hInstance : application instance
//           HWND hWndDialog : main window dialog handle
//     out :
//     return : 
//-----------------------------------------------------------------------------
INT_PTR CFilters::ShowFilterDialog(HINSTANCE hInstance,HWND hWndDialog)
{
    INT_PTR iRet;
    // only allow one instance of FiltersDialog for all threads
    if (pFiltersWndProc)
        return FILTERS_DLG_RES_FAILED;
    if (this->hWndFiltersDialog)
        return FILTERS_DLG_RES_FAILED;

    // show dialog
    iRet=DialogBoxParam(hInstance,(LPCTSTR)IDD_DIALOGAPPLICATIONFILTERS,hWndDialog,(DLGPROC)FiltersWndProc,(LPARAM)this);
    this->hWndFiltersDialog=NULL;
    pFiltersWndProc=NULL;
    return iRet;
}

//-----------------------------------------------------------------------------
// Name: SaveFiltersSettings
// Object: Save and apply settings
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CFilters::SaveFiltersSettings()
{
    DWORD dw;
    DWORD dwNbSelectedItems;
    DWORD dwNbItems;
    DWORD dwCnt;
    DWORD dwCnt2;
    DWORD dwOldProcIDSize;
    BOOL  bFound;
    LRESULT lResult;
    TCHAR ItemText[MAX_PATH];
    TCHAR ProcName[MAX_PATH];
    int iSscanfNbRes;
    DWORD* pdwOldProcID=NULL;

    // temporary store old process array
    dwOldProcIDSize=this->pdwFiltersCurrentPorcessIDSize;
    if (dwOldProcIDSize>0)
    {
        pdwOldProcID=new DWORD[dwOldProcIDSize];
        for (dwCnt=0;dwCnt<dwOldProcIDSize;dwCnt++)
            pdwOldProcID[dwCnt]=this->pdwFiltersCurrentPorcessID[dwCnt];
    }

    // free memory
    this->FreeMemory();

    // store data fields, to restore them at the next display of filter window 
    this->StoreOptions();

    ///////////////////////
    // retreive current selected processes
    ///////////////////////

    // get number of selected items in the listbox
    dwNbSelectedItems=(DWORD)SendMessage(this->hwndFilterDialogListBox,LB_GETSELCOUNT,0,0);
    // get number of items in the listbox
    dwNbItems=(DWORD)SendMessage(this->hwndFilterDialogListBox,LB_GETCOUNT,0,0);
    if ((dwNbSelectedItems!=LB_ERR)&&(dwNbSelectedItems>0)
        &&(dwNbItems!=LB_ERR)&&(dwNbItems>0))
    {
        this->pdwFiltersCurrentPorcessID=new DWORD[dwNbSelectedItems];

        // for each listbox item
        for (dwCnt=0;dwCnt<dwNbItems;dwCnt++)
        {
            // check selected state
            lResult=SendMessage(this->hwndFilterDialogListBox,LB_GETSEL,dwCnt,0);
            // if not selected
            if ((lResult==LB_ERR)||(lResult==0))
                // check next item
                continue;
            // selected
            
            // retreive text (we don't send the LB_GETTEXTLEN message as we have filled listbox with
            // MAX_PATH length buffer
            if (SendMessage(this->hwndFilterDialogListBox,LB_GETTEXT,dwCnt,(LPARAM)ItemText)!=LB_ERR)
            {
                // text is like "ProcName (0xPid)"
                // retreive Process ID
                dw=0;
                iSscanfNbRes=_stscanf(ItemText,_T("%s (0x%x)"),ProcName,&dw);
                if ((iSscanfNbRes!=2)||(dw==0))
                    continue;

                // store Process ID in pdwFiltersCurrentPorcessID array
                this->pdwFiltersCurrentPorcessID[this->pdwFiltersCurrentPorcessIDSize]=dw;
                this->pdwFiltersCurrentPorcessIDSize++;
            }
        }
    }

    //////////////////////////////////////
    // get parent ID filters
    //////////////////////////////////////
    if (GetDlgItemText(this->hWndFiltersDialog,IDC_EDIT_PARENT_PID_FILTERS,ItemText,MAX_PATH))
        this->pdwFiltersParentPorcessID=CMultipleElementsParsing::ParseDWORD(ItemText,&this->pdwFiltersParentPorcessIDSize);

    //////////////////////////////////////
    // get inclusion filters
    //////////////////////////////////////
    if (GetDlgItemText(this->hWndFiltersDialog,IDC_EDIT_INCLUSION_FILTERS,ItemText,MAX_PATH))
        this->pszFiltersInclusion=CMultipleElementsParsing::ParseString(ItemText,&this->pszFiltersInclusionSize);

    //////////////////////////////////////
    // get exclusion filters
    //////////////////////////////////////
    if (GetDlgItemText(this->hWndFiltersDialog,IDC_EDIT_EXCLUSION_FILTERS,ItemText,MAX_PATH))
        this->pszFiltersExclusion=CMultipleElementsParsing::ParseString(ItemText,&this->pszFiltersExclusionSize);

    CLinkList* pLinkList=new CLinkList(sizeof(DWORD));
    /////////////////////////////////////////////////////
    // get new process to watch (process ID present in this->pdwFiltersCurrentPorcessID 
    // but not in pdwOldProcID)
    /////////////////////////////////////////////////////
    // for each process in this->pdwFiltersCurrentPorcessID
    for (dwCnt=0;dwCnt<this->pdwFiltersCurrentPorcessIDSize;dwCnt++)
    {
        // search process number in old process array
        bFound=FALSE;
        for (dwCnt2=0;dwCnt2<dwOldProcIDSize;dwCnt2++)
        {
            // if item is found, process is already monitored
            if (this->pdwFiltersCurrentPorcessID[dwCnt]==pdwOldProcID[dwCnt2])
            {
                bFound=TRUE;
                break;
            }
        }
        // if not found the item have to be monitored
        if (!bFound)
            pLinkList->AddItem(&this->pdwFiltersCurrentPorcessID[dwCnt]);
    }
    // update pdwFiltersNewPorcessIDToWatch and pdwFiltersNewPorcessIDToWatchSize
    this->pdwFiltersNewPorcessIDToWatch=(DWORD*)pLinkList->ToSecureArray(&this->pdwFiltersNewPorcessIDToWatchSize);


    // remove items of the linked list (avoid to destroy and create a new one)
    pLinkList->RemoveAllItems();

    /////////////////////////////////////////////////////
    // get process to detach (process ID present in pdwOldProcID 
    // but not in this->pdwFiltersCurrentPorcessID and still alive)
    /////////////////////////////////////////////////////
    // for each process in pdwOldProcID
    for (dwCnt=0;dwCnt<dwOldProcIDSize;dwCnt++)
    {
        // search process number in pdwFiltersCurrentPorcessID array
        bFound=FALSE;
        for (dwCnt2=0;dwCnt2<this->pdwFiltersCurrentPorcessIDSize;dwCnt2++)
        {
            // if process ID is found we should still monitor process
            if (pdwOldProcID[dwCnt]==this->pdwFiltersCurrentPorcessID[dwCnt2])
            {
                bFound=TRUE;
                break;
            }
        }
        // if not found the item have to be released
        if (!bFound)
            pLinkList->AddItem(&pdwOldProcID[dwCnt]);
    }
    // update pdwFiltersNewPorcessIDToWatch and pdwFiltersNewPorcessIDToWatchSize
    this->pdwFiltersPorcessIDToRelease=(DWORD*)pLinkList->ToSecureArray(&this->pdwFiltersPorcessIDToReleaseSize);

    // free memory
    delete pLinkList;

    if (dwOldProcIDSize>0)
        delete[] pdwOldProcID;
}

//-----------------------------------------------------------------------------
// Name: DoesProcessNameMatchFilters
// Object: Check if process name matchs given filters
// Parameters :
//     in  : TCHAR* ProcessName : name of process
//     out :
//     return : TRUE if ProcessName matchs filters, else returns FALSE
//-----------------------------------------------------------------------------
BOOL CFilters::DoesProcessNameMatchFilters(TCHAR* ProcessName)
{
    DWORD dwCnt;

    BOOL bMatchInclude;
    if (this->pszFiltersInclusionSize!=0)
    {
        bMatchInclude=FALSE;
        // check include filters
        for (dwCnt=0;dwCnt<this->pszFiltersInclusionSize;dwCnt++)
        {
            // check if name doesn't match
            if (CWildCharCompare::WildICmp(this->pszFiltersInclusion[dwCnt],ProcessName))
                bMatchInclude=TRUE;
        }
        // if no one include filter matchs return
        if (!bMatchInclude)
            return FALSE;
    }

    // check exclude filters
    for (dwCnt=0;dwCnt<this->pszFiltersExclusionSize;dwCnt++)
    {
        // check if name matchs
        if (CWildCharCompare::WildICmp(this->pszFiltersExclusion[dwCnt],ProcessName))
            return FALSE;
    }

    // file names match inclusion and exclusion filters
    return TRUE;
}

//-----------------------------------------------------------------------------
// Name: DoesParentIDMatchFilters
// Object: Check if process parent ID matchs given filters
// Parameters :
//     in  : DWORD dwParentID : parent process ID
//     out :
//     return : TRUE if parent ID matchs filters, else returns FALSE
//-----------------------------------------------------------------------------
BOOL CFilters::DoesParentIDMatchFilters(DWORD dwParentID)
{
    // if no filters set
    if (this->pdwFiltersParentPorcessIDSize==0)
        return TRUE;

    DWORD dwCnt;
    // loop throw pdwFiltersParentPorcessID array
    for(dwCnt=0;dwCnt<this->pdwFiltersParentPorcessIDSize;dwCnt++)
    {
        // if item found
        if (this->pdwFiltersParentPorcessID[dwCnt]==dwParentID)
            return TRUE;
    }
    // item not found
    return FALSE;
}

//-----------------------------------------------------------------------------
// Name: RefreshProcessesList
// Object: refresh the list of current running processes
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CFilters::RefreshProcessesList()
{
    DWORD aProcesses[FILTERS_NB_MAX_PROCESSES];
    HMODULE hMod;
    DWORD cbNeeded;
    HANDLE hProcess;
    DWORD cProcesses;
    DWORD i;
    TCHAR pszProcessName[MAX_PATH] = _T("Unknown");
    TCHAR pszProcess[MAX_PATH];
    DWORD processID;

    // clear listbox
    SendMessage(this->hwndFilterDialogListBox,LB_RESETCONTENT,0,0);

    // Get the list of process identifiers.
    if (EnumProcesses( aProcesses, sizeof(aProcesses), &cbNeeded ))
    {
        // Calculate how many process identifiers were returned.
        cProcesses = cbNeeded / sizeof(DWORD);

        // get process name
        for ( i = 0; i < cProcesses; i++ )
        {
            processID=aProcesses[i];

            // check special case of the "Inactive" process and the "System" process
            if ((processID==FILTERS_INACTIVE_PROCESSES_PID)||(processID==FILTERS_SYSTEM_PID))
                continue;

            // Get a handle to the process.
            hProcess = OpenProcess( PROCESS_QUERY_INFORMATION |
                                        PROCESS_VM_READ,
                                        FALSE, processID );
            // Get the process name.
            if (hProcess)
            {
                // we just need first module handle (the application one)
                // so we don't send an array of HMODULE, but only address of hMod
                if (EnumProcessModules( hProcess, &hMod, sizeof(hMod),&cbNeeded))
                {
                    // retreive module base name
                    GetModuleBaseName(hProcess,hMod,pszProcessName,MAX_PATH);
                    // append PID to the process name
                    _sntprintf(pszProcess,MAX_PATH,_T("%s (0x%x)"),pszProcessName,processID);
                    // add process to the list box
                    CListbox::AddStringWithHScrollUpdate(this->hwndFilterDialogListBox,pszProcess);
                }
                // close process handle
                CloseHandle( hProcess );
            }
        }
    }
}

//-----------------------------------------------------------------------------
// Name: FiltersWndProc
// Object: dialog callback of the Filters dialog
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CALLBACK FiltersWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    DWORD dw;
    switch (uMsg)
    {
    case WM_INITDIALOG:
        pFiltersWndProc=(CFilters*)lParam;
        pFiltersWndProc->hWndFiltersDialog=hWnd;
        pFiltersWndProc->hwndFilterDialogListBox=GetDlgItem(hWnd,IDC_LIST_RUNNING_APP_FILTER);
        // load dlg icons
        CDialogHelper::SetIcon(hWnd,IDI_ICON_FILTERS);

        // update process list
        pFiltersWndProc->RefreshProcessesList();

        // reload old options if any
        pFiltersWndProc->ReloadOptions();

        break;
    case WM_CLOSE:
        EndDialog(hWnd,FILTERS_DLG_RES_CANCEL);
        break;
    case WM_COMMAND:
        switch (LOWORD(wParam))
        {
        case IDOK:
            // save filters settings
            pFiltersWndProc->SaveFiltersSettings();
            // exit
            EndDialog(hWnd,FILTERS_DLG_RES_OK);
            break;
        case IDCANCEL:
            EndDialog(hWnd,FILTERS_DLG_RES_CANCEL);
            break;
        case IDC_BUTTON_REFRESH_PROCESSES_LIST:
            // update process list
            pFiltersWndProc->RefreshProcessesList();

            // select process again
            pFiltersWndProc->ReSelectProcess();
            break;
        case IDC_BUTTON_FILTERS_SELECT_ALL:
            dw=(DWORD)SendMessage(pFiltersWndProc->hwndFilterDialogListBox,LB_GETCOUNT,0,0);
            if (dw==LB_ERR)
                break;
            // LB_SELITEMRANGE lParam : The low-order word specifies the zero-based index 
            // of the first item to select. The high-order word specifies the zero-based index 
            // of the last item to select.
            dw=dw<<16;// as the first index is 0
            SendMessage(pFiltersWndProc->hwndFilterDialogListBox,LB_SELITEMRANGE,TRUE,dw);

            break;
        case IDC_BUTTON_FILTERS_UNSELECT_ALL:
            dw=(DWORD)SendMessage(pFiltersWndProc->hwndFilterDialogListBox,LB_GETCOUNT,0,0);
            if (dw==LB_ERR)
                break;
            // LB_SELITEMRANGE lParam : The low-order word specifies the zero-based index 
            // of the first item to select. The high-order word specifies the zero-based index 
            // of the last item to select.
            dw=dw<<16;// as the first index is 0
            SendMessage(pFiltersWndProc->hwndFilterDialogListBox,LB_SELITEMRANGE,FALSE,dw);
            break;
        }
        break;
    }
    return FALSE;
}