/*
Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>
Dynamic aspect ratio code Copyright (C) 2004 Jacquelin POTIER <jacquelin.potier@free.fr>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include <windows.h>
#include <stdio.h>
#pragma warning (push)
#pragma warning(disable : 4005)// for '_stprintf' : macro redefinition in tchar.h
#include <TCHAR.h>
#pragma warning (pop)
#include "resource.h"
#include "about.h"
#include "filters.h"
#include "../tools/string/AnsiUnicodeConvert.h"
#include "../tools/Process/APIOverride/ApiOverride.h"
#include "../tools/Process/ProcessHelper/ProcessHelper.h"
#include "../tools/GUI/SelectWindow/selectwindow.h"
#include "../tools/GUI/ListBox/Listbox.h"
#include "../tools/GUI/ListView/ListView.h"
#include "../tools/GUI/SysLink/SysLink.h"
#include "../Tools/GUI/Dialog/DialogHelper.h"
#include "../Tools/Process/ProcessMonitor/ProcMonInterface.h"
#include "../Tools/LinkList/LinkList.h"

/////////////////////////////////////////
// Defines
/////////////////////////////////////////
// First listview column text for report messages
#define LISTVIEW_ITEM_TEXT_INFORMATION  _T("I")
#define LISTVIEW_ITEM_TEXT_ERROR        _T("E")
#define LISTVIEW_ITEM_TEXT_WARNING      _T("W")
#define LISTVIEW_ITEM_TEXT_MESSAGE_MAX_SIZE 2 // max size of all LISTVIEW_ITEM_TEXT_ ... messages (including \0)

// types of report messages
enum tagMsgTypes
{
    MSG_INFORMATION,
    MSG_WARNING,
    MSG_ERROR
};
/////////////////////////////////////////
// vars
/////////////////////////////////////////
CLinkList* pApiOverrideList=NULL; // linked list containing all used CAPIOverride objects
CSelectWindow* pSelectWindow=NULL;// Class to graphicaly select an other window and retreive information about selected window
CListview* pListview=NULL;// class helper for listview
CProcMonInterface* pProcMonInterface=NULL;// class helper for using ProcMon driver (driver used for signaling process creation and ends)
CFilters* pFilters=NULL;// class for the filters dialog

HINSTANCE mhInstance;// app instance
HWND mhWndDialog;// main dialog handle
LPSTR mlpCmdLine;// app command line
BOOL bMonitoring=TRUE;// flag to know if APIoverride is monitoring
BOOL bFaking=TRUE;// flag to know if APIoverride is faking
BOOL bStarted=FALSE;// flag to know if APIoverride is started
BOOL bExitingStop=FALSE;// flag to know user has asked to stop application and we are currently stopping APIoverride
BOOL bExit=FALSE;// flag to know that app has asked to exit

WNDPROC wndOriginalListviewDWL_MSGRESULT=NULL;// contains original DWL_MSGRESULT of Listview with a view to retore it
COLORREF ListViewColorInformation=RGB(0,128,0);// listview item color of information reports
COLORREF ListViewColorError=RGB(255,0,0);// listview item color of error reports
COLORREF ListViewColorWarning=RGB(250,190,16);// listview item color of warning reports
COLORREF ListViewColorMessageBackGround RGB(247,247,247);// listview item background color of reports

/////////////////////////////////////////
// functions
/////////////////////////////////////////
LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
LRESULT CALLBACK FiltersWndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
void CallBackProcessCreation(HANDLE hParentId,HANDLE hProcessId);
void BrowseApplicationPath();
void BrowseMonitoringFile();
void BrowseFakingFile();
void StartStop();
DWORD WINAPI Stop(LPVOID lpParameter);
void StartStopMonitoring();
void StartStopFaking();
void LoadMonitoringFile();
void UnloadMonitoringFile();
DWORD WINAPI UnloadMonitoringFile(LPVOID lpParameter);
void LoadFakeAPI();
void UnloadFakeAPI();
DWORD WINAPI UnloadFakeAPI(LPVOID lpParameter);
void CallBackBeforeAppResume(DWORD dwProcessID,PVOID pUserParam);
void CallBackUnexpectedUnload(DWORD dwProcessID,PVOID pUserParam);
void EnableMonitoringFakingInterface(BOOL bEnable);
void ClearLogs();
void LoadLogs();
void SaveLogs();
void About();
void Init();
DWORD WINAPI ParseCommandLine(LPVOID lpParameter);
void Exit(INT_PTR nExitCode);
void Resize();
void Resize(RECT* pWinRect);
void OnMouseDown(LPARAM lParam);
void OnMouseUp(LPARAM lParam);
void OnMouseMove(LPARAM lParam);
void MakePrettyName(TCHAR* pszFileName);
void DecodePrettyName(TCHAR* pszFileName);
void GetRemoteWindowInfos();
void SetApplicationsFilters();
LRESULT ProcessCustomDraw (LPARAM lParam);
void ReportMessage(TCHAR* pszMsg, tagMsgTypes MsgType);

//-----------------------------------------------------------------------------
// Name: WinMain
// Object: Entry point of app
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
int WINAPI WinMain(HINSTANCE hInstance,
    HINSTANCE hPrevInstance,
    LPSTR lpCmdLine,
    int nCmdShow
)
{
    UNREFERENCED_PARAMETER(hPrevInstance);
    UNREFERENCED_PARAMETER(nCmdShow);

    mhInstance=hInstance;
    mlpCmdLine=lpCmdLine;

    // enable Xp style
    InitCommonControls();

    // create a list for CApiOverride objects
    pApiOverrideList=new CLinkList(sizeof(CApiOverride*));

    // create ProcMon.sys driver interface
    pProcMonInterface=new CProcMonInterface();

    // create CSelectWindow object 
    pSelectWindow=new CSelectWindow();
    pSelectWindow->bSelectOnlyDialog=TRUE;
    pSelectWindow->bOwnerWindowSelectable=TRUE;

    // create a CFilters object
    pFilters=new CFilters();

    // show main window
    DialogBox(hInstance, (LPCTSTR)IDD_DIALOGMAIN, NULL, (DLGPROC)WndProc);

    // delete pFilters
    delete pFilters;

    // delete ProcMon.sys driver interface
    delete pProcMonInterface;

    // delete pApioverrideList
    delete pApiOverrideList;

    // delete CSelectWindow object
    delete pSelectWindow;

    return 0;
}

//-----------------------------------------------------------------------------
// Name: WndProc
// Object: Main dialog callback
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    LPNMLISTVIEW pnm;
    switch (uMsg)
    {
    case WM_INITDIALOG:
        mhWndDialog=hWnd;
        Init();
        break;
    case WM_CLOSE:
        Exit(LOWORD(wParam));
        break;
    case WM_SIZING:
        Resize((RECT*)lParam);
        break;
    case WM_SIZE:
        switch (wParam)
        {
        case SIZE_MAXIMIZED:
        case SIZE_RESTORED:
            Resize();
            break;
        }
        break;

    case WM_NOTIFY:
        pnm = (LPNMLISTVIEW)lParam;
        //we are only interested in NM_CUSTOMDRAW notifications for ListView
        if (pListview)
        {
            if (pnm->hdr.hwndFrom == pListview->GetControlHandle() &&
                pnm->hdr.code == NM_CUSTOMDRAW)
            {
                //must use SetWindowLong to return value from dialog proc
                wndOriginalListviewDWL_MSGRESULT=(WNDPROC)SetWindowLong(hWnd, DWL_MSGRESULT, (LONG)ProcessCustomDraw(lParam));
                return TRUE;
            }
        }
        break;

    case WM_LBUTTONDOWN:
        OnMouseDown(lParam);
        break;

    case WM_LBUTTONUP:
    case WM_KILLFOCUS:
        OnMouseUp(lParam);
        break;

    case WM_MOUSEMOVE:
        OnMouseMove(lParam);
        break;

    case WM_COMMAND:
        switch (LOWORD(wParam))
        {
        case IDC_BUTTON_START_STOP_HOOKS:
            StartStop();
            break;
        case IDC_BUTTON_BROWSE_APP_PATH:
            BrowseApplicationPath();
            break;
        case IDC_BUTTON_PROCESS_FILTERS:
            SetApplicationsFilters();
            break;

        case IDC_BUTTON_BROWSE_MONITORING_FILE:
            BrowseMonitoringFile();
            break;
        case IDC_BUTTON_LOAD_MONITORING_FILE:
            LoadMonitoringFile();
            break;
        case IDC_BUTTON_UNLOAD_MONITORING_FILE:
            UnloadMonitoringFile();
            break;
        case IDC_BUTTON_START_STOP_MONITORING:
            StartStopMonitoring();
            break;

        case IDC_BUTTON_BROWSE_FAKING_FILE:
            BrowseFakingFile();
            break;
        case IDC_BUTTON_LOAD_FAKING_FILE:
            LoadFakeAPI();
            break;
        case IDC_BUTTON_UNLOAD_FAKING_FILE:
            UnloadFakeAPI();
            break;
        case IDC_BUTTON_START_STOP_FAKING:
            StartStopFaking();
            break;

        case IDC_BUTTON_CLEAR_MONITORING_LISTVIEW:
            ClearLogs();
            break;
        case IDC_BUTTON_LOAD_LOGS:
            LoadLogs();
            break;
        case IDC_BUTTON_SAVE_LOGS:
            SaveLogs();
            break;
        case IDC_BUTTON_ABOUT:
            About();
            break;
        case IDC_BUTTON_EXIT:
            Exit(LOWORD(wParam));
            break;
        case IDC_CHECK_SUSPENDED_MODE:
            if (IsDlgButtonChecked(mhWndDialog,IDC_CHECK_SUSPENDED_MODE)==BST_CHECKED)
                CheckDlgButton(mhWndDialog,IDC_CHECK_START_APP_WAIT_TIME,BST_UNCHECKED);
            break;
        case IDC_CHECK_START_APP_WAIT_TIME:
            if (IsDlgButtonChecked(mhWndDialog,IDC_CHECK_START_APP_WAIT_TIME)==BST_CHECKED)
                CheckDlgButton(mhWndDialog,IDC_CHECK_SUSPENDED_MODE,BST_UNCHECKED);
            break;
         }
        break;
    }
    return FALSE;
}

//-----------------------------------------------------------------------------
// Name: Init
// Object: Initialize objects that requiers the Dialog exists
//          and sets some dialog proporties
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void Init()
{

    Resize();

    // load app icon
    CDialogHelper::SetIcon(mhWndDialog,IDI_ICON_MAINWINDOW);
    
    // check first radio
    SendDlgItemMessage(mhWndDialog,IDC_RADIO_PID,(UINT)BM_SETCHECK,BST_CHECKED,0);

    // Init list view columns
    pListview=new CListview(GetDlgItem(mhWndDialog,IDC_LISTLOGMONITORING));
    COLUMN_INFO mpColumnInfoProcesses[6]={  {_T("Dir"),30,LVCFMT_LEFT},
                                            {_T("Call"),360,LVCFMT_LEFT},
                                            {_T("Ret Value"),80,LVCFMT_CENTER},
                                            {_T("Caller Addr"),80,LVCFMT_CENTER},
                                            {_T("ProcessID"),80,LVCFMT_CENTER},
                                            {_T("Module Name"),100,LVCFMT_LEFT}
                                          };
    pListview->SetStyle(TRUE,TRUE,FALSE);
    pListview->SetView(LV_VIEW_DETAILS);
    pListview->InitListViewColumns(6,mpColumnInfoProcesses);

    // disable monitoring faking interface
    EnableMonitoringFakingInterface(FALSE);

    // init select window
    pSelectWindow->Initialize(mhInstance,mhWndDialog,IDC_STATIC_PICTURE_SELECT_TARGET,
                            IDB_BLANK,IDB_CROSS,IDC_CURSOR_TARGET);

    if (*mlpCmdLine!=0)
        CloseHandle(CreateThread(NULL,0,ParseCommandLine,NULL,0,NULL));
}


//-----------------------------------------------------------------------------
// Name: ParseCommandLine
// Object: allow to parse application command line in a thread
// Parameters :
//     in  : LPVOID lpParameter : unused
//     out :
//     return : 
//-----------------------------------------------------------------------------
DWORD WINAPI ParseCommandLine(LPVOID lpParameter)
{
    UNREFERENCED_PARAMETER(lpParameter);
    WCHAR* lpwCmdLine;
    LPWSTR* lppwargv;
    CApiOverride* pApiOverride;
    int argc=0;
    TCHAR* argv[2];
    int cnt;
    BOOL bRet=FALSE;
    // convert string param to unicode
    CAnsiUnicodeConvert::AnsiToUnicode(mlpCmdLine,&lpwCmdLine);
    // use CommandLineToArgvW
    lppwargv=CommandLineToArgvW(
            lpwCmdLine,
            &argc
            );
    // check parameters number
    if (argc==0)
    {
        free(lpwCmdLine);
        GlobalFree(lppwargv);
        return 0;
    }
    if (argc>2)
    {
        MessageBox(mhWndDialog,_T("Commande line syntax : FileName [ResumeTimeMs]"),_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
        free(lpwCmdLine);
        GlobalFree(lppwargv);
        return 0xFFFFFFFF;
    }
    
    for(cnt=0;cnt<argc;cnt++)
    {
#if (defined(UNICODE)||defined(_UNICODE))
        // just copy data to get the same use in ansi or unicode mode
        argv[cnt]=_tcsdup(lppwargv[cnt]);
#else
        // convert all params to ansi
        CAnsiUnicodeConvert::UnicodeToAnsi(lppwargv[cnt],&argv[cnt]);
#endif
    }

    if(argc>0)// if no params do nothing, else
    {
        // create a new CApiOverride object
        pApiOverride=new CApiOverride();
        // set the unexpected unload callback
        pApiOverride->SetUnexpectedUnload(CallBackUnexpectedUnload,pApiOverride);
        // Set a share pListview
        pApiOverride->SetMonitoringListview(pListview);
        // add it to pApiOverrideList
        pApiOverrideList->AddItem(&pApiOverride);

        // disable the start button
        EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_START_STOP_HOOKS),FALSE);

        if (argc==1)// only filename
            bRet=pApiOverride->Start(argv[0],_T(""),CallBackBeforeAppResume,pApiOverride,CApiOverride::StartWayAsSoonAsPossible,0);
        else // file name and resume time
            bRet=pApiOverride->Start(argv[0],_T(""),CallBackBeforeAppResume,pApiOverride,CApiOverride::StartWaySleep,(DWORD)_ttoi(argv[1]));

        // enable the start button
        EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_START_STOP_HOOKS),TRUE);
    }

    if (bRet)
    {
        bStarted=TRUE;
        // put IDC_BUTTON_START_STOP_HOOKS caption to Stop
        SetDlgItemText(mhWndDialog,IDC_BUTTON_START_STOP_HOOKS,_T("Stop"));
    }

    // free memory
    for(cnt=0;cnt<argc;cnt++)
        free(argv[cnt]);
    free(lpwCmdLine);
    GlobalFree(lppwargv);
    return 0;
}

//-----------------------------------------------------------------------------
// Name: OnMouseDown
// Object: WM_MOUSEDOWN callback
// Parameters :
//     in  : LPARAM lParam : Dialog callback LPARAM
//     out :
//     return : 
//-----------------------------------------------------------------------------
void OnMouseDown(LPARAM lParam)
{
    pSelectWindow->MouseDown(lParam);
}
//-----------------------------------------------------------------------------
// Name: OnMouseUp
// Object: WM_MOUSEUP callback
// Parameters :
//     in  : LPARAM lParam : Dialog callback LPARAM
//     out :
//     return : 
//-----------------------------------------------------------------------------
void OnMouseUp(LPARAM lParam)
{
    UNREFERENCED_PARAMETER(lParam);

    pSelectWindow->MouseUp();
}

//-----------------------------------------------------------------------------
// Name: OnMouseMove
// Object: WM_MOUSEMOVE callback
// Parameters :
//     in  : LPARAM lParam : Dialog callback LPARAM
//     out :
//     return : 
//-----------------------------------------------------------------------------
void OnMouseMove(LPARAM lParam)
{
    BOOL bWindowChanged=FALSE;
    pSelectWindow->MouseMove(lParam,&bWindowChanged);
    if (bWindowChanged)
    {
        // set process ID in IDC_EDIT_PID
        SetDlgItemInt(mhWndDialog,IDC_EDIT_PID,pSelectWindow->WindowProcessID,FALSE);
    }
}

//-----------------------------------------------------------------------------
// Name: StartStop
// Object: Start/Stop button callback. Start or stop hooking depending options
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void StartStop()
{
    BOOL bSuccess=FALSE;
    DWORD cnt;
    TCHAR psz[MAX_PATH];
    TCHAR ProcName[MAX_PATH];

    // if application is started, that means we have to do a stop
    if (bStarted)// Stop action
    {
        // create a new thread to allow unload of monitoring/overriding of DlgProc
        CloseHandle(CreateThread(NULL,0,Stop,NULL,0,NULL));
    }
    else // Start action
    {
        CApiOverride* pApiOverride;
        
        // if we want to hook all processes
        if (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_ALL_PROCESS)==BST_CHECKED)
        {
            // start the procmon driver
            bSuccess=pProcMonInterface->StartDriver();
            if (bSuccess)
            {
                // set a callback for process creation
                pProcMonInterface->SetProcessStartCallBack(CallBackProcessCreation);
                // start monitoring
                pProcMonInterface->StartMonitoring();

                // retreive wanted running application array
                for(cnt=0;cnt<pFilters->pdwFiltersCurrentPorcessIDSize;cnt++)
                {
                    // if process is no more alive
                    if (!CProcessHelper::IsAlive(pFilters->pdwFiltersCurrentPorcessID[cnt]))
                    {
                         // add information to listview
                        _sntprintf(psz,MAX_PATH,_T("Process 0x%x don't exists anymore, so it won't be hooked"),pFilters->pdwFiltersCurrentPorcessID[cnt]);
                        ReportMessage(psz,MSG_WARNING);
                        continue;
                    }
                    // create a new CApiOverride object
                    pApiOverride=new CApiOverride();
                    // set the unexpected unload callback
                    pApiOverride->SetUnexpectedUnload(CallBackUnexpectedUnload,pApiOverride);
                    // give handle of listview to pApiOverride so we don't worry about logging monitoring msg
                    pApiOverride->SetMonitoringListview(pListview);

                    // start hooking
                    if (pApiOverride->Start(pFilters->pdwFiltersCurrentPorcessID[cnt]))
                    {
                        // on success 

                        // add it to pApiOverrideList
                        pApiOverrideList->AddItem(&pApiOverride);
                        

                        // get proc name
                        *ProcName=0;
                        pApiOverride->GetProcessName(ProcName,MAX_PATH);
                        if (*ProcName)
                            _sntprintf(psz,MAX_PATH,_T("Process 0x%x (%s) successfully hooked"),pFilters->pdwFiltersCurrentPorcessID[cnt],ProcName);
                        else
                            _sntprintf(psz,MAX_PATH,_T("Process 0x%x successfully hooked"),pFilters->pdwFiltersCurrentPorcessID[cnt]);
                        // add info to listview
                        ReportMessage(psz,MSG_INFORMATION);
                    }
                    else
                    {
                        
                        // get proc name
                        if (CProcessHelper::GetProcessName(pFilters->pdwFiltersCurrentPorcessID[cnt],ProcName,MAX_PATH))
                            _sntprintf(psz,MAX_PATH,_T("Error hooking process 0x%x (%s)"),pFilters->pdwFiltersCurrentPorcessID[cnt],ProcName);
                        else
                            _sntprintf(psz,MAX_PATH,_T("Error hooking process 0x%x"),pFilters->pdwFiltersCurrentPorcessID[cnt]);
                        // add info to listview
                        ReportMessage(psz,MSG_ERROR);

                        // delete pApiOverride object
                        delete pApiOverride;
                    }

                }
            }
        }
        else
        {
            // only one pApiOverride creation is requiered

            // create a new CApiOverride object
            pApiOverride=new CApiOverride();
            // set the unexpected unload callback
            pApiOverride->SetUnexpectedUnload(CallBackUnexpectedUnload,pApiOverride);
            // give handle of listview to pApiOverride so we don't worry about logging monitoring msg
            pApiOverride->SetMonitoringListview(pListview);

            // if we want to hook application with the specified PID
            if (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_PID)==BST_CHECKED)
            {
                // Retreive PID we want to hook from dialog interface
                DWORD dwPID=GetDlgItemInt(mhWndDialog,IDC_EDIT_PID,&bSuccess,FALSE);

                if (bSuccess)
                    // check that process is still alive
                    bSuccess=CProcessHelper::IsAlive(dwPID);
                // if good pid value
                if (bSuccess)
                    bSuccess=pApiOverride->Start(dwPID);
                else
                    MessageBox(NULL,_T("Bad Process Id number"),_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                // if pApiOverride->Start successful
                if (bSuccess)
                {
                    // add it to pApiOverrideList
                    pApiOverrideList->AddItem(&pApiOverride);

                    // add info to listview
                    *ProcName=0;
                    pApiOverride->GetProcessName(ProcName,MAX_PATH);
                    if (*ProcName)
                        _sntprintf(psz,MAX_PATH,_T("Process 0x%x (%s) successfully hooked"),dwPID,ProcName);
                    else
                        _sntprintf(psz,MAX_PATH,_T("Process 0x%x successfully hooked"),dwPID);
                    ReportMessage(psz,MSG_INFORMATION);

                    // show an information message
                    MessageBox(NULL,_T("Application Ready to be Hooked\r\nYou can now load Monitoring and Faking files"),_T("Information"),MB_OK|MB_ICONINFORMATION|MB_TOPMOST);
                }
            }
            // if we want to hook the specified app at startup
            else if (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_APP_NAME)==BST_CHECKED)
            {
                TCHAR pszAppPath[MAX_PATH]=_T("");
                TCHAR pszCmdLine[MAX_PATH]=_T("");

                // get Application full path
                GetDlgItemText(mhWndDialog,IDC_EDIT_APP_PATH,pszAppPath,MAX_PATH);

                // get cmd line
                GetDlgItemText(mhWndDialog,IDC_EDIT_CMD_LINE,pszCmdLine,MAX_PATH);

                // if wait time before injection checked
                if (IsDlgButtonChecked(mhWndDialog,IDC_CHECK_START_APP_WAIT_TIME)==BST_CHECKED)
                {
                    // retreive resume time from dailog interface
                    DWORD dwResumeTime=GetDlgItemInt(mhWndDialog,IDC_EDIT_RESUMETIME,&bSuccess,FALSE);
                    // if good resume time value
                    if (bSuccess)
                    {
                        // disable StartStop button
                        EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_START_STOP_HOOKS),FALSE);
                        // start delay hooking
                        bSuccess=pApiOverride->Start(pszAppPath,pszCmdLine,CallBackBeforeAppResume,pApiOverride,CApiOverride::StartWaySleep,dwResumeTime);
                        // enable StartStop button
                        EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_START_STOP_HOOKS),TRUE);
                    }
                    else
                        MessageBox(NULL,_T("Bad resume time value"),_T("Error"),MB_OK|MB_ICONERROR|MB_TOPMOST);
                }
                // if suspended injection checked
                else if (IsDlgButtonChecked(mhWndDialog,IDC_CHECK_SUSPENDED_MODE)==BST_CHECKED)
                {
                    // disable StartStop button
                    EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_START_STOP_HOOKS),FALSE);
                    // start with infos
                    bSuccess=pApiOverride->Start(pszAppPath,pszCmdLine,CallBackBeforeAppResume,pApiOverride,CApiOverride::StartWaySuspended,0);
                    // enable StartStop button
                    EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_START_STOP_HOOKS),TRUE);
                }
                else // inject as soon as possible
                {
                    // disable StartStop button
                    EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_START_STOP_HOOKS),FALSE);
                    // start with infos
                    bSuccess=pApiOverride->Start(pszAppPath,pszCmdLine,CallBackBeforeAppResume,pApiOverride,CApiOverride::StartWayAsSoonAsPossible,0);
                    // enable StartStop button
                    EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_START_STOP_HOOKS),TRUE);
                }
            }
            // on error
            if (!bSuccess)
                // free memory 
                delete pApiOverride;
        }

        if (bSuccess)
        {
            bStarted=TRUE;
            // put IDC_BUTTON_START_STOP_HOOKS caption to Stop
            SetDlgItemText(mhWndDialog,IDC_BUTTON_START_STOP_HOOKS,_T("Stop"));

            // enable monitoring faking interface
            EnableMonitoringFakingInterface(TRUE);

            // if we have selected a window and hook a process ID
            if (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_PID)==BST_CHECKED)
                // get informations for the last selected window
                GetRemoteWindowInfos();
        }
        else
            // disable monitoring faking interface
            EnableMonitoringFakingInterface(FALSE);
    }
}

//-----------------------------------------------------------------------------
// Name: Stop
// Object: Allow to stop hooking of all hooked processes in a thread.
// Parameters :
//     in  : LPVOID lpParameter : unused
//     out :
//     return : 
//-----------------------------------------------------------------------------
DWORD WINAPI Stop(LPVOID lpParameter)
{
    UNREFERENCED_PARAMETER(lpParameter);

    BOOL bSuccess=TRUE;
    BOOL bItemSuccess;
    DWORD dwArraySize;
    DWORD dwProcessID;
    CApiOverride*** pApiOverrideArray;
    CApiOverride* pApiOverride;
    TCHAR psz[MAX_PATH];
    TCHAR ProcName[MAX_PATH];

    // lock start/stop button
    EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_START_STOP_HOOKS),FALSE);

    // convert link list to array
    pApiOverrideArray=(CApiOverride***)pApiOverrideList->ToInsecureArray(&dwArraySize);
    // for each hooked process
    for (DWORD dwCnt=0;dwCnt<dwArraySize;dwCnt++)
    {
        pApiOverride=(*pApiOverrideArray[dwCnt]);

        // retreive proc ID
        dwProcessID=pApiOverride->GetProcessID();        

        // stop
        bItemSuccess=pApiOverride->Stop();
        if (bItemSuccess)
        {
            
            *ProcName=0;
            pApiOverride->GetProcessName(ProcName,MAX_PATH);
            // try to get process name
            if (*ProcName)
                _sntprintf(psz,MAX_PATH,_T("Process 0x%x (%s) successfully unhooked"),dwProcessID,ProcName);
            else
                _sntprintf(psz,MAX_PATH,_T("Process 0x%x successfully unhooked"),dwProcessID);
            // add info to listview
            ReportMessage(psz,MSG_INFORMATION);

            // remove item from link list
            pApiOverrideList->RemoveItemFromItemData(&pApiOverride);

            // delete CApiOverride object
            delete pApiOverride;
        }
        else
        {
            // try to get process name
            if (CProcessHelper::GetProcessName(dwProcessID,ProcName,MAX_PATH))
                _sntprintf(psz,MAX_PATH,_T("Error unhooking process 0x%x (%s)"),dwProcessID,ProcName);
            else
                _sntprintf(psz,MAX_PATH,_T("Error unhooking process 0x%x"),dwProcessID);
            // add info to listview
            ReportMessage(psz,MSG_ERROR);
        }
        bSuccess=bSuccess&&bItemSuccess;
    }
    // free allocated array
    delete[] pApiOverrideArray;

    // if all process spy
    if (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_ALL_PROCESS)==BST_CHECKED)
    {
        // stop ProcMon.sys driver
        pProcMonInterface->StopMonitoring();
        pProcMonInterface->StopDriver();
    }

    if (bSuccess)
    {
        // put IDC_BUTTON_START_STOP_HOOKS caption to Start
        SetDlgItemText(mhWndDialog,IDC_BUTTON_START_STOP_HOOKS,_T("Start"));

        // disable monitoring faking interface
        EnableMonitoringFakingInterface(FALSE);
        
        // Clear monitoring and faking listbox
        SendDlgItemMessage(mhWndDialog,IDC_LIST_MONITORING_FILES,LB_RESETCONTENT,0,0);
        SendDlgItemMessage(mhWndDialog,IDC_LIST_FAKING_FILES,LB_RESETCONTENT,0,0);

        // clear last selected window informations
        SetDlgItemText(mhWndDialog,IDC_STATIC_SELECTED_WINDOW_INFO,_T(""));

        //////////////////////////////////////////////
        //reset state of start/stop monitoring faking
        //////////////////////////////////////////////
        bMonitoring=TRUE;
        // put IDC_BUTTON_START_STOP_MONITORING caption to Stop
        SetDlgItemText(mhWndDialog,IDC_BUTTON_START_STOP_MONITORING,_T("Stop Monitoring"));

        bFaking=TRUE;
        // put IDC_BUTTON_START_STOP_FAKING caption to Stop
        SetDlgItemText(mhWndDialog,IDC_BUTTON_START_STOP_FAKING,_T("Stop Faking"));

        // set the started flag to false
        bStarted=FALSE;
    }

    // unlock start/stop button
    EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_START_STOP_HOOKS),TRUE);

    // if we have to exit
    if (bExitingStop)
        Exit(NULL);

    return 0;
}


//-----------------------------------------------------------------------------
// Name: CallBackProcessCreation
// Object: call back called at each process creation
// Parameters :
//     in  : HANDLE hParentId : ID of process that create new process 
//           HANDLE hProcessId : new created process ID
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CallBackProcessCreation(HANDLE hParentId,HANDLE hProcessId)
{
    BOOL bSuccess=FALSE;
    DWORD dwSleepTime;
    HWND hWndListBox;
    DWORD dwCnt;
    DWORD dwNbItems;
    DWORD dwFirstTryTime;
    TCHAR psz[MAX_PATH];
    TCHAR psz2[MAX_PATH];
    TCHAR psz3[MAX_PATH];

    // if user have ask to exit do nothing
    if (bExit)
        return;

    // check parent id filters
    if(!pFilters->DoesParentIDMatchFilters((DWORD)hParentId))
        return;

    // get processname and check proc name filters
    // CProcessHelper::GetProcessName can failed at process startup, so we have to wait
    // a little until EnumProcessModules and GetModuleBaseName don't fail anymore
    bSuccess=FALSE;
   
    // until name retreival success or timeout
    for (int cnt=0;cnt<APIOVERRIDE_NB_SLEEP_FOR_GET_PROCESS_NAME_TIMEOUT;cnt++)
    {
        dwFirstTryTime=GetTickCount();
        while (((GetTickCount()-dwFirstTryTime)<(APIOVERRIDE_GET_PROCESS_NAME_TIMEOUT_IN_MS/APIOVERRIDE_NB_SLEEP_FOR_GET_PROCESS_NAME_TIMEOUT))
                && (!bSuccess))
        {
            // try to retreive process name
            bSuccess=CProcessHelper::GetProcessName((DWORD)hProcessId,psz,MAX_PATH);
        }
        // lets os do other job (like loading our target app)
        Sleep(10);
    }

    // if time out occurs 
    // that means we fail to retreive process name 
    // so query if the user want to hook the process
    if (!bSuccess)
    {
        // try to get process name
        if (CProcessHelper::GetProcessName((DWORD)hParentId,psz2,MAX_PATH))
            _sntprintf(
                psz,
                MAX_PATH,
                _T("Can't retreive name of process 0x%x launched by process 0x%x (%s).\r\nDo you want to hook it ?"),
                hProcessId,
                hParentId,
                psz2);
        else
            _sntprintf(
                psz,
                MAX_PATH,
                _T("Can't retreive name of process 0x%x launched by process 0x%x.\r\nDo you want to hook it ?"),
                hProcessId,
                hParentId);
       if (MessageBox(NULL,psz,_T("Question"),MB_YESNO|MB_ICONQUESTION|MB_TOPMOST)!=IDYES)
           return;
    }

    else // on name retreival success
    {
        // check filters based on process name
        if (!pFilters->DoesProcessNameMatchFilters(psz))
            return;
    }

    CApiOverride* pApiOverride;
    // create a new CApiOverride object
    pApiOverride=new CApiOverride();
    // set the unexpected unload callback
    pApiOverride->SetUnexpectedUnload(CallBackUnexpectedUnload,pApiOverride);
    // give handle of listview to pApiOverride so we don't worry about logging monitoring msg
    pApiOverride->SetMonitoringListview(pListview);

    // if sleeptime before injection
    if (IsDlgButtonChecked(mhWndDialog,IDC_CHECK_INJECT_IN_ALL_ONLY_AFTER)==BST_CHECKED)
    {
        // retreive wait time
        dwSleepTime=GetDlgItemInt(mhWndDialog,IDC_EDIT_INSERT_AFTER,&bSuccess,FALSE);

        if (bSuccess)
        {
            // sleep
            // Notice : we don't have to care of this blocking state 
            // as CallBackProcessCreation func are called in threads (see ProcmonInterface)
            Sleep(dwSleepTime); 
        }
    }

    // start overriding on new process
    if (!pApiOverride->Start((DWORD)hProcessId))
    {

        // try to get process name
        if (CProcessHelper::GetProcessName((DWORD)hProcessId,psz2,MAX_PATH))
            _sntprintf(psz,MAX_PATH,_T("Error hooking process 0x%x (%s)"),(DWORD)hProcessId,psz2);
        else
            _sntprintf(psz,MAX_PATH,_T("Error hooking process 0x%x"),(DWORD)hProcessId);

        // report error
        ReportMessage(psz,MSG_ERROR);

        // free memory
        delete pApiOverride;
        return;
    }

    // set monitoring and faking state
    if (!bMonitoring)
        pApiOverride->StopMonitoring();
    if (!bFaking)
        pApiOverride->StopFaking();

    // add it to pApiOverrideList
    pApiOverrideList->AddItem(&pApiOverride);

    // report information
    *psz2=0;
    pApiOverride->GetProcessName(psz2,MAX_PATH);
    if ((*psz2)
        &&CProcessHelper::GetProcessName((DWORD)hParentId,psz3,MAX_PATH))
    {
        _sntprintf(psz,MAX_PATH,_T("Process 0x%x (%s) launched by 0x%x (%s) successfully hooked"),
            (DWORD)hProcessId,psz2,
            (DWORD)hParentId,psz3);
    }
    else
    {
        _sntprintf(psz,MAX_PATH,_T("Process 0x%x launched by 0x%x successfully hooked"),(DWORD)hProcessId,(DWORD)hParentId);
    }
    ReportMessage(psz,MSG_INFORMATION);


    //////////////////////////////////////////
    // inject all monitoring files present in listbox
    //////////////////////////////////////////
    hWndListBox=GetDlgItem(mhWndDialog,IDC_LIST_MONITORING_FILES);
    dwNbItems=(DWORD)SendMessage(hWndListBox,LB_GETCOUNT,0,0);
    // for each listbox item
    for (dwCnt=0;dwCnt<dwNbItems;dwCnt++)
    {
        // retreive text (we don't send the LB_GETTEXTLEN message as we have filled listbox with
        // MAX_PATH length buffer)
        if (SendMessage(hWndListBox,LB_GETTEXT,dwCnt,(LPARAM)psz)==LB_ERR)
            continue;

        // get file name
        DecodePrettyName(psz);

        // load monitoring file
        pApiOverride->LoadMonitoringFile(psz);

    }

    //////////////////////////////////////////
    // inject all dll present in listbox
    //////////////////////////////////////////
    hWndListBox=GetDlgItem(mhWndDialog,IDC_LIST_FAKING_FILES);
    dwNbItems=(DWORD)SendMessage(hWndListBox,LB_GETCOUNT,0,0);
    // for each listbox item
    for (dwCnt=0;dwCnt<dwNbItems;dwCnt++)
    {
        // retreive text (we don't send the LB_GETTEXTLEN message as we have filled listbox with
        // MAX_PATH length buffer)
        if (SendMessage(hWndListBox,LB_GETTEXT,dwCnt,(LPARAM)psz)==LB_ERR)
            continue;

        // get file name
        DecodePrettyName(psz);

        // load faking dll
        pApiOverride->LoadFakeAPI(psz);
    }

}

//-----------------------------------------------------------------------------
// Name: GetRemoteWindowInfos
// Object: this proc contains samples of remote process proc execution
//         gives info only for last selected window
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void GetRemoteWindowInfos()
{
    CApiOverride* pApiOverride;

    if (IsBadReadPtr(pApiOverrideList,sizeof(CLinkList*)))
        return;
    if (IsBadReadPtr(pApiOverrideList->Head,sizeof(CLinkListItem*)))
        return;
    pApiOverride=*(CApiOverride**)pApiOverrideList->Head->ItemData;
    if (IsBadReadPtr(pApiOverride,sizeof(CApiOverride*)))
        return;

    // give info of selected window
    if (pSelectWindow->WindowHandle)
    {
        TCHAR psz[MAX_PATH];
        TCHAR psztmp[MAX_PATH];

        DWORD dwDlgProc=0;
        DWORD dwWndProc=0;

        int iParam;
        STRUCT_FUNC_PARAM pParams[2];

        // For the GetWindowLong func with DWLP_DLGPROC or GWLP_WNDPROC param,
        // just check the difference between call from this process and call in remote process

        // dwDlgProc=(DWORD)GetWindowLong (pSelectWindow->WindowHandle, DWLP_DLGPROC);
        iParam=DWLP_DLGPROC;
        pParams[0].bPassAsRef=FALSE;
        pParams[0].dwDataSize=4;
        pParams[0].pData=(PBYTE)&pSelectWindow->WindowHandle;
        pParams[1].bPassAsRef=FALSE;
        pParams[1].dwDataSize=4;
        pParams[1].pData=(PBYTE)&iParam;
#if (defined(UNICODE)||defined(_UNICODE))
        pApiOverride->ProcessInternalCall(_T("User32.dll"),_T("GetWindowLongW"),2,pParams,&dwDlgProc);
#else
        pApiOverride->ProcessInternalCall(_T("User32.dll"),_T("GetWindowLongA"),2,pParams,&dwDlgProc);
#endif
        

        // dwWndProc=(DWORD)GetWindowLong (pSelectWindow->WindowHandle, GWLP_WNDPROC);
        iParam=GWLP_WNDPROC;
        pParams[0].bPassAsRef=FALSE;
        pParams[0].dwDataSize=4;
        pParams[0].pData=(PBYTE)&pSelectWindow->WindowHandle;
        pParams[1].bPassAsRef=FALSE;
        pParams[1].dwDataSize=4;
        pParams[1].pData=(PBYTE)&iParam;
#if (defined(UNICODE)||defined(_UNICODE))
        pApiOverride->ProcessInternalCall(_T("User32.dll"),_T("GetWindowLongW"),2,pParams,&dwDlgProc);
#else
        pApiOverride->ProcessInternalCall(_T("User32.dll"),_T("GetWindowLongA"),2,pParams,&dwDlgProc);
#endif


/*
//////////////////////
// other samples of remote call
//////////////////////
DWORD dw;
// api stdcall
pApiOverride->ProcessInternalCall("Kernel32.dll","IsDebuggerPresent",0,NULL,&dw);
// exe internal cdecl
int i1=18;
int i2=22;
TCHAR pc[255];
pParams[0].bPassAsRef=FALSE;
pParams[0].dwDataSize=4;
pParams[0].pData=(PBYTE)&i1;
pParams[1].bPassAsRef=FALSE;
pParams[1].dwDataSize=4;
pParams[1].pData=(PBYTE)&i2;
pApiOverride->ProcessInternalCall(_T("EXE_INTERNAL@0x4114dd"),_T("MyFunc"),2,pParams,&dw);
// api stdcall with out pointer
i1=255;
pParams[0].bPassAsRef=FALSE;
pParams[0].dwDataSize=4;
pParams[0].pData=(PBYTE)&i1;
pParams[1].bPassAsRef=TRUE;
pParams[1].dwDataSize=255;
pParams[1].pData=(PBYTE)pc;
pApiOverride->ProcessInternalCall(_T("Kernel32.dll"),_T("GetTempPathA"),2,pParams,&dw);
// api stdcall with ref struct

RECT Rect;
pParams[0].bPassAsRef=FALSE;
pParams[0].dwDataSize=4;
pParams[0].pData=(PBYTE)&pSelectWindow->WindowHandle;
pParams[1].bPassAsRef=TRUE;
pParams[1].dwDataSize=sizeof(RECT);
pParams[1].pData=(PBYTE)&Rect;
pApiOverride->ProcessInternalCall(_T("User32.dll"),_T("GetWindowRect"),2,pParams,&dw);
GetWindowRect(pSelectWindow->WindowHandle,&Rect);
*/

        // give window information to user ( WindowProc and WindowDlgProc)
        *psz=0;
        if ((dwWndProc)||(dwDlgProc))
        {
            _tcscpy(psz,_T("Selected Window Info\r\n"));
            if (dwDlgProc)
            {
                _stprintf(psztmp,_T("DlgProc: 0x%x\t"),dwDlgProc);
                _tcscat(psz,psztmp);
            }
            if (dwWndProc)
            {
                _stprintf(psztmp,_T("WndProc: 0x%x"),dwWndProc);
                _tcscat(psz,psztmp);
            }
        }
        SetDlgItemText(mhWndDialog,IDC_STATIC_SELECTED_WINDOW_INFO,psz);
    }
}

//-----------------------------------------------------------------------------
// Name: StartStopMonitoring
// Object: start or stop monitoring of all hooked processes
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void StartStopMonitoring()
{
    BOOL bSuccess=TRUE;
    BOOL bItemSuccess;
    DWORD dwArraySize;
    DWORD dwCnt;
    CApiOverride*** pApiOverrideArray;
    // convert link list to array
    pApiOverrideArray=(CApiOverride***)pApiOverrideList->ToInsecureArray(&dwArraySize);

    // if we are monitoring
    if (bMonitoring)
    {
        // stop monitoring for all hooked processes
        for (dwCnt=0;dwCnt<dwArraySize;dwCnt++)
        {
            bItemSuccess=(*pApiOverrideArray[dwCnt])->StopMonitoring();
            bSuccess=bSuccess&&bItemSuccess;
        }
        if (bSuccess)
        {
            bMonitoring=FALSE;
            // put IDC_BUTTON_START_STOP_MONITORING caption to Stop
            SetDlgItemText(mhWndDialog,IDC_BUTTON_START_STOP_MONITORING,_T("Start Monitoring"));
        }
    }
    else // we are not monitoring
    {
        // start monitoring for all hooked processes
        for (dwCnt=0;dwCnt<dwArraySize;dwCnt++)
        {
            bItemSuccess=(*pApiOverrideArray[dwCnt])->StartMonitoring();
            bSuccess=bSuccess&&bItemSuccess;
        }
        if (bSuccess)
        {
            bMonitoring=TRUE;
            // put IDC_BUTTON_START_STOP_MONITORING caption to Stop
            SetDlgItemText(mhWndDialog,IDC_BUTTON_START_STOP_MONITORING,_T("Stop Monitoring"));
        }
    }

    // free memory
    delete[] pApiOverrideArray;
}

//-----------------------------------------------------------------------------
// Name: StartStopFaking
// Object: start or stop faking for all hooked processes
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void StartStopFaking()
{
    BOOL bSuccess=TRUE;
    BOOL bItemSuccess;
    DWORD dwArraySize;
    DWORD dwCnt;
    CApiOverride*** pApiOverrideArray;
    // convert link list to array
    pApiOverrideArray=(CApiOverride***)pApiOverrideList->ToInsecureArray(&dwArraySize);
    // if we are faking
    if (bFaking)
    {
        // stop faking for all hooked processes
        for (dwCnt=0;dwCnt<dwArraySize;dwCnt++)
        {
            bItemSuccess=(*pApiOverrideArray[dwCnt])->StopFaking();
            bSuccess=bSuccess&&bItemSuccess;
        }
        if (bSuccess)
        {
            bFaking=FALSE;
            // put IDC_BUTTON_START_STOP_FAKING caption to Stop
            SetDlgItemText(mhWndDialog,IDC_BUTTON_START_STOP_FAKING,_T("Start Faking"));
        }
    }
    else// we are not faking
    {
        // start faking for all hooked processes
        for (dwCnt=0;dwCnt<dwArraySize;dwCnt++)
        {
            bItemSuccess=(*pApiOverrideArray[dwCnt])->StartFaking();
            bSuccess=bSuccess&&bItemSuccess;
        }
        if (bSuccess)
        {
            bFaking=TRUE;
            // put IDC_BUTTON_START_STOP_FAKING caption to Stop
            SetDlgItemText(mhWndDialog,IDC_BUTTON_START_STOP_FAKING,_T("Stop Faking"));
        }
    }
    // free memory
    delete[] pApiOverrideArray;
}

//-----------------------------------------------------------------------------
// Name: LoadMonitoringFile
// Object: load a monitoring file in all hooked processes
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void LoadMonitoringFile()
{
    BOOL bSuccess=TRUE;
    BOOL bItemSuccess;
    DWORD dwArraySize;
    CApiOverride*** pApiOverrideArray;
    // convert link list to array
    pApiOverrideArray=(CApiOverride***)pApiOverrideList->ToInsecureArray(&dwArraySize);

    TCHAR pszFile[MAX_PATH]=_T("");
    GetDlgItemText(mhWndDialog,IDC_EDIT_MONITORING_FILE,pszFile,MAX_PATH);

    // for each hooked process
    for (DWORD dwCnt=0;dwCnt<dwArraySize;dwCnt++)
    {
        // load monitoring file
        bItemSuccess=(*pApiOverrideArray[dwCnt])->LoadMonitoringFile(pszFile);
        bSuccess=bSuccess&&bItemSuccess;
    }
    if (bSuccess)
    {
        MakePrettyName(pszFile);
        HWND hWndListBox=GetDlgItem(mhWndDialog,IDC_LIST_MONITORING_FILES);
        // add monitoring file name to listbox
        CListbox::AddStringWithHScrollUpdate(hWndListBox, pszFile);
    }
    // free memory
    delete[] pApiOverrideArray;
}

//-----------------------------------------------------------------------------
// Name: UnloadMonitoringFile
// Object: unload a monitoring file in all hooked processes
//          due to unload requiered time (sometime blocking call)
//          unload is done into a thread
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void UnloadMonitoringFile()
{
    // create a new thread to allow unload of monitoring/overriding of DlgProc
    CloseHandle(CreateThread(NULL,0,UnloadMonitoringFile,NULL,0,NULL));
}

DWORD WINAPI UnloadMonitoringFile(LPVOID lpParameter)
{
    UNREFERENCED_PARAMETER(lpParameter);
    TCHAR pszFile[MAX_PATH]=_T("");
    LONG lNbItems;
    HWND hWndListBox;

    BOOL bSuccess=TRUE;
    BOOL bItemSuccess;
    DWORD dwArraySize;
    CApiOverride*** pApiOverrideArray;
    // convert link list into array
    pApiOverrideArray=(CApiOverride***)pApiOverrideList->ToInsecureArray(&dwArraySize);

    hWndListBox=GetDlgItem(mhWndDialog,IDC_LIST_MONITORING_FILES);
    // get number of loaded files
    lNbItems=(LONG)SendMessage(hWndListBox,LB_GETCOUNT,0,0);
    if (lNbItems==LB_ERR)
    {
        delete[] pApiOverrideArray;
        return 0xFFFFFFFF;
    }

    // lock unload button
    EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_UNLOAD_MONITORING_FILE),FALSE);

    // parse listbox items to know if they are selected
    for (int cnt=0;cnt<lNbItems;cnt++)
    {
        // if item is selected
        if (SendMessage(hWndListBox,LB_GETSEL,cnt,0)>0)
        {
            // get item text (file name)
            SendMessage(hWndListBox,LB_GETTEXT,cnt,(LPARAM)pszFile);
            DecodePrettyName(pszFile);

            // try to unload
            for (DWORD dwCnt=0;dwCnt<dwArraySize;dwCnt++)
            {
                bItemSuccess=(*pApiOverrideArray[dwCnt])->UnloadMonitoringFile(pszFile);
                bSuccess=bSuccess&&bItemSuccess;
            }

            if (bSuccess)
            {
                // remove from listbox
                SendMessage(hWndListBox,LB_DELETESTRING,cnt,(LPARAM)pszFile);

                // update next item and item count
                cnt--;
                lNbItems--;
            }
        }
    }

    // update listbox horizontal scroll
    CListbox::UpdateListBoxHScroll(hWndListBox);

    // unlock unload button
    EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_UNLOAD_MONITORING_FILE),TRUE);

    // free memory
    delete[] pApiOverrideArray;
    return 0;
}

//-----------------------------------------------------------------------------
// Name: LoadFakeAPI
// Object: load a fake dll in all hooked processes
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void LoadFakeAPI()
{
    BOOL bSuccess=TRUE;
    BOOL bItemSuccess;
    DWORD dwArraySize;
    CApiOverride*** pApiOverrideArray;
    // convert link list into array
    pApiOverrideArray=(CApiOverride***)pApiOverrideList->ToInsecureArray(&dwArraySize);

    TCHAR pszFile[MAX_PATH]=_T("");
    // get file name to add
    GetDlgItemText(mhWndDialog,IDC_EDIT_FAKING_FILE,pszFile,MAX_PATH);

    // try to load it for all hooked processes
    for (DWORD dwCnt=0;dwCnt<dwArraySize;dwCnt++)
    {
        bItemSuccess=(*pApiOverrideArray[dwCnt])->LoadFakeAPI(pszFile);
        bSuccess=bSuccess&&bItemSuccess;
    }
    if (bSuccess)
    {
        // add to listbox
        MakePrettyName(pszFile);
        HWND hWndListBox=GetDlgItem(mhWndDialog,IDC_LIST_FAKING_FILES);
        CListbox::AddStringWithHScrollUpdate(hWndListBox, pszFile);
    }
    // free memory
    delete[] pApiOverrideArray;
}

//-----------------------------------------------------------------------------
// Name: UnloadFakeAPI
// Object: unload a monitoring file in all hooked processes
//          due to unload requiered time (sometime blocking call)
//          unload is done into a thread
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void UnloadFakeAPI()
{
    // create a new thread to allow unload of monitoring/overriding of DlgProc
    CloseHandle(CreateThread(NULL,0,UnloadFakeAPI,NULL,0,NULL));
}
DWORD WINAPI UnloadFakeAPI(LPVOID lpParameter)
{
    UNREFERENCED_PARAMETER(lpParameter);
    TCHAR pszFile[MAX_PATH]=_T("");
    LONG lNbItems;
    BOOL bSuccess=TRUE;
    BOOL bItemSuccess;
    DWORD dwArraySize;
    CApiOverride*** pApiOverrideArray;
    // convert link list into array
    pApiOverrideArray=(CApiOverride***)pApiOverrideList->ToInsecureArray(&dwArraySize);

    HWND hWndListBox;

    hWndListBox=GetDlgItem(mhWndDialog,IDC_LIST_FAKING_FILES);
    // get number of loaded files
    lNbItems=(LONG)SendMessage(hWndListBox,LB_GETCOUNT,0,0);
    if (lNbItems==LB_ERR)
    {
        delete[] pApiOverrideArray;
        return 0xFFFFFFFF;
    }
    // lock unload button
    EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_UNLOAD_FAKING_FILE),FALSE);

    // parse items to know if they are selected
    for (int cnt=0;cnt<lNbItems;cnt++)
    {
        // if item is selected
        if (SendMessage(hWndListBox,LB_GETSEL,cnt,0)>0)
        {
            // get item text (file name)
            SendMessage(hWndListBox,LB_GETTEXT,cnt,(LPARAM)pszFile);
            DecodePrettyName(pszFile);
            // try to unload for all processes
            for (DWORD dwCnt=0;dwCnt<dwArraySize;dwCnt++)
            {
                bItemSuccess=(*pApiOverrideArray[dwCnt])->UnloadFakeAPI(pszFile);
                bSuccess=bSuccess&&bItemSuccess;
            }
            if (bSuccess)
            {
                // remove from listbox
                SendMessage(hWndListBox,LB_DELETESTRING,cnt,(LPARAM)pszFile);

                // update next item and item count
                cnt--;
                lNbItems--;
            }
        }
    }
    // update listbox horizontal scroll
    CListbox::UpdateListBoxHScroll(hWndListBox);

    // unlock unload button
    EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_UNLOAD_FAKING_FILE),TRUE);

    // free memory
    delete[] pApiOverrideArray;

    return 0;
}

//-----------------------------------------------------------------------------
// Name: CallBackBeforeAppResume
// Object: called after apioverride.dll injection when process is hooked at startup
//         and just before process resume
// Parameters :
//     in  : DWORD dwProcessID : process ID that is suspended
//           PVOID pUserParam : associated CApiOverride object pointer
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CallBackBeforeAppResume(DWORD dwProcessID,PVOID pUserParam)
{
    TCHAR psz[MAX_PATH];
    TCHAR ProcName[MAX_PATH];

    // if user have ask to exit do nothing
    if (bExit)
        return;

    CApiOverride* pApiOverride=(CApiOverride*)pUserParam;

    // add pApiOverride to pApiOverrideList
    pApiOverrideList->AddItem(&pApiOverride);

    // add info to listview
    *ProcName=0;
    pApiOverride->GetProcessName(ProcName,MAX_PATH);

    if (*ProcName)
        _sntprintf(psz,MAX_PATH,_T("Process 0x%x (%s) successfully hooked"),dwProcessID,ProcName);
    else
        _sntprintf(psz,MAX_PATH,_T("Process 0x%x successfully hooked"),dwProcessID);
    ReportMessage(psz,MSG_INFORMATION);

    if (*ProcName)
        _sntprintf(psz,MAX_PATH,_T("Application %s (0x%x) Ready to be Hooked in a paused state\r\nYou can now load Monitoring and Faking files\r\nPress OK after having loading files to Resume Application"),ProcName,dwProcessID);
    else
        _sntprintf(psz,MAX_PATH,_T("Application with process ID 0x%x Ready to be Hooked in a paused state\r\nYou can now load Monitoring and Faking files\r\nPress OK after having loading files to Resume Application"),dwProcessID);

    // enable monitoring faking interface to allow user to load monitoring and faking files 
    // before app resume
    EnableMonitoringFakingInterface(TRUE);

    // cool tricks: msgbox don't return until we click OK
    // --> lets application in suspended state until user close messagebox
    MessageBox(NULL,psz,_T("Information"),MB_OK|MB_ICONINFORMATION|MB_TOPMOST);
}

//-----------------------------------------------------------------------------
// Name: CallBackUnexpectedUnload
// Object: called after when a process is closed and we don't have unhook it
// Parameters :
//     in  : DWORD dwProcessID : process ID that has been closed
//           PVOID pUserParam : associated CApiOverride object pointer
//     out :
//     return : 
//-----------------------------------------------------------------------------
void CallBackUnexpectedUnload(DWORD dwProcessID,PVOID pUserParam)
{
    CApiOverride* pApiOverride;
    TCHAR psz[2*MAX_PATH];
    TCHAR ProcName[MAX_PATH];
    
    pApiOverride=(CApiOverride*)pUserParam;

    // if user have ask to exit do nothing
    if (bExit)
        return;

    // retreive process name before deleting pApiOverride object
    *ProcName=0;
    pApiOverride->GetProcessName(ProcName,MAX_PATH);

    // remove item from link list
    pApiOverrideList->RemoveItemFromItemData(&pApiOverride);
    // free item's memory
    delete pApiOverride;

    if (*ProcName)
        _sntprintf(psz,2*MAX_PATH,_T("Unload of application 0x%x (%s)"),dwProcessID,ProcName);
    else
        _sntprintf(psz,2*MAX_PATH,_T("Unload of application 0x%x"),dwProcessID);
    // report unload
    ReportMessage(psz,MSG_WARNING);

    if (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_ALL_PROCESS)!=BST_CHECKED)
    // show a message box only one process is hooked --> call stop
    {
        MessageBox(NULL,psz,_T("Warning"),MB_OK|MB_ICONWARNING|MB_TOPMOST);
        // stop
        CloseHandle(CreateThread(NULL,0,Stop,NULL,0,NULL));
    }
}

//-----------------------------------------------------------------------------
// Name: ClearLogs
// Object: clear the list view
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void ClearLogs()
{
    pListview->Clear();
}
//-----------------------------------------------------------------------------
// Name: LoadLogs
// Object: load previously saved logs into listview
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void LoadLogs()
{
    pListview->Load();
}
//-----------------------------------------------------------------------------
// Name: SaveLogs
// Object: save current listview logs
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void SaveLogs()
{
    pListview->Save();    
}

//-----------------------------------------------------------------------------
// Name: Exit
// Object: Ends application. Stop hooking if not done
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void Exit(INT_PTR nExitCode)
{
    bExit=TRUE;
    // if apioverriding is started, we have to stop it
    if (bStarted)
    {
        // hide window to avoid user interaction
        ShowWindow(mhWndDialog,FALSE);

        // Exit process is already started
        if (bExitingStop)// should not appear as the window is hidden
        {
            MessageBox(mhWndDialog,_T("WinapiOverride is exiting.\r\nPlease wait until it release all hooked processes"),_T("Information"),MB_OK|MB_ICONINFORMATION|MB_TOPMOST);
            // just let started Exit process do its job
            return;
        }

        // set the bExitingStop flag to TRUE (it will be used in Stop proc)
        bExitingStop=TRUE;
        // call the Stop proc in another thread as Stop func us DlgProc msg and we already are in DlgProc
        CloseHandle(CreateThread(NULL,0,Stop,NULL,0,NULL));
        // dialog exit will be done in the Stop thread calling this Exit func,
        // it's ok as bStarted will be to FALSE, so just return
        return;
    }

    // remove notify notification
    SetWindowLong(mhWndDialog, DWL_MSGRESULT, (LONG)wndOriginalListviewDWL_MSGRESULT);

    // delete listview object
    delete pListview;
    pListview=NULL;

    // end dialog
    EndDialog(mhWndDialog,nExitCode);
}

//-----------------------------------------------------------------------------
// Name: About
// Object: Show the about dialog
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void About()
{
    ShowAboutDialog(mhInstance,mhWndDialog);
}

//-----------------------------------------------------------------------------
// Name: SetApplicationsFilters
// Object: Show the filter dialog and apply new filters
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void SetApplicationsFilters()
{
    DWORD dwCnt;
    DWORD dwCnt2;
    TCHAR psz[2*MAX_PATH];
    DWORD dwArraySize;
    BOOL bItemSuccess;
    DWORD dwProcId;
    TCHAR ProcName[MAX_PATH];

    CApiOverride*** pApiOverrideArray;
    CApiOverride* pApiOverride;

    // if all process watching is started
    if (bStarted
        && (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_ALL_PROCESS)==BST_CHECKED)
        )
    {
        /////////////////////////////////////
        // update pFilters->pdwFiltersCurrentPorcessID 
        // with new hooked process before showing the filter dialog
        /////////////////////////////////////

        // delete current pFilters->pdwFiltersCurrentPorcessID array
        delete[] pFilters->pdwFiltersCurrentPorcessID;

        // get hooked running processes
        pApiOverrideArray=(CApiOverride***)pApiOverrideList->ToInsecureArray(&dwArraySize);

        // allocate memory for pdwFiltersCurrentPorcessID and fill pdwFiltersCurrentPorcessIDSize
        pFilters->pdwFiltersCurrentPorcessIDSize=dwArraySize;
        pFilters->pdwFiltersCurrentPorcessID=new DWORD[dwArraySize];

        // copy hooked ProcessId value into pFilters->pdwFiltersCurrentPorcessID
        for (dwCnt=0;dwCnt<dwArraySize;dwCnt++)
        {
            pApiOverride=(*pApiOverrideArray[dwCnt]);
            pFilters->pdwFiltersCurrentPorcessID[dwCnt]=pApiOverride->GetProcessID();
        }

        // delete pApiOverrideArray
        delete[] pApiOverrideArray;
    }

    /////////////////////////////////////
    // show the filter dialog
    /////////////////////////////////////
    if (pFilters->ShowFilterDialog(mhInstance,mhWndDialog)!=FILTERS_DLG_RES_OK)
        return;

    // if all process watching is not started
    if ((!bStarted)
        || (IsDlgButtonChecked(mhWndDialog,IDC_RADIO_ALL_PROCESS)!=BST_CHECKED)
        )
        return;

    /////////////////////////////////////
    // if Started apply changes immediately by hooking new process
    // and unhooking no more used process
    /////////////////////////////////////


    ///////////////////////////////////
    // Hook new processes
    ///////////////////////////////////

    // if api override is stared, hook new process and release no more wanted one
    for(dwCnt=0;dwCnt<pFilters->pdwFiltersNewPorcessIDToWatchSize;dwCnt++)
    {
        // if process is no more alive
        if (!CProcessHelper::IsAlive(pFilters->pdwFiltersNewPorcessIDToWatch[dwCnt]))
        {
            _sntprintf(psz,MAX_PATH,_T("Process 0x%x don't exists anymore, so it won't be hooked"),pFilters->pdwFiltersCurrentPorcessID[dwCnt]);
            ReportMessage(psz,MSG_WARNING);

            continue;
        }

        // create a new CApiOverride object
        pApiOverride=new CApiOverride();
        // set the unexpected unload callback
        pApiOverride->SetUnexpectedUnload(CallBackUnexpectedUnload,pApiOverride);
        // give handle of listview to pApiOverride so we don't worry about logging monitoring msg
        pApiOverride->SetMonitoringListview(pListview);

        if (pApiOverride->Start(pFilters->pdwFiltersNewPorcessIDToWatch[dwCnt]))
        {
            // set monitoring and faking state
            if (!bMonitoring)
                pApiOverride->StopMonitoring();
            if (!bFaking)
                pApiOverride->StopFaking();

            // add it to pApiOverrideList
            pApiOverrideList->AddItem(&pApiOverride);

            // add information to listview
            *ProcName=0;
            pApiOverride->GetProcessName(ProcName,MAX_PATH);
            if (*ProcName)
                _sntprintf(psz,MAX_PATH,_T("Process 0x%x (%s) hooked successfully"),pFilters->pdwFiltersCurrentPorcessID[dwCnt],ProcName);
            else
                _sntprintf(psz,MAX_PATH,_T("Process 0x%x hooked successfully"),pFilters->pdwFiltersCurrentPorcessID[dwCnt]);
            ReportMessage(psz,MSG_INFORMATION);
        }
        else
        {
            // add information to listview
            if (CProcessHelper::GetProcessName(pFilters->pdwFiltersCurrentPorcessID[dwCnt],ProcName,MAX_PATH))
                _sntprintf(psz,MAX_PATH,_T("Error hooking process 0x%x (%s)"),pFilters->pdwFiltersCurrentPorcessID[dwCnt],ProcName);
            else
                _sntprintf(psz,MAX_PATH,_T("Error hooking process 0x%x"),pFilters->pdwFiltersCurrentPorcessID[dwCnt]);

            ReportMessage(psz,MSG_ERROR);

            // delete pApiOverride object
            delete pApiOverride;        
        }
    }

    ///////////////////////////////////
    // unhook no more wanted processes
    ///////////////////////////////////

    pApiOverrideArray=(CApiOverride***)pApiOverrideList->ToInsecureArray(&dwArraySize);

    // loop throught pApiOverrideList
    for (dwCnt=0;dwCnt<dwArraySize;dwCnt++)
    {
        pApiOverride=(*pApiOverrideArray[dwCnt]);

        // retreiv proc id
        dwProcId=pApiOverride->GetProcessID();

        // search proc to release throw pFilters->pdwFiltersPorcessIDToRelease
        for(dwCnt2=0;dwCnt2<pFilters->pdwFiltersPorcessIDToReleaseSize;dwCnt2++)
        {
            if (dwProcId!=pFilters->pdwFiltersPorcessIDToRelease[dwCnt2])
                continue;

            // else
            // process to unhook as been found

            // stop hook
            bItemSuccess=pApiOverride->Stop();
            if (bItemSuccess)
            {
                // add information to listview

                *ProcName=0;
                pApiOverride->GetProcessName(ProcName,MAX_PATH);
                if (*ProcName)
                    _sntprintf(psz,MAX_PATH,_T("Process 0x%x (%s) unhooked successfully"),dwProcId,ProcName);
                else
                    _sntprintf(psz,MAX_PATH,_T("Process 0x%x unhooked successfully"),dwProcId);
                ReportMessage(psz,MSG_INFORMATION);

                // remove item from link list
                pApiOverrideList->RemoveItemFromItemData(&pApiOverride);
                // delete CApiOverride object
                delete pApiOverride;
            }
            else
            {
                // add information to listview

                if (CProcessHelper::GetProcessName(dwProcId,ProcName,MAX_PATH))
                    _sntprintf(psz,MAX_PATH,_T("Error unhooking process 0x%x (%s)"),dwProcId,ProcName);
                else
                    _sntprintf(psz,MAX_PATH,_T("Error unhooking process 0x%x"),dwProcId);
                ReportMessage(psz,MSG_ERROR);
            }
            // item is found, so go out of 2nd loop
            break;
        }
    }

    // delete pApiOverrideArray
    delete[] pApiOverrideArray;

}

//-----------------------------------------------------------------------------
// Name: ReportMessage
// Object: add an information message into the listview
// Parameters :
//     in  : TCHAR* pszMsg : message to display
//     out : tagMsgTypes MsgType : type of message (inforamtion, error , warning)
//     return : 
//-----------------------------------------------------------------------------
void ReportMessage(TCHAR* pszMsg, tagMsgTypes MsgType)
{
    TCHAR** pszListViewMessage;
    pszListViewMessage=new TCHAR*[2];
    
    switch(MsgType)
    {
    case MSG_INFORMATION:
        pszListViewMessage[0]=_tcsdup(LISTVIEW_ITEM_TEXT_INFORMATION);
        break;
    case MSG_WARNING:
        pszListViewMessage[0]=_tcsdup(LISTVIEW_ITEM_TEXT_WARNING);
        break;
    case MSG_ERROR:
        pszListViewMessage[0]=_tcsdup(LISTVIEW_ITEM_TEXT_ERROR);
        break;
    default:
        pszListViewMessage[0]=_tcsdup(_T(""));
        break;
    }
    pszListViewMessage[1]=pszMsg;

    // add item to listview
    pListview->AddItemAndSubItems(2,pszListViewMessage,TRUE);

    // free memory
    free(pszListViewMessage[0]);
    delete[] pszListViewMessage;
}

//-----------------------------------------------------------------------------
// Name: MakePrettyName
// Object: getting full path and return "filename | path"
// Parameters :
//     inout  : TCHAR* pszFileName full path name
//     return : 
//-----------------------------------------------------------------------------
void MakePrettyName(TCHAR* pszFileName)
{
    TCHAR pszFile[MAX_PATH];
    TCHAR* psz;

    *pszFile=0;
    // search last '\\'
    psz=_tcsrchr(pszFileName,'\\');
    psz++;
    // copy filename
    _tcscpy(pszFile,psz);
    _tcscat(pszFile,_T(" | "));
    psz--;
    *psz=0;
    // copy path
    _tcscat(pszFile,pszFileName);
    // put filename | path to pszFileName
    _tcscpy(pszFileName,pszFile);
}

//-----------------------------------------------------------------------------
// Name: DecodePrettyName
// Object: getting "filename | path" and return full path
// Parameters :
//     inout  : TCHAR* pszFileName full path name
//     return : 
//-----------------------------------------------------------------------------
void DecodePrettyName(TCHAR* pszFileName)
{
    TCHAR pszFile[MAX_PATH];
    TCHAR* psz;

    *pszFile=0;
    // search first '|' (filname can't contain "|")
    psz=_tcschr(pszFileName,'|');
    psz+=2;
    // copy path
    _tcscpy(pszFile,psz);
    psz-=3;
    *psz=0;
    // copy filename
    _tcscat(pszFile,_T("\\"));
    _tcscat(pszFile,pszFileName);
    // put fullpath to pszFileName
    _tcscpy(pszFileName,pszFile);
}

//-----------------------------------------------------------------------------
// Name: EnableMonitoringFakingInterface
// Object: Enable / disable monitoring and faking user interface
// Parameters :
//     in  : BOOL bEnable : TRUE to enable, FALSE to disable
//     out :
//     return : 
//-----------------------------------------------------------------------------
void EnableMonitoringFakingInterface(BOOL bEnable)
{
    ///////////////////////////////////////////
    // lower interface enabling/disabling
    ///////////////////////////////////////////
    EnableWindow(GetDlgItem(mhWndDialog,IDC_EDIT_MONITORING_FILE),bEnable);
    EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_BROWSE_MONITORING_FILE),bEnable);
    EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_LOAD_MONITORING_FILE),bEnable);
    EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_UNLOAD_MONITORING_FILE),bEnable);
    EnableWindow(GetDlgItem(mhWndDialog,IDC_LIST_MONITORING_FILES),bEnable);

    EnableWindow(GetDlgItem(mhWndDialog,IDC_EDIT_FAKING_FILE),bEnable);
    EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_BROWSE_FAKING_FILE),bEnable);
    EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_LOAD_FAKING_FILE),bEnable);
    EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_UNLOAD_FAKING_FILE),bEnable);
    EnableWindow(GetDlgItem(mhWndDialog,IDC_LIST_FAKING_FILES),bEnable);

    EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_START_STOP_MONITORING),bEnable);
    EnableWindow(GetDlgItem(mhWndDialog,IDC_BUTTON_START_STOP_FAKING),bEnable);

    ///////////////////////////////////////////
    // upper interface part enabling/disabling
    ///////////////////////////////////////////
    // we do the inverse of lower part
    bEnable=!bEnable;
    // lock unlock radios
    EnableWindow(GetDlgItem(mhWndDialog,IDC_RADIO_ALL_PROCESS),bEnable);
    EnableWindow(GetDlgItem(mhWndDialog,IDC_RADIO_APP_NAME),bEnable);
    EnableWindow(GetDlgItem(mhWndDialog,IDC_RADIO_PID),bEnable);
}

//-----------------------------------------------------------------------------
// Name: BrowseApplicationPath
// Object: open browse dialog for selecting application to start
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void BrowseApplicationPath()
{
    TCHAR pszFile[MAX_PATH]=_T("");

    // open dialog
    OPENFILENAME ofn;
    memset(&ofn,0,sizeof (OPENFILENAME));
    ofn.lStructSize=sizeof (OPENFILENAME);
    ofn.hwndOwner=mhWndDialog;
    ofn.hInstance=mhInstance;
    ofn.lpstrFilter=_T("exe\0*.exe\0");
    ofn.nFilterIndex = 1;
    ofn.Flags=OFN_EXPLORER|OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST;
    ofn.lpstrDefExt=_T("exe");
    ofn.lpstrFile=pszFile;
    ofn.nMaxFile=MAX_PATH;
    
    if (!GetOpenFileName(&ofn))
        return;

    SetDlgItemText(mhWndDialog,IDC_EDIT_APP_PATH,pszFile);
}

//-----------------------------------------------------------------------------
// Name: BrowseMonitoringFile
// Object: open browse dialog for loading monitoring files
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void BrowseMonitoringFile()
{
    TCHAR pszFile[MAX_PATH]=_T("");

    // open dialog
    OPENFILENAME ofn;
    memset(&ofn,0,sizeof (OPENFILENAME));
    ofn.lStructSize=sizeof (OPENFILENAME);
    ofn.hwndOwner=mhWndDialog;
    ofn.hInstance=mhInstance;
    ofn.lpstrFilter=_T("txt\0*.txt\0");
    ofn.nFilterIndex = 1;
    ofn.Flags=OFN_EXPLORER|OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST;
    ofn.lpstrDefExt=_T("txt");
    ofn.lpstrFile=pszFile;
    ofn.nMaxFile=MAX_PATH;
    
    if (!GetOpenFileName(&ofn))
        return;

    SetDlgItemText(mhWndDialog,IDC_EDIT_MONITORING_FILE,pszFile);
}

//-----------------------------------------------------------------------------
// Name: BrowseFakingFile
// Object: open browse dialog for loading faking dll
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
void BrowseFakingFile()
{
    TCHAR pszFile[MAX_PATH]=_T("");

    // open dialog
    OPENFILENAME ofn;
    memset(&ofn,0,sizeof (OPENFILENAME));
    ofn.lStructSize=sizeof (OPENFILENAME);
    ofn.hwndOwner=mhWndDialog;
    ofn.hInstance=mhInstance;
    ofn.lpstrFilter=_T("dll\0*.dll\0");
    ofn.nFilterIndex = 1;
    ofn.Flags=OFN_EXPLORER|OFN_PATHMUSTEXIST|OFN_FILEMUSTEXIST;
    ofn.lpstrDefExt=_T("dll");
    ofn.lpstrFile=pszFile;
    ofn.nMaxFile=MAX_PATH;
    
    if (!GetOpenFileName(&ofn))
        return;

    SetDlgItemText(mhWndDialog,IDC_EDIT_FAKING_FILE,pszFile);
}

//-----------------------------------------------------------------------------
// Name: ProcessCustomDraw
// Object: Handle NM_CUSTOMDRAW message (set listview colors)
// Parameters :
//     in  : 
//     out :
//     return : 
//-----------------------------------------------------------------------------
LRESULT ProcessCustomDraw (LPARAM lParam)
{
    LPNMLVCUSTOMDRAW lplvcd = (LPNMLVCUSTOMDRAW)lParam;
    TCHAR ItemText[LISTVIEW_ITEM_TEXT_MESSAGE_MAX_SIZE+1];

    switch(lplvcd->nmcd.dwDrawStage) 
    {
        case CDDS_PREPAINT : //Before the paint cycle begins
            //request notifications for individual listview items
            return CDRF_NOTIFYITEMDRAW;
			
        case CDDS_ITEMPREPAINT: //Before an item is drawn
            // lplvcd->nmcd.dwItemSpec // item index
            // to request notification for subitems
            // return CDRF_NOTIFYSUBITEMDRAW;
            if (!pListview)
                return CDRF_DODEFAULT;

            // retreive item text
            pListview->GetItemText((int)lplvcd->nmcd.dwItemSpec,ItemText,LISTVIEW_ITEM_TEXT_MESSAGE_MAX_SIZE+1);
            
            // check len of item to avoid doing compare if not requiered
            if (_tcslen(ItemText)>LISTVIEW_ITEM_TEXT_MESSAGE_MAX_SIZE)
                return CDRF_DODEFAULT;

            //customize item appearance
            if (_tcscmp(ItemText,LISTVIEW_ITEM_TEXT_INFORMATION)==0)
            {
                // set a forground color
                lplvcd->clrText   = ListViewColorInformation;
            }
            else
            if (_tcscmp(ItemText,LISTVIEW_ITEM_TEXT_ERROR)==0)
            {
                // set a forground color
                lplvcd->clrText   = ListViewColorError;
            }
            else
            if (_tcscmp(ItemText,LISTVIEW_ITEM_TEXT_WARNING)==0)
            {
                // set a forground color
                lplvcd->clrText   = ListViewColorWarning;
            }
            else
            {
                return CDRF_DODEFAULT;
            }

            // set a background color
            lplvcd->clrTextBk = ListViewColorMessageBackGround;

            //To set a custom font:
            //SelectObject(lplvcd->nmcd.hdc, <your custom HFONT>);

            return CDRF_NEWFONT;
	
        // for subitem customization
        // case CDDS_SUBITEM | CDDS_ITEMPREPAINT: //Before a subitem is drawn
            // lplvcd->nmcd.dwItemSpec // item number
            // lplvcd->iSubItem // subitem number
            // // customize subitem appearance for column 0
            // lplvcd->clrText   = RGB(255,255,255);
            // lplvcd->clrTextBk = RGB(255,0,0);
            // //To set a custom font:
            // //SelectObject(lplvcd->nmcd.hdc, <your custom HFONT>);

            // return CDRF_NEWFONT;
    }
    return CDRF_DODEFAULT;
}