;Definizioni delle costati usate nel progetto

;************************************************************************
;*	MASCHERE DEL PROTOCOLLO												*
;************************************************************************
HEADER_COMM		equ	'C'		;Codice dell'header comando
HEADER_DATO		equ	'D'		;Codice dell'header dato
STOP_BYTE		equ	'S'		;Codice del byte di stop
RET_COUNT		equ	'c'		;Codice istruzione
RUN_ACQ			equ	'a'		;Codice istruzione
STOP_ACQ		equ	'A'		;Codice istruzione
READ_TO_MEM		equ	'r'		;Codice istruzione
WRITE_TO_MEM	equ	'w'		;Codice istruzione
BIT_WORD		equ 'b'		;Codice istruzione
RESET			equ	'R'		;Codice istruzione
;************************************************************************
;*																		*
;************************************************************************

;************************************************************************
;*	PIN DEFINE PORT USATI 												*
;************************************************************************
FEEDBACK		equ 	3		;RB3 gestisce la retroazione termica;OUT
ISVALID			equ 	4		;RB4 segnala che sta per arrivare una sequenza valida;OUT
INRANDOM		equ		5		;RB5 legge il dato del generatore random hardware;IN
;************************************************************************
;*																		*
;************************************************************************

;************************************************************************
;*	PARAMETRI CASCINE   												*
;************************************************************************
;imposta la velocit della seriale
VELOCITA		equ 	25
				;BRGH= 1 ---> BR= f/(16(x+1));f= 4 MHz;x= VELOCITA
				;25 --> 9600 baud  @4MHz
				;6  --> 32.6 Kbaud @4MHz
				;3  --> 55.6 Kbaud @4MHz
;imposta il valore di matching del timer2
MATCHING		equ 	250
				;impulso alla frequenza di Fosc/4*1/1*1/2*1/MATCHING
				;250 --> fornisce un impulso corrispondente alla frequenza di 2  Khz @4Mhz
				;125 --> fornisce un impulso corrispondente alla frequenza di 4  Khz @4Mhz
				;62  --> fornisce un impulso corrispondente alla frequenza di 8,064 Khz @4Mhz
				;50  --> fornisce un impulso corrispondente alla frequenza di 10  Khz @4Mhz
				;25  --> fornisce un impulso corrispondente alla frequenza di 20 Khz @4Mhz
;************************************************************************
;*																		*
;************************************************************************

;************************************************************************
;*	COSTANTI USATE DAL PROTOCOLLO DI COMUNICAZIONE	    				*
;************************************************************************
ERR				equ	0	;STAPRO.0 -> ERR		errore nel protocollo
HEADER			equ	1	;STAPRO.1 -> HEADER		verificato header valido
RDATA			equ	2	;STAPRO.2 -> RDATA		verificata istr che legge dato
BYTPAC			equ	3	;STAPRO.3 -> BYTPAC		verificato byte lung. istr/dato
PARS			equ	5	;STAPRO.5 -> PARS		effettuato il parsing
WDATA			equ	6	;STAPRO.6 -> WDATA		verificata istr che scrive dato
BUSY			equ	7	;STAPRO.7 -> BUSY		il pic non riceve comandi/dati

STARTACQ		equ	0	;FLAGREG.0 -> STARTACQ	start della routine di acquisizione
RETCOU			equ	1	;FLAGREG.1 -> RETCOU	start comando ritorno contatori
STAACQ			equ	2	;FLAGREG.2 -> STAACQ	start comando acquisizione
STOACQ			equ	3	;FLAGREG.3 -> STOACQ	stop comando acquisizione
RST				equ	4	;FLAGREG.4 -> RST		start comando reset

;************************************************************************
;*																		*
;************************************************************************

;************************************************************************
;*	CODICI ERRORE														*
;************************************************************************
ERRORE			equ	'e'						;Codice di errore

;10		Header non valido
LowHeaderError 					equ '1'		;il modulo di ricezione viene predisposto per accettare un nuovo comando oppure
HighHeaderError 				equ '0'		;dato (se il comando precedente prevedeva un scambio dati)

;11 	Lunghezza errata
LowDataLenMisError 				equ '1'		;Mismatch tra lunghezza dato prevista e quella effettiva								
HighDataLenMisError 			equ '1'		;il modulo di ricezione viene predisposto per accettare un nuovo dato

;12		Lunghezza pacchetto non valida
LowPacketLenghtError			equ '1'		;il modulo di ricezione viene predisposto per accettare un nuovo comando oppure
HighPacketLenghtError 			equ '2'		;dato (se il comando precedente prevedeva un scambio dati)

;13		Pacchetto dato inatteso
LowDataSeReError 				equ '1'		;il pacchetto ricevuto non era atteso
HighDataSeReError 				equ '3'		;il modulo di ricezione viene predisposto per accettare un nuovo comando

;14		Pacchetto comando inatteso
LowCommSeReError 				equ '1'		;il modulo di ricezione viene predisposto per accettare un nuovo comando
HighCommSeReError 				equ '4'		;se il comando precedente prevedeva un scambio dati, tale comando verr ignorato

;15		Memoria non valida
LowOutOfDataMemoryError 		equ '1'		;si tenta di accedere all' esterno della memoria dati valida
HighOutOfDataMemoryError 		equ '5'		;il modulo di ricezione viene predisposto per accettare un nuovo comando

;16		Errore USART
LowUsartError 					equ '1'		;l' usart ha generato un errore
HighUsartError 					equ '6'		;il modulo di ricezione viene predisposto per accettare un nuovo comando

;17		Contatore in overflow
LowCounterOverflowError 		equ '1'		;un contatore ha generato un overflow
HighCounterOverflowError 		equ '7'		;l'acquisizione si ferma mantenendo i valori di conteggio memorizzati

;18		Busy
LowBusyError 					equ '1'		;si  fatta una richiesta ma la periferica  occupata
HighBusyError 					equ '8'		;ricezione bloccata fino al termine della condizione di busy

;19		Stop byte atteso e non ricevuto
LowStopByteError 				equ '1'		;il modulo di ricezione viene predisposto per accettare un nuovo comando oppure
HighStopByteError 				equ '9'		;dato (se il comando precedente prevedeva un scambio dati)

;1A		Comando non valido
LowCommandError 				equ '1'		;Comando non valido
HighCommandError 				equ 'A'		;il modulo di ricezione viene predisposto per accettare un nuovo comando

;1B		Lunghezza del dato errata
LowDataLenCommError 			equ '1'		;il modulo di ricezione viene predisposto per accettare un nuovo comando
HighDataLenCommError 			equ 'B'		;(max 64 bytes a comando in r/w)

;1C		Lunghezza simbolo errata
LowSymbLenError  				equ '1'		;Lunghezza simbolo errata
HighSymbLenError  				equ 'C'		;il modulo di ricezione viene predisposto per accettare un nuovo comando
;************************************************************************
;*																		*
;************************************************************************