;Definizioni delle funzioni usate nel progetto

InitAll
	movlw	0x01				;Viene impostato il valore di default di BITWORD (1)
	movwf	BITWORD
	return
;-----------------------------------------------------------------------

BitsAcqInit
	call	DataMemReset
	clrf	BITCOUNT
	return
;-----------------------------------------------------------------------

ProAllReset
	clrf	STAPRO
	clrf	BUFLEN				;Reset completo dello stato del protocollo (attesa nuovo comando).
	clrf	LENPAC
	clrf	TEMPREG
	clrf	BUFREAD
	clrf	DML
	clrf	DMC
	clrf	DATMEM
	return
;-----------------------------------------------------------------------

ProDataReset
	movlw	b'01000100'			;Reset dello stato del protocollo relativamente al comando/dato corrente
	andwf	STAPRO,1			; (attesa nuovo comando/dato).
	clrf	BUFLEN
	clrf	LENPAC
	clrf	TEMPREG
	clrf	BUFREAD
	clrf	DMC
	return
;-----------------------------------------------------------------------
CounterOverflowError
	bcf		T2CON,TMR2ON		;Spegnimento del Timer2 ed azzeramento
	bcf		FLAGREG,STARTACQ	;Disabilitazione routine di acquisizione
	movlw	ERRORE				;Invio del codice di errore '17'
	movwf	TXREG
	movlw	LowCounterOverflowError
	call	TxWait
	movwf	TXREG
	movlw	HighCounterOverflowError
	call	TxWait
	movwf	TXREG
	return

;-----------------------------------------------------------------------


WaitNewByte
Loop0
	movf	BUFLEN,w			;Controllo della condizione BUFLEN = 0: se valida viene effettuato
	btfsc	STATUS,Z			; il salto a Jump21, diversamente si prosegue
	goto	Jump21
	movf	BUFLEN,w			;Controllo della condizione BUFLEN - BUFREAD: se il risultato  > 0
	bcf		STATUS,C			; avviene il ritorno, diversamente si prosegue
	subwf	BUFREAD,w
	btfss	STATUS,C
	return
Jump21
	btfsc	FLAGREG,STARTACQ	;Se STARTACQ = 1, la routine di acquisizione viene chiamata
	call	AcqBits
	goto	Loop0
;-----------------------------------------------------------------------

BinNumToASCII
	clrf	CENTINAIA			;Azzeramento del contatore
Loop11
	incf	CENTINAIA,f			;Memorizza le sottrazioni di 100		
	addlw	-d'100'				;Sottrazione di 100 -> se carry = 0 si ha l'uscita dal
	btfsc	STATUS,C			; loop (ottenuto un numero negativo), se = 1 viene
	goto	Loop11				; eseguito un altro conteggio/sottrazione
	decf	CENTINAIA,f			;Compensazione dell'ultimo incremento
	addlw	d'100'				;Compensazione dell'ultima sottrazione
	clrf	DECINE				;Azzeramento del contatore
Loop12
	incf	DECINE,f			;Memorizza le sottrazioni di 10
	addlw	-d'10'				;Sottrazione di 10 -> se carry = 0 si ha l'uscita dal
	btfsc	STATUS,C			; loop (ottenuto un numero negativo), se = 1 viene
	goto	Loop12				; eseguito un altro conteggio/sottrazione
	decf	DECINE,f			;Compensazione dell'ultimo incremento
	addlw	d'10'				;Compensazione dell'ultima sottrazione
	movwf	UNITA				;In w rimangono le unit

	movlw	D'48'				;Somma dell'offset adeguato per la rappresentazione
	addwf	CENTINAIA,f			; in ASCII
	addwf	DECINE,f
	addwf	UNITA,f
	
	return
;-----------------------------------------------------------------------

TxWait
	bcf		INTCON,GIE
	bsf		STATUS,RP0
	bcf		STATUS,RP1
Loop1
	btfss	TXSTA,TRMT			;Finch TMRT  settato il buffer di trasmissione  pieno -> attesa
	goto	Loop1
	bcf		STATUS,RP0
	bsf		INTCON,GIE
	return
;-----------------------------------------------------------------------


SendACK
	movlw	'A'				;Invio del codice "ACK", quale comprensione del comando ricevuto
	movwf	TXREG
	call	TxWait

	movlw	'C'
	movwf	TXREG
	call	TxWait

	movlw	'V'
	movwf	TXREG
	call	TxWait

	return
;-----------------------------------------------------------------------

ClearAll
	bcf		STATUS,IRP
	movlw	0x20				;Settaggio degli indirizzi di inizio e (fine + 1) memoria del banco 0
	movwf	FSR
	movlw	0x7F
	movwf	TEMPREG
Loop8
	clrf	INDF				;Routine di azzeramento della porzione di memoria
	incf	FSR,f
	movf	TEMPREG,w
	bcf		STATUS,Z
	subwf	FSR,w
	btfsc	STATUS,Z
	goto	Jump22
	goto	Loop8
Jump22
	movlw	0xA0				;Settaggio degli indirizzi di inizio e (fine + 1) memoria del banco 1
	movwf	FSR
	movlw	0xF0
	movwf	TEMPREG
Loop7
	clrf	INDF				;Routine di azzeramento della porzione di memoria
	incf	FSR,f
	movf	TEMPREG,w
	bcf		STATUS,Z
	subwf	FSR,w
	btfss	STATUS,Z
	goto	Loop7
	call	DataMemReset

	return
;-----------------------------------------------------------------------

DataMemReset
	bsf		STATUS,IRP
	movlw	0x10				;Settaggio degli indirizzi di inizio e (fine + 1) memoria del banco 2
	movwf	FSR
	movlw	0x70
	movwf	TEMPREG
Loop4
	clrf	INDF				;Routine di azzeramento della porzione di memoria
	incf	FSR,f
	movf	TEMPREG,w
	bcf		STATUS,Z
	subwf	FSR,w
	btfsc	STATUS,Z
	goto	Jump19
	goto	Loop4
Jump19
	movlw	0x20				;Salto al banco 3 sommando un offset adeguato (20h) per inziare l'exec
	addwf	FSR,f				; della routine alla locazione 190h
	movlw	0xE5				;Locazione di fine + 1 (1E5h:1EFh sono riservate) del banco 3
	movwf	TEMPREG
Loop5
	clrf	INDF				;Routine di azzeramento della porzione di memoria
	incf	FSR,f
	movf	TEMPREG,w
	bcf		STATUS,Z
	subwf	FSR,w
	btfss	STATUS,Z
	goto	Loop5
	bcf		STATUS,IRP
	clrf	0x7F				;Azzeramento di TEMPREG (va fatto per ultimo)
	return
;-----------------------------------------------------------------------


;************************************************************************
;*	ROUTINE DI ACQUISIZIONE												*
;************************************************************************

AcqBits
	return
;-----------------------------------------------------------------------


DatMemOk
	movf	DATMEM,w			;Controllo della condizione DATMEM>=Data_Start e
	movwf	TEMPREG				; DATMEM<=Data_End
	movlw	Data_Start
	bcf		STATUS,C
	subwf	TEMPREG,w
	btfsc	STATUS,C
	goto	Jump16
	call	OutOfDataMemoryError
	return
Jump16
	movf	TEMPREG,w
	bcf		STATUS,C
	sublw	Data_End
	btfss	STATUS,C	
	call	OutOfDataMemoryError
	return
;-----------------------------------------------------------------------

InMemory
	decf	DML,1				;(64 bytes -> da 0 a 63)
	movlw	0x40				;(64)
	bcf		STATUS,C			;Viene verificata la condizione DML<=64 (DML-64 -> C sempre 0)
	subwf	DML,w
	btfss	STATUS,C
	goto	Jump7
	call	DataLenCommError
	return
Jump7
	movf	DATMEM,w			;Viene verificata la condizione (DATMEM+DML)>Data_Start
	addwf	DML,w
	movwf	TEMPREG
	movlw	Data_Start
	bcf		STATUS,C
	subwf	TEMPREG,w
	btfsc	STATUS,C
	goto	Jump10
	call	OutOfDataMemoryError
	return
Jump10
	movlw	Data_End			;Viene verificata la condizione (DATMEM+DML)<(Data_End+1)
	addlw	0x01
	bcf		STATUS,C
	subwf	TEMPREG,w
	btfsc	STATUS,C
	call	OutOfDataMemoryError
	incf	DML,1
	return
;-----------------------------------------------------------------------

UsartError
	call	ProAllReset
	SendError LowUsartError, HighUsartError
	return
;-----------------------------------------------------------------------

HeaderError
	call	ProDataReset
	SendError LowHeaderError, HighHeaderError
	return
;-----------------------------------------------------------------------

OutOfDataMemoryError
	call	ProAllReset
	SendError LowOutOfDataMemoryError, HighOutOfDataMemoryError
	return
;-----------------------------------------------------------------------

CommSeReError
	call	ProAllReset	
	SendError LowCommSeReError, HighCommSeReError
	return
;-----------------------------------------------------------------------

DataSeReError
	call	ProAllReset	
	SendError LowDataSeReError, HighDataSeReError
	return
;-----------------------------------------------------------------------

DataLenMisError
	call	ProDataReset
	SendError LowDataLenMisError, HighDataLenMisError
	return
;-----------------------------------------------------------------------

DataLenCommError
	call	ProAllReset
	SendError LowDataLenCommError, HighDataLenCommError
	return
;-----------------------------------------------------------------------

SymbLenError
	call	ProAllReset
	SendError LowSymbLenError, HighSymbLenError 
	return
;-----------------------------------------------------------------------

PacketLenghtError
	call	ProDataReset
	SendError LowPacketLenghtError, HighPacketLenghtError
	return
;-----------------------------------------------------------------------

BusyError
	SendError LowBusyError,HighBusyError
	return
;-----------------------------------------------------------------------

CommandError
	call	ProAllReset
	SendError LowCommandError, HighCommandError 
	return
;-----------------------------------------------------------------------

StopByteError
	btfss	STAPRO,WDATA
	goto	Jump29
	call	ProAllReset

	goto	Jump28
Jump29	
	btfss	STAPRO,RDATA
	goto	Jump30
	call	ProAllReset
	goto	Jump28
Jump30
	call	ProDataReset
Jump28
	SendError LowStopByteError, HighStopByteError
	return

;-----------------------------------------------------------------------